@@ -12,7 +12,7 @@
 
 (set-verify-guards-eagerness 2)
 (include-book "std/util/bstar" :dir :system)
-(include-book "../utilities")
+(include-book "acl2s/utilities" :dir :system)
 ;(include-book "basis")
 
 ;; PETE: add the global cgen::cgen-guard-checking to control how
@@ -58,17 +58,6 @@
     (cons (modify-symbol prefix (car syms) postfix)
           (modify-symbol-lst prefix (cdr syms) postfix))))
 
-
-
-; utility fn to print if verbose flag is true
-(defmacro cw? (verbose-flag &rest rst)
-  `(if ,verbose-flag
-     (cw ,@rst)
-     nil))
-
-
-
-
 (defmacro   debug-flag  (vl)
   `(> ,vl 3))
 
@@ -81,7 +70,6 @@
 (defmacro   verbose-flag ( vl)
   `(> ,vl 1))
 
-
 (defmacro   inhibit-output-flag ( vl)
   `(<= ,vl 0))
 
@@ -1162,6 +1150,7 @@ Mainly to be used for evaluating enum lists "
 
 (defun is-var-equality-hyp (term)
   (and (equal (len term) 3)
-       (member-equal (car term) '(EQUAL EQ EQL = INT= STRING-EQUAL ACL2::HONS-EQUAL))
-       (or (proper-symbolp (second term))
-           (proper-symbolp (third term)))))
+       (member-equal (car term) '(EQUAL EQ EQL = INT= STRING-EQUAL ACL2::HONS-EQUAL acl2s::==))
+       (and (or (proper-symbolp (second term))
+                (proper-symbolp (third term)))
+            (not (equal (second term) (third term))))))
