@@ -12,6 +12,7 @@
 (include-book "defunc" :ttags :all)
 (include-book "definec" :ttags :all)
 (include-book "defintrange" :ttags :all)
+(include-book "acl2s-sigs" :ttags :all)
 
 ;; (defun allp (x)
 ;;   (declare (ignore x)
@@ -239,6 +240,10 @@
   ;;Cgen settings
   (acl2s::acl2s-defaults :set acl2s::testing-enabled t)
   (acl2s::acl2s-defaults :set acl2s::num-trials 500)
+  (acl2s::acl2s-defaults :set acl2s::use-fixers t)
+
+  ;; Defdata settings
+  (acl2s::acl2s-defaults :set acl2s::defdata-aliasing-enabled t)
   ))
 
 #!ACL2
