@@ -165,12 +165,6 @@
   (declare (xargs :guard (symbolp sym)))
   (acl2s::fix-intern-in-pkg-of-sym (symbol-name sym) :a))
 
-; utility fn to print if verbose flag is true
-(defmacro cw? (verbose-flag &rest rst)
-  `(if ,verbose-flag
-     (cw ,@rst)
-     nil))
-
 (defmacro acl2-getprop (name prop w &key default)
  `(getprop ,name ,prop ,default 'acl2::current-acl2-world ,w))
 
@@ -201,9 +195,14 @@
                 (booleanp x);t or nil
                 (acl2::legal-constantp x)))))
 
+(defthm keyword-list-is-symbol-list
+  (implies (keyword-listp x)
+           (symbol-listp x))
+  :rule-classes (:forward-chaining))
+
 (defthm proper-symbol-is-symbol
-  (and (implies (proper-symbolp x)
-                (symbolp x)))
+  (implies (proper-symbolp x)
+           (symbolp x))
   :rule-classes (:compound-recognizer
                  :forward-chaining))
 
@@ -658,7 +657,8 @@ see (defdata foo rational)
                (and lst (cadr lst))))
        (- (cw "~%")))
     `(with-output
-      ,@(and (not verbosep) '(:off :all :on (summary error) :summary (acl2::form acl2::time)))
+      ,@(and (not verbosep)
+             '(:off :all :on (summary error) :summary-off (:other-than acl2::form acl2::time)))
       :gag-mode t :stack :push
       (make-event
        (b* ((pkg (current-package state))
@@ -666,8 +666,10 @@ see (defdata foo rational)
             (A (type-alias-table (w state)))
             (pred (if ',pred ',pred (make-predicate-symbol ',alias pkg)))
             (type (base-alias-type ',type A))
-            (predicate (acl2s::get-alist
-                        :predicate (acl2s::get-alist type M)))
+            (type-alist (acl2s::get-alist type M))
+            (predicate (acl2s::get-alist :predicate type-alist))
+            (def (acl2s::get-alist :def type-alist))
+            (record? (and (consp def) (equal 'record (car def))))
             (base-enum (enumerator-name type A M))
             (base-enum/acc (enum/acc-name type A M))
             (alias-enum (acl2s::make-symbl `(nth- ,',alias) pkg))
@@ -676,7 +678,10 @@ see (defdata foo rational)
             (seed (acl2s::fix-intern$ "SEED" pkg))
             ((unless predicate)
              (er hard 'defdata-alias
-                 "~%**Unknown type**: ~x0 is not a known type name.~%" ',type)))
+                 "~%**Unknown type**: ~x0 is not a known type name.~%" ',type))
+            ((when record?)
+             (er hard 'defdata-alias
+                 "~%**Record type**: ~x0 is a record and records cannot be aliased.~%" ',type)))
          `(encapsulate
            ()
            (table type-alias-table
@@ -720,7 +725,7 @@ see (defdata foo rational)
             (er hard 'defdata-alias
                 "~%**Unknown type**: ~x0 is not a known type name.~%" ',type )))
         `(with-output
-          ,@(and (not ,verbosep) '(:off :all :on (summary error) :summary (acl2::form acl2::time)))
+          ,@(and (not ,verbosep) '(:off :all :on (summary error) :summary-off (:other-than acl2::form acl2::time)))
           :gag-mode t :stack :push
           (encapsulate
            ()
@@ -1059,7 +1064,7 @@ see (defdata foo rational)
 
 #|
 
-PETE: seems like this is not used. If used, have to 
+PETE: seems like this is not used. If used, have to
 fix s+ form so that it has access to pkg.
 
 (defmacro acl2s::defun-attach (&rest args)
@@ -1137,7 +1142,7 @@ fix s+ form so that it has access to pkg.
                          ans))
           ((equal (car term) 'ACL2::QUOTE) ans)
           (t (get-vars1-lst (cdr term) ans))))
- 
+
  (defun get-vars1-lst (terms ans)
    (declare (xargs :verify-guards nil
                    :guard (and (true-listp terms)
@@ -1164,3 +1169,4 @@ fix s+ form so that it has access to pkg.
   (declare (xargs :guard (true-listp xs)))
   (for ((x in xs)) (always (or (proper-symbolp x)
                                (rquotep x)))))
+
