@@ -17,7 +17,7 @@ data last modified: [2014-08-06]
 (include-book "register-combinator")
 
 
-(defconst *listof-export-defthms*
+(def-const *listof-export-defthms*
   '((defthm _PRED_-IMPLIES-TLP
       (implies (_PRED_ x)
                (true-listp x))
@@ -34,17 +34,25 @@ data last modified: [2014-08-06]
           )))
 
 (program)
+
 (defun listof-theory-events (name cbody new-types kwd-alist wrld)
   (declare (xargs :mode :program))
 ;assumption: cbody is a core defdata exp (holds because user-combinators occur only at top-level)
   (b* ((M (append new-types (table-alist 'type-metadata-table wrld)))
-       (pred (predicate-name name M))
-       ((when (not (proper-symbolp pred))) (er hard? 'listof-theory-events "~| Couldnt find predicate name for ~x0.~%" name))
+       (A (table-alist 'type-alias-table wrld))
+       (pred (predicate-name name A M))
+       ((when (not (proper-symbolp pred)))
+        (er hard? 'listof-theory-events
+            "~| Couldnt find predicate name for ~x0.~%" name))
        ((mv atom-list-subtypep ?cpred) 
         (if (and (proper-symbolp cbody) (assoc-eq cbody M))
-            (mv (subtype-p (predicate-name cbody M) 'acl2::atom wrld) (predicate-name cbody M))
+            (mv (subtype-p (predicate-name cbody A M)
+                           'acl2::atom wrld)
+                (predicate-name cbody A M))
           (mv nil :undef)))
        (disabled (get1 :disabled  kwd-alist))
+       (curr-pkg (get1 :current-package kwd-alist))
+       (pkg-sym (pkg-witness curr-pkg))
        (local-events-template nil)
        (export-defthms-template *listof-export-defthms*)
 
@@ -57,13 +65,13 @@ data last modified: [2014-08-06]
                                      :splice-alist splice-alist
                                      :atom-alist atom-alist
                                      :str-alist str-alist
-                                     :pkg-sym 'acl2::asdf))
+                                     :pkg-sym pkg-sym))
        (export-defthms (template-subst export-defthms-template
                                      :features features
                                      :splice-alist splice-alist
                                      :atom-alist atom-alist
                                      :str-alist str-alist
-                                     :pkg-sym 'acl2::asdf))
+                                     :pkg-sym pkg-sym))
 
        (all-defthm-names (get-event-names export-defthms))
        (theory-name (get1 :theory-name  kwd-alist))
@@ -74,7 +82,6 @@ data last modified: [2014-08-06]
       (acl2::def-ruleset! ,theory-name ',all-defthm-names)
       )))
 
-
 (defun listof-theory-ev (p top-kwd-alist wrld)
   (b* (((cons name A) p)
        ((acl2::assocs pdef new-types kwd-alist) A) ;replace odef with pdef
@@ -82,8 +89,6 @@ data last modified: [2014-08-06]
     (case-match pdef
       (('LISTOF cbody) (listof-theory-events name cbody new-types kwd-alist wrld))
       (& '()))))
-     
-             
 
 (defloop user-listof-theory-events1 (ps kwd-alist wrld)
   (for ((p in ps)) (append (listof-theory-ev p kwd-alist wrld))))
@@ -103,8 +108,6 @@ data last modified: [2014-08-06]
               :invisible)))
           events))))
 
-                      
-
 (logic)
 (deflabel listof)
 (register-user-combinator listof 
