@@ -13,3 +13,6 @@
 (include-book "acl2s/acl2s-sigs" :dir :system :ttags :all)
 (include-book "acl2s/mode-acl2s-dependencies" :dir :system :ttags :all)
 (include-book "acl2s/defdata-testing" :dir :system :ttags :all)
+;(include-book "projects/smtlink/top" :dir :system :ttags :all)
+;(include-book "projects/smtlink/examples/basictypes" :dir :system :ttags :all)
+(include-book "acl2s/sorting/sorting" :dir :system :ttags :all)
