@@ -340,12 +340,12 @@ Now in defthm.lisp
                 (print-base-p b))
            (character-listp (explode-atom x b))))
 
-(verify-termination fix-pkg)
+; (verify-termination fix-pkg) ; Matt K. mod: now comes in :logic mode
 (verify-termination fix-sym)
 (verify-termination pack-to-string)
 (verify-termination gen-sym-sym)
 
-(verify-guards fix-pkg)
+; (verify-guards fix-pkg) ; Matt K. mod: now comes guard-verified
 (verify-guards fix-sym)
 (verify-guards pack-to-string)
 (verify-guards gen-sym-sym)
@@ -367,8 +367,9 @@ Now in defthm.lisp
   (fix-intern$ (pack-to-string l)
                (if pkg pkg (best-package-symbl-list l "ACL2"))))
 
-(defun mk-acl2s-sym (lsym)
-  (make-symbl lsym "ACL2S"))
+(defun mk-acl2s-sym (l)
+  (declare (xargs :guard (good-atom-listp l)))
+  (make-symbl l "ACL2S"))
 
 (defmacro make-sym (s suf &optional pkg)
 ; Returns the symbol s-suf.
@@ -634,3 +635,110 @@ functions over natural numbers.
        `(with-output
          :stack :pop 
          (defconst ,',name ',form ,@(and ,doc '(,doc))))))))
+
+; Utilities to stage rules.
+
+(mutual-recursion
+ (defun find-first-call (fn term)
+ ; Find the first call of fn in term.
+  (cond ((acl2::variablep term) nil)
+        ((acl2::fquotep term) nil)
+        ((equal (acl2::ffn-symb term) fn)
+         term)
+        (t (find-first-call-lst fn (acl2::fargs term)))))
+
+ (defun find-first-call-lst (fn lst)
+ ; Find the first call of fn in a list of terms.
+  (cond ((endp lst) nil)
+        (t (or (find-first-call fn (car lst))
+               (find-first-call-lst fn (cdr lst)))))))
+
+(defun stage1 (fn max clause flg)
+; If the clause is stable under simplification and there is a call of
+; fn in it, expand it.  But don't do it more than max times.
+ (let ((temp (and flg
+                  (find-first-call-lst fn clause))))
+   (if temp
+       (if (zp max)
+           (cw "~%~%HINT PROBLEM:  The maximum repetition count of ~
+                your STAGE hint been reached without eliminating ~
+                all of the calls of ~x0.  You could supply a larger ~
+                count with the optional second argument to STAGE ~
+                (which defaults to 100).  But think about what is ~
+                happening! Is each stage permanently eliminating a ~
+                call of ~x0?~%~%"
+               fn)
+         `(:computed-hint-replacement
+            ((stage1 ',fn ,(- max 1)
+                     clause
+                     stable-under-simplificationp))
+           :expand (,temp)))
+     nil)))
+
+(defmacro stage (fn &optional (max '100))
+ `(stage1 ',fn ,max clause stable-under-simplificationp))
+
+; see custom.lisp where the stage hints have been added.
+
+(defun pair-tl (x y)
+  (if (or (endp x) (endp y))
+      nil
+    (cons (list (car x) (car y)) (pair-tl (cdr x) (cdr y)))))
+
+(defun stage-rule1 (fn def-rule vars max clause flg)
+; If the clause is stable under simplification and there is a call of
+; fn in it, expand it.  But don't do it more than max times.
+  (let ((temp (and flg
+                   (find-first-call-lst fn clause))))
+    (if temp
+        (if (zp max)
+            (cw "~%~%HINT PROBLEM:  The maximum repetition count of ~
+                your STAGE hint been reached without eliminating ~
+                all of the calls of ~x0.  You could supply a larger ~
+                count with the optional second argument to STAGE ~
+                (which defaults to 100).  But think about what is ~
+                happening! Is each stage permanently eliminating a ~
+                call of ~x0?~%~%"
+                fn)
+          `(:computed-hint-replacement
+            ((stage-rule1 ',fn
+                          ',def-rule
+                          ',vars
+                          ,(- max 1)
+                          clause
+                          stable-under-simplificationp))
+            :expand (:with ,def-rule ,temp)))
+      nil)))
+
+(defmacro stage-rule (fn def-rule &optional (max '100) vars)
+  `(stage-rule1 ',fn
+                ',def-rule
+                (or ,vars (formals ',fn (w state)))
+                ,max
+                clause
+                stable-under-simplificationp))
+
+#|
+ Here is an example of how this is used
+
+ (add-default-hints!
+  '((stage-rule cfix cfix-definition-rule))
+  :at-end t)
+
+|#
+
+; utility fn to print if verbose flag is true
+(defmacro cw? (verbose-flag &rest rst)
+  `(if ,verbose-flag
+     (cw ,@rst)
+     nil))
+
+(defmacro ecw (&rest rst)
+  `(prog2$ (cw ,@(butlast rst 1))
+           (mv t ,(car (last rst)) state)))
+
+(defmacro ecw? (verbose-flag &rest rst)
+  `(if ,verbose-flag
+       (ecw ,@rst)
+     nil))
+
