@@ -247,7 +247,9 @@
               (equal (mod x 1)
                      0))))
 
-(defthm floor-mod-elim
+; Name changed 3/15/2016 by Matt K. to avoid name clash with lemma
+; floor-mod-elim in ihs/quotient-remainder-lemmas.
+(defthm floor-mod-elim-weak
   (implies (rationalp-guard x y)
 	   (equal (+ (mod x y)
 		     (* y (floor x y)))
@@ -773,7 +775,9 @@
 ;;
 ;;-----------------------------------------------------------------
 
-(defthm floor-+
+; Name changed 3/15/2016 by Matt K. to avoid name clash with lemma
+; floor-+ in ihs/quotient-remainder-lemmas.
+(defthmd floor-+-weak
   (implies (and (rationalp-guard x y z)
                 (syntaxp (not (equal (fn-symb x) 'MOD)))
                 (syntaxp (not (equal (fn-symb x) 'MOD))))
@@ -781,9 +785,9 @@
 		  (+ (floor (+ (mod x z) (mod y z)) z)
 		     (+ (floor x z) (floor y z))))))
 
-(in-theory (disable floor-+))
-
-(defthm mod-+
+; Name changed 3/15/2016 by Matt K. to avoid name clash with lemma
+; mod-+ in ihs/quotient-remainder-lemmas.
+(defthmd mod-+-weak
   (implies (and (rationalp-guard x y z)
                 (syntaxp (not (equal (fn-symb x) 'MOD)))
                 (syntaxp (not (equal (fn-symb x) 'MOD))))
@@ -791,8 +795,6 @@
 		  (mod (+ (mod x z) (mod y z)) z)))
   :hints (("Goal" :in-theory (enable mod))))
 
-(in-theory (disable mod-+))
-
 ;; This rule below should probably be used to ``normalize''
 ;; expressions such as
 ;; (equal (floor (+ (floor a 2) (* f2 (expt 2 i)))
@@ -805,22 +807,22 @@
 
 ;; Add other direction of the next two rules, or generalize them.
 
-(defthm rewrite-floor-x*y-z-right
+; Name changed 3/15/2016 by Matt K. to avoid name clash with lemma
+; rewrite-floor-x*y-z-right in ihs/quotient-remainder-lemmas.
+(defthmd rewrite-floor-x*y-z-right-weak
   (implies (rationalp-guard x y z)
 	   (equal (floor (* x y) z)
 		  (floor x (/ z y)))))
 
-(in-theory (disable rewrite-floor-x*y-z-right))
-
-(defthm rewrite-mod-x*y-z-right
+(defthmd rewrite-mod-x*y-z-right
   (implies (rationalp-guard x y z)
 	   (equal (mod (* x y) z)
 		  (* y (mod x (/ z y)))))
   :hints (("Goal" :in-theory (enable mod))))
 
-(in-theory (disable rewrite-mod-x*y-z-right))
-
-(defthm floor-minus
+; Name changed 3/15/2016 by Matt K. to avoid name clash with lemma
+; floor-minus in ihs/quotient-remainder-lemmas.
+(defthm floor-minus-weak
   (implies (and (rationalp-guard x y)
                 (syntaxp (negative-addends-p x)))
            (equal (floor x y)
@@ -857,21 +859,25 @@
 		    (- (mod x (- y)) (- y)))))
   :hints (("Goal" :in-theory (enable mod))))
 
-(defthm rewrite-floor-mod
+; Name changed 3/15/2016 by Matt K. to avoid name clash with lemma
+; rewrite-floor-mod in ihs/quotient-remainder-lemmas.
+(defthm rewrite-floor-mod-weak
   (implies (and (rationalp-guard x y z)
 		(equal i (/ y z))
 		(integerp i))
 	   (equal (floor (mod x y) z)
 		  (- (floor x z) (* i (floor x y))))))
 
-(defthm rewrite-mod-mod
+; Name changed 3/15/2016 by Matt K. to avoid name clash with lemma
+; rewrite-mod-mod in ihs/quotient-remainder-lemmas.
+(defthm rewrite-mod-mod-weak
   (implies (and (rationalp-guard x y z)
                 (not (equal z 0))
 		(equal i (/ y z))
 		(integerp i))
 	   (equal (mod (mod x y) z)
 		  (mod x z)))
-  :hints (("Goal" :in-theory (enable mod-+)
+  :hints (("Goal" :in-theory (enable mod-+-weak)
                   :cases ((equal (floor x y) 0)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -903,12 +909,14 @@
                           (equal (mod x z) x))))
    :hints (("Goal" :cases ((< 0 z) (< z 0) (equal z 0)))
            ("Subgoal 3.10" :in-theory (enable mod))
-           ("Subgoal 3.4" :in-theory (enable mod-+))
+           ("Subgoal 3.4" :in-theory (enable mod-+-weak))
            ("Subgoal 2.5" :in-theory (enable mod))
-           ("Subgoal 2.2" :in-theory (enable mod-+)))
+           ("Subgoal 2.2" :in-theory (enable mod-+-weak)))
    :rule-classes nil))
 
-(defthm mod-+-cancel-0
+; Name changed 3/15/2016 by Matt K. to avoid name clash with lemma
+; mod-+-cancel-0 in ihs/quotient-remainder-lemmas.
+(defthm mod-+-cancel-0-weak
     (implies (and (rationalp-guard x y z)
                   (syntaxp (mod-+-cancel-0-fn x z)))
              (equal (equal (mod x y) z)
@@ -961,7 +969,9 @@
         (list (cons 'common-factors (make-product-ccc common-factors)))
       nil)))
 
-(defthm floor-cancel-*
+; Name changed 3/15/2016 by Matt K. to avoid name clash with lemma
+; floor-cancel-* in ihs/quotient-remainder-lemmas.
+(defthm floor-cancel-*-weak
     (implies (and (rationalp-guard x y)
                   (bind-free
                    (find-common-factors x y mfc state)
@@ -970,6 +980,7 @@
                   (not (equal common-factors 0)))
              (equal (floor x y)
                     (floor (* x (/ common-factors)) (* y (/ common-factors))))))
+
 (defthm mod-cancel-*
     (implies (and (rationalp-guard x y)
                   (bind-free
@@ -1063,7 +1074,9 @@
         (t
          nil)))
 
-(defthm simplify-mod-+-mod
+; Name changed 3/15/2016 by Matt K. to avoid name clash with lemma
+; simplify-mod-+-mod in ihs/quotient-remainder-lemmas.
+(defthm simplify-mod-+-mod-weak
     (implies (and (not (equal y 0))
                   (bind-free
                    (simplify-mod-+-mod-fn x y mfc state)
@@ -1136,7 +1149,11 @@
   (equal (floor 0 x)
          0)))
 
-(defthm floor-floor-integer
+; Name changed 3/15/2016 by Matt K. to avoid name clash with lemma
+; floor-floor-integer in ihs/quotient-remainder-lemmas.  (Interestingly, other
+; than the use of real-rationalp in that other lemma, the lemma below is the
+; stronger of the two.)
+(defthm floor-floor-integer-alt
     (implies (and (rationalp x)
                   (integerp i)
                   (integerp j)
