@@ -149,6 +149,7 @@ executed; it uses a SAT solver to answer the question.</p>
                                    (aig-sat-when-sat))
                    :use ((:instance aig-sat-when-sat
                           (aig (aig-not (faig-purebool-aig x)))
+                          (gatesimp (aignet::default-gatesimp))
                           (transform-config nil)))))))
 
   (local (defthm l1
@@ -161,8 +162,9 @@ executed; it uses a SAT solver to answer the question.</p>
                    :in-theory (e/d (faig-purebool-p-as-aig-eval)
                                    (aig-sat-when-unsat))
                    :use ((:instance aig-sat-when-unsat
-                                    (aig (aig-not (faig-purebool-aig x)))
-                                    (env (faig-purebool-p-witness x))
+                          (aig (aig-not (faig-purebool-aig x)))
+                          (gatesimp (aignet::default-gatesimp))
+                          (env (faig-purebool-p-witness x))
                           (transform-config nil)))))))
 
   (defthm faig-purebool-check-correct
@@ -188,7 +190,8 @@ executed; it uses a SAT solver to answer the question.</p>
                                     faig-eval)
                                    (aig-sat-when-sat))
                    :use ((:instance aig-sat-when-sat
-                                    (aig (aig-not (faig-purebool-aig x)))
+                          (aig (aig-not (faig-purebool-aig x)))
+                          (gatesimp (aignet::default-gatesimp))
                           (transform-config nil)))))))
 
   (defthm faig-purebool-counterexample-correct
@@ -324,7 +327,8 @@ faig-purebool-list-check).</p>"
                    :in-theory (e/d (faig-purebool-list-p-as-aig-eval)
                                    (aig-sat-when-sat))
                    :use ((:instance aig-sat-when-sat
-                                    (aig (aig-not (faig-purebool-list-aig x)))
+                          (gatesimp (aignet::default-gatesimp))
+                          (aig (aig-not (faig-purebool-list-aig x)))
                           (transform-config nil)))))))
 
   (local (defthm l1
@@ -337,8 +341,9 @@ faig-purebool-list-check).</p>"
                    :in-theory (e/d (faig-purebool-list-p-as-aig-eval)
                                    (aig-sat-when-unsat))
                    :use ((:instance aig-sat-when-unsat
-                                    (aig (aig-not (faig-purebool-list-aig x)))
-                                    (env (faig-purebool-list-witness x))
+                          (gatesimp (aignet::default-gatesimp))
+                          (aig (aig-not (faig-purebool-list-aig x)))
+                          (env (faig-purebool-list-witness x))
                           (transform-config nil)))))))
 
   (defthm faig-purebool-list-check-correct
