@@ -217,7 +217,7 @@
  (progn
    (defun my-glucose-config ()
      (declare (xargs :guard t))
-     (satlink::make-config :cmdline "glucose"
+     (satlink::make-config :cmdline "glucose -model"
                            :verbose t
                            :mintime 1/2
                            :remove-temps t))
