@@ -55,7 +55,8 @@
 (defun get-fixtypes-alist (world)
   (cdr (assoc 'fixtype-alist (table-alist 'fixtypes world))))
 
-(defun deffixtype-fn (name predicate fix equiv execp definep inline equal topic verbosep hints forward)
+(defun deffixtype-fn (name predicate fix equiv executablep definep inline equal no-rewrite-quoted-constant
+                           topic verbosep hints forward)
   (if definep
       `(with-output ,@(and (not verbosep) '(:off :all :on acl2::error)) :stack :push
          (encapsulate nil
@@ -67,14 +68,14 @@
                      (value-triple
                       (er hard? 'deffixtype
                           "Failed to prove that ~x0 is idempotent.~%" ',fix)))))
-           (,(cond ((not execp)  'defun-nx)
+           (,(cond ((not executablep) 'defun-nx)
                    ((not inline) 'defun)
                    (t            'defun-inline))
             ,equiv (x y)
-             (declare (xargs :normalize nil
-                             ,@(and execp `(:guard (and (,predicate x) (,predicate y))))
-                             :verify-guards ,execp))
-             (,equal (,fix x) (,fix y)))
+            (declare (xargs :normalize nil
+                            ,@(and executablep `(:guard (and (,predicate x) (,predicate y))))
+                            :verify-guards ,executablep))
+            (,equal (,fix x) (,fix y)))
            (local (in-theory '(,equiv tmp-deffixtype-idempotent
                                       booleanp-compound-recognizer)))
            (defequiv ,equiv :package :legacy)
@@ -83,7 +84,10 @@
                      (concatenate 'string
                                   (symbol-name fix) "-UNDER-" (symbol-name equiv))
                      equiv)
-             (,equiv (,fix x) x))
+             (,equiv (,fix x) x)
+             :rule-classes (:rewrite
+                            . ,(and (not no-rewrite-quoted-constant)
+                                    '(:rewrite-quoted-constant))))
            ,@(and forward
                   `((defthm ,(intern-in-package-of-symbol
                               (concatenate 'string "EQUAL-OF-" (symbol-name fix) "-1-FORWARD-TO-" (symbol-name equiv))
@@ -114,10 +118,10 @@
                                                      :pred predicate
                                                      :fix fix
                                                      :equiv equiv
-                                                     :executablep execp
+                                                     :executablep executablep
                                                      :equiv-means-fixes-equal
                                                      ;; BOZO stupid ACL2 is so awful...
-                                                     (if (and execp inline)
+                                                     (if (and executablep inline)
                                                          (intern-in-package-of-symbol
                                                           (concatenate 'string (symbol-name equiv) "$INLINE")
                                                           equiv)
@@ -154,7 +158,7 @@
                                                    :pred predicate
                                                    :fix fix
                                                    :equiv equiv
-                                                   :executablep execp
+                                                   :executablep executablep
                                                    :equiv-means-fixes-equal thmname
                                                    :inline inline
                                                    :equal equal
@@ -163,19 +167,23 @@
                              (get-fixtypes-alist world))))))))
 
 
-(defmacro deffixtype (name &key pred fix equiv (execp 't)
-                           ;; optional
+(defmacro deffixtype (name &key
+                           pred
+                           fix
+                           equiv
+                           (executablep 't)
                            define
                            verbosep
                            hints
                            forward
+                           (no-rewrite-quoted-constant 'nil)
                            (inline 't)
                            (equal 'equal)
                            (topic 'nil topic-p))
 ; We contemplated making "equal" the default equivalence relation but decided
 ; against it.  See Github Issue 240 for relevant discussion.
   (declare (xargs :guard (and pred fix equiv)))
-  (deffixtype-fn name pred fix equiv execp define inline equal
+  (deffixtype-fn name pred fix equiv executablep define inline equal no-rewrite-quoted-constant
     (if topic-p topic name)
     verbosep hints forward))
 
@@ -250,10 +258,7 @@
           (raise "Not a fixtype name or predicate: ~x0" type)))
        (fix (fixtype->fix fixtype))
        (equiv (fixtype->equiv fixtype))
-       (pred (fixtype->pred fixtype))
        (hints (getarg :hints nil kwd-alist))
-       (skip-const-thm (or (getarg :skip-const-thm nil kwd-alist)
-                           (not (fixtype->executablep fixtype))))
        (skip-cong-thm (getarg :skip-cong-thm nil kwd-alist))
        ((unless (and (consp form) (symbolp (car form))))
         (raise "Form should be a function call term, but it's ~x0" form))
@@ -278,11 +283,6 @@
           'string (symbol-name basename) "-OF-" (symbol-name fix) "-" (symbol-name arg)
           under-out-equiv suffix)
          pkg))
-       (const-thmname
-        (intern-in-package-of-symbol
-         (concatenate
-          'string (symbol-name basename) "-OF-" (symbol-name fix) "-" (symbol-name arg) "-NORMALIZE-CONST" under-out-equiv suffix)
-         pkg))
        (congruence-thmname
         (intern-in-package-of-symbol
          (concatenate
@@ -302,24 +302,17 @@
                      (member arg vars)))
         (raise "Expected ~x0 to be among variables of ~x1" arg form))
 
-       (fix-body `(,out-equiv ,(subst `(,fix ,arg) arg form)
+       (subst-alist `((,arg . (,fix ,arg))))
+       (fix-body `(,out-equiv ,(acl2::sublis-var subst-alist form)
                               ,form))
        (fix-thm `(defthm ,fix-thmname
                    ,fix-body
                    :hints ,hints))
-       (const-thm (and (not skip-const-thm)
-                       `(defthm ,const-thmname
-                          (implies (syntaxp (and (quotep ,arg)
-                                                 (not (,pred (cadr ,arg)))))
-                                   (,out-equiv ,form
-                                               ,(subst `(,fix ,arg) arg form)))
-                          :hints (("Goal" :in-theory
-                                   '(,out-equiv-equiv-rune ,fix-thmname))))))
        (cong-thm (and (not skip-cong-thm)
                       `(defthm ,congruence-thmname
                          (implies (,equiv ,arg ,argequiv)
                                   (,out-equiv ,form
-                                              ,(subst argequiv arg form)))
+                                              ,(acl2::sublis-var `((,arg . ,argequiv)) form)))
                          :hints (("Goal" :in-theory nil
                                   :do-not '(preprocess)
                                   :use ((:instance ,fix-thmname)
@@ -334,7 +327,6 @@
      :kwd-alist kwd-alist
      :fix-body fix-body
      :fix-thm fix-thm
-     :const-thm const-thm
      :cong-thm cong-thm)))
 
 (defun fixequiv-events (fixequiv)
@@ -406,4 +398,3 @@
 
 
 ||#
-
