@@ -101,6 +101,41 @@
 
          (table g-functions ',',fn ',gfn)))))
 
+(defun def-g-binary-op-body (a b fn gfn body)
+  `(cond ((and (general-concretep ,a) (general-concretep ,b))
+          (gret (mk-g-concrete
+                 (ec-call (,fn (general-concrete-obj ,a)
+                               (general-concrete-obj ,b))))))
+         ((mbe :logic (eq (tag ,a) :g-ite)
+               :exec (and (consp ,a) (eq (tag ,a) :g-ite)))
+          (if (zp clk)
+              (gret (g-apply ',fn (gl-list ,a ,b)))
+            (let* ((test (g-ite->test ,a))
+                   (then (g-ite->then ,a))
+                   (else (g-ite->else ,a)))
+              (g-if (gret test)
+                    (,gfn then ,b hyp clk config bvar-db state)
+                    (,gfn else ,b hyp clk config bvar-db state)))))
+         ((mbe :logic (eq (tag ,b) :g-ite)
+               :exec (and (consp ,b) (eq (tag ,b) :g-ite)))
+          (if (zp clk)
+              (gret (g-apply ',fn (gl-list ,a ,b)))
+            (let* ((test (g-ite->test ,b))
+                   (then (g-ite->then ,b))
+                   (else (g-ite->else ,b)))
+              (g-if (gret test)
+                    (,gfn ,a then hyp clk config bvar-db state)
+                    (,gfn ,a else hyp clk config bvar-db state)))))
+         ((mbe :logic (not (member-eq (tag ,a) '(:g-var :g-apply)))
+               :exec (or (atom ,a)
+                         (not (member-eq (tag ,a) '(:g-var :g-apply)))))
+          (cond ((mbe :logic (not (member-eq (tag ,b) '( :g-var :g-apply)))
+                      :exec (or (atom ,b)
+                                (not (member-eq (tag ,b) '( :g-var :g-apply)))))
+                 ,body)
+                (t (gret (g-apply ',fn (gl-list ,a ,b))))))
+         (t (gret (g-apply ',fn (gl-list ,a ,b))))))
+
 
 
 (defmacro def-g-binary-op (fn body &key hyp-hints pres-hints hyp-fix-hints)
@@ -112,35 +147,7 @@
            (b (cadr formals)))
       `(def-g-fn ,fn
          (let ((a ',a) (b ',b) (fn ',fn))
-           `(cond ((and (general-concretep ,a) (general-concretep ,b))
-                   (gret (mk-g-concrete
-                          (ec-call (,fn (general-concrete-obj ,a)
-                                        (general-concrete-obj ,b))))))
-                  ((and (consp ,a) (eq (tag ,a) :g-ite))
-                   (if (zp clk)
-                       (gret (g-apply ',fn (gl-list ,a ,b)))
-                     (let* ((test (g-ite->test ,a))
-                            (then (g-ite->then ,a))
-                            (else (g-ite->else ,a)))
-                       (g-if (gret test)
-                             (,gfn then ,b hyp clk config bvar-db state)
-                             (,gfn else ,b hyp clk config bvar-db state)))))
-                  ((and (consp ,b) (eq (tag ,b) :g-ite))
-                   (if (zp clk)
-                       (gret (g-apply ',fn (gl-list ,a ,b)))
-                     (let* ((test (g-ite->test ,b))
-                            (then (g-ite->then ,b))
-                            (else (g-ite->else ,b)))
-                       (g-if (gret test)
-                             (,gfn ,a then hyp clk config bvar-db state)
-                             (,gfn ,a else hyp clk config bvar-db state)))))
-                  ((or (atom ,a)
-                       (not (member-eq (tag ,a) '(:g-var :g-apply))))
-                   (cond ((or (atom ,b)
-                              (not (member-eq (tag ,b) '( :g-var :g-apply))))
-                          ,',',body)
-                         (t (gret (g-apply ',fn (gl-list ,a ,b))))))
-                  (t (gret (g-apply ',fn (gl-list ,a ,b))))))
+           (def-g-binary-op-body a b fn gfn ',',body))
          :hyp-hints ,',hyp-hints
          :pres-hints ,',pres-hints
          :hyp-fix-hints ,',hyp-fix-hints))))
