@@ -1,4 +1,32 @@
-
+; GL - A Symbolic Simulation Framework for ACL2
+; Copyright (C) 2008-2013 Centaur Technology
+;
+; Contact:
+;   Centaur Technology Formal Verification Group
+;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
+;   http://www.centtech.com/
+;
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
+;
+; Original author: Sol Swords <sswords@centtech.com>
 
 (in-package "GL")
 (include-book "generic-geval")
@@ -16,6 +44,7 @@
 (def-eval-g glcp-generic-geval
   (
    ;; used in shape specs
+   acl2::logtail$inline
    logapp int-set-sign maybe-integer
           cons car cdr consp if not equal nth len iff
           shape-spec-slice-to-env
@@ -39,7 +68,7 @@
       SYMBOL-PACKAGE-NAME
       INTERN-IN-PACKAGE-OF-SYMBOL
       CODE-CHAR
-;   DENOMINATOR
+   DENOMINATOR
       CDR
 ;   COMPLEX
       CAR
@@ -49,7 +78,7 @@
       IMAGPART
       SYMBOLP
       REALPART
-;   NUMERATOR
+   NUMERATOR
       EQUAL
       STRINGP
       RATIONALP
@@ -85,7 +114,10 @@
 
       ;; force checks
       gl-force-check-fn
-      binary-minus-for-gl))
+      binary-minus-for-gl
+
+      ;; measure stuff
+      o< o-p))
 
 (in-theory (disable glcp-generic-geval))
 
