@@ -34,7 +34,7 @@
 (include-book "std/util/define" :dir :system)
 (include-book "std/basic/defs" :dir :system)
 (include-book "tools/flag" :dir :system)
-(include-book "std/misc/two-nats-measure" :dir :system)
+(include-book "std/basic/two-nats-measure" :dir :system)
 (include-book "misc/hons-help" :dir :system) ;; for alist-keys/vals
 (include-book "tools/templates" :dir :system)
 (local (include-book "std/lists/sets" :dir :system))
