@@ -45,7 +45,10 @@
   :short "Turning a hierarchical SVEX design into a finite state machine."
   :long " <p>The function @(see svex-design-compile) extracts a finite state
 machine representation from a hierarchical SVEX design.  See also @(see
-defsvtv).</p>
+defsvtv), @(see defsvtv$), and @(see defcycle), which all perform this process
+and subsequently perform some amount of unrolling of the resulting finite state
+machine in order to produce either a pipeline representation (@(see defsvtv)
+and @(see defsvtv$)) or a cycle FSM (@(see defcycle)).</p>
 
 <p>We follow these steps to turn an svex module hierarchy into a finite-state
 machine representation:</p>
@@ -281,11 +284,18 @@ svex-assigns-compose)).</li>
       (implies (maybe-svar-fix x)
                (svar-p (maybe-svar-fix x))))))
 
+(defprod indname-result
+  ((varname svar-p)
+   (decl wire-p))
+  :layout :fulltree)
+
+(fty::defoption maybe-indname-result indname-result)
+
 (acl2::def-1d-arr indnamememo
   :slotname indname
-  :pred maybe-svar-p
-  :fix maybe-svar-fix$inline
-  :type-decl (satisfies maybe-svar-p)
+  :pred maybe-indname-result-p
+  :fix maybe-indname-result-fix$inline
+  :type-decl (satisfies maybe-indname-result-p)
   :pkg-sym sv::svex
   :default-val nil)
 
@@ -302,14 +312,18 @@ svex-assigns-compose)).</li>
        (in-bounds (< idx (indnames-length indnamememo)))
        (look (and in-bounds
                   (get-indname idx indnamememo)))
-       ((when (and look (svar-addr-p look))) (mv look indnamememo))
-       (name (moddb-wireidx->path idx (modscope->modidx scope) moddb))
+       ((when (and look (svar-addr-p (indname-result->varname look))))
+        (mv (indname-result->varname look) indnamememo))
+       ((mv name wire) (moddb-wireidx->path/decl idx (modscope->modidx scope) moddb))
        (res (address->svar (make-address :path name)))
        (indnamememo (if in-bounds
-                        (set-indname idx res indnamememo)
+                        (set-indname idx (make-indname-result :varname res :decl wire) indnamememo)
                       indnamememo)))
     (mv res indnamememo)))
 
+
+(local (in-theory (disable lhs-vars-when-consp)))
+
 (define lhs-indexed->named ((x lhs-p) (scope modscope-p) (moddb moddb-ok) indnamememo)
   :guard (and (modscope-okp scope moddb)
               (svarlist-boundedp (lhs-vars x) (modscope-local-bound scope moddb)))
@@ -367,6 +381,9 @@ svex-assigns-compose)).</li>
                    :do-not-induct t)))))
 
 
+
+
+
 (define aliases-indexed->named-aux ((n natp) (aliases) (scope modscope-p) (moddb moddb-ok) indnamememo)
   :guard (and (<= (lnfix n) (aliass-length aliases))
               (modscope-okp scope moddb)
@@ -385,6 +402,12 @@ svex-assigns-compose)).</li>
        (aliases (set-alias n lhs1 aliases)))
     (aliases-indexed->named-aux (1+ (lnfix n)) aliases scope moddb indnamememo))
   ///
+  (local
+   (defthm lemma
+     (implies (<= (len aliases) (nfix m))
+              (not (consp (nth m aliases))))
+     :hints (("goal" :in-theory (enable nth len nfix)))))
+
   (defthm aliases-indexed->named-preserves-lesser-indices
     (implies (< (nfix m) (nfix n))
              (equal (nth m (mv-nth 0 (aliases-indexed->named-aux
@@ -399,21 +422,41 @@ svex-assigns-compose)).</li>
                                           n aliases scope moddb indnamememo))))))))
 
 
+(fty::defmap var-decl-map :key-type svar :val-type wire :true-listp t)
+
+(define indnamememo-to-var-decl-map ((n natp) indnamememo (acc var-decl-map-p))
+  :returns (map var-decl-map-p)
+  :guard (<= n (indnames-length indnamememo))
+  :measure (nfix (- (indnames-length indnamememo) (nfix n)))
+  (b* (((when (mbe :logic (zp (- (indnames-length indnamememo) (nfix n)))
+                   :exec (eql n (indnames-length indnamememo))))
+        (var-decl-map-fix acc))
+       (entry (get-indname n indnamememo))
+       (acc (if entry
+                (b* (((indname-result entry)))
+                  (hons-acons entry.varname entry.decl (var-decl-map-fix acc)))
+              (var-decl-map-fix acc))))
+    (indnamememo-to-var-decl-map (1+ (lnfix n)) indnamememo acc)))
+
 
 (define aliases-indexed->named (aliases (scope modscope-p) (moddb moddb-ok))
-  :returns (aliases1 (equal (len aliases1) (len aliases)))
+  :returns (mv (new-aliases (equal (len new-aliases) (len aliases)))
+               (varmap var-decl-map-p))
   :guard (and (modscope-okp scope moddb)
               (<= (aliass-length aliases) (modscope-local-bound scope moddb))
               (aliases-normorderedp aliases))
   :verbosep t
   (b* (((acl2::local-stobjs indnamememo)
-        (mv aliases indnamememo))
-       (indnamememo (resize-indnames (aliass-length aliases) indnamememo)))
-    (aliases-indexed->named-aux 0 aliases scope moddb indnamememo))
+        (mv aliases varmap indnamememo))
+       (indnamememo (resize-indnames (aliass-length aliases) indnamememo))
+       ((mv aliases indnamememo)
+        (aliases-indexed->named-aux 0 aliases scope moddb indnamememo))
+       (varmap (indnamememo-to-var-decl-map 0 indnamememo nil)))
+    (mv aliases varmap indnamememo))
   ///
-  (defthm vars-of-aliases-indexed->named
+  (defret vars-of-aliases-indexed->named
     (implies (< (nfix m) (len aliases))
-             (svarlist-addr-p (lhs-vars (nth m (aliases-indexed->named aliases scope moddb))))))
+             (svarlist-addr-p (lhs-vars (nth m new-aliases)))))
 
 
   (local (defun ind (n)
@@ -421,18 +464,90 @@ svex-assigns-compose)).</li>
                n
              (ind (1- n)))))
   (local
-   (defthm aliases-vars-aux-of-aliases-indexed->named
+   (defret aliases-vars-aux-of-aliases-indexed->named
      (implies (<= (nfix n) (len aliases))
-              (svarlist-addr-p (aliases-vars-aux n (aliases-indexed->named aliases scope moddb))))
+              (svarlist-addr-p (aliases-vars-aux n new-aliases)))
      :hints(("Goal" :in-theory (e/d (aliases-vars-aux)
                                     (aliases-indexed->named)) :induct (ind n)))))
 
-  (defthm aliases-vars-of-aliases-indexed->named
-    (svarlist-addr-p (aliases-vars (aliases-indexed->named aliases scope moddb)))
+  (defret aliases-vars-of-aliases-indexed->named
+    (svarlist-addr-p (aliases-vars new-aliases))
     :hints(("Goal" :in-theory (e/d (aliases-vars)
                                    (aliases-indexed->named))))))
 
 
+
+;; when keeping aliases indexed rather than named
+(define svar-register-indnamememo ((x svar-p) (scope modscope-p) (moddb moddb-ok) indnamememo)
+  :guard (and (modscope-okp scope moddb)
+              (svar-boundedp x (modscope-local-bound scope moddb)))
+  :guard-hints ((and stable-under-simplificationp
+                     '(:in-theory (enable svar-boundedp modscope-local-bound))))
+  :returns (indnamememo1 (equal (len indnamememo1)
+                                (len indnamememo)))
+  (b* ((idx (lnfix (svar-index x)))
+       (in-bounds (< idx (indnames-length indnamememo)))
+       (look (and in-bounds
+                  (get-indname idx indnamememo)))
+       ((when look)
+        indnamememo)
+       ((mv ?name wire) (moddb-wireidx->path/decl idx (modscope->modidx scope) moddb)))
+    (if in-bounds
+        (set-indname idx (make-indname-result :varname x :decl wire) indnamememo)
+      indnamememo)))
+
+(define lhs-register-indnamememo ((x lhs-p) (scope modscope-p) (moddb moddb-ok) indnamememo)
+  :guard (and (modscope-okp scope moddb)
+              (svarlist-boundedp (lhs-vars x) (modscope-local-bound scope moddb)))
+  :verify-guards nil
+  :returns (indnamememo1 (equal (len indnamememo1)
+                                (len indnamememo)))
+  :measure (len x)
+  (b* (((mv first rest) (lhs-decomp x))
+       ((unless first) indnamememo)
+       ((lhrange first) first)
+       (indnamememo
+        (lhatom-case first.atom
+          :z indnamememo
+          :var (svar-register-indnamememo first.atom.name scope moddb indnamememo))))
+    (lhs-register-indnamememo rest scope moddb indnamememo))
+  ///
+  (verify-guards lhs-register-indnamememo
+    :hints(("Goal" :in-theory (enable lhatom-vars)))))
+
+
+
+(define aliases-to-var-decl-map-aux ((n natp) (aliases) (scope modscope-p) (moddb moddb-ok) indnamememo)
+  :guard (and (<= (lnfix n) (aliass-length aliases))
+              (modscope-okp scope moddb)
+              (aliases-boundedp-aux n aliases (modscope-local-bound scope moddb)))
+  :guard-hints (("goal" :expand ((aliases-boundedp-aux n aliases (modscope-local-bound scope moddb)))))
+  :returns (indnamememo1 (equal (len indnamememo1) (len indnamememo)))
+  :measure (nfix (- (aliass-length aliases) (nfix n)))
+  :guard-debug t
+  (b* (((when (mbe :logic (zp (- (aliass-length aliases) (nfix n)))
+                   :exec (eql (aliass-length aliases) n)))
+        indnamememo)
+       (lhs (get-alias n aliases))
+       (indnamememo (lhs-register-indnamememo lhs scope moddb indnamememo)))
+    (aliases-to-var-decl-map-aux (1+ (lnfix n)) aliases scope moddb indnamememo)))
+
+
+(define aliases-to-var-decl-map (aliases (scope modscope-p) (moddb moddb-ok))
+  :returns (varmap var-decl-map-p)
+  :guard (and (modscope-okp scope moddb)
+              (<= (aliass-length aliases) (modscope-local-bound scope moddb))
+              (aliases-normorderedp aliases))
+  :verbosep t
+  (b* (((acl2::local-stobjs indnamememo)
+        (mv varmap indnamememo))
+       (indnamememo (resize-indnames (aliass-length aliases) indnamememo))
+       (indnamememo
+        (aliases-to-var-decl-map-aux 0 aliases scope moddb indnamememo))
+       (varmap (indnamememo-to-var-decl-map 0 indnamememo nil)))
+    (mv varmap indnamememo)))
+
+
 ;; (defines svex-indexed->named
 ;;   :Verify-guards nil
 ;;   ;; Don't want to memoize this since it will slow down moddb operations
@@ -597,7 +712,7 @@ svex-assigns-compose)).</li>
        (ans (make-lhs-override :lhs lhs :test 1 :val rhs.value))
        (nrev (acl2::nrev-push ans nrev)))
     (assigns-to-overrides-nrev (cdr x) nrev)))
-  
+
 
 (define assigns-to-overrides ((x assigns-p))
   :returns (override lhs-overridelist-p)
@@ -628,7 +743,7 @@ svex-assigns-compose)).</li>
                                       assigns-vars
                                       lhs-overridelist-keys
                                       lhs-override-vars)))))
-  
+
 
 ;; (define svex-overrides-boundedp ((x svex-overridelist-p) (bound natp))
 ;;   (if (atom x)
@@ -636,77 +751,77 @@ svex-assigns-compose)).</li>
 ;;     (and (svex-boundedp (svex-override->wire (car x)) bound)
 ;;          (svex-overrides-boundedp (cdr x) bound))))
 
-(define svex->normed-lhs ((x svex-p)
-                          (aliases))
-  :guard (and (svarlist-boundedp (svex-vars x) (aliass-length aliases))
-              (lhssvex-unbounded-p x))
-  :verify-guards nil
-  :returns (lhs lhs-p)
-  :measure (svex-count x)
-  (svex-case x
-    :var (get-alias (svar-index x.name) aliases)
-    :quote nil
-    :call
-    (case x.fn
-      (concat (b* (((list w lo hi) x.args)
-                   (width (2vec->val (svex-quote->val w)))
-                   (lo-lhs (svex->normed-lhs lo aliases))
-                   (hi-lhs (svex->normed-lhs hi aliases)))
-                (lhs-concat (lnfix width) lo-lhs hi-lhs)))
-      (rsh (b* (((list sh xx) x.args)
-                (shamt (2vec->val (svex-quote->val sh)))
-                (sub-lhs (svex->normed-lhs xx aliases)))
-             (lhs-rsh (lnfix shamt) sub-lhs)))))
-  ///
-  (local (defthm equal-of-len
-           (implies (syntaxp (quotep n))
-                    (equal (equal (len x) n)
-                           (if (zp n)
-                               (and (eql n 0)
-                                    (atom x))
-                             (and (consp x)
-                                  (equal (len (cdr x)) (1- n))))))))
-  (verify-guards svex->normed-lhs
-    :hints (("goal" :expand ((svex-vars x)
-                             (lhssvex-unbounded-p x))
-             :in-theory (enable svar-boundedp svexlist-vars))))
+;; (define svex->normed-lhs ((x svex-p)
+;;                           (aliases))
+;;   :guard (and (svarlist-boundedp (svex-vars x) (aliass-length aliases))
+;;               (lhssvex-unbounded-p x))
+;;   :verify-guards nil
+;;   :returns (lhs lhs-p)
+;;   :measure (svex-count x)
+;;   (svex-case x
+;;     :var (get-alias (svar-index x.name) aliases)
+;;     :quote nil
+;;     :call
+;;     (case x.fn
+;;       (concat (b* (((list w lo hi) x.args)
+;;                    (width (2vec->val (svex-quote->val w)))
+;;                    (lo-lhs (svex->normed-lhs lo aliases))
+;;                    (hi-lhs (svex->normed-lhs hi aliases)))
+;;                 (lhs-concat (lnfix width) lo-lhs hi-lhs)))
+;;       (rsh (b* (((list sh xx) x.args)
+;;                 (shamt (2vec->val (svex-quote->val sh)))
+;;                 (sub-lhs (svex->normed-lhs xx aliases)))
+;;              (lhs-rsh (lnfix shamt) sub-lhs)))))
+;;   ///
+;;   (local (defthm equal-of-len
+;;            (implies (syntaxp (quotep n))
+;;                     (equal (equal (len x) n)
+;;                            (if (zp n)
+;;                                (and (eql n 0)
+;;                                     (atom x))
+;;                              (and (consp x)
+;;                                   (equal (len (cdr x)) (1- n))))))))
+;;   (verify-guards svex->normed-lhs
+;;     :hints (("goal" :expand ((svex-vars x)
+;;                              (lhssvex-unbounded-p x))
+;;              :in-theory (enable svar-boundedp svexlist-vars))))
 
-  (defthm vars-of-svex->normed-lhs
-    (implies (not (member v (aliases-vars aliases)))
-             (not (member v (lhs-vars (svex->normed-lhs x aliases)))))))
+;;   (defthm vars-of-svex->normed-lhs
+;;     (implies (not (member v (aliases-vars aliases)))
+;;              (not (member v (lhs-vars (svex->normed-lhs x aliases)))))))
 
 
-(define svex-overrides-alias-norm ((x svex-overridelist-p) aliases)
-  :guard (svarlist-boundedp (svex-overridelist-keys x) (aliass-length aliases))
-  :prepwork ((local (in-theory (enable svex-overridelist-keys
-                                       svex-overridelist-vars
-                                       lhs-overridelist-vars))))
-  :returns (mv err (newx lhs-overridelist-p))
-  (b* (((when (atom x)) (mv nil nil))
-       ((svex-override xf) (car x))
-       ((unless (lhssvex-unbounded-p xf.wire))
-        (mv (msg "Error: Expression to be overridden can't be expressed as a concatenation of part-selects: ~x0~%" xf.wire)
-            nil))
-       (lhs (svex->normed-lhs xf.wire aliases))
-       (first (make-lhs-override :lhs lhs
-                                 :test xf.test
-                                 :val xf.val))
-       ((mv err rest) (svex-overrides-alias-norm (cdr x) aliases))
-       ((when err) (mv err nil)))
-    (mv nil (cons first rest)))
-  ///
-  (more-returns
-   (newx :name vars-of-svex-overrides-alias-norm
-         (implies (not (member v (svex-overridelist-vars x)))
-                  (not (member v (lhs-overridelist-vars newx))))
-         :hints(("Goal" :in-theory (enable lhs-overridelist-vars
-                                           svex-overridelist-vars)))))
+;; (define svex-overrides-alias-norm ((x svex-overridelist-p) aliases)
+;;   :guard (svarlist-boundedp (svex-overridelist-keys x) (aliass-length aliases))
+;;   :prepwork ((local (in-theory (enable svex-overridelist-keys
+;;                                        svex-overridelist-vars
+;;                                        lhs-overridelist-vars))))
+;;   :returns (mv err (newx lhs-overridelist-p))
+;;   (b* (((when (atom x)) (mv nil nil))
+;;        ((svex-override xf) (car x))
+;;        ((unless (lhssvex-unbounded-p xf.wire))
+;;         (mv (msg "Error: Expression to be overridden can't be expressed as a concatenation of part-selects: ~x0~%" xf.wire)
+;;             nil))
+;;        (lhs (svex->normed-lhs xf.wire aliases))
+;;        (first (make-lhs-override :lhs lhs
+;;                                  :test xf.test
+;;                                  :val xf.val))
+;;        ((mv err rest) (svex-overrides-alias-norm (cdr x) aliases))
+;;        ((when err) (mv err nil)))
+;;     (mv nil (cons first rest)))
+;;   ///
+;;   (more-returns
+;;    (newx :name vars-of-svex-overrides-alias-norm
+;;          (implies (not (member v (svex-overridelist-vars x)))
+;;                   (not (member v (lhs-overridelist-vars newx))))
+;;          :hints(("Goal" :in-theory (enable lhs-overridelist-vars
+;;                                            svex-overridelist-vars)))))
 
-  (more-returns
-   (newx :name keys-of-svex-overrides-alias-norm
-         (implies (not (member v (aliases-vars aliases)))
-                  (not (member v (lhs-overridelist-keys newx))))
-         :hints(("Goal" :in-theory (enable lhs-overridelist-keys))))))
+;;   (more-returns
+;;    (newx :name keys-of-svex-overrides-alias-norm
+;;          (implies (not (member v (aliases-vars aliases)))
+;;                   (not (member v (lhs-overridelist-keys newx))))
+;;          :hints(("Goal" :in-theory (enable lhs-overridelist-keys))))))
 
 
 (define cap-length ((n natp) x)
@@ -730,11 +845,12 @@ svex-assigns-compose)).</li>
                                      (aliases-normorderedp new-aliases))))
   :guard (svarlist-addr-p (modalist-vars (design->modalist x)))
   :verify-guards nil
-  :prepwork ((local (in-theory (enable modscope->top modscope->modidx modscope-okp
-                                       modscope-top-bound modscope-okp))))
+  :prepwork ((local (in-theory (e/d (modscope->top modscope->modidx modscope-okp
+                                                   modscope-top-bound modscope-okp)
+                                    ((moddb-clear))))))
 
   (b* ((moddb (moddb-clear moddb))
-       (aliases (aliases-fix aliases))
+       (aliases (resize-lhss 0 aliases))
        ((design x) x)
        (modalist x.modalist)
        (topmod x.top)
@@ -756,7 +872,6 @@ svex-assigns-compose)).</li>
         ((elab-mod (moddb->modsi modidx moddb)))
         (elab-mod->totalwires elab-mod))
        ;; (- (cw "Total wires: ~x0~%" totalwires))
-       (aliases (resize-lhss 0 aliases))
        (aliases (resize-lhss totalwires aliases))
 
        ;; ((unless modidx)
@@ -821,7 +936,13 @@ svex-assigns-compose)).</li>
            (svarlist-boundedp (constraintlist-vars flat-constraints) bound)
            (svarlist-boundedp (assigns-vars flat-fixups) bound)
            ;; (svarlist-boundedp (svar-map-vars res-delays) (len aliases))
-           ))))
+           )))
+
+  (defretd normalize-stobjs-of-<fn>
+    (implies (syntaxp (not (and (equal aliases ''nil)
+                                (equal moddb ''nil))))
+             (equal <call>
+                    (let ((moddb nil) (aliases nil)) <call>)))))
 
 
 (local (include-book "tools/trivial-ancestors-check" :dir :system))
@@ -858,9 +979,34 @@ svex-assigns-compose)).</li>
              (not (member v (assigns-vars new-x))))
     :hints(("Goal" :in-theory (enable assigns-vars)))))
 
+
+(define svex-alist-truncate-by-var-decls ((x svex-alist-p)
+                                          (var-decls var-decl-map-p)
+                                          (acc svex-alist-p))
+  :returns (new-x svex-alist-p)
+  (b* (((when (atom x)) (svex-alist-fix acc))
+       ((unless (mbt (and (consp (car x)) (svar-p (caar x)))))
+        (svex-alist-truncate-by-var-decls (cdr x) var-decls acc))
+       ((cons var val) (car x))
+       (wire (cdr (hons-get var (var-decl-map-fix var-decls))))
+       ((unless wire)
+        (svex-alist-truncate-by-var-decls (cdr x) var-decls acc))
+       ((wire wire))
+       (acc (cons (cons var (svex-concat wire.width val 0)) (svex-alist-fix acc))))
+    (svex-alist-truncate-by-var-decls (cdr x) var-decls acc))
+  ///
+  (defret vars-of-<fn>
+    (implies (and (svarlist-addr-p (svex-alist-vars x))
+                  (svarlist-addr-p (svex-alist-vars acc)))
+             (svarlist-addr-p (svex-alist-vars new-x)))
+    :hints(("Goal" :in-theory (enable svex-alist-vars))))
+
+  (local (in-theory (enable svex-alist-fix))))
+
 (define svex-normalize-assigns ((assigns assigns-p)
                                 (fixups assigns-p)
                                 (constraints constraintlist-p)
+                                (var-decls var-decl-map-p)
                                 (aliases))
   :guard (and ;; (svarlist-boundedp (svar-map-vars delays) (aliass-length aliases))
           (svarlist-boundedp (assigns-vars assigns) (aliass-length aliases))
@@ -909,10 +1055,11 @@ svex-assigns-compose)).</li>
        ;; compose norm-fixups with res-assigns
        (final-fixups (assigns-compose norm-fixups res-assigns))
 
-       (final-assigns (fast-alist-free
-                       (fast-alist-clean
-                        (svex-apply-overrides (assigns-to-overrides final-fixups) res-assigns))))
+       (final-assigns1 (fast-alist-free
+                        (fast-alist-clean
+                         (svex-apply-overrides (assigns-to-overrides final-fixups) res-assigns))))
 
+       (final-assigns (svex-alist-truncate-by-var-decls final-assigns1 var-decls nil))
        ;; Collect all variables referenced and add delays as needed.
        (delayvars (svarlist-collect-delays (svexlist-collect-vars (svex-alist-vals res-assigns))))
        (res-delays (delay-svarlist->delays delayvars)))
@@ -1044,7 +1191,7 @@ should address this again later.</p>"
                                            (indexedp 'nil)
                                            ((moddb "overwritten") 'moddb)
                                            ((aliases "overwritten") 'aliases))
-  
+
   :parents (svex-compilation)
   :short "Flatten a hierarchical SV design and apply alias normalization to it."
   :long "<p>This does all of the steps of @(see svex-design-compile) except for
@@ -1064,14 +1211,20 @@ should address this again later.</p>"
        ((when err)
         (mv err nil nil nil moddb aliases))
        (modidx (moddb-modname-get-index (design->top x) moddb))
-       (aliases (if indexedp
-                    aliases
-                  (cwtime (aliases-indexed->named aliases
-                                                  (make-modscope-top :modidx modidx)
-                                                  moddb)
-                          :mintime 1)))
+       ((mv aliases var-decl-map)
+        (if indexedp
+            (b* ((map (cwtime
+                       (aliases-to-var-decl-map aliases
+                                                (make-modscope-top :modidx modidx)
+                                                moddb)
+                       :mintime 1)))
+              (mv aliases map))
+          (cwtime (aliases-indexed->named aliases
+                                          (make-modscope-top :modidx modidx)
+                                          moddb)
+                  :mintime 1)))
        ((mv res-assigns res-delays res-constraints)
-        (svex-normalize-assigns assigns fixups constraints aliases)))
+        (svex-normalize-assigns assigns fixups constraints var-decl-map aliases)))
     (mv nil res-assigns res-delays res-constraints moddb aliases))
   ///
   (verify-guards svex-design-flatten-and-normalize-fn
@@ -1107,7 +1260,14 @@ should address this again later.</p>"
   (defret vars-of-svex-design-flatten-and-normalize-constraints
     (implies (and (modalist-addr-p (design->modalist x))
                   (not indexedp))
-             (svarlist-addr-p (constraintlist-vars flat-constraints)))))
+             (svarlist-addr-p (constraintlist-vars flat-constraints))))
+
+  (defretd normalize-stobjs-of-<fn>
+    (implies (syntaxp (not (and (equal aliases ''nil)
+                                (equal moddb ''nil))))
+             (equal <call>
+                    (let ((moddb nil) (aliases nil)) <call>)))
+    :hints(("Goal" :in-theory (enable normalize-stobjs-of-svex-design-flatten)))))
 
 
 (define svex-design-compile ((x design-p)
