@@ -36,7 +36,7 @@
 (include-book "centaur/vl/mlib/strip" :dir :system)
 (local (include-book "std/basic/arith-equivs" :dir :system))
 (local (include-book "../../vl/util/arithmetic"))
-(local (include-book "std/testing/assert" :dir :system))
+(local (include-book "std/testing/assert-bang" :dir :system))
 (local (std::add-default-post-define-hook :fix))
 (local (in-theory (disable (:e tau-system))))
 (local (in-theory (enable tag-reasoning)))
