@@ -42,6 +42,24 @@
   use-arithmetic-5
   :disabled t))
 
+(def-rp-rule bits-of-3vec-fix
+  (implies (and (natp a)
+                (natp b))
+           (equal (bits (sv::3vec-fix x) a b)
+                  (sv::3vec-fix (bits x a b))))
+  :otf-flg t
+  :hints (("Goal"
+           :use ((:instance 4vec-part-select-of-3vec-fix))
+           :do-not-induct t
+           :in-theory (e/d
+                       (bits
+                        ;;logior-of-logtail-same-size
+                        )
+                       (4vec-part-select-of-3vec-fix
+                        ;;BITOPS::LOGHEAD-OF-LOGIOR
+                        ;;BITOPS::LOGTAIL-OF-LOGIOR
+                        )))))
+
 (encapsulate nil
 
   ;;low priority lemmas:
@@ -404,6 +422,23 @@
                               4vec-bitnot-of-4vec-concat)
                              ())))))
 
+(encapsulate
+  nil
+
+  (local
+   (use-arithmetic-5 t))
+  (def-rp-rule 4vec-concat$-bit-and-zero
+    (implies (and (posp size)
+                  (bitp num1))
+             (equal (4vec-concat$ size num1 0)
+                    num1))
+    :hints (("Goal"
+             :in-theory (e/d (4VEC-CONCAT$
+                              4VEC-CONCAT
+                              SV::4VEC->UPPER
+                              SV::4VEC->LOWER)
+                             ())))))
+
 (encapsulate
   nil
 
@@ -962,6 +997,15 @@
     (implies (natp size)
              (equal (4vec-part-install start size old new)
                     (sbits start size (bits new 0 size ) old)))
+    :hints (("Goal"
+             :in-theory (e/d (sbits
+                              4vec-part-install-of-4vec-part-select) ()))))
+
+  (def-rp-rule$ t t
+    4vec-part-install-is-sbits-without-inserting-bits
+    (implies (natp size)
+             (equal (4vec-part-install start size old new)
+                    (sbits start size new old)))
     :hints (("Goal"
              :in-theory (e/d (sbits
                               4vec-part-install-of-4vec-part-select) ())))))
@@ -1605,8 +1649,24 @@
                              (
                               4vec-part-select-of-negated-bit
                               convert-4vec-concat-to-4vec-concat$
+                              )))))
+  (defthm bits-of-bitp-size=posp-start=0
+    (implies (and (bitp val)
+                  (posp size))
+             (equal (bits val 0 size)
+                    val))
+    :hints (("Goal"
+             :in-theory (e/d (bits
+                              4VEC-PART-SELECT
+                              SV::4VEC->UPPER
+                              sv::4vec->lower
+                              4VEC-CONCAT
+                              4VEC-CONCAT$)
+                             (
                               ))))))
 
+
+
 (encapsulate
   nil
 
@@ -1875,12 +1935,11 @@
     bits-of-sbits-4-no-syntaxp
     bits-of-sbits-5-no-syntaxp))
 
-
 (encapsulate
   nil
   (local
    (use-arithmetic-5 t))
-  
+
   (def-rp-rule$ t nil
     logbit-to-bits
     (implies (and (natp index)
@@ -1905,3 +1964,23 @@
 
   (rp-attach-sc logbit-to-bits
                 logbit-to-bits-side-cond))
+
+
+(def-rp-rule 4vec-parity-of-bitp
+  (implies (bitp x)
+           (equal (sv::4vec-parity x)
+                  (- x)))
+  :hints (("Goal"
+           :in-theory (e/d (bitp) ()))))
+
+(def-rp-rule 4vec-parity-of-bits-to-4vec-bitxor
+  (implies (and (integerp x)
+                (natp start))
+           (and (equal (sv::4vec-parity (bits x start 2))
+                       (- (sv::4vec-bitxor (bits x start 1)
+                                           (bits x (1+ start) 1))))
+                (equal (sv::4vec-parity (bits x start 1))
+                       (- (bits x start 1)))))
+  :hints (("Goal"
+           :do-not-induct t
+           :use ((:instance 4vec-parity-of-4vec-part-select-to-4vec-bitxor)))))
