@@ -108,9 +108,7 @@
    (let* ((x.kind (svex-kind-wog x)))
      (case
        x.kind
-       (:quote (mv (cond ((atom x) (list 'quote x))
-                         ((atom (cdr x)) (list 'quote (sv::4vec-x)))
-                         (t (list 'quote (cadr x))))
+       (:quote (mv (list 'quote x)
                    t))
        (:var (if good-env-flg
                  (mv (let* ((val (hons-get x env-falist)))
@@ -186,9 +184,7 @@
    (let* ((x.kind (svexl-node-kind-wog x)))
      (case
        x.kind
-       (:quote (mv (cond ((atom x) (list 'quote x))
-                         ((atom (cdr x)) (list 'quote (sv::4vec-x)))
-                         (t (list 'quote (cadr x))))
+       (:quote (mv (list 'quote x)
                    t))
        (:var (if good-env-flg
                  (mv (let* ((val (hons-get x env-falist)))
@@ -314,15 +310,15 @@
     :gag-mode nil
     (rp::def-formula-checks
      svex-eval-wog-formula-checks
-     (svex-eval-wog-meta-main
+     (;;svex-eval-wog-meta-main
       sv::4vec-fix$inline
       svex-env-fastlookup-wog
-      svexl-node-eval-wog-meta-main
+      ;;svexl-node-eval-wog-meta-main
       svexl-node-eval-wog
       svex-eval-wog))))
 
 (local
- (defthmd svex-eval-wog-eval 
+ (defthmd svex-eval-wog-eval
    (implies (and (rp-evl-meta-extract-global-facts)
                  (svex-eval-wog-formula-checks state)
                  (consp x)
@@ -532,8 +528,6 @@
                                (:rewrite
                                 sv::4vec-p-when-member-equal-of-4veclist-p)
                                (:rewrite sv::maybe-4vec-p-when-4vec-p)
-                               (:rewrite
-                                sv::car-of-4veclist-fix-x-normalize-const-under-4vec-equiv)
                                (:rewrite sv::4vec-p-of-nth-when-4veclist-p)
                                (:type-prescription svex-eval-wog-formula-checks)
                                (:type-prescription o<)
@@ -856,7 +850,7 @@
                                svex-eval-wog-meta-lst
                                rp::is-falist)
                               ()))))
-   
+
    (defthm-svexl-node-eval-wog-meta
      (defthmd rp-termp-svexl-node-eval-wog-meta
        (implies (and (if good-env-flg
@@ -889,7 +883,7 @@
                                rp::is-falist)
                               ()))))
 
-   
+
 
    (local
     (defthm lemma2
@@ -1172,7 +1166,7 @@
                                (:REWRITE LEMMA3)
                                (:DEFINITION RP::FALIST-CONSISTENT))))))))
 
-(defthm valid-rp-meta-rulep-svex-eval-wog-meta-main
+#|(defthm valid-rp-meta-rulep-svex-eval-wog-meta-main
   (implies (and (rp-evl-meta-extract-global-facts)
                 (svex-eval-wog-formula-checks state))
            (let ((rule (make rp::rp-meta-rule-rec
@@ -1189,10 +1183,10 @@
                             svex-eval-wog-meta-main
                             rp::rp-term-listp
                             rp::valid-sc
-                            rp::valid-sc)))))
+                            rp::valid-sc)))))||#
 
 
-(defthm valid-rp-meta-rulep-svexl-node-eval-wog-meta-main
+#|(defthm valid-rp-meta-rulep-svexl-node-eval-wog-meta-main
   (implies (and (rp-evl-meta-extract-global-facts)
                 (svex-eval-wog-formula-checks state))
            (let ((rule (make rp::rp-meta-rule-rec
@@ -1208,9 +1202,9 @@
                            (rp::rp-termp
                             svexl-node-eval-wog-meta-main
                             rp::rp-term-listp
-                            rp::valid-sc)))))
+                            rp::valid-sc)))))||#
 
-(rp::add-meta-rules svex-eval-wog-formula-checks
+#|(rp::add-meta-rules svex-eval-wog-formula-checks
                     (list (make rp::rp-meta-rule-rec
                                 :fnc 'svex-eval-wog-meta-main
                                 :trig-fnc 'svex-eval-wog
@@ -1220,4 +1214,19 @@
                                 :fnc 'svexl-node-eval-wog-meta-main
                                 :trig-fnc 'svexl-node-eval-wog
                                 :dont-rw t
-                                :valid-syntax t)))
+                                :valid-syntax t)))||#
+
+
+(rp::add-meta-rule
+ :meta-fnc svexl-node-eval-wog-meta-main
+ :trig-fnc svexl-node-eval-wog
+ :formula-checks svex-eval-wog-formula-checks
+ :valid-syntaxp t
+ :returns (mv term dont-rw))
+
+(rp::add-meta-rule
+ :meta-fnc svex-eval-wog-meta-main
+ :trig-fnc svex-eval-wog
+ :formula-checks svex-eval-wog-formula-checks
+ :valid-syntaxp t
+ :returns (mv term dont-rw))
