@@ -33,6 +33,7 @@
 (include-book "shell")
 (include-book "gather")
 (include-book "zip")
+(include-book "json")
 (include-book "server")
 (include-book "oslib/argv" :dir :system)
 (include-book "centaur/misc/intern-debugging" :dir :system)
@@ -130,6 +131,7 @@ commands.</p>
           (cons "server" *vl-server-help*)
           (cons "shell"  *vl-shell-help*)
           (cons "zip"    *vl-zip-help*)
+          (cons "json"   *vl-json-help*)
           ))
 
   (encapsulate
@@ -258,6 +260,11 @@ vl-toolkit-other-command).</p>"
           (exit-ok)
           state))
 
+       ((when (equal cmd "json"))
+        (b* ((state (vl-json-top args)))
+          (exit-ok)
+          state))
+
        ((when (equal cmd "server"))
         (b* ((state (vl-server-top args)))
           ;; Do not call exit here, same reason as 'shell'
