@@ -311,127 +311,7 @@ tests at the bottom of this file.</p>")
             (vl-idtoken-p (car (vl-tokstream->tokens))))
    :hints(("Goal" :in-theory (enable vl-is-token?)))))
 
-(defparser vl-parse-param-expression ()
-  ;; Verilog-2005:       Matches mintypmax_expression
-  ;; SystemVerilog-2012: Matches mintypmax_expression | data_type | $
-  ;;
-  ;; Except that our SystemVerilog expression parser already accepts $ as an
-  ;; expression, so we really just match:
-  ;;
-  ;; param_expression ::= mintypmax_expression | data_type
-  :result (vl-paramvalue-p val)
-  :resultp-of-nil nil
-  :fails gracefully
-  :count strong
-  ;; We use backtracking to try to match expression first, then datatype only
-  ;; if that fails.  This order is important, to handle the ambiguous case of a
-  ;; plain identifier in the way that the type-disambiguation transform
-  ;; expects.
-  (b* ((backup (vl-tokstream-save))
-       ((mv err expr tokstream)
-        (vl-parse-mintypmax-expression))
-       ((unless err)
-        (mv err (make-vl-paramvalue-expr :expr expr) tokstream))
-       (tokstream (vl-tokstream-restore backup)))
-    (seq tokstream
-         (type := (vl-parse-datatype))
-         (return (make-vl-paramvalue-type :type type)))))
 
-(defparser vl-parse-named-parameter-assignment ()
-  :result (vl-namedparamvalue-p val)
-  :resultp-of-nil nil
-  :fails gracefully
-  :count strong
-  (seq tokstream
-        (:= (vl-match-token :vl-dot))
-        (id := (vl-match-token :vl-idtoken))
-        (:= (vl-match-token :vl-lparen))
-        (unless (vl-is-token? :vl-rparen)
-          (value := (vl-parse-param-expression)))
-        (:= (vl-match-token :vl-rparen))
-        (return (make-vl-namedparamvalue :name (vl-idtoken->name id)
-                                         :value value))))
-
-(defparser vl-parse-list-of-named-parameter-assignments ()
-  :result (vl-namedparamvaluelist-p val)
-  :resultp-of-nil t
-  :true-listp t
-  :fails gracefully
-  :count strong
-  (seq tokstream
-        (first := (vl-parse-named-parameter-assignment))
-        (when (vl-is-token? :vl-comma)
-          (:= (vl-match-token :vl-comma))
-          (rest := (vl-parse-list-of-named-parameter-assignments)))
-        (return (cons first rest))))
-
-(defparser vl-parse-list-of-ordered-parameter-assignments ()
-  :result (vl-paramvaluelist-p val)
-  :resultp-of-nil t
-  :true-listp t
-  :fails gracefully
-  :count strong
-  (seq tokstream
-        (first := (vl-parse-param-expression))
-        (when (vl-is-token? :vl-comma)
-          (:= (vl-match-token :vl-comma))
-          (rest := (vl-parse-list-of-ordered-parameter-assignments)))
-        (return (cons first rest))))
-
-(defprojection vl-expressions->paramvalues ((x vl-exprlist-p))
-  :returns (paramvals vl-paramvaluelist-p)
-  (vl-paramvalue-expr x))
-
-(defparser vl-parse-list-of-parameter-assignments ()
-  :result (vl-paramargs-p val)
-  :resultp-of-nil nil
-  :fails gracefully
-  :count strong
-  (seq tokstream
-        (when (vl-is-token? :vl-dot)
-          (args := (vl-parse-list-of-named-parameter-assignments))
-          (return (make-vl-paramargs-named :args args)))
-        (exprs := (if (eq (vl-loadconfig->edition config) :verilog-2005)
-                      ;; Verilog-2005 doesn't allow mintypmax exprs here.
-                      (b* (((mv err val tokstream)
-                            (vl-parse-1+-expressions-separated-by-commas))
-                           ((when err) (mv err nil tokstream)))
-                        (mv err (vl-expressions->paramvalues val) tokstream))
-                    ;; SystemVerilog-2012 does.
-                    (vl-parse-list-of-ordered-parameter-assignments)))
-        (return (make-vl-paramargs-plain :args exprs))))
-
-(defparser vl-parse-parameter-value-assignment ()
-  :result (vl-paramargs-p val)
-  :resultp-of-nil nil
-  :fails gracefully
-  :count strong
-  (seq tokstream
-        (:= (vl-match-token :vl-pound))
-
-        (unless (vl-is-token? :vl-lparen)
-          ;; Surprisingly, even though the grammar seems to clearly require
-          ;; parentheses, other tools accept things like myadder #3 (o, a,
-          ;; b).  So, we'll also try to tolerate a single expression that comes
-          ;; immediately after a pound here.  In this case there shouldn't be
-          ;; any closing paren.
-          (expr := (vl-parse-expression))
-          (return (make-vl-paramargs-plain :args
-                                           (list (make-vl-paramvalue-expr :expr expr)))))
-
-        ;; Otherwise we found #(, so match the arguments and so on.
-        (:= (vl-match))
-
-        (when (and (vl-is-token? :vl-rparen)
-                   (not (eq (vl-loadconfig->edition config) :verilog-2005)))
-          ;; In SystemVerilog, #() is allowed.  However, in Verilog-2005 it's a
-          ;; parse error.
-          (:= (vl-match))
-          (return (make-vl-paramargs-plain :args nil)))
-
-        (args := (vl-parse-list-of-parameter-assignments))
-        (:= (vl-match-token :vl-rparen))
-        (return args)))
 
 
 ; Verilog-2005:
