@@ -39,6 +39,10 @@
 ; Matt K.: Avoid ACL2(p) error (the job died after vl-maybe-scopeitem-p).
 (set-waterfall-parallelism nil)
 
+; Matt K. addition: Important for speed, given the change after v8-0 to keep
+; LET expressions in right-hand sides of rewrite rules.
+(add-default-hints '('(:do-not '(preprocess))) :at-end t)
+
 (defxdoc scopestack
   :parents (mlib)
   :short "Scopestacks deal with namespaces in SystemVerilog by tracking the
@@ -316,7 +320,8 @@ other kinds of scopes (e.g., compilation units?) we could add them here.</p>"
 at the top level.  However, if we ever want to allow, e.g., nested modules,
 then we will need to extend this.</p>"
     '((design ()
-              (module :acc mods) udp interface program)))
+              (module :acc mods) udp interface program
+              (class :acc classes))))
 
   (defval *vl-scopes->portdecls*
     :parents (scopestack-constants)
@@ -459,6 +464,14 @@ in it, such as a function, task, or block statement."
     (make-vl-blockscope :vardecls x.initdecls
                         :scopetype :vl-forstmt)))
 
+(define vl-foreachstmt->blockscope ((x vl-stmt-p))
+  :guard (vl-stmt-case x :vl-foreachstmt)
+  :returns (scope vl-blockscope-p)
+  :parents (vl-blockscope vl-scopestack-push)
+  (b* (((vl-foreachstmt x)))
+    (make-vl-blockscope :vardecls x.vardecls
+                        :scopetype :vl-foreachstmt)))
+
 
 ;; Notes on name spaces -- from SV spec 3.13
 ;; SV spec lists 8 namespaces:
@@ -959,6 +972,10 @@ be very cheap in the single-threaded case.</p>"
                      (equal (tag x) :vl-scopeinfo)))
         :rule-classes :forward-chaining))))
 
+; Matt K. addition: Undo the added add-default-hints above to avoid failure in
+; next event.
+(remove-default-hints '('(:do-not '(preprocess))))
+
 (define vl-scope->scopetype ((x vl-scope-p))
   :returns (type vl-scopetype-p
                  :hints(("Goal" :in-theory (enable vl-scopetype-p))))
@@ -1726,7 +1743,8 @@ be very cheap in the single-threaded case.</p>"
              (or (equal (tag def) :vl-module)
                  (equal (tag def) :vl-udp)
                  (equal (tag def) :vl-interface)
-                 (equal (tag def) :vl-program))))
+                 (equal (tag def) :vl-program)
+                 (equal (tag def) :vl-class))))
   :rule-classes ((:forward-chaining))
   :hints(("Goal"
           :use ((:instance tag-when-vl-scopedef-p-forward
@@ -1856,7 +1874,6 @@ useful or meant for debugging purposes.</p>"
 
 
 
-
 ; Definition of vl-design-toplevel
 ;
 ; For resolving top-level hierarchical identifiers like topmod.foo.bar, we need
@@ -1865,7 +1882,15 @@ useful or meant for debugging purposes.</p>"
 ; place for vl-design-toplevel, and decided to memoize it and free it alongside
 ; of the other scopestack memo tables in vl-scopestacks-free.
 
+(define vl-bindlist->modinsts ((x vl-bindlist-p))
+  :returns (modinsts vl-modinstlist-p)
+  (if (atom x)
+      nil
+    (append-without-guard (vl-bind->modinsts (car x))
+                          (vl-bindlist->modinsts (cdr x)))))
+
 (def-genblob-transform vl-genblob->flatten-modinsts ((acc vl-modinstlist-p))
+  ;; Warning: rarely sensible -- returns modinsts from many scopes!
   :no-new-x t
   :returns ((acc vl-modinstlist-p))
   :apply-to-generates vl-generates->flatten-modinsts
@@ -1876,19 +1901,23 @@ useful or meant for debugging purposes.</p>"
                                         vl-modinstlist-p-when-subsetp-equal))))
   (vl-generates->flatten-modinsts (vl-genblob->generates x)
                                   (append-without-guard
+                                   (vl-bindlist->modinsts (vl-genblob->binds x))
                                    (vl-genblob->modinsts x)
                                    (vl-modinstlist-fix acc))))
 
+
 (define vl-module->flatten-modinsts ((x vl-module-p))
   :parents (vl-modulelist-everinstanced)
-  :short "Gather modinsts from the module, including its generate blocks."
+  :short "Gather modinsts from the module, including its generate blocks and bind
+          constructs (which don't even belong to it) -- rarely sensible!"
   :returns (modinsts vl-modinstlist-p)
   (b* ((genblob (vl-module->genblob x)))
     (vl-genblob->flatten-modinsts genblob nil)))
 
 (define vl-interface->flatten-modinsts ((x vl-interface-p))
   :parents (vl-interfacelist-everinstanced)
-  :short "Gather modinsts from the interface, including its generate blocks."
+  :short "Gather modinsts from the module, including its generate blocks and bind
+          constructs (which don't even belong to it) -- rarely sensible!"
   :returns (modinsts vl-modinstlist-p)
   (b* ((genblob (vl-interface->genblob x)))
     (vl-genblob->flatten-modinsts genblob nil)))
