@@ -192,169 +192,203 @@ found in the datatype of a variable, parameter, or typedef declaration.</p>")
         (append-without-guard decls decls-rest))))
 
 
+(fty::defmap vl-datatype-map :key-type vl-datatype :val-type vl-datatype :true-listp t)
+
 (defines vl-datatype-enumname-declarations
   (define vl-datatype-enumname-declarations ((x           vl-datatype-p)
                                              (typedefname maybe-stringp)
-                                             (loc         vl-location-p))
+                                             (loc         vl-location-p)
+                                             (seen        vl-datatype-map-p))
     :returns (mv (new-x vl-datatype-p)
                  (warnings vl-warninglist-p)
-                 (paramdecls vl-paramdecllist-p))
+                 (paramdecls vl-paramdecllist-p)
+                 (seen-out vl-datatype-map-p))
     :measure (vl-datatype-count x)
     :verify-guards nil
-    (vl-datatype-case x
-      :vl-enum (b* ((warnings    nil)
-                    (typedefname (maybe-string-fix typedefname))
-                    ;; For debugging, mark each declaration that we introduce
-                    ;; with a VL_ENUMITEM attribute that, if possible, will say
-                    ;; the name of the typedef that declares the enum.
-                    (atts (list (cons "VL_ENUMITEM"
-                                      (and typedefname
-                                           (make-vl-literal :val (make-vl-string :value typedefname))))))
-                    ((wmv warnings decls :ctx (vl-datatype-fix x))
-                     (vl-enumitemlist-enumname-declarations x.items (vl-make-index 0) x.basetype
-                                                            atts loc)))
-                 (mv (change-vl-enum x :values (vl-make-idexpr-list (vl-paramdecllist->names decls)))
-                     warnings decls))
-      :vl-struct (b* (((mv new-membs warnings paramdecls)
-                       (vl-structmemberlist-enumname-declarations x.members typedefname loc)))
-                   (mv (change-vl-struct x :members new-membs) warnings paramdecls))
-      :vl-union (b* (((mv new-membs warnings paramdecls)
-                      (vl-structmemberlist-enumname-declarations x.members typedefname loc)))
-                  (mv (change-vl-union x :members new-membs) warnings paramdecls))
-      :otherwise (mv (vl-datatype-fix x) nil nil)))
+    (b* ((seen (vl-datatype-map-fix seen)))
+      (vl-datatype-case x
+        :vl-enum (b* ((look (hons-get (vl-datatype-fix x) seen))
+                      ((when look)
+                       (mv (cdr look) nil nil seen))
+                      (warnings nil)
+                      (typedefname (maybe-string-fix typedefname))
+                      ;; For debugging, mark each declaration that we introduce
+                      ;; with a VL_ENUMITEM attribute that, if possible, will say
+                      ;; the name of the typedef that declares the enum.
+                      (atts (list (cons "VL_ENUMITEM"
+                                        (and typedefname
+                                             (make-vl-literal :val (make-vl-string :value typedefname))))))
+                      ((wmv warnings decls :ctx (vl-datatype-fix x))
+                       (vl-enumitemlist-enumname-declarations x.items (vl-make-index 0) x.basetype
+                                                              atts loc))
+                      (ans (change-vl-enum x :values (vl-make-idexpr-list (vl-paramdecllist->names decls)))))
+                   (mv ans
+                       warnings decls
+                       (hons-acons (vl-datatype-fix x) ans seen)))
+        :vl-struct (b* (((mv new-membs warnings paramdecls seen)
+                         (vl-structmemberlist-enumname-declarations x.members typedefname loc seen)))
+                     (mv (change-vl-struct x :members new-membs) warnings paramdecls seen))
+        :vl-union (b* (((mv new-membs warnings paramdecls seen)
+                        (vl-structmemberlist-enumname-declarations x.members typedefname loc seen)))
+                    (mv (change-vl-union x :members new-membs) warnings paramdecls seen))
+        :otherwise (mv (vl-datatype-fix x) nil nil seen))))
   (define vl-structmemberlist-enumname-declarations ((x vl-structmemberlist-p)
                                                      (typedefname maybe-stringp)
-                                                     (loc vl-location-p))
+                                                     (loc vl-location-p)
+                                                     (seen        vl-datatype-map-p))
     :returns (mv (new-x vl-structmemberlist-p)
                  (warnings vl-warninglist-p)
-                 (paramdecls vl-paramdecllist-p))
+                 (paramdecls vl-paramdecllist-p)
+                 (seen-out vl-datatype-map-p))
     :measure (vl-structmemberlist-count x)
-    (b* (((when (atom x)) (mv nil nil nil))
-         ((mv new-type warnings1 decls1)
-          (vl-datatype-enumname-declarations (vl-structmember->type (car x)) typedefname loc))
-         ((mv new-rest warnings-rest decls-rest)
-          (vl-structmemberlist-enumname-declarations (cdr x) typedefname loc)))
+    (b* (((when (atom x)) (mv nil nil nil (vl-datatype-map-fix seen)))
+         ((mv new-type warnings1 decls1 seen)
+          (vl-datatype-enumname-declarations (vl-structmember->type (car x)) typedefname loc seen))
+         ((mv new-rest warnings-rest decls-rest seen)
+          (vl-structmemberlist-enumname-declarations (cdr x) typedefname loc seen)))
       (mv (cons (change-vl-structmember (car x) :type new-type) new-rest)
           (append-without-guard warnings1 warnings-rest)
-          (append-without-guard decls1 decls-rest))))
+          (append-without-guard decls1 decls-rest)
+          seen)))
   ///
   (deffixequiv-mutual vl-datatype-enumname-declarations)
   (verify-guards vl-datatype-enumname-declarations))
 
-(define vl-paramdecl-enumname-declarations ((x vl-paramdecl-p))
+(define vl-paramdecl-enumname-declarations ((x vl-paramdecl-p)
+                                            (seen vl-datatype-map-p))
   :returns (mv (new-x vl-paramdecl-p)
                (warnings vl-warninglist-p)
-               (paramdecls vl-paramdecllist-p))
+               (paramdecls vl-paramdecllist-p)
+               (seen-out vl-datatype-map-p))
   (b* (((vl-paramdecl x))
-       ((mv new-type warnings paramdecls)
+       ((mv new-type warnings paramdecls seen)
         (vl-paramtype-case x.type
           ;; Note: Not sure how this should work for type parameters.  Presumably
           ;; it shouldn't hurt to make the declarations, even if it's going to get
           ;; overridden?
           :vl-typeparam
-          (b* (((mv type warnings paramdecls)
+          (b* (((mv type warnings paramdecls seen)
                 (if x.type.default
-                    (vl-datatype-enumname-declarations x.type.default nil x.loc)
-                  (mv nil nil nil))))
+                    (vl-datatype-enumname-declarations x.type.default nil x.loc seen)
+                  (mv nil nil nil (vl-datatype-map-fix seen)))))
             (mv (change-vl-typeparam x.type :default type)
-                warnings paramdecls))
+                warnings paramdecls seen))
           :vl-explicitvalueparam
-          (b* (((mv type warnings paramdecls)
-                (vl-datatype-enumname-declarations x.type.type nil x.loc)))
-            (mv (change-vl-explicitvalueparam x.type :type type) warnings paramdecls))
-          :otherwise (mv x.type nil nil))))
-    (mv (change-vl-paramdecl x :type new-type) warnings paramdecls)))
+          (b* (((mv type warnings paramdecls seen)
+                (vl-datatype-enumname-declarations x.type.type nil x.loc seen)))
+            (mv (change-vl-explicitvalueparam x.type :type type) warnings paramdecls seen))
+          :otherwise (mv x.type nil nil (vl-datatype-map-fix seen)))))
+    (mv (change-vl-paramdecl x :type new-type) warnings paramdecls seen)))
 
-(define vl-vardecl-enumname-declarations ((x vl-vardecl-p))
+(define vl-vardecl-enumname-declarations ((x vl-vardecl-p)
+                                          (seen vl-datatype-map-p))
   :returns (mv (new-x vl-vardecl-p)
                (warnings vl-warninglist-p)
-               (paramdecls vl-paramdecllist-p))
+               (paramdecls vl-paramdecllist-p)
+               (seen-out vl-datatype-map-p))
   (b* (((vl-vardecl x))
-       ((mv new-type warnings paramdecls)
-        (vl-datatype-enumname-declarations x.type nil x.loc)))
-    (mv (change-vl-vardecl x :type new-type) warnings paramdecls)))
+       ((mv new-type warnings paramdecls seen)
+        (vl-datatype-enumname-declarations x.type nil x.loc seen)))
+    (mv (change-vl-vardecl x :type new-type) warnings paramdecls seen)))
 
-(define vl-typedef-enumname-declarations ((x vl-typedef-p))
+(define vl-typedef-enumname-declarations ((x vl-typedef-p)
+                                          (seen vl-datatype-map-p))
   :returns (mv (new-x vl-typedef-p)
                (warnings vl-warninglist-p)
-               (paramdecls vl-paramdecllist-p))
+               (paramdecls vl-paramdecllist-p)
+               (seen-out vl-datatype-map-p))
   (b* (((vl-typedef x))
-       ((mv new-type warnings paramdecls)
-        (vl-datatype-enumname-declarations x.type x.name x.minloc)))
-    (mv (change-vl-typedef x :type new-type) warnings paramdecls)))
+       ((mv new-type warnings paramdecls seen)
+        (vl-datatype-enumname-declarations x.type x.name x.minloc seen)))
+    (mv (change-vl-typedef x :type new-type) warnings paramdecls seen)))
 
 ;; BOZO Do we need to also cover function/task decls, portdecls, etc?
 
 
-(define vl-paramdecllist-enumname-declarations ((x vl-paramdecllist-p))
+(define vl-paramdecllist-enumname-declarations ((x vl-paramdecllist-p)
+                                                (seen vl-datatype-map-p))
   :returns (mv (new-x vl-paramdecllist-p)
                (warnings vl-warninglist-p)
-               (paramdecls vl-paramdecllist-p))
-  (b* (((when (atom x)) (mv nil nil nil))
-       ((mv new-x1 warnings1 decls1) (vl-paramdecl-enumname-declarations (car x)))
-       ((mv new-x2 warnings2 decls2) (vl-paramdecllist-enumname-declarations (cdr x))))
+               (paramdecls vl-paramdecllist-p)
+               (seen-out vl-datatype-map-p))
+  (b* (((when (atom x)) (mv nil nil nil (vl-datatype-map-fix seen)))
+       ((mv new-x1 warnings1 decls1 seen) (vl-paramdecl-enumname-declarations (car x) seen))
+       ((mv new-x2 warnings2 decls2 seen) (vl-paramdecllist-enumname-declarations (cdr x) seen)))
     (mv (cons new-x1 new-x2)
         (append-without-guard warnings1 warnings2)
-        (append-without-guard decls1 decls2))))
+        (append-without-guard decls1 decls2)
+        seen)))
 
-(define vl-vardecllist-enumname-declarations ((x vl-vardecllist-p))
+(define vl-vardecllist-enumname-declarations ((x vl-vardecllist-p)
+                                              (seen vl-datatype-map-p))
   :returns (mv (new-x vl-vardecllist-p)
                (warnings vl-warninglist-p)
-               (paramdecls vl-paramdecllist-p))
-  (b* (((when (atom x)) (mv nil nil nil))
-       ((mv new-x1 warnings1 decls1) (vl-vardecl-enumname-declarations (car x)))
-       ((mv new-x2 warnings2 decls2) (vl-vardecllist-enumname-declarations (cdr x))))
+               (paramdecls vl-paramdecllist-p)
+               (seen-out vl-datatype-map-p))
+  (b* (((when (atom x)) (mv nil nil nil (vl-datatype-map-fix seen)))
+       ((mv new-x1 warnings1 decls1 seen) (vl-vardecl-enumname-declarations (car x) seen))
+       ((mv new-x2 warnings2 decls2 seen) (vl-vardecllist-enumname-declarations (cdr x) seen)))
     (mv (cons new-x1 new-x2)
         (append-without-guard warnings1 warnings2)
-        (append-without-guard decls1 decls2))))
+        (append-without-guard decls1 decls2)
+        seen)))
 
-(define vl-typedeflist-enumname-declarations ((x vl-typedeflist-p))
+(define vl-typedeflist-enumname-declarations ((x vl-typedeflist-p)
+                                              (seen vl-datatype-map-p))
   :returns (mv (new-x vl-typedeflist-p)
                (warnings vl-warninglist-p)
-               (paramdecls vl-paramdecllist-p))
-  (b* (((when (atom x)) (mv nil nil nil))
-       ((mv new-x1 warnings1 decls1) (vl-typedef-enumname-declarations (car x)))
-       ((mv new-x2 warnings2 decls2) (vl-typedeflist-enumname-declarations (cdr x))))
+               (paramdecls vl-paramdecllist-p)
+               (seen-out vl-datatype-map-p))
+  (b* (((when (atom x)) (mv nil nil nil (vl-datatype-map-fix seen)))
+       ((mv new-x1 warnings1 decls1 seen) (vl-typedef-enumname-declarations (car x) seen))
+       ((mv new-x2 warnings2 decls2 seen) (vl-typedeflist-enumname-declarations (cdr x) seen)))
     (mv (cons new-x1 new-x2)
         (append-without-guard warnings1 warnings2)
-        (append-without-guard decls1 decls2))))
+        (append-without-guard decls1 decls2)
+        seen)))
 
 
 (defines vl-genelementlist-enumname-declarations
   :verify-guards nil
 
-  (define vl-genelementlist-enumname-declarations ((x vl-genelementlist-p))
+  (define vl-genelementlist-enumname-declarations ((x vl-genelementlist-p)
+                                                   (seen vl-datatype-map-p))
     :returns (mv (warnings  vl-warninglist-p)
-                 (new-x     vl-genelementlist-p))
+                 (new-x     vl-genelementlist-p)
+                 (seen-out vl-datatype-map-p))
     :measure (two-nats-measure (vl-genelementlist-count x) 0)
     (b* (((when (atom x))
-          (mv nil nil))
-         ((mv warnings first)  (vl-genelement-enumname-declarations (car x)))
-         ((wmv warnings rest) (vl-genelementlist-enumname-declarations (cdr x))))
+          (mv nil nil (vl-datatype-map-fix seen)))
+         ((mv warnings first seen)  (vl-genelement-enumname-declarations (car x) seen))
+         ((wmv warnings rest seen) (vl-genelementlist-enumname-declarations (cdr x) seen)))
       (mv warnings
-          (append first rest))))
+          (append first rest)
+          seen)))
 
   (define vl-genblock-enumname-declarations ((x vl-genblock-p))
     :returns (mv (warnings  vl-warninglist-p)
                  (new-x     vl-genblock-p))
     :measure (two-nats-measure (vl-genblock-count x) 0)
     (b* (((vl-genblock x))
-         ((mv warnings new-elems) (vl-genelementlist-enumname-declarations x.elems))
+         ((mv warnings new-elems seen) (vl-genelementlist-enumname-declarations x.elems nil))
+         (- (fast-alist-free seen))
          (new-x (change-vl-genblock x :elems new-elems)))
       (mv warnings new-x)))
 
-  (define vl-genelement-enumname-declarations ((x vl-genelement-p))
+  (define vl-genelement-enumname-declarations ((x vl-genelement-p)
+                                               (seen vl-datatype-map-p))
     :returns (mv (warnings  vl-warninglist-p)
-                 (new-elems     vl-genelementlist-p))
+                 (new-elems     vl-genelementlist-p)
+                 (seen-out vl-datatype-map-p))
     :measure (two-nats-measure (vl-genelement-count x) 0)
     (vl-genelement-case x
       :vl-genbase
-      (b* (((mv new-item warnings decls)
+      (b* (((mv new-item warnings decls seen)
             (case (tag x.item)
-              (:vl-paramdecl (vl-paramdecl-enumname-declarations x.item))
-              (:vl-typedef   (vl-typedef-enumname-declarations x.item))
-              (:vl-vardecl   (vl-vardecl-enumname-declarations x.item))
-              (otherwise     (mv x.item nil nil))))
+              (:vl-paramdecl (vl-paramdecl-enumname-declarations x.item seen))
+              (:vl-typedef   (vl-typedef-enumname-declarations x.item seen))
+              (:vl-vardecl   (vl-vardecl-enumname-declarations x.item seen))
+              (otherwise     (mv x.item nil nil (vl-datatype-map-fix seen)))))
            ;; Remove duplicate decls because when we find something like:
           ;;    enum { FOO, BAR } a, b;
           ;; the declaration for "a" will try to introduce paramdecls for FOO
@@ -367,29 +401,29 @@ found in the datatype of a variable, parameter, or typedef declaration.</p>")
            (decls (remove-duplicates-equal (list-fix decls)))
            (new-elems (cons (change-vl-genbase x :item new-item)
                         (vl-modelementlist->genelements decls))))
-        (mv warnings new-elems))
+        (mv warnings new-elems seen))
       :vl-genbegin
       (b* (((mv warnings new-block) (vl-genblock-enumname-declarations x.block))
            (new-x                   (change-vl-genbegin x :block new-block)))
-        (mv warnings (list new-x)))
+        (mv warnings (list new-x) (vl-datatype-map-fix seen)))
       :vl-genif
       (b* (((mv warnings then)  (vl-genblock-enumname-declarations x.then))
            ((wmv warnings else) (vl-genblock-enumname-declarations x.else))
            (new-x               (change-vl-genif x :then then :else else)))
-        (mv warnings (list new-x)))
+        (mv warnings (list new-x) (vl-datatype-map-fix seen)))
       :vl-gencase
       (b* (((mv warnings cases)    (vl-gencaselist-enumname-declarations x.cases))
            ((wmv warnings default) (vl-genblock-enumname-declarations x.default))
            (new-x                  (change-vl-gencase x :cases cases :default default)))
-        (mv warnings (list new-x)))
+        (mv warnings (list new-x) (vl-datatype-map-fix seen)))
       :vl-genloop
       (b* (((mv warnings body) (vl-genblock-enumname-declarations x.body))
            (new-x              (change-vl-genloop x :body body)))
-        (mv warnings (list new-x)))
+        (mv warnings (list new-x) (vl-datatype-map-fix seen)))
       :vl-genarray
       (b* (((mv warnings blocks) (vl-genblocklist-enumname-declarations x.blocks))
            (new-x                (change-vl-genarray x :blocks blocks)))
-        (mv warnings (list new-x)))))
+        (mv warnings (list new-x) (vl-datatype-map-fix seen)))))
 
   (define vl-gencaselist-enumname-declarations ((x vl-gencaselist-p))
     :returns (mv (warnings vl-warninglist-p)
@@ -419,8 +453,9 @@ found in the datatype of a variable, parameter, or typedef declaration.</p>")
   :returns (new-x vl-module-p)
   (b* (((vl-module x) (vl-module-fix x))
        ((unless x.parse-temps) x)
-       ((mv warnings items)
-        (vl-genelementlist-enumname-declarations (vl-parse-temps->loaditems x.parse-temps))))
+       ((mv warnings items seen)
+        (vl-genelementlist-enumname-declarations (vl-parse-temps->loaditems x.parse-temps) nil))
+       (- (fast-alist-free seen)))
     ;; BOZO Enums first used in ports and paramports don't work yet
     (change-vl-module x
                       :parse-temps (change-vl-parse-temps x.parse-temps :loaditems items)
@@ -434,8 +469,9 @@ found in the datatype of a variable, parameter, or typedef declaration.</p>")
   :returns (new-x vl-interface-p)
   (b* (((vl-interface x) (vl-interface-fix x))
        ((unless x.parse-temps) x)
-       ((mv warnings items)
-        (vl-genelementlist-enumname-declarations (vl-parse-temps->loaditems x.parse-temps))))
+       ((mv warnings items seen)
+        (vl-genelementlist-enumname-declarations (vl-parse-temps->loaditems x.parse-temps) nil))
+       (- (fast-alist-free seen)))
     ;; BOZO Enums first used in ports and paramports don't work yet
     (change-vl-interface x
                          :parse-temps (change-vl-parse-temps x.parse-temps :loaditems items)
@@ -448,9 +484,11 @@ found in the datatype of a variable, parameter, or typedef declaration.</p>")
 (define vl-package-add-enumname-declarations ((x vl-package-p))
   :returns (new-x vl-package-p)
   (b* (((vl-package x) (vl-package-fix x))
-       ((mv typedefs warnings1 decls1) (vl-typedeflist-enumname-declarations x.typedefs))
-       ((mv vardecls warnings2 decls2) (vl-vardecllist-enumname-declarations x.vardecls))
-       ((mv paramdecls warnings3 decls3) (vl-paramdecllist-enumname-declarations x.paramdecls))
+       (seen nil)
+       ((mv typedefs warnings1 decls1 seen) (vl-typedeflist-enumname-declarations x.typedefs seen))
+       ((mv vardecls warnings2 decls2 seen) (vl-vardecllist-enumname-declarations x.vardecls seen))
+       ((mv paramdecls warnings3 decls3 seen) (vl-paramdecllist-enumname-declarations x.paramdecls seen))
+       (- (fast-alist-free seen))
        (decls (remove-duplicates-equal (append-without-guard decls1 decls2 (list-fix decls3)))))
     (change-vl-package
      x
@@ -467,9 +505,11 @@ found in the datatype of a variable, parameter, or typedef declaration.</p>")
 (define vl-design-add-enumname-declarations ((x vl-design-p))
   :returns (new-x vl-design-p)
   (b* (((vl-design x))
-       ((mv typedefs warnings1 decls1) (vl-typedeflist-enumname-declarations x.typedefs))
-       ((mv vardecls warnings2 decls2) (vl-vardecllist-enumname-declarations x.vardecls))
-       ((mv paramdecls warnings3 decls3) (vl-paramdecllist-enumname-declarations x.paramdecls))
+       (seen nil)
+       ((mv typedefs warnings1 decls1 seen)   (vl-typedeflist-enumname-declarations x.typedefs seen))
+       ((mv vardecls warnings2 decls2 seen)   (vl-vardecllist-enumname-declarations x.vardecls seen))
+       ((mv paramdecls warnings3 decls3 seen) (vl-paramdecllist-enumname-declarations x.paramdecls seen))
+       (- (fast-alist-free seen))
        (decls (remove-duplicates-equal (append-without-guard decls1 decls2 (list-fix decls3)))))
     (change-vl-design x
                       :paramdecls (append decls paramdecls)
