@@ -35,7 +35,7 @@
 (include-book "std/strings/url-encode" :dir :system)
 (include-book "std/strings/pretty" :dir :system)
 (local (include-book "arithmetic"))
-(local (include-book "std/testing/assert" :dir :system))
+(local (include-book "std/testing/assert-bang" :dir :system))
 (local (include-book "std/io/base" :dir :system))
 (local (include-book "std/basic/arith-equivs" :dir :system))
 (local (std::add-default-post-define-hook :fix))
@@ -2007,4 +2007,3 @@ it right away.</p>"
         (t
          (progn$ (raise "vl-basic-cw-obj is limited to 10 arguments.")
                  ps))))
-
