@@ -1571,11 +1571,9 @@ is a @(see vl-maybe-exprtype-p).</p>"
   ;;               (vl-maybe-expr-p (cdr a))
   ;;               (vl-atts-p x))))
 
-  (defthm alistp-when-vl-atts-p-rewrite
-    ;; This is potentially expensive, but without it we sometimes fail to
-    ;; relieve guards for things like assoc-equal into (vl-whatever->atts x).
-    (implies (vl-atts-p x)
-             (alistp x)))
+  ;; This is potentially expensive, but without it we sometimes fail to
+  ;; relieve guards for things like assoc-equal into (vl-whatever->atts x).
+  (in-theory (enable alistp-when-vl-atts-p-rewrite)) ; generated by defalist
 
   (defthm vl-expr-p-of-cdr-of-hons-assoc-equal-when-vl-atts-p
     (implies (vl-atts-p atts)
@@ -1812,12 +1810,7 @@ fairly easily solve the HIDEXPR problem.</p>"
     :hints(("Goal" :in-theory (enable acl2::take))))
 
   (defcong vl-exprlist-equiv vl-exprlist-equiv (list-fix x) 1)
-  (defcong vl-exprlist-equiv vl-exprlist-equiv (append x y) 1)
-  (defcong vl-exprlist-equiv vl-exprlist-equiv (append x y) 2)
-  (defcong vl-exprlist-equiv vl-exprlist-equiv (rev x) 1)
-  (defcong vl-exprlist-equiv vl-exprlist-equiv (take n x) 2
-    :hints(("Goal" :in-theory (enable acl2::take))))
-  (defcong vl-exprlist-equiv vl-exprlist-equiv (nthcdr n x) 2))
+  (defcong vl-exprlist-equiv vl-exprlist-equiv (rev x) 1))
 
 
 (local (defun make-cases (ops)
