@@ -1,6 +1,7 @@
 (in-package "ACL2")
 
-(include-book "xdoc/top" :dir :system)
+(include-book "instantiate")
+(include-book "autohide")
 
 (defxdoc clause-processor-tools
   :parents (clause-processor proof-automation)
@@ -24,12 +25,30 @@ your evaluator.</dd>
 <dt>clause-processors/unify-subst</dt>
 
 <dd>The @('def-unify') macro proves a unify/substitution algorithm correct for
-your evaluator</dd>
+your evaluator.</dd>
 
 
 <dt>clause-processors/meta-extract-user</dt>
 
 <dd>The @('def-meta-extract') macro sets up support for using the @(see
-meta-extract) stuff to assume facts from the world are correct</dd>
+meta-extract) stuff to assume facts from the world are correct.</dd>
 
-</dl>")
+
+<dt>misc/beta-reduce</dt>
+
+<dd>Implements a beta-reduction function, proves it preserves pseudo-termp, and
+provides a macro for proving correctness of beta-reduction for your
+evaluator.</dd>
+
+
+<dt>centaur/misc/beta-reduce-full</dt>
+
+<dd>Implements a recursive beta-reducer to expand away all lambdas, proves it
+preserves pseudo-termp, proves it is correct for an evaluator.</dd>
+
+</dl>
+
+<p>See also the clause-processors directory in general; many books are
+documented only in comments.</p>
+
+")
