@@ -26,8 +26,7 @@
     (local (defun p (x y) (declare (ignore x y)) t))
     (defthm p-axiom (p (g x) (g y))))
 
-; Define must-succeed and must-fail macros.
-   (local (include-book "std/testing/eval" :dir :system))
+   (local (include-book "std/testing/must-fail" :dir :system))
 
    (must-fail ; illustrates why we need a hint
     (defthm p-thm-fail
