@@ -32,7 +32,7 @@
 (in-package "ACL2")
 
 (include-book "extra-info")
-(include-book "std/testing/eval" :dir :system)
+(include-book "std/testing/must-fail" :dir :system)
 
 ; We have to disable tau for this whole file.  The reason is that the
 ; author is taking advantage of disable to make a defined function, e.g.,
@@ -132,4 +132,4 @@
  (defthmd test
    (zoo x)
    :otf-flg t
-   :hints (("Goal" :cases ((koo x))))))
\ No newline at end of file
+   :hints (("Goal" :cases ((koo x))))))
