@@ -43,11 +43,12 @@
 (set-enforce-redundancy t)
 
 (local (include-book "memory-impl"))
+
 (include-book "private")
 (include-book "misc/records" :dir :system)
 
 (defxdoc memory
-  :parents (acl2::data-structures)
+  :parents (acl2::data-structures acl2::misc/records)
   :short "Special records designed for array-like usage."
   :long "<p>Loading the library:</p>
 
@@ -165,8 +166,8 @@ substitution for memories depending upon your needs.</p>")
   (declare (xargs :guard (and (natp depth)
                               (_address-p addr depth)
                               (_memtree-p mtree depth)))
-           (type (signed-byte 30) depth)
-           (type (signed-byte 30) addr))
+           (type (signed-byte 29) depth)
+           (type (signed-byte 29) addr))
   (mbe :logic (_memtree-load addr mtree depth)
        :exec (if (= depth 0)
                  mtree
@@ -181,9 +182,9 @@ substitution for memories depending upon your needs.</p>")
   (declare (xargs :guard (and (natp depth)
                               (_address-p addr depth)
                               (_memtree-p mtree depth)))
-           (type (signed-byte 30) depth))
+           (type (signed-byte 29) depth))
   (mbe :logic (_memtree-load addr mtree depth)
-       :exec (if (<= depth 29)
+       :exec (if (<= depth 28)
                  (_fix-addr/depth-memtree-load addr mtree depth)
                (_fixnum-memtree-load (ash addr -1)
                                      (if (= (the-fixnum (logand addr 1)) 0)
@@ -214,8 +215,8 @@ substitution for memories depending upon your needs.</p>")
                               (not (null elem))
                               (_address-p addr depth)
                               (_memtree-p mtree depth)))
-           (type (signed-byte 30) depth)
-           (type (signed-byte 30) addr))
+           (type (signed-byte 29) depth)
+           (type (signed-byte 29) addr))
   (mbe :logic (_memtree-store addr elem mtree depth)
        :exec (if (= depth 0)
                  elem
@@ -234,9 +235,9 @@ substitution for memories depending upon your needs.</p>")
                               (not (null elem))
                               (_address-p addr depth)
                               (_memtree-p mtree depth)))
-           (type (signed-byte 30) depth))
+           (type (signed-byte 29) depth))
   (mbe :logic (_memtree-store addr elem mtree depth)
-       :exec (if (<= depth 29)
+       :exec (if (<= depth 28)
                  (_fix-addr/depth-memtree-store addr elem mtree depth)
                (let ((quotient (ash addr -1)))
                  (if (= (the-fixnum (logand addr 1)) 0)
@@ -277,8 +278,8 @@ substitution for memories depending upon your needs.</p>")
   (declare (xargs :guard (and (natp depth)
                               (_address-p addr depth)
                               (_memtree-p mtree depth)))
-           (type (signed-byte 30) depth)
-           (type (signed-byte 30) addr))
+           (type (signed-byte 29) depth)
+           (type (signed-byte 29) addr))
   (mbe :logic (_memtree-store-nil addr mtree depth)
        :exec (if (= depth 0)
                  nil
@@ -307,9 +308,9 @@ substitution for memories depending upon your needs.</p>")
   (declare (xargs :guard (and (natp depth)
                               (_address-p addr depth)
                               (_memtree-p mtree depth)))
-           (type (signed-byte 30) depth))
+           (type (signed-byte 29) depth))
   (mbe :logic (_memtree-store-nil addr mtree depth)
-       :exec (if (<= depth 29)
+       :exec (if (<= depth 28)
                  (_fix-addr/depth-memtree-store-nil addr mtree depth)
                (if (null mtree)
                    nil
@@ -349,7 +350,7 @@ substitution for memories depending upon your needs.</p>")
          (and (natp size)
               (natp depth)
               (booleanp fast)
-              (implies fast (signed-byte-p 30 depth))
+              (implies fast (signed-byte-p 29 depth))
               (<= size (expt 2 depth))
               (_memtree-p mtree depth)))))
 
@@ -471,7 +472,7 @@ of @('new') is @(see mem::private).</p>"
                (cons (cons nil t) (cons 1 (cons 1 nil)))
              (let ((depth (_log2 (1- size))))
                (cons
-                (cons nil (signed-byte-p 30 depth))
+                (cons nil (signed-byte-p 29 depth))
                 (cons size
                       (cons depth nil)))))))
 
@@ -602,7 +603,7 @@ mem::private).</p>"
 ; if the stated size is invalid), and that storing into a memory does not
 ; change its size.
 
-(defthm size-is-positive
+(defthm size-positive
   (and (integerp (size m))
        (< 0 (size m)))
   :rule-classes :type-prescription)
