@@ -100,7 +100,7 @@
   (or (getpropc name 'theorem nil wrld)
       (mv-let (flg prop)
         (constraint-info name wrld)
-        (cond ((eq prop *unknown-constraints*)
+        (cond ((unknown-constraints-p prop)
                *t*)
               (flg (ec-call (conjoin prop)))
               (t prop)))))
