@@ -10,9 +10,8 @@
 
 (in-package "APT")
 
-(include-book "kestrel/apt/utilities/xdoc-constructors" :dir :system)
 (include-book "kestrel/event-macros/xdoc-constructors" :dir :system)
-(include-book "isodata")
+(include-book "utilities/xdoc-constructors")
 
 ; (depends-on "design-notes/isodata.pdf")
 ; (depends-on "kestrel/design-notes/notation.pdf" :dir :system)
@@ -20,7 +19,9 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (defconst *isodata-design-notes*
-  (xdoc::ahref "res/kestrel-apt-design-notes/isodata.pdf" "design notes"))
+  (xdoc::&& "@('isodata') "
+            (xdoc::ahref "res/kestrel-apt-design-notes/isodata.pdf"
+                         "design notes")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -56,7 +57,8 @@
       as being changed via an indentity isomorphism,
       we can say that this transformation changes the representation of
       (the tuple of) all the function's arguments
-      into a new representation that is element-wise isomorphic.
+      into a new representation that consists of
+      (the tuple of) all the new function's arguments.
       In this case, there are two variants of this transformation:")
     (xdoc::ul
      (xdoc::li
@@ -109,7 +111,26 @@
 
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-   (xdoc::evmac-section-form-auto isodata)
+   (xdoc::evmac-section-form
+    (xdoc::codeblock
+     "(isodata old"
+     "         isomaps"
+     "         :predicate           ; default nil"
+     "         :new-name            ; default :auto"
+     "         :new-enable          ; default :auto"
+     "         :old-to-new-name     ; default from table"
+     "         :old-to-new-enable   ; default from table"
+     "         :new-to-old-name     ; default from table"
+     "         :new-to-old-enable   ; default from table"
+     "         :newp-of-new-name    ; default :auto"
+     "         :newp-of-new-enable  ; default t"
+     "         :verify-guards       ; default :auto"
+     "         :untranslate         ; default :nice"
+     "         :hints               ; default nil"
+     "         :print               ; default :result"
+     "         :show-only           ; default nil"
+     "         :compatibility       ; default nil"
+     "  )"))
 
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -119,7 +140,7 @@
      (xdoc::p
       "@('old') must
        be in logic mode,
-       be defined, and
+       be " (xdoc::seetopic "acl2::function-definedness" "defined") ", and
        have no input or output "
       (xdoc::seetopic "acl2::stobj" "stobjs")
       ". If the @(':predicate') input (see below) is @('t'),
@@ -211,8 +232,8 @@
         "Each @('isok') denotes the isomorphic mapping
          to apply to the arguments and results in @('arg/res-listk').
          Each @('isok') specifies
-         old and new isomorphic representations
-         and the isomorphisms between them.")
+         old and new representations
+         and the conversions between them.")
        (xdoc::p
         "It must be one of the following:")
        (xdoc::ul
@@ -220,7 +241,7 @@
          "A symbol that references
           a previous successful call of @(tsee defiso),
           i.e. the symbol must be the @('name') input of that call.
-          The domains and isomorphisms recorded under that name specify:
+          The domains and conversions recorded under that name specify:
           the recognizer of the old representation (@('doma')),
           which we call @('oldp') here;
           the recognizer of the new representation (@('domb')),
@@ -332,11 +353,11 @@
        the resulting isomorphic mapping over all function arguments
        is denoted as consisting of
        the domains @($A$) and @($A'$) and
-       the isomorphisms @($\\alpha$) and @($\\alpha'$),
+       the conversions @($\\alpha$) and @($\\alpha'$),
        and the resulting isomorphic mapping over all function results
        is denoted as consisting of
        the domains @($B$) and @($B'$) and
-       the isomorphisms @($\\beta$) and @($\\beta'$)."))
+       the conversions @($\\beta$) and @($\\beta'$)."))
 
     (xdoc::desc
      "@(':predicate') &mdash; default @('nil')"
@@ -365,32 +386,52 @@
 
     (xdoc::desc-apt-input-new-enable)
 
-    (xdoc::desc-apt-input-thm-name :never)
+    (xdoc::desc-apt-input-old-to-new-name)
 
-    (xdoc::desc-apt-input-thm-enable :never)
+    (xdoc::desc-apt-input-old-to-new-enable)
+
+    (xdoc::desc-apt-input-new-to-old-name)
+
+    (xdoc::desc-apt-input-new-to-old-enable)
 
     (xdoc::desc
-     "@(':new-to-old') &mdash; default @('nil')"
+     "@(':newp-of-new-name') &mdash; default @(':auto')"
      (xdoc::p
-      "Determines the name of the theorem that
-       relates the new function to the old function:")
+      "Determines the name of the theorem asserting that @('new') maps
+       arguments in the new representation
+       to results in the new representation.")
+     (xdoc::p
+      "It must be one of the following:")
      (xdoc::ul
       (xdoc::li
-       "@('nil'), to use the value from the APT defaults table,
-        which is set via @(tsee set-default-input-new-to-old).")
-      (xdoc::li
-       "A keyword, to use as separator between
-        the names of @('new') and @('old').
-        A keyword @(':kwd') specifies the theorem name @('newkwdold'),
+       "@(':auto'), to use the concatenation of
+        the name of @('new') followed by @('-new-representation'),
         in the same package as @('new').")
       (xdoc::li
-       "A non-@('nil'), non-keyword symbol,
-        to use as the name of the theorem."))
+       "Any other symbol, to use as the name of the theorem."))
+     (xdoc::p
+      "This input may be present only if
+       @('isomaps') includes some @(':resultj').")
      (xdoc::p
       "In the rest of this documentation page,
-       let @('new-to-old') be the name of this theorem."))
+       let @('newp-of-new') be the name of this theorem."))
 
-    (xdoc::desc-apt-input-verify-guards :never)
+    (xdoc::desc
+     "@(':newp-of-new-enable') &mdash; default @('t')"
+     (xdoc::p
+      "Determines whether @('newp-of-new') is enabled.")
+     (xdoc::p
+      "It must be one of the following:")
+     (xdoc::ul
+      (xdoc::li
+       "@('t'), to enable the theorem.")
+      (xdoc::li
+       "@('nil'), to disable the theorem."))
+     (xdoc::p
+      "This input may be present only if
+       @('isomaps') includes some @(':resultj')."))
+
+    (xdoc::desc-apt-input-verify-guards :plural-functions nil)
 
     (xdoc::desc-apt-input-untranslate)
 
@@ -779,9 +820,7 @@
       "In the " *isodata-design-notes* ",
        @('new-to-old') is denoted by
        @($f'f$) when @(':predicate') is @('nil'),
-       and @($'pp$) when @(':predicate') is @('t').")
-     (xdoc::p
-      "This theorem is disabled by default."))
+       and @($'pp$) when @(':predicate') is @('t')."))
 
     (xdoc::desc
      "@('old-to-new')"
@@ -837,10 +876,41 @@
        @($ff'$) when @(':predicate') is @('nil'),
        and @($pp'$) when @(':predicate') is @('t')."))
 
+    (xdoc::desc
+     "@('newp-of-new')"
+     (xdoc::p
+      "Theorem asserting that @('new') maps
+       arguments in the new representation
+       to results in the new representation:")
+     (xdoc::codeblock
+      ";; when m = 1:"
+      "(defthm newp-of-new"
+      "  (implies (and (newp1 x1)"
+      "                ..."
+      "                (newpn xn))"
+      "           (newp_r1 (new x1 ... xn))))"
+      ""
+      ";; when m > 1:"
+      "(defthm newp-of-new"
+      "  (implies (and (newp1 x1)"
+      "                ..."
+      "                (newpn xn))"
+      "           (mv-let (y1 ... ym)"
+      "             (new x1 ... xn)"
+      "             (and (newp_r1 y1)"
+      "                  ..."
+      "                  (newp_rm ym)))))")
+     (xdoc::p
+      "In the " *isodata-design-notes* ",
+       @('newp-of-new') is denoted by @($f'A'B'$).")
+     (xdoc::p
+      "This is generated if and only if
+       @('isomaps') includes some @(':resultj')."))
+
     (xdoc::p
      "A theory invariant is also generated to prevent
-       both @('new-to-old') and @('old-to-new')
-       from being enabled at the same time."))
+      both @('new-to-old') and @('old-to-new')
+      from being enabled at the same time."))
 
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
