@@ -1,7 +1,7 @@
 ; A lightweight book about the built-in function floor.
 ;
 ; Copyright (C) 2008-2011 Eric Smith and Stanford University
-; Copyright (C) 2013-2019 Kestrel Institute
+; Copyright (C) 2013-2020 Kestrel Institute
 ;
 ; License: A 3-clause BSD license. See the file books/3BSD-mod.txt.
 ;
@@ -19,7 +19,20 @@
 (local (include-book "divides"))
 (local (include-book "times-and-divides"))
 (local (include-book "nonnegative-integer-quotient"))
+(local (include-book "integerp"))
+(local (include-book "expt"))
 (local (include-book "../../meta/meta-plus-lessp"))
+(local (include-book "kestrel/utilities/equal-of-booleans" :dir :system))
+
+;move
+(defthm <-of-numerator-and-denominator-same
+  (implies (rationalp x)
+           (equal (< (numerator x) (denominator x))
+                  (if (<= x 0)
+                      t
+                    (< x 1))))
+  :hints (("Goal" :use (:instance rational-implies2)
+           :in-theory (disable rational-implies2))))
 
 (in-theory (disable floor))
 
@@ -72,6 +85,24 @@
                             (i (- (numerator (* i1 (/ j)))))
                             (j (denominator (* i1 (/ j)))))))))
 
+(defthm floor-weak-monotone-linear-1
+  (implies (and (<= free i)
+                (<= 0 j)
+                (rationalp free)
+                (rationalp i)
+                (rationalp j))
+           (<= (floor free j) (floor i j)))
+  :rule-classes ((:linear :trigger-terms ((floor i j)))))
+
+(defthm floor-weak-monotone-linear=-2
+  (implies (and (<= i free)
+                (<= 0 j)
+                (rationalp free)
+                (rationalp i)
+                (rationalp j))
+           (<= (floor i j) (floor free j)))
+  :rule-classes ((:linear :trigger-terms ((floor i j)))))
+
 (defthmd floor-when-multiple
   (implies (integerp (* i (/ j)))
            (equal (floor i j)
@@ -108,7 +139,15 @@
                 (rationalp j)
                 (not (equal 0 j)))
            (< (+ -1 (/ i j)) (floor i j)))
-  :hints (("Goal" :in-theory (enable floor))))
+  :hints (("Goal" :in-theory (e/d (floor) (<-OF-*-OF-/-ARG1)))))
+
+(defthm my-floor-lower-bound-linear
+  (implies (and (rationalp i)
+                (rationalp j)
+                (not (equal 0 j)))
+           (< (+ -1 (/ i j)) (floor i j)))
+  :rule-classes ((:linear :trigger-terms ((floor i j))))
+  :hints (("Goal" :by my-floor-lower-bound)))
 
 ;; In this version, we have multiplied through by j.
 (defthm my-floor-lower-bound-alt
@@ -138,7 +177,7 @@
                 (rationalp j))
            ;; the phrasing of the * term matches our normal form
            (<= (floor i j) (* i (/ j))))
-  :hints (("Goal" :in-theory (enable floor))))
+  :hints (("Goal" :in-theory (e/d (floor) (<-of-*-of-/-arg1)))))
 
 (defthm floor-upper-bound-linear
   (implies (and (rationalp i)
@@ -146,14 +185,15 @@
            ;; the phrasing of the * term matches our normal form
            (<= (floor i j) (* i (/ j))))
   :rule-classes ((:linear :trigger-terms ((floor i j))))
-  :hints (("Goal" :in-theory (enable floor))))
+  :hints (("Goal" :by my-floor-upper-bound)))
 
-(defthm floor-upper-bound-strong-linear
-  (implies (and (not (integerp (* (/ j) i)))
-                (rationalp i)
-                (rationalp j))
-           (< (floor i j) (* (/ j) i)))
-  :rule-classes ((:linear :backchain-limit-lst (0 nil nil))))
+(defthm *-of-floor-upper-bound-linear
+  (implies (and (rationalp i)
+                (rationalp j)
+                (< 0 j))
+           (<= (* j (floor i j)) i))
+  :rule-classes (:linear)
+  :hints (("Goal" :in-theory (enable floor))))
 
 ;; In this version, we have multiplied through by j.
 (defthmd my-floor-upper-bound-alt
@@ -161,7 +201,6 @@
                 (rationalp j)
                 (< 0 j))
            (<= (* j (floor i j)) i))
-  :rule-classes (:rewrite (:linear :trigger-terms ((* j (floor i j)))))
   :hints (("Goal" :use ((:instance my-floor-upper-bound)
                         (:instance <-of-*-and-*-cancel
                                    (x2 (floor i j))
@@ -170,6 +209,20 @@
            :in-theory (disable my-floor-upper-bound
                                <-of-*-and-*-cancel))))
 
+(defthm floor-upper-bound-strict
+  (implies (and (not (integerp (/ i j)))
+                (rationalp i)
+                (rationalp j))
+           (< (floor i j) (/ i j))))
+
+(defthm floor-upper-bound-strong-linear-cheap
+  (implies (and (not (integerp (* (/ j) i)))
+                (rationalp i)
+                (rationalp j))
+           (< (floor i j) (* (/ j) i)))
+  :rule-classes ((:linear :backchain-limit-lst (0 nil nil)))
+  :hints (("Goal" :in-theory (disable <-OF-*-OF-/-ARG2))))
+
 ;; generalizing this is hard since even if j is not rational, the quotient may be.
 (defthm floor-when-not-rationalp-arg1
   (implies (and (not (rationalp i))
@@ -305,6 +358,21 @@
                        (<= 0 i))))
   :hints (("Goal" :in-theory (enable floor))))
 
+;drop the non-gen one?
+(defthm equal-of-0-and-floor-gen
+  (implies (and (rationalp i)
+                (rationalp j))
+           (equal (equal 0 (floor i j))
+                  (if (< 0 j)
+                      (and (< i j)
+                           (<= 0 i))
+                    (if (equal 0 j)
+                        t
+                      ;; (< j 0):
+                      (and (< j i)
+                           (<= i 0))))))
+  :hints (("Goal" :in-theory (enable floor))))
+
 (defthm floor-of-1-arg1
   (implies (natp j) ;allow non nats somehow? ;allow negatives?
            (equal (floor 1 j)
@@ -318,10 +386,13 @@
 ;proved by ihs/quotient-remainder-lemmas
 (defthm floor-of-floor
   (implies (and (rationalp i)
-                (posp j1)
-                (posp j2))
+                (natp j1)
+                (natp j2))
            (equal (floor (floor i j1) j2)
-                  (floor i (* j1 j2)))))
+                  (floor i (* j1 j2))))
+  :hints (("Goal" :cases ((and (equal j1 0) (equal j2 0))
+                          (and (not (equal j1 0)) (equal j2 0))
+                          (and (not (equal j1 0)) (not (equal j2 0)))))))
 
 (local (include-book "arithmetic/inequalities" :dir :system)) ;for <-*-/-LEFT
 
@@ -382,8 +453,9 @@
 ;floor-minus should be split into two lemmas
 
 ;FIXME all-vars-negated returns true for a constant (even 0).  quotep help fixes it for this rule
-;todo: i thnk i've seen this loop
-(defthm floor-minus-eric-better
+;todo: i think i've seen this loop
+;ifix this - it fired on 1 <-- old comment?
+(defthmd floor-minus-eric-better
   (implies (and (syntaxp (and (all-vars-negated i)
                               (not (quotep i))))
                 (rationalp i)
@@ -456,7 +528,7 @@
                            (i2 (+ y n))
                            (j y)))))
 
-(local (include-book "ihs/quotient-remainder-lemmas" :dir :system)) ;why? for FLOOR-BOUNDED-BY-/? and to prove FLOOR-PEEL-OFF-CONSTANT
+;(local (include-book "ihs/quotient-remainder-lemmas" :dir :system)) ;why? for FLOOR-BOUNDED-BY-/? and to prove FLOOR-PEEL-OFF-CONSTANT
 
 (defthm floor-peel-off-constant
   (implies (and (syntaxp (quotep k))
@@ -493,6 +565,7 @@
   (implies (integerp x)
            (<= (floor x 2) (/ x 2))))
 
+;drop?
 (defthm floor-bound-better-linear
   (implies (integerp x)
            (<= (floor x 2) (/ x 2)))
@@ -503,12 +576,6 @@
          (floor a 2))
   :hints (("Goal" :in-theory (enable floor))))
 
-(defthm floor-upper-bound-strict
-  (implies (and (not (integerp (/ i j)))
-                (rationalp i)
-                (rationalp j))
-           (< (floor i j) (/ i j))))
-
 ;gen!
 (defthm equal-of-floor-same
   (implies (and (integerp j) ;(rationalp j);
@@ -542,14 +609,17 @@
   (implies (and (< i j)
                 (>= i 0)
                 (> j 0)
-                (force (rationalp i))
                 (force (rationalp j)))
            (equal (floor i j)
-                  0)))
+                  0))
+  :hints (("Goal" :cases ((rationalp i)))))
 
+;todo: compare to *-of-floor-upper-bound-linear
 (defthm floor-upper-bound-alt-linear
-  (implies (and (natp i)
-                (natp j))
+  (implies (and (<= 0 i)
+                (rationalp i)
+                (<= 0 j)
+                (rationalp j))
            (<= (* j (floor i j)) i))
   :rule-classes ((:linear))
   :hints (("Goal" :in-theory (enable mod))))
@@ -675,14 +745,17 @@
            :in-theory (disable <-*-left-cancel))))
 
 (defthmd floor-bound-lemma-2
-  (implies (and (< (floor x j) (+ 1 k))
-                (rationalp x)
+  (implies (and (< (floor i j) (+ 1 k))
+                (rationalp i)
                 (rationalp j)
                 (< 0 j)
-                (integerp k))
-           (< x (+ j (* j k))))
-  :hints (("Goal" :use ((:instance my-floor-lower-bound-alt (i x))
-                        (:instance <-*-left-cancel (x k) (y (floor x j)) (z j)))
+                (integerp k) ; gen to (rationalp k)?
+                )
+           ;; says that k+1 is greater than the quotient
+           (< i (+ j (* j k))))
+  :hints (("Goal" :cases ((integerp k))
+           :use ((:instance my-floor-lower-bound-alt (i i))
+                        (:instance <-*-left-cancel (x k) (y (floor i j)) (z j)))
            :do-not '(generalize eliminate-destructors)
            :in-theory (disable <-*-left-cancel
                                *-preserves->=-for-nonnegatives
@@ -695,9 +768,12 @@
                 (integerp k) ;gen?
                 )
            (equal (< k (floor i j))
-                  (<= (* j (+ 1 k)) i)))
+                  (if (integerp k)
+                      (<= (* j (+ 1 k)) i) ; the quotient is at least k+1
+                    (<= (+ 1 (floor k 1)) (/ i j)))
+                    ))
   :hints (("Goal":use ((:instance floor-bound-lemma-1 (x i))
-                       (:instance floor-bound-lemma-2 (x i))))))
+                       (:instance floor-bound-lemma-2)))))
 
 ;ffixme other way?
 ;kill the version for 4
@@ -719,54 +795,85 @@
                   (<= y x))))
 
 (defthmd bound-from-floor-bound
-  (implies (and (<= m (floor x n))
-                (< 0 n)
-                (rationalp x)
-                (rationalp m)
-                (rationalp n))
-           (<= (* m n) x))
+  (implies (and (<= k (floor i j))
+                (< 0 j)
+                (rationalp i)
+                (rationalp k)
+                (rationalp j))
+           (<= (* k j) i))
   :hints (("Goal" :in-theory (disable my-floor-upper-bound
                                       <-*-left-cancel)
-           :use ((:instance <-*-left-cancel (x (floor x n)) (y  m) (z n))
-                 (:instance my-floor-upper-bound (i x) (j n))))))
+           :use ((:instance <-*-left-cancel (x (floor i j)) (y m) (z j))
+                 (:instance my-floor-upper-bound)))))
 
 (defthmd bound-from-floor-bound-back
-  (implies (and (<= (* m n) x)
-                (< 0 n)
-                (rationalp x)
-                (integerp m)
-                (integerp n))
-           (<= m (floor x n)))
+  (implies (and (<= (* k j) i)
+                (< 0 j)
+                (rationalp i)
+                (integerp k) ;gen?
+                (integerp j))
+           (<= k (floor i j)))
   :hints  (("Goal" :in-theory (disable ;floor-bound-lemma2
                                        my-floor-lower-bound-alt
                                        <-*-left-cancel)
             :use (;(:instance <-*-left-cancel (x  (floor x n)) (y  m) (z n))
 ;                  (:instance floor-lower-bound (y n))
-                  (:instance floor-weak-monotone (j n) (i1 (* m n)) (i2 x))
+                  (:instance floor-weak-monotone (i1 (* k j)) (i2 i))
                   ))))
 
+;do we want this?
+;; (defthmd bound-from-floor-bound-back-gen
+;;   (implies (and (<= (* (floor k 1) j) i)
+;;                 (< 0 j)
+;;                 (rationalp i)
+;;                 (rationalp k) ; not this
+;;                 (integerp j))
+;;            (<= (floor k 1) (floor i j)))
+;;   :hints (("Goal" :use (:instance bound-from-floor-bound-back
+;;                                   (k (floor k 1)))
+;;            :in-theory (disable bound-from-floor-bound-back))))
+
 (defthmd <-of-floor-arg1
-  (implies (and (< 0 n)
-                (rationalp x)
-                (integerp m) ;gen?
-                (integerp n))
-           (equal (< (floor x n) m)
-                  (< x (* m n))))
-  :hints (("Goal" :use ((:instance bound-from-floor-bound)
+  (implies (and (< 0 j)
+                (rationalp i)
+                (integerp k) ;gen?
+                (integerp j))
+           (equal (< (floor i j) k)
+                  (< i (* k j))))
+  :hints (("Goal" :use (bound-from-floor-bound
                         bound-from-floor-bound-back))))
 
+(defthmd <-of-floor-arg1-gen
+  (implies (and (< 0 j)
+                (rationalp i)
+                (rationalp k)
+                (integerp j) ;gen?
+                )
+           (equal (< (floor i j) k)
+                  (if (integerp k)
+                      ;; check whether k is bigger than the quotient:
+                      (< i (* k j))
+                    ;; round k up to an integer, and check whether that is
+                    ;; bigger than the quotient:
+                    (< (/ i j) (+ 1 (floor k 1))))))
+  :hints (("Goal" :use ((:instance <-of-floor-arg1)
+                        (:instance <-of-floor-arg1 (k (floor k 1))))
+           :cases ((equal (floor k 1) (floor i j))))))
+
 (defthmd <-of-floor-and-constant
-  (implies (and (syntaxp (and (quotep m)
-                              (quotep n)))
-                (< 0 n)
-                (rationalp x)
-                (integerp m)
-                (integerp n))
-           (equal (< (floor x n) m)
-                  (< x (* m n))))
+  (implies (and (syntaxp (and (quotep k)
+                              (quotep j)))
+                (< 0 j)
+                (rationalp i)
+                (integerp k)
+                (integerp j))
+           (equal (< (floor i j) k)
+                  (< i (* k j))))
   :hints (("Goal" :use (:instance <-of-floor-arg1)
            :in-theory (disable <-of-floor-arg1))))
 
+
+
 ;disable?
 (defthm <-of-times-of-floor-and-same
   (implies (and (rationalp i)
@@ -845,6 +952,16 @@
   :hints (("Goal" :in-theory (enable floor)
            :cases ((rationalp j)))))
 
+(defthm floor-type-when-nonpositive-and-nonnegative
+  (implies (and (<= i 0)
+                (<= 0 j)
+                (or (rationalp i)
+                    (rationalp j)))
+           (<= (floor i j) 0))
+  :rule-classes (:type-prescription)
+  :hints (("Goal" :in-theory (enable floor)
+           :cases ((rationalp j)))))
+
 ;almost subsumed by <-of-floor-and-0
 (defthm <-of-floor-and-0-2
   (implies (and (<= 0 i)
@@ -876,28 +993,52 @@
                                                 <-*-/-left
                                                 <-y-*-y-x
                                                 <-*-/-right
-                                                floor-bound-hack-eric)))))
+                                                floor-bound-hack-eric
+                                                <-OF-*-OF-/-ARG1
+                                                <-OF-*-OF-/-ARG2
+                                                <-OF-*-SAME-ARG2)))))
 
 (defthm floor-bound-arg1-linear
-  (implies (and (rationalp i)
-                (<= 0 i)
-                (integerp j)
-                (<= 0 j))
+  (implies (and (<= 0 i)
+                ;; The only bad values of j are in in interval (0,1).
+                (or (integerp j)
+                    (<= 1 j)
+                    (<= j 0))
+                (rationalp i))
            (<= (floor i j) i))
-  :rule-classes ((:linear :trigger-terms ((floor i j)))))
+  :rule-classes ((:linear :trigger-terms ((floor i j))))
+  :hints (("Goal" :cases ((< j 0))))
+  )
 
-(defthm equal-of-floor-and-i
-  (implies (and (natp i)
-                (< 1 j)
-                (integerp j))
-           (equal (equal (floor i j) i)
-                  (equal i 0)))
-  :hints (("Goal"
-           :use (:instance floor-upper-bound-strict)
-           :in-theory (disable floor-upper-bound-strict
-                               <-of-times-of-floor-and-same
-                               floor-mod-elim))))
+;todo
+;; (defthm floor-bound-arg1-linear-negative
+;;   (implies (and (rationalp i)
+;;                 (<= i 0) ; this case
+;;                 ;; The only bad values of j are in in interval (0,1).
+;;                 (or (integerp j)
+;;                     (<= 1 j)
+;;                     (<= j 0)))
+;;            (<= i (floor i j)))
+;;   :rule-classes ((:linear :trigger-terms ((floor i j))))
+;;   :hints (("Goal" :cases ((< j 0))
+;;            :use (:instance my-floor-lower-bound)
+;;            :in-theory (disable my-floor-lower-bound
+;;                                my-floor-lower-bound-alt))))
 
+;todo
+;; (defthm equal-of-floor-and-i
+;;   (implies (and (integerp i)
+;;                 (< 1 j)
+;;                 (integerp j))
+;;            (equal (equal (floor i j) i)
+;;                   (if (<= 0 i)
+;;                       (equal i 0)
+;;                     (equal i -1))))
+;;   :hints (("Goal"
+;;            :use (:instance floor-upper-bound-strict)
+;;            :in-theory (disable floor-upper-bound-strict
+;;                                <-of-times-of-floor-and-same
+;;                                floor-mod-elim))))
 
 ;; ;we now have a more general version
 ;; ;gen!
@@ -917,14 +1058,14 @@
 (defthm <-of-floor-of-constant-and-constant-gen
   (implies (and (syntaxp (and (quotep k1)
                               (quotep k)))
-                (rationalp x)
+                (rationalp i)
                 (integerp k)
                 (posp k1))
-           (equal (< (floor x k1) k)
-                  (< x (* k k1))))
+           (equal (< (floor i k1) k)
+                  (< i (* k k1))))
   :hints (("Goal"
-           :use ((:instance bound-from-floor-bound (n k1) (m k))
-                 (:instance bound-from-floor-bound-back (n k1) (m k))))))
+           :use ((:instance bound-from-floor-bound (j k1) (k k))
+                 (:instance bound-from-floor-bound-back (j k1) (k k))))))
 
 ;slow?
 (defthm *-of-floor-of-same-when-multiple
@@ -1055,3 +1196,153 @@
            :in-theory (e/d (floor)
                            (<=-of-denominator-of-*-of-/
                             <-of-+-arg2-when-negative-constant)))))
+
+;; Disabled since it turns floor into division
+(defthmd floor-when-integerp-of-quotient
+  (implies (integerp (* x (/ y)))
+           (equal (floor x y)
+                  (* x (/ y))))
+  :hints (("Goal" :in-theory (enable floor))))
+
+(defthm floor-when-not-rationalp-of-quotient
+  (implies (not (rationalp (* x (/ y))))
+           (equal (floor x y)
+                  0))
+  :hints (("Goal" :in-theory (enable floor))))
+
+(defthm split-low-bit
+  (implies (rationalp i)
+           (equal i (+ (* 2 (floor i 2)) (mod i 2))))
+  :rule-classes nil
+  :hints (("Goal" :in-theory (enable mod))))
+
+(defthmd floor-of-2-cases
+  (implies (integerp i)
+           (equal (floor i 2)
+                  (if (equal 0 (mod i 2))
+                      (/ i 2)
+                    (+ -1/2 (/ i 2)))))
+  :hints (("Goal" :use ((:instance floor-unique
+                                   (j 2)
+                                   (n (if (equal 0 (mod i 2))
+                                          (/ i 2)
+                                        (+ 1/2 (/ i 2)))))
+                        (:instance split-low-bit)))))
+
+(defthmd floor-when-evenp
+  (implies (and (evenp x)
+                (integerp x))
+           (equal (floor x 2)
+                  (/ x 2)))
+  :hints (("Goal" :in-theory (enable floor evenp))))
+
+;; this one uses evenp
+(defthmd floor-of-2-cases-2
+   (implies (integerp i)
+            (equal (floor i 2)
+                   (if (evenp i)
+                       (/ i 2)
+                     (+ -1/2 (/ i 2)))))
+   :hints (("Goal" :in-theory (enable floor-of-2-cases evenp))))
+
+(defthm unsigned-byte-p-of-floor-by-2-strong
+  (implies (integerp x)
+           (equal (unsigned-byte-p n (floor x 2))
+                  (and (natp n)
+                       (unsigned-byte-p (+ 1 n) x))))
+  :hints (("Goal" :in-theory (enable unsigned-byte-p
+                                     expt-of-+
+                                     <-of-floor-arg1
+                                     ))))
+
+(defthm equal-of-floor-and-*-of-/
+  (equal (equal (floor i j) (* i (/ j)))
+         (integerp (* i (/ j)))))
+
+;; quite strong!
+;rephrase the rhs?
+;; If something is equal to the floor, it must be an integer, be no bigger than
+;; the quotient, and be strictly bigger than the quotient minus 1.
+(defthmd equal-of-floor
+  (implies (and (rationalp i)
+                (rationalp j))
+           (equal (equal val (floor i j))
+                  (and (integerp val)
+                       (<= val (/ i j))
+                       (< (/ i j) (+ 1 val)))))
+  :hints (("Goal" :in-theory (disable <-OF-*-OF-/-ARG1
+                                      <-OF-*-OF-/-ARG1-alt))))
+
+;disable?
+(defthm equal-x-times-2-floor-x
+  (implies (rationalp x)
+           (equal (equal x (* 2 (floor x 2)))
+                  (evenp x)))
+  :hints (("Goal" :in-theory (enable evenp floor))))
+
+;; Not sure exactly where this should go
+(defthm unsigned-byte-p-of-floor
+  (implies (and (unsigned-byte-p size x)
+                (natp y))
+           (unsigned-byte-p size (floor x y)))
+  :hints (("Goal"
+           :cases ((equal 0 y))
+           :in-theory (enable unsigned-byte-p))))
+
+(defthm floor-of-*-and-*-cancel-arg2-arg2
+  (equal (floor (* x z) (* y z))
+         (if (equal (fix z) 0)
+             0
+           (floor x y)))
+  :hints (("Goal" :in-theory (enable floor))))
+
+(local
+ (defthm helper1
+   (IMPLIES (AND (RATIONALP I)
+                 (RATIONALP J)
+                 (< 0 J)
+                 (RATIONALP K)
+                 (< k (floor i j)))
+            (<= (+ 1 (FLOOR K 1)) (FLOOR I J)))))
+
+(local
+ (defthm helper2
+   (IMPLIES (AND (RATIONALP I)
+                 (RATIONALP J)
+                 (< 0 J)
+                 (integerp Kup)
+                 (< kup (/ i j)))
+            (<= kup (FLOOR I J)))))
+
+(defthmd <-of-floor-arg2-gen
+  (implies (and (rationalp i)
+                (rationalp j)
+                (< 0 j)
+                (rationalp k))
+           (equal (< k (floor i j))
+                  ;; the quotient is at least k-rounded-up:
+                  (<= (+ 1 (floor k 1)) (/ i j))))
+  :hints (("Goal" :do-not '(generalize eliminate-destructors)
+           :use (helper1
+                 (:instance helper2 (kup (+ 1 (floor k 1)))))
+           :in-theory (disable <-*-/-LEFT
+                               <-OF-*-OF-/-ARG1-ARG2
+                               <-OF-*-OF-/-ARG1
+                               helper1))))
+
+(defthm floor-bound-strict-linear
+  (implies (and (< 1 j)
+                (< 0 i) ;i can't be 0
+                (rationalp i)
+                (rationalp j))
+           (< (floor i j) i))
+  :rule-classes (:linear))
+
+;strengthen?
+(defthm floor-bound-strict
+  (implies (and (< 1 j)
+                (<= 0 i)
+                (rationalp i)
+                (rationalp j))
+           (equal (< (floor i j) i)
+                  (not (equal i 0)))))
