@@ -263,10 +263,10 @@
    @(def symbol-package-name-of-symbol-is-not-empty-string)
    @(def symbol-equality)
    @(def default-pkg-imports)
-   @(def symbol-<-asymmetric)
-   @(def symbol-<-transitive)
-   @(def symbol-<-trichotomy)
-   @(def symbol-<-irreflexive)
+   @(def symbol<-asymmetric)
+   @(def symbol<-transitive)
+   @(def symbol<-trichotomy)
+   @(def symbol<-irreflexive)
    @(def default-intern-in-package-of-symbol)
    @(def default-symbol-name)
    @(def default-symbol-package-name)
