@@ -1,7 +1,7 @@
 ; BV Library: Theorems about bvchop.
 ;
 ; Copyright (C) 2008-2011 Eric Smith and Stanford University
-; Copyright (C) 2013-2019 Kestrel Institute
+; Copyright (C) 2013-2021 Kestrel Institute
 ;
 ; License: A 3-clause BSD license. See the file books/3BSD-mod.txt.
 ;
@@ -12,6 +12,8 @@
 (in-package "ACL2")
 
 (include-book "bvchop-def")
+(include-book "../arithmetic-light/power-of-2p")
+(local (include-book "unsigned-byte-p"))
 (local (include-book "../arithmetic-light/expt2"))
 (local (include-book "../arithmetic-light/times-and-divides"))
 (local (include-book "../arithmetic-light/divides"))
@@ -47,6 +49,13 @@
            (equal (bvchop n x) 0))
   :hints (("Goal" :in-theory (enable bvchop))))
 
+(defthm bvchop-when-not-natp-arg1-cheap
+  (implies (not (natp n))
+           (equal (bvchop n x)
+                  0))
+  :rule-classes ((:rewrite :backchain-limit-lst (0)))
+  :hints (("Goal" :in-theory (enable bvchop))))
+
 (defthm bvchop-when-i-is-not-an-integer
   (implies (not (integerp i))
            (equal (bvchop size i)
@@ -69,6 +78,11 @@
   :rule-classes (:rewrite :linear)
   :hints (("Goal" :in-theory (enable bvchop))))
 
+(defthm bvchop-upper-bound-linear-strong
+  (implies (natp n)
+           (<= (bvchop n x) (+ -1 (expt 2 n))))
+  :rule-classes :linear)
+
 (local
  (defthm bvchop-of-bvchop2
    (implies (and (<= 0 size1)
@@ -109,23 +123,21 @@
   :hints (("Goal" :use (:instance bvchop-bvchop-better-helper (size (ifix size)) (size1 (ifix size1)))
            :in-theory (disable bvchop-bvchop-better-helper))))
 
+;allow the sizes to differ?
+;or just use the meta rule...
 (defthm bvchop-of-*-of-bvchop
    (implies (and (integerp x)
-                 (integerp y)
-                 (integerp n))
-            (equal (bvchop n (* (bvchop n y) x))
+                 (integerp y))
+            (equal (bvchop n (* (bvchop n x) y))
                    (bvchop n (* x y))))
-   :otf-flg t
    :hints (("Goal" :do-not '(generalize eliminate-destructors)
             :in-theory (enable bvchop))))
 
 (defthm bvchop-of-*-of-bvchop-arg2
    (implies (and (integerp x)
-                 (integerp y)
-                 (integerp n))
+                 (integerp y))
             (equal (bvchop n (* x (bvchop n y)))
                    (bvchop n (* x y))))
-   :otf-flg t
    :hints (("Goal" :do-not '(generalize eliminate-destructors)
             :in-theory (enable bvchop))))
 
@@ -210,8 +222,7 @@
                 (syntaxp (quotep freesize))
                 ;gets computed:
                 (not (equal free (bvchop freesize const))))
-           (equal (equal const x)
-                  nil)))
+           (not (equal const x))))
 
 (defthm bvchop-of-1
   (equal (bvchop n 1)
@@ -222,12 +233,13 @@
 
 (defthmd bvchop-of-sum-cases
   (implies (and (integerp i2)
-                (integerp i1)
-                (natp size))
+                (integerp i1))
            (equal (bvchop size (+ i1 i2))
-                  (if (< (+ (bvchop size i1) (bvchop size i2)) (expt 2 size))
-                      (+ (bvchop size i1) (bvchop size i2))
-                    (+ (- (expt 2 size)) (bvchop size i1) (bvchop size i2)))))
+                  (if (not (natp size))
+                      0
+                    (if (< (+ (bvchop size i1) (bvchop size i2)) (expt 2 size))
+                        (+ (bvchop size i1) (bvchop size i2))
+                      (+ (- (expt 2 size)) (bvchop size i1) (bvchop size i2))))))
   :hints (("Goal" :in-theory (enable bvchop mod-sum-cases))))
 
 (defthm bvchop-of-minus-of-bvchop
@@ -249,7 +261,7 @@
   (implies (not (posp size))
            (equal (bvchop size i) 0))
   :rule-classes ((:rewrite :backchain-limit-lst (0)))
-  :hints (("Goal" :in-theory (e/d (bvchop) nil))))
+  :hints (("Goal" :in-theory (enable bvchop))))
 
 (defthm bvchop-of-minus
   (equal (bvchop size (- x))
@@ -263,30 +275,12 @@
                                bvchop-when-size-is-not-posp
                                expt))))
 
-;rename
-(defthm bvchop-n-times-drop
-   (implies (and (integerp x)
-                 (integerp y))
-            (equal (bvchop n (* (bvchop n x) y))
-                   (bvchop n (* x y))))
-   :hints (("Goal" :in-theory (enable bvchop))))
-
 ;i guess this one is an abbreviation rule
 (defthm unsigned-byte-p-bvchop-same
   (equal (unsigned-byte-p size (bvchop size i))
          (natp size))
   :hints (("Goal" :in-theory (enable unsigned-byte-p))))
 
-;allow the sizes to differ?
-;or just use the meta rule...
-(defthm bvchop-times-cancel
-  (implies (and (integerp x)
-                (integerp y)
-                (natp n))
-           (equal (bvchop n (* (bvchop n x) y))
-                  (bvchop n (* x y))))
-  :hints (("Goal" :in-theory (e/d (bvchop) (mod-cancel)))))
-
 ;rename
 (defthm bvchop-shift-gen
   (implies (and (integerp x)
@@ -373,7 +367,8 @@
            :in-theory (enable (:i expt)
                               bvchop ;mod-cancel
                               mod-of-mod-when-mult
-                              unsigned-byte-p))))
+                              unsigned-byte-p
+                              mod-sum-cases))))
 
 ;make a constant version? maybe not for this one?
 (defthm bvchop-of-mask-gen
@@ -389,8 +384,7 @@
   (implies (and (syntaxp (quotep k))
                 (not (unsigned-byte-p size k))
                 (natp size))
-           (equal (equal k (bvchop size x))
-                  nil)))
+           (not (equal k (bvchop size x)))))
 
 (defthm bvchop-of-expt-0
   (implies (and (<= size1 size2)
@@ -422,7 +416,7 @@
 
 ;gen
 ;strength reduction
-(defthm mod-by-4-becomes-bvchop
+(defthmd mod-by-4-becomes-bvchop
   (implies (integerp i)
            (equal (mod i 4)
                   (bvchop 2 i)))
@@ -474,6 +468,7 @@
                       (+ -1 (bvchop size x)))))
   :hints (("Goal" :in-theory (enable bvchop-of-sum-cases))))
 
+;rename
 (defthm bvchop-chop-leading-constant
   (implies (and (syntaxp (and (quotep k)
                               (quotep size)))
@@ -486,24 +481,24 @@
                   (bvchop size (+ (bvchop size k) x))))
   :hints (("Goal" :cases ((natp size)))))
 
+;rename
 (defthm bvchop-times-cancel-better
   (implies (and  (<= m n)
                  (integerp x)
                  (integerp y)
-                 (natp n)
-                 (natp m))
+                 (natp n))
            (equal (bvchop m (* (bvchop n x) y))
                   (bvchop m (* x y))))
   :hints (("Goal" :in-theory (e/d (bvchop) (mod-of-*-of-mod))
            :use ((:instance mod-of-*-of-mod (z (expt 2 m)) (x y) (y x))
                  (:instance mod-of-*-of-mod (z (expt 2 m)) (x y) (y (mod x (expt 2 n))))))))
 
+;rename
 (defthm bvchop-times-cancel-better-alt
   (implies (and  (<= m n)
                  (integerp x)
                  (integerp y)
-                 (natp n)
-                 (natp m))
+                 (natp n))
            (equal (bvchop m (* y (bvchop n x)))
                   (bvchop m (* y x))))
   :hints (("Goal" :use (:instance bvchop-times-cancel-better)
@@ -526,8 +521,7 @@
 (defthm bvchop-numeric-bound
   (implies (and (syntaxp (quotep k))
                 (<= k 0))
-           (equal (< (bvchop size x) k)
-                  nil)))
+           (not (< (bvchop size x) k))))
 
 (defthm bvchop-subst-constant
   (implies (and (syntaxp (not (quotep x)))
@@ -553,27 +547,24 @@
 ;can this be expensive?
 ;rename?
 (defthm bvchop-bound-rw
-  (implies (and (natp x)
-                (natp size))
-           (equal (< x (bvchop size x))
-                  nil))
+  (implies (<= 0 x)
+           (not (< x (bvchop size x))))
   :hints (("Goal" :in-theory (enable bvchop))))
 
+(defthm <=-of-bvchop-same-linear
+  (implies (<= 0 x)
+           (<= (bvchop n x) x))
+  :rule-classes :linear)
+
 (defthm <-of-bvchop-and-bvchop-same
   (implies (and (<= s1 s2)
                 (natp s1)
                 (natp s2))
-           (equal (< (bvchop s2 x) (bvchop s1 x))
-                  nil))
+           (not (< (bvchop s2 x) (bvchop s1 x))))
   :hints (("Goal"
            :use (:instance bvchop-bound-rw (x (bvchop s2 x)) (size s1))
            :in-theory (disable bvchop-bound-rw))))
 
-(defthm <=-of-bvchop-same-linear
-  (implies (natp x)
-           (<= (bvchop n x) x))
-  :rule-classes ((:linear))
-  :hints (("Goal" :cases ((posp n)))))
 
 ;Not sure this will fire if SMALL and BIG are constants, due to the free var.
 (defthm <=-of-bvchop-same-linear-2
@@ -611,14 +602,16 @@
                 (natp size))
            (equal (bvchop size (+ (* x (expt 2 size)) y))
                   (bvchop size y)))
-  :hints (("Goal" :in-theory (enable bvchop equal-of-0-and-mod))))
+  :hints (("Goal" :in-theory (enable bvchop equal-of-0-and-mod
+                                     mod-sum-cases))))
 
 (defthm bvchop-of-+-of-minus-of-expt
   (implies (and (integerp x)
                 (natp size))
            (equal (bvchop size (+ x (- (expt 2 size))))
                   (bvchop size x)))
-  :hints (("Goal" :in-theory (enable bvchop))))
+  :hints (("Goal" :in-theory (enable bvchop
+                                     mod-sum-cases))))
 
 (defthm bvchop-of-mod-of-expt-2
   (implies (and (< j size)
@@ -660,3 +653,116 @@
 
 (theory-invariant (incompatible (:definition bvchop)
                                 (:rewrite mod-of-expt-of-2)))
+
+(defthm unsigned-byte-p-of-bvchop-when-already
+  (implies (and (unsigned-byte-p n x)
+                (natp n)
+                (integerp x))
+          (unsigned-byte-p n (bvchop m x)))
+  :hints (("Goal" :in-theory (enable unsigned-byte-p))))
+
+;; Replaces mod with bvchop
+;; rename
+;kill the version with 4 hard-coded
+(defthmd mod-of-expt-of-2-constant-version
+  (implies (and (syntaxp (quotep k)) ;new..
+                (power-of-2p k) ;(equal k (expt 2 (+ -1 (integer-length k))))
+                (integerp x)
+                ;(natp k)
+                )
+           (equal (mod x k)
+                  (bvchop (+ -1 (integer-length k)) x)))
+  :hints (("Goal" :in-theory (e/d (power-of-2p) (mod-of-expt-of-2))
+           :use (:instance mod-of-expt-of-2
+                           (m (+ -1 (integer-length k)))))))
+
+(theory-invariant (incompatible (:definition bvchop) (:rewrite MOD-OF-EXPT-OF-2-CONSTANT-VERSION)))
+
+(defthm bitp-of-bvchop-of-1
+  (bitp (acl2::bvchop 1 x)))
+
+(defthm bvchop-+-cancel-cross
+  (implies (and (force (integerp size))
+                (>= size 0)
+                (force (integerp i))
+                (force (integerp j))
+                (force (integerp k)))
+           (equal (equal (bvchop size (+ j i))
+                         (bvchop size (+ i k)))
+                  (equal (bvchop size j)
+                         (bvchop size k)))))
+
+(defthm bvchop-+-cancel-cross2
+  (implies (and (force (integerp size))
+                (>= size 0)
+                (force (integerp i))
+                (force (integerp j))
+                (force (integerp k)))
+           (equal (equal (bvchop size (+ i j))
+                         (bvchop size (+ k i)))
+                  (equal (bvchop size j)
+                         (bvchop size k)))))
+
+(defthm bvchop-of-*-of-expt-when-<=
+  (implies (and (<= size n)
+                (integerp x)
+                (natp n)
+                ;(natp size)
+                )
+           (equal (bvchop size (* x (expt 2 n)))
+                  0))
+  :hints (("Goal" :cases ((natp size)))))
+
+(defthm bvchop-identity-cheap
+  (implies (and (unsigned-byte-p freesize i)
+                (<= freesize size)
+                (integerp size))
+           (equal (bvchop size i)
+                  i))
+  :hints (("Goal" :in-theory (enable unsigned-byte-p))))
+
+(defthm bvchop-of-both-sides
+  (implies (equal x y)
+           (equal (bvchop size x)
+                  (bvchop size y)))
+  :rule-classes nil)
+
+(defthmd bvchop-upper-bound-3
+  (implies (and (<= (+ -1 (expt 2 n)) k)
+                (integerp k)
+                (natp n))
+           (not (< k (bvchop n x))))
+  :hints (("Goal" :cases ((<= low high))
+           :use (:instance bound-when-usb (x (bvchop n x)))
+           :in-theory (disable bound-when-usb))))
+
+;bozo more like this?
+(defthmd bvchop-upper-bound-3-constant-version
+  (implies (and (syntaxp (quotep k))
+                (<= (+ -1 (expt 2 n)) k)
+                (integerp k)
+                (natp n))
+           (not (< k (bvchop n x))))
+  :hints (("Goal" :cases ((<= low high))
+           :use (:instance bound-when-usb (x (bvchop n x)))
+           :in-theory (disable bound-when-usb))))
+
+(defthmd bvchop-of-*-when-unsigned-byte-p-of-*-of-bvchop-and-bvchop
+  (implies (and (integerp x)
+                (integerp y))
+           (implies (unsigned-byte-p size (* (bvchop size x) (bvchop size y)))
+                    (equal (bvchop size (* x y))
+                           (* (bvchop size x) (bvchop size y)))))
+  :hints (("Goal" :use ((:instance bvchop-of-*-of-bvchop (n size))
+                        (:instance bvchop-of-*-of-bvchop-arg2 (n size)
+                                   (x (bvchop size x))))
+           :in-theory (disable bvchop-of-*-of-bvchop
+                               bvchop-of-*-of-bvchop-arg2
+                               bvchop-times-cancel-better-alt))))
+
+;gen the exponent
+(defthm bvchop-of-plus-of-expt-bigger
+  (implies (and (posp size)
+                (integerp x))
+           (equal (bvchop (+ -1 size) (+ x (expt 2 size)))
+                  (bvchop (+ -1 size) x))))
