@@ -18,8 +18,8 @@
 ;; See tests in sha-256-tests.lisp.
 
 (include-book "../../bv/ops32")
-(include-book "../../bv/rightrotate")
-(include-book "../../bv/defs-shifts")
+(include-book "../../bv/rightrotate32")
+(include-book "../../bv/bvshr-def")
 (include-book "../../bv-lists/packbv")
 (include-book "../../bv-lists/unpackbv")
 (include-book "../../bv-lists/bytes-to-bits")
@@ -68,7 +68,7 @@
                               (natp n)
                               (< n 32))))
   ;; Note the order of the arguments here:
-  (shr 32 x n))
+  (bvshr 32 x n))
 
 ;; Equation (4.2)
 (defun ch32 (x y z)
@@ -146,12 +146,14 @@
 (defthm mod-of-len-of-sha-256-pad-message-and-512
   (equal (mod (len (sha-256-pad-message msg)) 512)
          0)
-  :hints (("Goal" :in-theory (enable sha-256-pad-message))))
+  :hints (("Goal" :in-theory (enable sha-256-pad-message
+                                     acl2::mod-sum-cases))))
 
 (defthm mod-of-len-of-sha-256-pad-message-and-32
   (equal (mod (len (sha-256-pad-message msg)) 32)
          0)
-  :hints (("Goal" :in-theory (enable sha-256-pad-message))))
+  :hints (("Goal" :in-theory (enable sha-256-pad-message
+                                     acl2::mod-sum-cases))))
 
 ;; Divide MSG (a sequence of bits) into a sequence of 32-bit words.  See
 ;; Section 5.2.1.
