@@ -1,6 +1,6 @@
 ; Event Macros Library
 ;
-; Copyright (C) 2020 Kestrel Institute (http://www.kestrel.edu)
+; Copyright (C) 2021 Kestrel Institute (http://www.kestrel.edu)
 ;
 ; License: A 3-clause BSD license. See the LICENSE file distributed with ACL2.
 ;
@@ -11,11 +11,11 @@
 (in-package "ACL2")
 
 (include-book "evmac-input-hints-p")
-(include-book "evmac-input-print-p")
+(include-book "screen-printing")
 
+(include-book "kestrel/event-macros/cw-event" :dir :system)
+(include-book "kestrel/event-macros/try-event" :dir :system)
 (include-book "kestrel/std/system/fresh-logical-name-with-dollars-suffix" :dir :system)
-(include-book "kestrel/utilities/orelse" :dir :system)
-(include-book "kestrel/utilities/user-interface" :dir :system)
 (include-book "std/util/defaggregate" :dir :system)
 (include-book "xdoc/defxdoc-plus" :dir :system)
 
@@ -147,7 +147,7 @@
   :returns (mv (event "A @(tsee pseudo-event-formp).")
                (thm-name "A @(tsee symbolp).")
                (new-hints "An @(tsee evmac-input-hints-p).")
-               (new-names-to-avoid "A @(tsee symbol-listp)."))
+               (updated-names-to-avoid "A @(tsee symbol-listp)."))
   :mode :program
   :short "Generate a theorem event for an applicability condition."
   :long
@@ -202,6 +202,11 @@
    (xdoc::p
     "The theorem is wrapped into @(tsee try-event)
      in order to provide a terser error message if the proof fails.
+     This wrapping is not performed if @(':print') is @(':all'),
+     to avoid the suppression of the error output done by @(tsee try-event).
+     This causes the print levels to be not quite ``monotonic'',
+     in the sense that @(':print :all') will not print
+     the custom error message passed to @(tsee try-event).
      This choice may be revisited at some point.")
    (xdoc::p
     "This function also takes a print specifier as input,
@@ -245,13 +250,13 @@
      it is still, mainly, a theorem event."))
   (b* ((wrld (w state))
        ((evmac-appcond appcond) appcond)
-       (thm-name (fresh-logical-name-with-$s-suffix (intern-in-package-of-symbol
-                                                     (symbol-name appcond.name)
-                                                     (pkg-witness "ACL2"))
-                                                    nil
-                                                    names-to-avoid
-                                                    wrld))
-       (new-names-to-avoid (cons thm-name names-to-avoid))
+       ((mv thm-name updated-names-to-avoid)
+        (fresh-logical-name-with-$s-suffix (intern-in-package-of-symbol
+                                            (symbol-name appcond.name)
+                                            (pkg-witness "ACL2"))
+                                           nil
+                                           names-to-avoid
+                                           wrld))
        (thm-formula (untranslate appcond.formula t wrld))
        ((mv thm-hints new-hints) (if (keyword-truelist-alistp hints)
                                      (mv (cdr (assoc-eq appcond.name hints))
@@ -264,7 +269,9 @@
        (error-msg (msg
                    "The proof of the ~x0 applicability condition fails:~%~x1~|"
                    appcond.name thm-formula))
-       (try-thm-event (try-event thm-event ctx t nil error-msg))
+       (try?-thm-event (if (eq print :all)
+                           thm-event
+                         (try-event thm-event ctx t nil error-msg)))
        (show-progress-p (member-eq print '(:info :all)))
        (progress-start? (and show-progress-p
                              `((cw-event
@@ -274,9 +281,9 @@
        (progress-end? (and show-progress-p
                            `((cw-event "Done.~%"))))
        (event `(local (progn ,@progress-start?
-                             ,try-thm-event
+                             ,try?-thm-event
                              ,@progress-end?))))
-    (mv event thm-name new-hints new-names-to-avoid)))
+    (mv event thm-name new-hints updated-names-to-avoid)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -289,7 +296,7 @@
   :returns (mv (events "A @(tsee pseudo-event-form-listp).")
                (thm-names "A @(tsee keyword-symbol-alistp).")
                (new-hints "An @(tsee evmac-input-hints-p).")
-               (new-names-to-avoid "A @(tsee symbol-listp)."))
+               (updated-names-to-avoid "A @(tsee symbol-listp)."))
   :mode :program
   :short "Lift @(tsee evmac-appcond-theorem)
           to lists of applicability conditions."
@@ -405,8 +412,8 @@
      first one calls @(tsee evmac-appcond-theorem-list)
      and then @(tsee evmac-ensure-no-extra-hints) on the remaining hints.
      This combining function returns no hints result."))
-  (b* (((mv events thm-names remaining-hints new-names-to-avoid)
+  (b* (((mv events thm-names remaining-hints updated-names-to-avoid)
         (evmac-appcond-theorem-list
          appconds hints names-to-avoid print ctx state))
        ((er &) (evmac-ensure-no-extra-hints remaining-hints ctx state)))
-    (value (list events thm-names new-names-to-avoid))))
+    (value (list events thm-names updated-names-to-avoid))))
