@@ -1,6 +1,6 @@
-; FTY -- Byte List Fixtype Generator
+; FTY Library
 ;
-; Copyright (C) 2019 Kestrel Institute (http://www.kestrel.edu)
+; Copyright (C) 2020 Kestrel Institute (http://www.kestrel.edu)
 ;
 ; License: A 3-clause BSD license. See the LICENSE file distributed with ACL2.
 ;
@@ -10,6 +10,7 @@
 
 (in-package "FTY")
 
+(include-book "kestrel/event-macros/proof-preparation" :dir :system)
 (include-book "std/typed-lists/unsigned-byte-listp" :dir :system)
 (include-book "std/typed-lists/signed-byte-listp" :dir :system)
 (include-book "std/util/deflist" :dir :system)
@@ -306,8 +307,7 @@
        ()
        (logic)
        ,deflist-event
-       (set-default-hints nil)
-       (set-override-hints nil)
+       (acl2::evmac-prepare-proofs)
        ,theorems-event)))
 
 (defsection defbytelist-macro-definition
