@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018 Kestrel Institute (http://www.kestrel.edu)
+ * Copyright (C) 2020 Kestrel Institute (http://www.kestrel.edu)
  * License: A 3-clause BSD license. See the LICENSE file distributed with ACL2.
  * Author: Alessandro Coglio (coglio@kestrel.edu)
  */
@@ -41,6 +41,18 @@ public abstract class Acl2Rational extends Acl2Number {
         return Acl2Symbol.T;
     }
 
+    /**
+     * Checks if this rational is a rational, which is always true,
+     * returning a Java boolean instead of an ACL2 symbol.
+     * This is consistent with the {@code rationalp} ACL2 function.
+     *
+     * @return {@code true}.
+     */
+    @Override
+    boolean rationalpBoolean() {
+        return true;
+    }
+
     /**
      * Negates (arithmetically) this rational,
      * consistently with the {@code unary--} ACL2 function.
