@@ -10,7 +10,7 @@
 
 (in-package "JAVA")
 
-(include-book "types")
+(include-book "type-macros")
 
 (include-book "../language/primitive-operations")
 (include-book "../language/primitive-conversions")
@@ -61,91 +61,7 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defsection atj-java-primitive-values-representation-check
-  :short "Checking the representation of Java primitive values."
-  :long
-  (xdoc::topstring
-   (xdoc::p
-    "The fixtypes for the Java primitive types (e.g. @(tsee int-value))
-     provide the representation of Java primitive types in ACL2,
-     which ATJ maps to Java primitive values.
-     While values of those ACL2 fixtypes can be treated abstractly,
-     nothing prevents the use of ACL2 operations on them
-     that expose their internal structure, e.g. taking the @(tsee car).
-     Thus, Java code generated by ATJ must, if needed,
-     convert Java primitive values to
-     the Java representation of their ACL2 representation.")
-   (xdoc::p
-    "The following theorems ensure that the aforementioned fixtypes
-     have the representation assumed by ATJ."))
-
-  (defrule atj-java-boolean-value-representation-check
-    (equal (boolean-value-p x)
-           (and (tuplep 2 x)
-                (eq (first x) :boolean)
-                (booleanp (second x))))
-    :rule-classes nil
-    :enable boolean-value-p)
-
-  (defrule atj-java-char-value-representation-check
-    (equal (char-value-p x)
-           (and (tuplep 2 x)
-                (eq (first x) :char)
-                (ubyte16p (second x))))
-    :rule-classes nil
-    :enable char-value-p)
-
-  (defrule atj-java-byte-value-representation-check
-    (equal (byte-value-p x)
-           (and (tuplep 2 x)
-                (eq (first x) :byte)
-                (sbyte8p (second x))))
-    :rule-classes nil
-    :enable byte-value-p)
-
-  (defrule atj-java-short-value-representation-check
-    (equal (short-value-p x)
-           (and (tuplep 2 x)
-                (eq (first x) :short)
-                (sbyte16p (second x))))
-    :rule-classes nil
-    :enable short-value-p)
-
-  (defrule atj-java-int-value-representation-check
-    (equal (int-value-p x)
-           (and (tuplep 2 x)
-                (eq (first x) :int)
-                (sbyte32p (second x))))
-    :rule-classes nil
-    :enable int-value-p)
-
-  (defrule atj-java-long-value-representation-check
-    (equal (long-value-p x)
-           (and (tuplep 2 x)
-                (eq (first x) :long)
-                (sbyte64p (second x))))
-    :rule-classes nil
-    :enable long-value-p)
-
-  (defrule atj-java-float-value-representation-check
-    (equal (float-value-p x)
-           (and (tuplep 2 x)
-                (eq (first x) :float)
-                (float-value-abs-p (second x))))
-    :rule-classes nil
-    :enable float-value-p)
-
-  (defrule atj-java-double-value-representation-check
-    (equal (double-value-p x)
-           (and (tuplep 2 x)
-                (eq (first x) :double)
-                (double-value-abs-p (second x))))
-    :rule-classes nil
-    :enable double-value-p))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-
-(defval *atj-java-primitive-constrs*
+(defval *atj-jprim-constr-fns*
   :short "List of (the names of) the ACL2 functions that model
           the construction of Java primitive values."
   :long
@@ -162,7 +78,7 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defval *atj-java-primitive-deconstrs*
+(defval *atj-jprim-deconstr-fns*
   :short "List of (the names of) the ACL2 functions that model
           the deconstruction of Java primitive values."
   :long
@@ -180,7 +96,7 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defval *atj-java-primitive-unops*
+(defval *atj-jprim-unop-fns*
   :short "List of (the names of) the ACL2 functions that model
           Java primitive unary operations."
   :long
@@ -205,7 +121,7 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defval *atj-java-primitive-binops*
+(defval *atj-jprim-binop-fns*
   :short "List of (the names of) the ACL2 functions that model
           Java primitive binary operations."
   :long
@@ -286,7 +202,7 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defval *atj-java-primitive-convs*
+(defval *atj-jprim-conv-fns*
   :short "List of (the names of) the ACL2 functions that model
           Java primitive conversions."
   :long
@@ -341,65 +257,65 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defval *atj-java-primitive-fns*
+(defval *atj-jprim-fns*
   :short "List of (the names of) the ACL2 functions that model
           Java primitive value constructions and operations."
-  (append *atj-java-primitive-constrs*
-          *atj-java-primitive-deconstrs*
-          *atj-java-primitive-unops*
-          *atj-java-primitive-binops*
-          *atj-java-primitive-convs*)
+  (append *atj-jprim-constr-fns*
+          *atj-jprim-deconstr-fns*
+          *atj-jprim-unop-fns*
+          *atj-jprim-binop-fns*
+          *atj-jprim-conv-fns*)
   ///
-  (assert-event (function-name-listp *atj-java-primitive-fns* (w state)))
-  (assert-event (no-duplicatesp-eq *atj-java-primitive-fns*)))
+  (assert-event (function-name-listp *atj-jprim-fns* (w state)))
+  (assert-event (no-duplicatesp-eq *atj-jprim-fns*)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define atj-java-primitive-constr-p (fn)
+(define atj-jprim-constr-fn-p (fn)
   :returns (yes/no booleanp)
   :short "Recognize the ACL2 function symbols that model
           the construction of Java primitive types."
-  (and (member-eq fn *atj-java-primitive-constrs*) t))
+  (and (member-eq fn *atj-jprim-constr-fns*) t))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define atj-java-primitive-deconstr-p (fn)
+(define atj-jprim-deconstr-fn-p (fn)
   :returns (yes/no booleanp)
   :short "Recognize the ACL2 function symbols that model
           the deconstruction of Java primitive types."
-  (and (member-eq fn *atj-java-primitive-deconstrs*) t))
+  (and (member-eq fn *atj-jprim-deconstr-fns*) t))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define atj-java-primitive-unop-p (fn)
+(define atj-jprim-unop-fn-p (fn)
   :returns (yes/no booleanp)
   :short "Recognize the ACL2 function symbols that model
           the Java primitive unary operations."
-  (and (member-eq fn *atj-java-primitive-unops*) t))
+  (and (member-eq fn *atj-jprim-unop-fns*) t))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define atj-java-primitive-binop-p (fn)
+(define atj-jprim-binop-fn-p (fn)
   :returns (yes/no booleanp)
   :short "Recognize the ACL2 function symbols that model
           the Java primitive binary operations."
-  (and (member-eq fn *atj-java-primitive-binops*) t))
+  (and (member-eq fn *atj-jprim-binop-fns*) t))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define atj-java-primitive-conv-p (fn)
+(define atj-jprim-conv-fn-p (fn)
   :returns (yes/no booleanp)
   :short "Recognize the ACL2 function symbols that model
           the Java primitive conversions."
-  (and (member-eq fn *atj-java-primitive-convs*) t))
+  (and (member-eq fn *atj-jprim-conv-fns*) t))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define atj-java-primitive-fn-p (fn)
+(define atj-jprim-fn-p (fn)
   :returns (yes/no booleanp)
-  :short "Recognize the ACL2 funcion (ymbols that model
+  :short "Recognize the ACL2 function symbols that model
           the Java primitive value constructions, operations, and conversions."
-  (and (member-eq fn *atj-java-primitive-fns*) t))
+  (and (member-eq fn *atj-jprim-fns*) t))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -408,7 +324,7 @@
 
   ;; constructors:
 
-  (atj-main-function-type boolean-value (:asymbol) :jboolean)
+  (atj-main-function-type boolean-value (:aboolean) :jboolean)
 
   (atj-main-function-type char-value (:ainteger) :jchar)
 
@@ -422,7 +338,7 @@
 
   ;; deconstructors:
 
-  (atj-main-function-type boolean-value->bool$inline (:jboolean) :asymbol)
+  (atj-main-function-type boolean-value->bool$inline (:jboolean) :aboolean)
 
   (atj-main-function-type char-value->nat$inline (:jchar) :ainteger)
 
@@ -685,3 +601,37 @@
   ;; widening and narrowing conversions:
 
   (atj-main-function-type byte-to-char (:jbyte) :jchar))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define atj-jprim-constr-fn-to-ptype ((fn atj-jprim-constr-fn-p))
+  :returns (ptype primitive-typep)
+  :short "Map an ACL2 function that models a Java primitive constructor
+          to the corresponding Java primitive type."
+  (case fn
+    (boolean-value (primitive-type-boolean))
+    (char-value (primitive-type-char))
+    (byte-value (primitive-type-byte))
+    (short-value (primitive-type-short))
+    (int-value (primitive-type-int))
+    (long-value (primitive-type-long))
+    (t (prog2$ (impossible) (ec-call (primitive-type-fix :irrelevant)))))
+  :guard-hints (("Goal" :in-theory (enable atj-jprim-constr-fn-p)))
+  :hooks (:fix))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define atj-jprim-deconstr-fn-to-ptype ((fn atj-jprim-deconstr-fn-p))
+  :returns (ptype primitive-typep)
+  :short "Map an ACL2 function that models a Java primitive deconstructor
+          to the corresponding Java primitive type."
+  (case fn
+    (boolean-value->bool$inline (primitive-type-boolean))
+    (char-value->nat$inline (primitive-type-char))
+    (byte-value->int$inline (primitive-type-byte))
+    (short-value->int$inline (primitive-type-short))
+    (int-value->int$inline (primitive-type-int))
+    (long-value->int$inline (primitive-type-long))
+    (t (prog2$ (impossible) (ec-call (primitive-type-fix :irrelevant)))))
+  :guard-hints (("Goal" :in-theory (enable atj-jprim-deconstr-fn-p)))
+  :hooks (:fix))
