@@ -317,7 +317,7 @@
    "This is a private static final field with an initializer,
     which constructs the character value.")
   (b* ((name (atj-gen-shallow-char-field-name char))
-       (init (atj-gen-char char))
+       (init (atj-gen-char char t nil))
        (type *aij-type-char*))
     (make-jfield :access (jaccess-private)
                  :static? t
@@ -359,7 +359,7 @@
    "This is a private static final field with an initializer,
     which constructs the string value.")
   (b* ((name (atj-gen-shallow-string-field-name string))
-       (init (atj-gen-string string))
+       (init (atj-gen-string string t nil))
        (type *aij-type-string*))
     (make-jfield :access (jaccess-private)
                  :static? t
@@ -422,7 +422,7 @@
     This field cannot be private,
     otherwise the classes for other packages could not access it.")
   (b* ((name (atj-gen-shallow-symbol-field-name symbol))
-       (init (atj-gen-symbol symbol))
+       (init (atj-gen-symbol symbol t nil))
        (type *aij-type-symbol*))
     (make-jfield :access (jaccess-default)
                  :static? t
@@ -520,35 +520,54 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define atj-gen-shallow-char ((char characterp))
+(define atj-gen-shallow-char ((char characterp) (guards$ booleanp))
   :returns (expr jexprp)
   :short "Generate a shallowly embedded ACL2 quoted character."
   :long
-  (xdoc::topstring-p
-   "This is just a reference to the field for the quoted character.")
-  (jexpr-name (atj-gen-shallow-char-field-name char)))
+  (xdoc::topstring
+   (xdoc::p
+    "If guards are assumed,
+     we generate the corresponding Java character literal.")
+   (xdoc::p
+    "Otherwise,
+     we generate a reference to the field for the quoted character."))
+  (if guards$
+      (jexpr-literal-character char)
+    (jexpr-name (atj-gen-shallow-char-field-name char))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define atj-gen-shallow-string ((string stringp))
+(define atj-gen-shallow-string ((string stringp) (guards$ booleanp))
   :returns (expr jexprp)
   :short "Generate a shallowly embedded ACL2 quoted string."
   :long
-  (xdoc::topstring-p
-   "This is just a reference to the field for the quoted string.")
-  (jexpr-name (atj-gen-shallow-string-field-name string)))
+  (xdoc::topstring
+   (xdoc::p
+    "If guards are assumed,
+     we generate the corresponding Java string expression.")
+   (xdoc::p
+    "Otherwise,
+     we generate a reference to the field for the quoted string."))
+  (if guards$
+      (atj-gen-jstring string)
+    (jexpr-name (atj-gen-shallow-string-field-name string))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define atj-gen-shallow-symbol ((symbol symbolp)
                                 (pkg-class-names string-string-alistp)
-                                (curr-pkg stringp))
+                                (curr-pkg stringp)
+                                (guards$ booleanp))
   :returns (expr jexprp)
   :short "Generate a shallowly embedded ACL2 quoted symbol."
   :long
   (xdoc::topstring
    (xdoc::p
-    "This depends on the current package,
+    "If guards are assumed and the symbol is an ACL2 boolean,
+     we generate the corresponding Java boolean literal.")
+   (xdoc::p
+    "In all other cases,
+     the generated expression depends on the current package,
      i.e. the package of the function where the quoted symbol occurs.
      If the current package is the same as the symbol's package,
      or if the current package imports the symbol,
@@ -557,7 +576,10 @@
      Otherwise, we generate a qualified name,
      preceded by the name of the class for the symbol's package.
      This mirrors the rules for symbol references in ACL2."))
-  (b* ((sym-pkg (symbol-package-name symbol))
+  (b* (((when (and guards$
+                   (booleanp symbol)))
+        (if symbol (jexpr-literal-true) (jexpr-literal-false)))
+       (sym-pkg (symbol-package-name symbol))
        (simple-name (atj-gen-shallow-symbol-field-name symbol))
        ((when (or (equal sym-pkg curr-pkg)
                   (member-eq symbol (pkg-imports curr-pkg))))
@@ -580,15 +602,17 @@
 (define atj-gen-shallow-value (value
                                (qpairs cons-pos-alistp)
                                (pkg-class-names string-string-alistp)
-                               (curr-pkg stringp))
+                               (curr-pkg stringp)
+                               (guards$ booleanp))
   :returns (expr jexprp)
   :short "Generate a shallowly embedded ACL2 value."
   (cond ((acl2-numberp value) (atj-gen-shallow-number value))
-        ((characterp value) (atj-gen-shallow-char value))
-        ((stringp value) (atj-gen-shallow-string value))
+        ((characterp value) (atj-gen-shallow-char value guards$))
+        ((stringp value) (atj-gen-shallow-string value guards$))
         ((symbolp value) (atj-gen-shallow-symbol value
                                                  pkg-class-names
-                                                 curr-pkg))
+                                                 curr-pkg
+                                                 guards$))
         ((consp value) (atj-gen-shallow-cons value qpairs))
         (t (prog2$ (raise "Internal error: ~x0 is not a recognized value."
                           value)
