@@ -46,13 +46,14 @@
   ((bool bool))
   :tag :boolean
   :layout :list
+  :pred boolean-valuep
   ///
 
-  (defruled boolean-value-p-alt-def
-    (equal (boolean-value-p x)
+  (defruled boolean-valuep-alt-def
+    (equal (boolean-valuep x)
            (or (equal x (boolean-value t))
                (equal x (boolean-value nil))))
-    :enable boolean-value-p))
+    :enable boolean-valuep))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -61,6 +62,7 @@
   ((nat ubyte16))
   :tag :char
   :layout :list
+  :pred char-valuep
   ///
 
   (defrule char-value->nat-upper-bound
@@ -77,6 +79,7 @@
   ((int sbyte8))
   :tag :byte
   :layout :list
+  :pred byte-valuep
   ///
 
   (defrule byte-value->int-lower-bound
@@ -100,6 +103,7 @@
   ((int sbyte16))
   :tag :short
   :layout :list
+  :pred short-valuep
   ///
 
   (defrule short-value->int-lower-bound
@@ -123,6 +127,7 @@
   ((int sbyte32))
   :tag :int
   :layout :list
+  :pred int-valuep
   ///
 
   (defrule int-value->int-lower-bound
@@ -146,6 +151,7 @@
   ((int sbyte64))
   :tag :long
   :layout :list
+  :pred long-valuep
   ///
 
   (defrule long-value->int-lower-bound
@@ -169,7 +175,8 @@
           in the float value set [JLS:4.2.3]."
   ((float float-value-abs))
   :tag :float
-  :layout :list)
+  :layout :list
+  :pred float-valuep)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -178,12 +185,13 @@
           in the double value set [JLS:4.2.3]."
   ((double double-value-abs))
   :tag :double
-  :layout :list)
+  :layout :list
+  :pred double-valuep)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (fty::deflist boolean-value-list
-  :short "Fixtype of true lists of Java @('boolean') values."
+  :short "Fixtype of lists of Java @('boolean') values."
   :elt-type boolean-value
   :true-listp t
   :elementp-of-nil nil
@@ -192,7 +200,7 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (fty::deflist char-value-list
-  :short "Fixtype of true lists of Java @('char') values."
+  :short "Fixtype of lists of Java @('char') values."
   :elt-type char-value
   :true-listp t
   :elementp-of-nil nil
@@ -201,7 +209,7 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (fty::deflist byte-value-list
-  :short "Fixtype of true lists of Java @('byte') values."
+  :short "Fixtype of lists of Java @('byte') values."
   :elt-type byte-value
   :true-listp t
   :elementp-of-nil nil
@@ -210,7 +218,7 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (fty::deflist short-value-list
-  :short "Fixtype of true lists of Java @('short') values."
+  :short "Fixtype of lists of Java @('short') values."
   :elt-type short-value
   :true-listp t
   :elementp-of-nil nil
@@ -219,7 +227,7 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (fty::deflist int-value-list
-  :short "Fixtype of true lists of Java @('int') values."
+  :short "Fixtype of lists of Java @('int') values."
   :elt-type int-value
   :true-listp t
   :elementp-of-nil nil
@@ -228,7 +236,7 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (fty::deflist long-value-list
-  :short "Fixtype of true lists of Java @('long') values."
+  :short "Fixtype of lists of Java @('long') values."
   :elt-type long-value
   :true-listp t
   :elementp-of-nil nil
@@ -237,7 +245,7 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (fty::deflist float-value-list
-  :short "Fixtype of true lists of Java @('float') values."
+  :short "Fixtype of lists of Java @('float') values."
   :elt-type float-value
   :true-listp t
   :elementp-of-nil nil
@@ -246,7 +254,7 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (fty::deflist double-value-list
-  :short "Fixtype of true lists of Java @('double') values."
+  :short "Fixtype of lists of Java @('double') values."
   :elt-type double-value
   :true-listp t
   :elementp-of-nil nil
@@ -271,7 +279,7 @@
      We also define a conditional constructor and a conditional destructor,
      analogous to the ones of the fixtypes of other Java primitive values."))
 
-  (define floatx-value-p (x)
+  (define floatx-valuep (x)
     :returns (yes/no booleanp)
     :parents (floatx-value-fns)
     :short "Recognizer of Java @('float') values
@@ -282,34 +290,34 @@
     ///
     (defrule no-floatx-value-when-unsupported
       (implies (not (floatx-param))
-               (not (floatx-value-p x)))))
+               (not (floatx-valuep x)))))
 
-  (local (in-theory (enable floatx-value-p)))
+  (local (in-theory (enable floatx-valuep)))
 
-  (define floatx-value-fix ((x floatx-value-p))
-    :returns (fixed-x floatx-value-p :hyp (floatx-param))
+  (define floatx-value-fix ((x floatx-valuep))
+    :returns (fixed-x floatx-valuep :hyp (floatx-param))
     :parents (floatx-value-fns)
     :short "Fixer of Java @('float') values
             in the float-extended-exponent value set."
-    (mbe :logic (if (floatx-value-p x)
+    (mbe :logic (if (floatx-valuep x)
                     x
                   (list :floatx (floatx-value-abs-pos-zero)))
          :exec x)
     ///
-    (defrule floatx-value-fix-when-floatx-value-p
-      (implies (floatx-value-p x)
+    (defrule floatx-value-fix-when-floatx-valuep
+      (implies (floatx-valuep x)
                (equal (floatx-value-fix x) x))))
 
   (local (in-theory (enable floatx-value-fix)))
 
   (define floatx-value ((x floatx-value-abs-p))
-    :returns (value floatx-value-p :hyp (floatx-param))
+    :returns (value floatx-valuep :hyp (floatx-param))
     :parents (floatx-value-fns)
     :short "Constructor of Java @('float') values
             in the float-extended-exponent value set."
     (list :floatx (floatx-value-abs-fix x)))
 
-  (define floatx-value->floatx ((value floatx-value-p))
+  (define floatx-value->floatx ((value floatx-valuep))
     :returns (x floatx-value-abs-p :hyp (floatx-param))
     :parents (floatx-value-fns)
     :short "Destructor of Java @('float') values
@@ -341,7 +349,7 @@
      We also define a conditional constructor and a conditional destructor,
      analogous to the ones of the fixtypes of other Java primitive values."))
 
-  (define doublex-value-p (x)
+  (define doublex-valuep (x)
     :returns (yes/no booleanp)
     :parents (doublex-value-fns)
     :short "Recognizer of Java @('double') values
@@ -352,34 +360,34 @@
     ///
     (defrule no-doublex-value-when-unsupported
       (implies (not (doublex-param))
-               (not (doublex-value-p x)))))
+               (not (doublex-valuep x)))))
 
-  (local (in-theory (enable doublex-value-p)))
+  (local (in-theory (enable doublex-valuep)))
 
-  (define doublex-value-fix ((x doublex-value-p))
-    :returns (fixed-x doublex-value-p :hyp (doublex-param))
+  (define doublex-value-fix ((x doublex-valuep))
+    :returns (fixed-x doublex-valuep :hyp (doublex-param))
     :parents (doublex-value-fns)
     :short "Fixer of Java @('double') values
             in the double-extended-exponent value set."
-    (mbe :logic (if (doublex-value-p x)
+    (mbe :logic (if (doublex-valuep x)
                     x
                   (list :doublex (doublex-value-abs-pos-zero)))
          :exec x)
     ///
-    (defrule doublex-value-fix-when-doublex-value-p
-      (implies (doublex-value-p x)
+    (defrule doublex-value-fix-when-doublex-valuep
+      (implies (doublex-valuep x)
                (equal (doublex-value-fix x) x))))
 
   (local (in-theory (enable doublex-value-fix)))
 
   (define doublex-value ((x doublex-value-abs-p))
-    :returns (value doublex-value-p :hyp (doublex-param))
+    :returns (value doublex-valuep :hyp (doublex-param))
     :parents (doublex-value-fns)
     :short "Constructor of Java @('double') values
             in the double-extended-exponent value set."
     (list :doublex (doublex-value-abs-fix x)))
 
-  (define doublex-value->doublex ((value doublex-value-p))
+  (define doublex-value->doublex ((value doublex-valuep))
     :returns (x doublex-value-abs-p :hyp (doublex-param))
     :parents (doublex-value-fns)
     :short "Destructor of Java @('double') values
@@ -400,7 +408,8 @@
   (:byte byte-value)
   (:short short-value)
   (:int int-value)
-  (:long long-value))
+  (:long long-value)
+  :pred integral-valuep)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -408,7 +417,8 @@
   :short "Fixtype of Java floating-point values [JLS:4.2.3],
           excluding extended-exponent values [JLS:4.2.3]."
   (:float float-value)
-  (:double double-value))
+  (:double double-value)
+  :pred floating-point-valuep)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -416,34 +426,34 @@
   :short "Fixtype of Java floating-point values [JLS:4.2.3],
           including extended-exponent values [JLS:4.2.3]."
 
-  (define floating-pointx-value-p (x)
+  (define floating-pointx-valuep (x)
     :returns (yes/no booleanp)
     :parents (floating-pointx-value)
     :short "Recognizer for @(tsee floating-pointx-value)."
-    (or (floating-point-value-p x)
-        (floatx-value-p x)
-        (doublex-value-p x))
+    (or (floating-point-valuep x)
+        (floatx-valuep x)
+        (doublex-valuep x))
     ///
 
-    (defrule floating-pointx-value-p-when-floating-point-value-p
-      (implies (floating-point-value-p x)
-               (floating-pointx-value-p x)))
+    (defrule floating-pointx-valuep-when-floating-point-valuep
+      (implies (floating-point-valuep x)
+               (floating-pointx-valuep x)))
 
-    (defrule floating-pointx-value-p-when-floatx-value-p
-      (implies (floatx-value-p x)
-               (floating-pointx-value-p x)))
+    (defrule floating-pointx-valuep-when-floatx-valuep
+      (implies (floatx-valuep x)
+               (floating-pointx-valuep x)))
 
-    (defrule floating-pointx-value-p-when-doublex-value-p
-      (implies (doublex-value-p x)
-               (floating-pointx-value-p x))))
+    (defrule floating-pointx-valuep-when-doublex-valuep
+      (implies (doublex-valuep x)
+               (floating-pointx-valuep x))))
 
   (std::deffixer floating-pointx-value-fix
-    :pred floating-pointx-value-p
+    :pred floating-pointx-valuep
     :body-fix (float-value (float-value-abs-pos-zero))
     :parents nil)
 
   (fty::deffixtype floating-pointx-value
-    :pred floating-pointx-value-p
+    :pred floating-pointx-valuep
     :fix floating-pointx-value-fix
     :equiv floating-pointx-value-equiv
     :define t
@@ -460,14 +470,15 @@
   (:int int-value)
   (:long long-value)
   (:float float-value)
-  (:double double-value))
+  (:double double-value)
+  :pred numeric-valuep)
 
 (defsection numeric-value-ext
   :extension numeric-value
-  (defrule numeric-value-p-when-integral-value-p
-    (implies (integral-value-p x)
-             (numeric-value-p x))
-    :enable integral-value-p))
+  (defrule numeric-valuep-when-integral-valuep
+    (implies (integral-valuep x)
+             (numeric-valuep x))
+    :enable integral-valuep))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -475,34 +486,34 @@
   :short "Fixtype of Java numeric values [JLS:4.2],
           including extended-exponent values [JLS:4.2.3]."
 
-  (define numericx-value-p (x)
+  (define numericx-valuep (x)
     :returns (yes/no booleanp)
     :parents (numericx-value)
     :short "Recognizer for @(tsee numericx-value)."
-    (or (numeric-value-p x)
-        (floatx-value-p x)
-        (doublex-value-p x))
+    (or (numeric-valuep x)
+        (floatx-valuep x)
+        (doublex-valuep x))
     ///
 
-    (defrule numericx-value-p-when-numeric-value-p
-      (implies (numeric-value-p x)
-               (numericx-value-p x)))
+    (defrule numericx-valuep-when-numeric-valuep
+      (implies (numeric-valuep x)
+               (numericx-valuep x)))
 
-    (defrule numericx-value-p-when-floatx-value-p
-      (implies (floatx-value-p x)
-               (numericx-value-p x)))
+    (defrule numericx-valuep-when-floatx-valuep
+      (implies (floatx-valuep x)
+               (numericx-valuep x)))
 
-    (defrule numericx-value-p-when-doublex-value-p
-      (implies (doublex-value-p x)
-               (numericx-value-p x))))
+    (defrule numericx-valuep-when-doublex-valuep
+      (implies (doublex-valuep x)
+               (numericx-valuep x))))
 
   (std::deffixer numericx-value-fix
-    :pred numericx-value-p
+    :pred numericx-valuep
     :body-fix (char-value 0)
     :parents nil)
 
   (fty::deffixtype numericx-value
-    :pred numericx-value-p
+    :pred numericx-valuep
     :fix numericx-value-fix
     :equiv numericx-value-equiv
     :define t
@@ -520,14 +531,15 @@
   (:int int-value)
   (:long long-value)
   (:float float-value)
-  (:double double-value))
+  (:double double-value)
+  :pred primitive-valuep)
 
 (defsection primitive-value-ext
   :extension primitive-value
-  (defrule primitive-value-p-when-numeric-value-p
-    (implies (numeric-value-p x)
-             (primitive-value-p x))
-    :enable numeric-value-p))
+  (defrule primitive-valuep-when-numeric-valuep
+    (implies (numeric-valuep x)
+             (primitive-valuep x))
+    :enable numeric-valuep))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -535,32 +547,32 @@
   :short "Fixtype of Java primitive values [JLS:4.2],
           including extended-exponent values [JLS:4.2.3]."
 
-  (define primitivex-value-p (x)
+  (define primitivex-valuep (x)
     :returns (yes/no booleanp)
     :parents (primitivex-value)
     :short "Recognizer for @(tsee primitivex-value)."
-    (or (primitive-value-p x)
-        (floatx-value-p x)
-        (doublex-value-p x))
+    (or (primitive-valuep x)
+        (floatx-valuep x)
+        (doublex-valuep x))
     ///
 
-    (defrule primitivex-value-p-when-primitive-value-p
-      (implies (primitive-value-p x)
-               (primitivex-value-p x)))
+    (defrule primitivex-valuep-when-primitive-valuep
+      (implies (primitive-valuep x)
+               (primitivex-valuep x)))
 
-    (defrule primitivex-value-p-when-numericx-value-p
-      (implies (numericx-value-p x)
-               (primitivex-value-p x))
-      :enable numericx-value-p))
+    (defrule primitivex-valuep-when-numericx-valuep
+      (implies (numericx-valuep x)
+               (primitivex-valuep x))
+      :enable numericx-valuep))
 
   (std::deffixer primitivex-value-fix
-    :pred primitivex-value-p
+    :pred primitivex-valuep
     :body-fix (char-value 0)
     :parents (primitivex-value)
     :short "Fixer for @(tsee primitivex-value).")
 
   (fty::deffixtype primitivex-value
-    :pred primitivex-value-p
+    :pred primitivex-valuep
     :fix primitivex-value-fix
     :equiv primitivex-value-equiv
     :define t
@@ -570,67 +582,67 @@
 
 (defruled disjoint-primitive-values
   :short "The tagging keywords make all the primitive values disjoint."
-  (and (implies (boolean-value-p x)
-                (and (not (char-value-p x))
-                     (not (byte-value-p x))
-                     (not (short-value-p x))
-                     (not (int-value-p x))
-                     (not (long-value-p x))
-                     (not (float-value-p x))
-                     (not (double-value-p x))
-                     (not (floatx-value-p x))
-                     (not (doublex-value-p x))))
-       (implies (char-value-p x)
-                (and (not (byte-value-p x))
-                     (not (short-value-p x))
-                     (not (int-value-p x))
-                     (not (long-value-p x))
-                     (not (float-value-p x))
-                     (not (double-value-p x))
-                     (not (floatx-value-p x))
-                     (not (doublex-value-p x))))
-       (implies (byte-value-p x)
-                (and (not (short-value-p x))
-                     (not (int-value-p x))
-                     (not (long-value-p x))
-                     (not (float-value-p x))
-                     (not (double-value-p x))
-                     (not (floatx-value-p x))
-                     (not (doublex-value-p x))))
-       (implies (short-value-p x)
-                (and (not (int-value-p x))
-                     (not (long-value-p x))
-                     (not (float-value-p x))
-                     (not (double-value-p x))
-                     (not (floatx-value-p x))
-                     (not (doublex-value-p x))))
-       (implies (int-value-p x)
-                (and (not (long-value-p x))
-                     (not (float-value-p x))
-                     (not (double-value-p x))
-                     (not (floatx-value-p x))
-                     (not (doublex-value-p x))))
-       (implies (long-value-p x)
-                (and (not (float-value-p x))
-                     (not (double-value-p x))
-                     (not (floatx-value-p x))
-                     (not (doublex-value-p x))))
-       (implies (float-value-p x)
-                (and (not (double-value-p x))
-                     (not (floatx-value-p x))
-                     (not (doublex-value-p x))))
-       (implies (double-value-p x)
-                (and (not (floatx-value-p x))
-                     (not (doublex-value-p x))))
-       (implies (floatx-value-p x)
-                (not (doublex-value-p x))))
-  :enable (boolean-value-p
-           char-value-p
-           byte-value-p
-           short-value-p
-           int-value-p
-           long-value-p
-           float-value-p
-           double-value-p
-           floatx-value-p
-           doublex-value-p))
+  (and (implies (boolean-valuep x)
+                (and (not (char-valuep x))
+                     (not (byte-valuep x))
+                     (not (short-valuep x))
+                     (not (int-valuep x))
+                     (not (long-valuep x))
+                     (not (float-valuep x))
+                     (not (double-valuep x))
+                     (not (floatx-valuep x))
+                     (not (doublex-valuep x))))
+       (implies (char-valuep x)
+                (and (not (byte-valuep x))
+                     (not (short-valuep x))
+                     (not (int-valuep x))
+                     (not (long-valuep x))
+                     (not (float-valuep x))
+                     (not (double-valuep x))
+                     (not (floatx-valuep x))
+                     (not (doublex-valuep x))))
+       (implies (byte-valuep x)
+                (and (not (short-valuep x))
+                     (not (int-valuep x))
+                     (not (long-valuep x))
+                     (not (float-valuep x))
+                     (not (double-valuep x))
+                     (not (floatx-valuep x))
+                     (not (doublex-valuep x))))
+       (implies (short-valuep x)
+                (and (not (int-valuep x))
+                     (not (long-valuep x))
+                     (not (float-valuep x))
+                     (not (double-valuep x))
+                     (not (floatx-valuep x))
+                     (not (doublex-valuep x))))
+       (implies (int-valuep x)
+                (and (not (long-valuep x))
+                     (not (float-valuep x))
+                     (not (double-valuep x))
+                     (not (floatx-valuep x))
+                     (not (doublex-valuep x))))
+       (implies (long-valuep x)
+                (and (not (float-valuep x))
+                     (not (double-valuep x))
+                     (not (floatx-valuep x))
+                     (not (doublex-valuep x))))
+       (implies (float-valuep x)
+                (and (not (double-valuep x))
+                     (not (floatx-valuep x))
+                     (not (doublex-valuep x))))
+       (implies (double-valuep x)
+                (and (not (floatx-valuep x))
+                     (not (doublex-valuep x))))
+       (implies (floatx-valuep x)
+                (not (doublex-valuep x))))
+  :enable (boolean-valuep
+           char-valuep
+           byte-valuep
+           short-valuep
+           int-valuep
+           long-valuep
+           float-valuep
+           double-valuep
+           floatx-valuep
+           doublex-valuep))
