@@ -1,6 +1,6 @@
 ; Standard System Library
 ;
-; Copyright (C) 2019 Kestrel Institute (http://www.kestrel.edu)
+; Copyright (C) 2020 Kestrel Institute (http://www.kestrel.edu)
 ;
 ; License: A 3-clause BSD license. See the LICENSE file distributed with ACL2.
 ;
@@ -12,8 +12,8 @@
 
 (include-book "non-executablep-plus")
 
-(include-book "std/testing/assert" :dir :system)
-(include-book "std/testing/eval" :dir :system)
+(include-book "std/testing/assert-bang" :dir :system)
+(include-book "std/testing/must-succeed-star" :dir :system)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -30,6 +30,7 @@
  (assert! (non-executablep+ 'g (w state))))
 
 (must-succeed*
- (defun-sk h (x y) (exists z (equal z (cons x y)))
-   :witness-dcls ((declare (xargs :non-executable nil))))
+ (defun-sk h (x y)
+   (declare (xargs :non-executable nil))
+   (exists z (equal z (cons x y))))
  (assert! (not (non-executablep+ 'h (w state)))))
