@@ -20,16 +20,22 @@
 (include-book "all-vars")
 (include-book "all-vars-in-untranslated-term")
 (include-book "all-vars-open")
+(include-book "check-and-call")
+(include-book "check-if-call")
 (include-book "check-lambda-call")
 (include-book "check-list-call")
+(include-book "check-mbt-call")
+(include-book "check-mbt-dollar-call")
 (include-book "check-mv-let-call")
 (include-book "check-nary-lambda-call")
+(include-book "check-or-call")
 (include-book "check-unary-lambda-call")
 (include-book "check-user-lambda")
 (include-book "check-user-term")
 (include-book "dumb-occur-var-open")
 (include-book "guard-verified-exec-fnsp")
 (include-book "guard-verified-fnsp")
+(include-book "if-tree-leaf-terms")
 (include-book "lambda-closedp")
 (include-book "lambda-guard-verified-exec-fnsp")
 (include-book "lambda-guard-verified-fnsp")
