@@ -19,7 +19,7 @@
 ;; XARGS (or several), etc.
 
 (include-book "keyword-value-lists2")
-(include-book "kestrel/utilities/conjunctions" :dir :system)
+(include-book "conjunctions")
 (include-book "std/lists/list-defuns" :dir :system) ;for flatten
 (include-book "std/util/bstar" :dir :system)
 
@@ -103,6 +103,18 @@
                 (consp declares))
            (declarep (car declares))))
 
+(defthm all-declarep-of-cons
+  (equal (all-declarep (cons declare declares))
+         (and (declarep declare)
+              (all-declarep declares)))
+  :hints (("Goal" :in-theory (enable all-declarep))))
+
+(defthm all-declarep-of-take
+  (implies (and (all-declarep declares)
+                (<= (nfix n) (len declares)))
+           (all-declarep (take n declares)))
+  :hints (("Goal" :in-theory (enable all-declarep))))
+
 ;;; Extract the xargs from some declares
 
 ;returns a keyword-value-listp
@@ -386,6 +398,8 @@
               (xargs ,@xargs-key-vals))))
 
 ;If there is already a "verify-guards t", we remove it from the declares.
+;; TODO: Preserve more of the original order of things?
+;; TODO: Generalize to any xarg
 (defun set-verify-guards-in-declares (declares verify-guards)
   (declare (xargs :guard (and (true-listp declares)
                               (all-declarep declares)
@@ -397,6 +411,7 @@
     `((declare ,@other-declare-args
                (xargs ,@xargs-key-vals)))))
 
+;; todo: rename to set-verify-guards-nil
 (defund add-verify-guards-nil (declares)
   (declare (xargs :guard (and (true-listp declares)
                               (all-declarep declares))))
@@ -407,6 +422,7 @@
            (all-declarep (add-verify-guards-nil declares)))
   :hints (("Goal" :in-theory (enable add-verify-guards-nil))))
 
+;; todo: rename to set-verify-guards-t
 (defund add-verify-guards-t (declares)
   (declare (xargs :guard (and (true-listp declares)
                               (all-declarep declares))))
@@ -458,8 +474,6 @@
     `((declare ,@other-declare-args
                (xargs ,@xargs-key-vals)))))
 
-
-
 (defun remove-declare-args (type declare-args)
   (declare (xargs :guard (all-declare-argp declare-args)))
   (if (atom declare-args)
@@ -506,7 +520,6 @@
            (first-declare `(declare ,declare-arg ,@declare-args)))
       (cons first-declare (rest declares)))))
 
-
 (defthm all-declare-argp-of-get-non-xargs-from-declares
   (implies (all-declarep declares)
            (all-declare-argp (get-non-xargs-from-declares declares))))
@@ -652,8 +665,16 @@
          1)
   :hints (("Goal" :in-theory (enable add-xarg-in-declares len))))
 
+(defthm all-declarep-of-add-xarg-in-declares
+  (implies (and (keywordp xarg)
+                (true-listp declares)
+                (all-declarep declares))
+           (all-declarep (add-xarg-in-declares xarg val declares)))
+  :hints (("Goal" :in-theory (enable add-xarg-in-declares))))
+
 ;;  Replace the value of XARG with VAL in DECLARES if it's present.  If it's
 ;;  not present, add it with value VAL.
+;; TODO: Preserve more of the original structure of things?
 (defund replace-xarg-in-declares (xarg val declares)
   (declare (xargs :guard (and (keywordp xarg)
                               (true-listp declares)
@@ -665,6 +686,12 @@
          1)
   :hints (("Goal" :in-theory (enable replace-xarg-in-declares))))
 
+(defthm all-declarep-of-replace-xarg-in-declares
+  (implies (and (keywordp xarg)
+                (true-listp declares)
+                (all-declarep declares))
+           (all-declarep (replace-xarg-in-declares xarg val declares)))
+  :hints (("Goal" :in-theory (enable replace-xarg-in-declares))))
 
 
 (defthm get-non-xargs-from-declare-args-of-append
