@@ -1,6 +1,6 @@
-; Ordered Maps (Omaps)
+; Ordered Maps (Omaps) Library
 ;
-; Copyright (C) 2019 Kestrel Institute (http://www.kestrel.edu)
+; Copyright (C) 2020 Kestrel Institute (http://www.kestrel.edu)
 ;
 ; License: A 3-clause BSD license. See the LICENSE file distributed with ACL2.
 ;
@@ -10,31 +10,35 @@
 
 (in-package "OMAP")
 
-(include-book "xdoc/constructors" :dir :system)
 (include-book "std/osets/top" :dir :system)
 (include-book "std/util/define" :dir :system)
 (include-book "std/util/defrule" :dir :system)
+(include-book "xdoc/defxdoc-plus" :dir :system)
+(local (include-book "misc/total-order" :dir :system))
+(local (include-book "std/lists/acl2-count" :dir :system))
+(local (include-book "tools/rulesets" :dir :system))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defxdoc omaps
+(defxdoc+ omaps
   :parents (acl2::kestrel-utilities set::std/osets acl2::alists)
   :short "A library of omaps (ordered maps),
           i.e. finite maps represented as strictly ordered alists."
   :long
   (xdoc::topstring
    (xdoc::p
-    "This is related to the library of
-     <see topic='@(url set::std/osets)'>osets (ordered sets)</see>,
-     i.e. finite sets represented as strictly ordered lists.
+    "This is related to the library of "
+    (xdoc::seetopic "set::std/osets" "osets (ordered sets)")
+    ", i.e. finite sets represented as strictly ordered lists.
      Like osets capture (up to isomorphism)
      the mathematical notion of finite sets (over ACL2 objects),
      omaps capture (up to isomorphism)
      the mathematical notion of finite maps (over ACL2 objects).
      In particular, omap equality is @(tsee equal).")
    (xdoc::p
-    "Since the <see topic='@(url <<)'>total order on ACL2 values</see>
-     is lexicographic on @(tsee cons) pairs,
+    "Since the "
+    (xdoc::seetopic "<<" "total order on ACL2 values")
+    " is lexicographic on @(tsee cons) pairs,
      an omap is an oset of @(tsee cons) pairs;
      furthermore, an omap is an alist.
      While then, in principle,
@@ -44,9 +48,9 @@
      (i.e. requiring omaps instead of just osets or alists)
      and that treat non-omaps as the empty omap.
      That is, the omap operations respect a `non-map convention'
-     analogous to the
-     <see topic='@(url set::primitives)'>non-set convention</see>
-     respected by the oset operations;
+     analogous to the "
+    (xdoc::seetopic "set::primitives" "non-set convention")
+    " respected by the oset operations;
      some of the latter are, analogously,
      versions of list operations (e.g. @(tsee set::head) for @(tsee car)),
      motivated by the fact that the list operations
@@ -59,9 +63,9 @@
      @(tsee head),
      @(tsee tail), and
      @(tsee update)
-     are ``primitive'' in the same sense as the
-     <see topic='@(url set::primitives)'>oset primitives</see>:
-     their logical definitions depend on
+     are ``primitive'' in the same sense as the "
+    (xdoc::seetopic "set::primitives" "oset primitives")
+    ": their logical definitions depend on
      the underlying representation as alists,
      and provide replacements of alist operations
      that respect the `non-map convention'.
@@ -96,13 +100,12 @@
      Perhaps the @('\"SET\"') package for osets
      could be renamed to @('\"OSET\"') at some point,
      for consistency.
-     (A similar issue will arise for a library of obags, i.e. ordered bags,
+     (A similar issue applies to the library of obags, i.e. ordered bags,
      and the @('[books]/coi/bags') library,
      which defines a @('\"BAG\"') package.)")
    (xdoc::p
     "This omap library could become a new @('std/omaps') library,
-     part of <see topic='@(url std)'>STD</see>,
-     parallel to @(tsee set::std/osets).")
+     part of @(csee std), parallel to @(tsee set::std/osets).")
    (xdoc::p
     "Compared to using the built-in @(see acl2::alists) to represent maps,
      omaps are closer to the mathematical notion of maps,
@@ -112,22 +115,22 @@
      The map library in @('[books]/coi/maps/')
      operates on possibly unordered alists.")
    (xdoc::p
-    "Compared to the records in
-     <see topic='@(url acl2::misc/records)'>@('[books]/misc/records.lisp')</see>
-     and @('[books]/coi/records/'),
+    "Compared to the records in "
+    (xdoc::seetopic "acl2::misc/records" "@('[books]/misc/records.lisp')")
+    " and @('[books]/coi/records/'),
      omaps allow any value to be associated to keys,
      without having to exclude @('nil') or some other fixed value.
      Furthermore, as noted in the comments in @('[books]/coi/maps/maps.lisp'),
      the `get' operation on those records
      does not always yield a value smaller than the map.
      On the other hand, theorems about omaps have generally more hypotheses
-     than the theorems about records.")))
+     than the theorems about records."))
+  :default-parent t)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define mapp (x)
   :returns (yes/no booleanp)
-  :parents (omaps)
   :short "Recognize omaps."
   :long
   (xdoc::topstring-p
@@ -153,13 +156,17 @@
   (defrule alistp-when-mapp
     (implies (mapp x)
              (alistp x))
-    :rule-classes (:rewrite :forward-chaining)))
+    :rule-classes (:rewrite :forward-chaining))
+
+  (defruled consp-car-when-non-empty-mapp
+    (implies (and map (mapp map))
+             (consp (car map))))
+)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define mfix ((x mapp))
   :returns (fixed-x mapp)
-  :parents (omaps)
   :short "Fixing function for omaps."
   :long
   (xdoc::topstring-p
@@ -170,13 +177,16 @@
 
   (defrule mfix-when-mapp
     (implies (mapp x)
-             (equal (mfix x) x))))
+             (equal (mfix x) x)))
+
+  (defrule mfix-implies-mapp
+    (implies (mfix x)
+             (mapp x))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define empty ((map mapp))
   :returns (yes/no booleanp)
-  :parents (omaps)
   :short "Check if an omap is empty."
   :long
   (xdoc::topstring-p
@@ -191,14 +201,25 @@
 
   (defrule mfix-when-empty
     (implies (empty x)
-             (equal (mfix x) nil))))
+             (equal (mfix x) nil)))
+
+  (defrule mapp-non-nil-implies-non-empty
+    (implies (and (mapp map) map)
+             (not (empty map))))
+  
+  (defrule acl2-count-head-when-non-empty
+    (implies (not (empty map))
+             (< (+ (acl2-count (car (car map)))
+                   (acl2-count (cdr (car map))))
+                (acl2-count map)))
+    :hints (("Goal" :in-theory (enable mfix))))
+)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define head ((map mapp))
   :guard (not (empty map))
   :returns (mv key val)
-  :parents (omaps)
   :short "Smallest key, and associated value, of a non-empty omap."
   :long
   (xdoc::topstring-p
@@ -248,12 +269,28 @@
     :rule-classes :built-in-clause
     :enable (empty mfix)))
 
+
+(define head-key ((map mapp))
+  :guard (not (empty map))
+  :enabled t
+  (mv-let (key val)
+      (head map)
+    (declare (ignore val))
+    key))
+
+(define head-val ((map mapp))
+  :guard (not (empty map))
+  :enabled t
+  (mv-let (key val)
+      (head map)
+    (declare (ignore key))
+    val))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define tail ((map mapp))
   :guard (not (empty map))
   :returns (map1 mapp :hints (("Goal" :in-theory (enable mfix mapp))))
-  :parents (omaps)
   :short "Rest of a non-empty omap after removing its smallest pair."
   :long
   (xdoc::topstring-p
@@ -282,13 +319,24 @@
     :rule-classes :built-in-clause
     :enable (empty mfix)))
 
+
+(defruled head-tail-order
+  (implies (not (empty (tail X)))
+           (<< (mv-nth 0 (head X))
+               (mv-nth 0 (head (tail X)))))
+  :enable (mapp head tail mfix))
+(defruled head-tail-order-contrapositive
+  (implies (not (<< (mv-nth 0 (head X))
+                    (mv-nth 0 (head (tail X)))))
+           (empty (tail X)))
+  :enable head-tail-order)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define update (key val (map mapp))
   :returns (map1 mapp
                  :hints (("Goal"
                           :in-theory (enable mapp mfix empty head tail))))
-  :parents (omaps)
   :short "Set a key to a value in an omap."
   :long
   (xdoc::topstring
@@ -306,7 +354,6 @@
                    (t (cons (cons key0 val0)
                             (update key val (tail map))))))))
   ///
-
   (defrule update-of-head-and-tail
     (implies (not (empty map))
              (equal (update (mv-nth 0 (head map))
@@ -353,20 +400,26 @@
            nil)
     :enable (update tail mfix))
 
-  (defrule head-key-of-update
-    (equal (mv-nth 0 (head (update key val map)))
-           (cond ((empty map) key)
-                 ((<< key (mv-nth 0 (head map))) key)
-                 (t (mv-nth 0 (head map)))))
+  (defrule head-of-update
+    (equal (head (update key val map))
+           (cond ((empty map) (mv key val))
+                 ((<< (mv-nth 0 (head map)) key) (head map))
+                 (t (mv key val))))
     :enable (update head tail))
 
-  (defrule head-value-of-update
+  (defruled head-key-of-update
+    (equal (mv-nth 0 (head (update key val map)))
+           (cond ((empty map) key)
+                 ((<< (mv-nth 0 (head map)) key) (mv-nth 0 (head map)))
+                 (t key)))
+    :enable (head-of-update))
+
+  (defruled head-value-of-update
     (equal (mv-nth 1 (head (update key val map)))
            (cond ((empty map) val)
-                 ((<< key (mv-nth 0 (head map))) val)
-                 ((equal key (mv-nth 0 (head map))) val)
-                 (t (mv-nth 1 (head map)))))
-    :enable (update head tail))
+                 ((<< (mv-nth 0 (head map)) key) (mv-nth 1 (head map)))
+                 (t val)))
+    :enable (head-of-update))
 
   (defrule tail-of-update
     (equal (tail (update key val map))
@@ -374,13 +427,64 @@
                  ((<< key (mv-nth 0 (head map))) map)
                  ((equal key (mv-nth 0 (head map))) (tail map))
                  (t (update key val (tail map)))))
-    :enable (update tail)))
+    :enable (update tail))
+ )
+
+(defrule head-value-of-update-empty
+  (implies (empty map)
+           (equal (mv-nth 1 (head (update key val map)))
+                  val))
+  :enable (head-value-of-update))
+(defrule head-value-of-update-key-<<
+  (implies (and (not (empty map))
+                (or (<< key (mv-nth 0 (head map)))
+                    (equal key (mv-nth 0 (head map)))))
+           (equal (mv-nth 1 (head (update key val map)))
+                  val))
+  :enable (head-value-of-update))
+
+(defrule head-value-of-update-when-head-key-equal
+  (implies (equal (mv-nth 0 (head (update key val x)))
+                  key)
+           (equal (mv-nth 1 (head (update key val x)))
+                           val))
+  :hints (("Goal" :in-theory (enable acl2::<<-irreflexive head-of-update))))
+
+(defrule head-value-of-update-not-<<
+  (implies (and (not (empty map))
+                (<< (mv-nth 0 (head map)) key))
+           (equal (mv-nth 1 (head (update key val map)))
+                  (mv-nth 1 (head map))))
+  :enable (head-value-of-update))
+
+(defrule tail-of-update-empty
+  (implies (empty map)
+           (equal (tail (update key val map))
+                  nil))
+  :enable (tail-of-update))
+(defrule tail-of-update-<<
+  (implies (and (not (empty map))
+                (<< key (mv-nth 0 (head map))))
+           (equal (tail (update key val map))
+                  map))
+  :enable (tail-of-update))
+(defrule tail-of-update-equal
+  (implies (and (not (empty map))
+                (equal key (mv-nth 0 (head map))))
+           (equal (tail (update key val map))
+                  (tail map)))
+  :enable (tail-of-update))
+(defrule tail-of-update-<<-rev
+  (implies (and (not (empty map))
+                (<< (mv-nth 0 (head map)) key))
+           (equal (tail (update key val map))
+                  (update key val (tail map))))
+  :enable (tail-of-update))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define update* ((new mapp) (old mapp))
   :returns (map mapp)
-  :parents (omaps)
   :short "Update a map with another map."
   :long
   (xdoc::topstring-p
@@ -417,7 +521,6 @@
 
 (define delete (key (map mapp))
   :returns (map1 mapp)
-  :parents (omaps)
   :short "Remove a key, and associated value, from an omap."
   :long
   (xdoc::topstring-p
@@ -441,7 +544,6 @@
 
 (define delete* ((keys set::setp) (map mapp))
   :returns (map1 mapp)
-  :parents (omaps)
   :short "Remove keys, and associated values, from an omap."
   :long
   (xdoc::topstring-p
@@ -466,7 +568,6 @@
 
 (define in (key (map mapp))
   :returns (pair? listp)
-  :parents (omaps)
   :short "Check if a key is in an omap."
   :long
   (xdoc::topstring
@@ -495,6 +596,12 @@
     (implies (in key (tail map))
              (in key map)))
 
+  (defrule acl2-count-in-<-map
+    (implies (not (empty map))
+             (< (acl2-count (in key map))
+                (acl2-count map)))
+    :enable (mv-nth head empty mfix consp-car-when-non-empty-mapp))
+
   (defrule in-of-update
     (equal (in key1 (update key val map))
            (if (equal key1 key)
@@ -506,7 +613,6 @@
 
 (define in* ((keys set::setp) (map mapp))
   :returns (yes/no booleanp)
-  :parents (omaps)
   :short "Check if every key in a non-empty set is in an omap."
   :long
   (xdoc::topstring-p
@@ -531,12 +637,63 @@
     (implies (in* keys map)
              (in* (set::tail keys) map))))
 
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(def-ruleset order-rules
+  '(acl2::<<-irreflexive
+    acl2::<<-transitive
+    acl2::<<-asymmetric
+    acl2::<<-trichotomy
+    acl2::<<-implies-lexorder
+    (:induction update)
+  ;  update-induction-case
+  ;  head-update
+    head-of-update
+    head-key-of-update
+    head-value-of-update
+    head-tail-order
+    head-tail-order-contrapositive))
+
+(defthm weak-update-induction-helper-1
+  (implies (and (mapp M)
+                (not (in key M))
+                (not (equal (mv-nth 0 (head (update key val M))) key)))
+           (equal (head (update key val M))
+                  (head M)))
+  :hints (("Goal" :in-theory (acl2::enable* order-rules))))
+
+(defthm weak-update-induction-helper-2
+  (implies (and (not (in key M))
+                (not (equal (mv-nth 0 (head (update key val M))) key)))
+           (equal (tail (update key val M))
+                  (update key val (tail M))))
+  :hints (("Goal" :in-theory (acl2::enable* order-rules))))
+
+(defthm weak-update-induction-helper-3
+  (implies (and (not (in key M))
+                (equal (mv-nth 0 (head (update key val M))) key))
+           (equal (tail (update key val M))
+                  (mfix M)))
+  :hints(("Goal" :in-theory (acl2::enable* order-rules))))
+
+(defun weak-update-induction (key val M)
+  (declare (xargs :guard (mapp M)))
+  (cond ((empty M) nil)
+        ((in key M) nil)
+        ((equal (head-key (update key val M)) key) nil)
+        (t (list (weak-update-induction key val (tail M))))))
+
+(defthm use-weak-update-induction t
+  :rule-classes ((:induction
+                  :pattern (update key val M)
+                  :scheme (weak-update-induction key val M))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define lookup (key (map mapp))
   :guard (in key map)
   :returns (val)
-  :parents (omaps)
   :short "Value associated to a key in an omap."
   (cdr (in key map))
   ///
@@ -544,14 +701,21 @@
   (defrule lookup-when-empty
     (implies (empty map)
              (not (lookup key map)))
-    :rule-classes (:rewrite :type-prescription)))
+    :rule-classes (:rewrite :type-prescription))
+
+  (defrule acl2-count-lookup-<-map
+    (implies (not (empty map))
+             (< (acl2-count (lookup key map))
+                (acl2-count map)))
+    :hints (("Goal" :in-theory (disable acl2-count-in-<-map)
+             :use acl2-count-in-<-map)))
+)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define lookup* ((keys set::setp) (map mapp))
   :guard (in* keys map)
   :returns (vals set::setp)
-  :parents (omaps)
   :short "Set of values associated to a set of keys in an omap."
   :long
   (xdoc::topstring-p
@@ -580,7 +744,6 @@
 
 (define rlookup (val (map mapp))
   :returns (keys set::setp)
-  :parents (omaps)
   :short "Set of keys to which a value is associated."
   :long
   (xdoc::topstring
@@ -611,7 +774,6 @@
 
 (define rlookup* ((vals set::setp) (map mapp))
   :returns (keys set::setp)
-  :parents (omaps)
   :short "Set of keys to which any value in a set is associated."
   :long
   (xdoc::topstring-p
@@ -637,7 +799,6 @@
 
 (define restrict ((keys set::setp) (map mapp))
   :returns (map1 mapp)
-  :parents (omaps)
   :short "Restrict an omap to a set of keys."
   :long
   (xdoc::topstring-p
@@ -668,7 +829,6 @@
 (define keys ((map mapp))
   :returns (keys set::setp
                  :hints (("Goal" :in-theory (enable mfix mapp set::setp))))
-  :parents (omaps)
   :short "Oset of the keys of an omap."
   (cond ((empty map) nil)
         (t (mv-let (key val)
@@ -683,11 +843,17 @@
     :rule-classes (:rewrite :type-prescription)
     :enable empty))
 
+(defthm keys-of-update
+  (equal (keys (update key val m))
+         (set::insert key (keys m)))
+  ;; This ugly list suggests a need for useful lemmas!
+  :hints (("Goal" :in-theory (enable keys update empty insert head tail mfix mapp set::insert
+                                     set::head set::tail set::empty setp))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define values ((map mapp))
   :returns (vals set::setp)
-  :parents (omaps)
   :short "Oset of the values of an omap."
   (cond ((empty map) nil)
         (t (mv-let (key val)
@@ -707,7 +873,6 @@
 
 (define compatiblep ((map1 mapp) (map2 mapp))
   :returns (yes/no booleanp)
-  :parents (omaps)
   :short "Check if two omaps are compatible, in the sense that
           they map their common keys to the same values."
   :long
@@ -739,7 +904,6 @@
 
 (define submap ((sub mapp) (sup mapp))
   :returns (yes/no booleanp)
-  :parents (omaps)
   :short "Check if an omap is a submap of another omap."
   :long
   (xdoc::topstring
@@ -769,7 +933,6 @@
 
 (define size ((map mapp))
   :returns (size natp)
-  :parents (omaps)
   :short "Size of an omap."
   :long
   (xdoc::topstring
@@ -781,9 +944,9 @@
      into assertions about @(tsee empty) and @(tsee tail);
      the expansion terminates because of the @(tsee syntaxp) restriction.
      These theorems are disabled by default.
-     These are the omap analogous of
-     <see topic='@(url acl2::list-len-const-theorems)'>these theorems</see>
-     for lists."))
+     These are the omap analogous of "
+    (xdoc::seetopic "acl2::list-len-const-theorems" "these theorems")
+    " for lists."))
   (cond ((empty map) 0)
         (t (1+ (size (tail map)))))
   ///
@@ -822,4 +985,26 @@
                          (> (size (tail map))
                             (1- c))))
                 (> (size map) c))
-       :rule-classes nil))))
+       :rule-classes nil)))
+  ;; (defrule size-update
+  ;;   (equal (size (update key val m))
+  ;;          (if (in key m)
+  ;;              (size m)
+  ;;            (1+ (size m)))))
+  )
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define from-lists ((keys true-listp) (vals true-listp))
+  :guard (= (len keys) (len vals))
+  :returns (map mapp)
+  :short "Build an omap from
+          a list of keys and a list of corresponding values."
+  :long
+  (xdoc::topstring
+   (xdoc::p
+    "If there are duplicate keys in the first list,
+     the leftmost key, and the corresponding value,
+     will be in the resulting omap."))
+  (cond ((endp keys) nil)
+        (t (update (car keys) (car vals) (from-lists (cdr keys) (cdr vals))))))
