@@ -10,8 +10,9 @@
 
 (in-package "ACL2")
 
+(include-book "kestrel/std/system/fresh-namep" :dir :system)
 (include-book "kestrel/std/system/pseudo-event-formp" :dir :system)
-(include-book "kestrel/utilities/strings/char-kinds" :dir :system)
+(include-book "kestrel/utilities/strings/nondigit-chars" :dir :system)
 (include-book "std/strings/decimal" :dir :system)
 (include-book "std/util/defval" :dir :system)
 (include-book "system/kestrel" :dir :system)
@@ -303,12 +304,12 @@
        (name-chars-without-end-marker
         (take len-of-name-without-end-marker name-chars))
        (digits-of-index
-        (reverse (str::take-leading-digits (reverse
-                                            name-chars-without-end-marker)))))
+        (reverse (str::take-leading-dec-digit-chars
+                  (reverse name-chars-without-end-marker)))))
     (if digits-of-index
         (b* (((when (eql (car digits-of-index) #\0))
               (mv nil nil nil))
-             (index (str::digit-list-value digits-of-index))
+             (index (str::dec-digit-chars-value digits-of-index))
              (name-chars-without-index-and-end-marker
               (take (- (len name-chars-without-end-marker)
                        (len digits-of-index))
@@ -380,6 +381,25 @@
         (intern (implode name-chars) "ACL2")
       (intern-in-package-of-symbol (implode name-chars) base))))
 
+(define make-numbered-name-list ((bases symbol-listp)
+                                 (indices/wildcards nat-listp)
+                                 (wrld plist-worldp))
+  :guard (= (len bases) (len indices/wildcards))
+  :returns (names symbol-listp)
+  :verify-guards nil
+  :parents (numbered-names)
+  :short "Lift @(tsee make-numbered-name) to lists."
+  (cond ((endp bases) nil)
+        (t (cons (make-numbered-name (car bases)
+                                     (car indices/wildcards)
+                                     wrld)
+                 (make-numbered-name-list (cdr bases)
+                                          (cdr indices/wildcards)
+                                          wrld))))
+  ///
+  (defret len-of-make-numbered-name-list
+    (equal (len names) (len bases))))
+
 (define set-numbered-name-index
   ((name symbolp) (index posp) (wrld plist-worldp))
   :returns (new-name symbolp)
@@ -549,6 +569,38 @@
            (next-numbered-name-aux base (1+ current-index) wrld)
          current-index)))))
 
+(define next-fresh-numbered-names ((bases symbol-listp)
+                                   (index posp)
+                                   (names-to-avoid symbol-listp)
+                                   (wrld plist-worldp))
+  :returns (mv (names "A @(tsee symbol-listp).")
+               (new-names-to-avoid "A @(tsee symbol-listp)."))
+  :mode :program
+  :parents (numbered-names)
+  :short "Add to each of the given bases the lowest index,
+          starting with the given index,
+          such that the resulting names are not already in use."
+  (b* ((names (make-numbered-name-list bases (repeat (len bases) index) wrld))
+       ((when (and (null (fresh-name-listp-msg-weak names 'function wrld))
+                   (not (intersectp-eq names names-to-avoid))))
+        (mv names (append names names-to-avoid))))
+    (next-fresh-numbered-names bases (1+ index) names-to-avoid wrld)))
+
+(define next-fresh-numbered-name ((base symbolp)
+                                  (index posp)
+                                  (names-to-avoid symbol-listp)
+                                  (wrld plist-worldp))
+  :returns (mv (name "A @(tsee symbolp).")
+               (new-names-to-avoid "A @(tsee symbol-listp)."))
+  :mode :program
+  :parents (numbered-names)
+  :short "Specialize @(tsee next-fresh-numbered-names) to a single name."
+  (b* (((mv names names-to-avoid) (next-fresh-numbered-names (list base)
+                                                             index
+                                                             names-to-avoid
+                                                             wrld)))
+    (mv (car names) names-to-avoid)))
+
 (defxdoc global-numbered-name-index
   :parents (numbered-names)
   :short "Global index for numbered names."
