@@ -1,7 +1,7 @@
 ; Utilities for manipulating terms
 ;
 ; Copyright (C) 2008-2011 Eric Smith and Stanford University
-; Copyright (C) 2013-2020 Kestrel Institute
+; Copyright (C) 2013-2021 Kestrel Institute
 ;
 ; License: A 3-clause BSD license. See the file books/3BSD-mod.txt.
 ;
@@ -15,15 +15,25 @@
 
 (include-book "forms") ;for farg1, etc.
 (include-book "kestrel/alists-light/lookup" :dir :system)
-(include-book "substitution")
+(include-book "kestrel/terms-light/sublis-var-simple" :dir :system)
+(include-book "symbol-term-alistp")
+(include-book "expand-lambdas-in-term")
 (include-book "tools/flag" :dir :system)
-(local (include-book "kestrel/lists-light/symbol-listp" :dir :system))
+(local (include-book "kestrel/typed-lists-light/symbol-listp" :dir :system))
 (local (include-book "kestrel/lists-light/member-equal" :dir :system))
 (local (include-book "kestrel/lists-light/add-to-set-equal" :dir :system))
 (local (include-book "kestrel/alists-light/pairlis-dollar" :dir :system))
+(local (include-book "kestrel/lists-light/last" :dir :system))
 
 ;todo: use list fix to combine these into a nice rule?
 
+(defthm pseudo-termp-of-lookup-equal-when-symbol-term-alistp
+  (implies (and (symbol-term-alistp subst)
+;                (assoc-equal term subst)
+                )
+           (pseudo-termp (lookup-equal term subst)))
+  :hints (("Goal" :in-theory (enable symbol-term-alistp))))
+
 (defthm pseudo-term-listp-of-append1
   (implies (true-listp l1)
            (equal (pseudo-term-listp (append l1 l2))
@@ -39,86 +49,6 @@
 
 ;dups (but some of these are now guard verified)
 
-;I guess this works for lambdas too, since they must be complete (i.e., the lambda formals must include all vars that are free in the lambda body)
-;TODO: Compare to all-vars1
-(mutual-recursion
- (defund get-vars-from-term-aux (term acc)
-   (declare (xargs :guard (and (true-listp acc)
-                               (pseudo-termp term))
-                   :verify-guards nil ;;done below
-                   ))
-   (if (atom term)
-       (add-to-set-eq term acc)
-     (let ((fn (ffn-symb term)))
-       (if (eq 'quote fn)
-           acc
-         (get-vars-from-terms-aux (fargs term) acc)))))
-
- (defund get-vars-from-terms-aux (terms acc)
-   (declare (xargs :guard (and (true-listp acc)
-                               (true-listp terms)
-                               (pseudo-term-listp terms))))
-   (if (endp terms)
-       acc
-     (get-vars-from-terms-aux (cdr terms)
-                          (get-vars-from-term-aux (car terms) acc)))))
-
-(make-flag get-vars-from-term-aux)
-
-(defthm-flag-get-vars-from-term-aux
-  (defthm true-listp-of-get-vars-from-term-aux
-    (implies (true-listp acc)
-             (true-listp (get-vars-from-term-aux term acc)))
-    :flag get-vars-from-term-aux)
-  (defthm true-listp-of-get-vars-from-terms-aux
-    (implies (true-listp acc)
-             (true-listp (get-vars-from-terms-aux terms acc)))
-    :flag get-vars-from-terms-aux)
-  :hints (("Goal" :in-theory (enable get-vars-from-term-aux get-vars-from-terms-aux))))
-
-(verify-guards get-vars-from-term-aux)
-
-(defun get-vars-from-term (term)
-  (declare (xargs :guard (pseudo-termp term)))
-  (get-vars-from-term-aux term nil))
-
-(defun get-vars-from-terms (terms)
-  (declare (xargs :guard (pseudo-term-listp terms)))
-  (get-vars-from-terms-aux terms nil))
-
-(defthm-flag-get-vars-from-term-aux
-  (defthm symbol-listp-of-get-vars-from-term-aux
-    (implies (and (pseudo-termp term)
-                  (symbol-listp acc))
-             (symbol-listp (get-vars-from-term-aux term acc)))
-    :flag get-vars-from-term-aux)
-  (defthm symbol-listp-of-get-vars-from-terms-aux
-    (implies (and (pseudo-term-listp terms)
-                  (symbol-listp acc))
-             (symbol-listp (get-vars-from-terms-aux terms acc)))
-    :flag get-vars-from-terms-aux)
-  :hints (("Goal" :in-theory (enable get-vars-from-term-aux get-vars-from-terms-aux))))
-
-(defthm symbol-listp-of-get-vars-from-terms
-  (implies (pseudo-term-listp terms)
-           (symbol-listp (get-vars-from-terms terms)))
-  :hints (("Goal" :in-theory (enable get-vars-from-terms))))
-
-(defthm symbol-listp-of-get-vars-from-term
-  (implies (pseudo-termp term)
-           (symbol-listp (get-vars-from-term term)))
-  :hints (("Goal" :in-theory (enable get-vars-from-term))))
-
-(defthm true-listp-of-get-vars-from-terms
-  (implies (pseudo-term-listp terms)
-           (true-listp (get-vars-from-terms terms)))
-  :hints (("Goal" :in-theory (enable get-vars-from-terms))))
-
-(defthm true-listp-of-get-vars-from-term
-  (implies (pseudo-termp term)
-           (true-listp (get-vars-from-term term)))
-  :hints (("Goal" :in-theory (enable get-vars-from-term))))
-
 ;recognize an alist from pseudo-terms to pseudo-terms
 (defun pseudo-term-alistp (alist)
   (declare (xargs :guard t))
@@ -150,8 +80,8 @@
     (let* ((pair (first alist))
            (key (car pair))
            (val (cdr pair)))
-      (if (or (intersection-eq (get-vars-from-term key) vars)
-              (intersection-eq (get-vars-from-term val) vars)) ;todo: do we want this check?  if not, we can simplify this routine
+      (if (or (intersection-eq (free-vars-in-term key) vars)
+              (intersection-eq (free-vars-in-term val) vars)) ;todo: do we want this check?  if not, we can simplify this routine
           (drop-pairs-that-mention-vars (rest alist) vars) ;drop the pair
         (cons pair (drop-pairs-that-mention-vars (rest alist) vars))))))
 
@@ -218,173 +148,11 @@
   (if (endp terms)
       nil
     (let ((term (first terms)))
-      (if (intersection-eq (get-vars-from-term term) vars)
+      (if (intersection-eq (free-vars-in-term term) vars)
           (drop-terms-that-mention-vars (rest terms) vars) ;drop the term
         (cons term (drop-terms-that-mention-vars (rest terms) vars))))))
 
-(make-flag my-sublis-var)
 
-(defund symbol-term-alistp (x)
-  (declare (xargs :guard t))
-  (cond ((atom x) (eq x nil))
-        (t (and (consp (car x))
-                (symbolp (car (car x)))
-                (pseudo-termp (cdr (car x)))
-                (symbol-term-alistp (cdr x))))))
-
-(defthm symbol-term-alistp-of-cdr
-  (implies (symbol-term-alistp alist)
-           (symbol-term-alistp (cdr alist)))
-  :hints (("Goal" :in-theory (enable symbol-term-alistp))))
-
-(defthm symbolp-of-car-of-car-when-symbol-term-alistp
-  (implies (symbol-term-alistp x)
-           (symbolp (car (car x))))
-  :rule-classes :type-prescription
-  :hints (("Goal" :in-theory (enable symbol-term-alistp))))
-
-(defthm pseudo-termp-of-cdr-of-car-when-symbol-term-alistp-cheap
-  (implies (symbol-term-alistp x)
-           (pseudo-termp (cdr (car x))))
-  :hints (("Goal" :in-theory (enable symbol-term-alistp))))
-
-(defthm symbol-alistp-when-symbol-term-alistp-cheap
-  (implies (symbol-term-alistp x)
-           (symbol-alistp x))
-  :rule-classes ((:rewrite :backchain-limit-lst (0)))
-  :hints (("Goal" :in-theory (enable symbol-term-alistp))))
-
-(defthm symbol-term-alistp-forward-to-symbol-alistp
-  (implies (symbol-term-alistp x)
-           (symbol-alistp x))
-  :rule-classes :forward-chaining)
-
-(defthm consp-of-car-when-symbol-term-alistp-cheap
-  (implies (symbol-term-alistp alist)
-           (equal (consp (car alist))
-                  (consp alist)))
-  :rule-classes ((:rewrite :backchain-limit-lst (0)))
-  :hints (("Goal" :in-theory (enable symbol-term-alistp))))
-
-(defthm iff-of-car-when-symbol-term-alistp-cheap
-  (implies (symbol-term-alistp alist)
-           (iff (car alist)
-                alist))
-  :rule-classes ((:rewrite :backchain-limit-lst (0)))
-  :hints (("Goal" :in-theory (enable symbol-term-alistp))))
-
-(defthm pseudo-termp-of-cdr-of-assoc-equal
-  (implies (and ;(assoc-equal form alist)
-                ;(symbolp form)
-                (symbol-term-alistp alist))
-           (pseudo-termp (cdr (assoc-equal form alist))))
-  :hints (("Goal" :in-theory (enable assoc-equal symbol-term-alistp))))
-
-(defthm pseudo-termp-of-lookup-equal-when-symbol-term-alistp
-  (implies (and (symbol-term-alistp subst)
-;                (assoc-equal term subst)
-                )
-           (pseudo-termp (lookup-equal term subst)))
-  :hints (("Goal" :in-theory (enable symbol-term-alistp))))
-
-(defthm symbol-term-alistp-of-pairlis$
-  (implies (and (symbol-listp syms)
-                (pseudo-term-listp terms))
-           (symbol-term-alistp (pairlis$ syms terms)))
-  :hints (("Goal" :in-theory (enable symbol-term-alistp))))
-
-(defthm symbol-term-alistp-of-pairlis$-alt
-  (implies (equal (len vars) (len terms))
-           (equal (symbol-term-alistp (pairlis$ vars terms))
-                  (and (symbol-listp (true-list-fix vars))
-                       (pseudo-term-listp (true-list-fix terms)))))
-  :hints (("Goal" :in-theory (enable symbol-term-alistp))))
-
-;dup
-(defthm len-of-my-sublis-var-lst
-  (equal (len (my-sublis-var-lst alist l))
-         (len l))
-  :hints (("Goal"
-           :induct (len l)
-           :in-theory (enable my-sublis-var-lst len))))
-
-;TODO change the formals of my-sublis-var and my-sublis-var-lst to be term and terms, not form and l?
-(defthm-flag-my-sublis-var
-  (defthm pseudo-termp-of-my-sublis-var
-    (implies (and (pseudo-termp form)
-                  (symbol-term-alistp alist))
-             (pseudo-termp (my-sublis-var alist form)))
-    :flag my-sublis-var)
-  (defthm pseudo-term-listp-of-my-sublis-var-lst
-    (implies (and (pseudo-term-listp l)
-                  (symbol-term-alistp alist))
-             (pseudo-term-listp (my-sublis-var-lst alist l)))
-    :flag my-sublis-var-lst)
-  :hints (("Goal" :expand ((PSEUDO-TERMP (CONS (CAR FORM)
-                                               (MY-SUBLIS-VAR-LST ALIST (CDR FORM))))))))
-
-;(later we may handle non-pseudo-terms that still include lets).
-;This is similar to remove-lambdas, but we don't use remove-lambdas,
-;because it has to preserve quote normal form. For example:
-;(remove-lambdas '((lambda (x y) (binary-+ x y)) '1 '2)) produces '3
-;whereas we want (binary-+ '1 '2).
-(mutual-recursion
- (defund expand-lambdas-in-term (term)
-   (declare (xargs :measure (acl2-count term)
-                   :guard (pseudo-termp term)
-                   :verify-guards nil)) ;see verify-guards form below
-   (if (variablep term)
-       term
-     (if (quotep term)
-         term
-       ;;it's a function call (maybe a lambda application):
-       (let* ((args (fargs term))
-              (args (expand-lambdas-in-terms args)) ;process the args first
-              (fn (ffn-symb term)))
-         (if (flambdap fn) ;test for lambda application.  term is: ((lambda (formals) body) ... args ...)
-             (let* ((lambda-body (expand-lambdas-in-term (lambda-body fn)))) ;;apply recursively to the lambda body
-               ;; beta-reduce:
-               (my-sublis-var (pairlis$ (lambda-formals fn) args) lambda-body))
-           ;;not a lambda application, so just rebuild the function call:
-           `(,fn ,@args))))))
-
- (defund expand-lambdas-in-terms (terms)
-   (declare (xargs :measure (acl2-count terms)
-                   :guard (pseudo-term-listp terms)))
-   (if (endp terms)
-       nil
-     (cons (expand-lambdas-in-term (first terms))
-           (expand-lambdas-in-terms (rest terms))))))
-
-(make-flag expand-lambdas-in-term)
-
-;TODO: Automate some of this?
-
-(defthm len-of-expand-lambdas-in-terms
-  (equal (len (expand-lambdas-in-terms terms))
-         (len terms))
-  :hints (("Goal" :induct (len terms)
-           :in-theory (enable len expand-lambdas-in-terms))))
-
-(defthm consp-of-expand-lambdas-in-terms
-  (equal (consp (expand-lambdas-in-terms terms))
-         (consp terms))
-  :hints (("Goal" :induct (len terms)
-           :in-theory (enable len expand-lambdas-in-terms))))
-
-(defthm-flag-expand-lambdas-in-term
-  (defthm pseudo-termp-of-expand-lambdas-in-term
-    (implies (pseudo-termp term)
-             (pseudo-termp (expand-lambdas-in-term term)))
-    :flag expand-lambdas-in-term)
-  (defthm pseudo-term-listp-of-expand-lambdas-in-terms
-    (implies (pseudo-term-listp terms)
-             (pseudo-term-listp (expand-lambdas-in-terms terms)))
-    :flag expand-lambdas-in-terms)
-  :hints (("Goal" :in-theory (enable expand-lambdas-in-term
-                                     expand-lambdas-in-terms))))
-
-(verify-guards expand-lambdas-in-term)
 
 ;replace sym with its binding in the alist (if any).  otherwise, return sym
 (defun replace-if-bound (sym alist)
@@ -395,8 +163,7 @@
         (cdr res)
       sym)))
 
-;todo: guard could be (function-renamingp function-renaming)
-;Rename all functions called in TERM that are mapped to new names by ALIST.
+;; Renames function symbols in TERM if indicated by ALIST.  Functions not mapped to anything ALIST are left alone.  Does not support renaming lambdas but does support introducing lambdas.
 ;; (RENAME-FNs  '(foo '1 (baz (foo x y))) '((foo . bar)))
 ;; This is similar to sublis-fn-lst-simple but does not evaluate ground terms for primitives, etc.
 (mutual-recursion
@@ -420,7 +187,7 @@
                     (replace-if-bound fn alist))))
          (fcons-term fn (rename-fns-lst (fargs term) alist))))))
 
-;Rename all functions called in TERMS according that are mapped to new names by ALIST.
+ ;;Renames function symbols in TERMS if indicated by ALIST.
  (defun rename-fns-lst (terms alist)
    (declare (xargs :guard (and (pseudo-term-listp terms)
                                (symbol-alistp alist))))
@@ -500,6 +267,11 @@
            (symbol-listp (get-fns-in-term term)))
   :hints (("Goal" :in-theory (enable get-fns-in-term))))
 
+(defthm true-listp-of-get-fns-in-term
+  (true-listp (get-fns-in-term term))
+  :rule-classes :type-prescription
+  :hints (("Goal" :in-theory (enable get-fns-in-term))))
+
 (defund get-fns-in-terms (terms)
   (declare (xargs :guard (pseudo-term-listp terms)))
   (get-fns-in-terms-aux terms nil))
@@ -581,33 +353,42 @@
 
 ;returns a term with one level of lambda removed...
 ;fixme what if the body is also a lambda?
-(defun beta-reduce (lambda-expr)
+(defund beta-reduce (lambda-expr)
   (declare (xargs :guard (my-lambda-applicationp lambda-expr)))
   (let* ((lambda-part (first lambda-expr))
          (formals (second lambda-part))
          (body (car (last lambda-part))) ;previously we took the third part, but declares sometime intervene?
          (actuals (cdr lambda-expr)))
-    (my-sublis-var (pairlis$ formals actuals)
-                   body)))
+    (sublis-var-simple (pairlis$ formals actuals)
+                       body)))
 
-;todo: what about lambdas?
-(mutual-recursion
- (defun subtermp (a b)
-   (declare (xargs :guard (pseudo-termp b)))
-   (if (atom b)
-       (equal b a)
-     (let ((fn (ffn-symb b)))
-       (if (eq 'quote fn)
-           (equal a b)
-         (or (equal a b)
-             (subterm-of-anyp a (fargs b)))))))
+;move
+(local
+ (defthm car-last-when-length-known
+   (implies (and (equal (len x) k)
+                 (posp k))
+            (equal (car (last x))
+                   (nth (+ -1 k) x)))))
 
- (defun subterm-of-anyp (a b-lst)
-   (declare (xargs :guard (pseudo-term-listp b-lst)))
-   (if (endp b-lst)
-       nil
-     (or (subtermp a (car b-lst))
-         (subterm-of-anyp a (cdr b-lst))))))
+;move
+(local
+ (defthm pseudo-termp-of-car-of-last-of-car
+  (implies (and (pseudo-termp term)
+                (consp term)
+                (consp (cdr (car term))))
+           (pseudo-termp (car (last (car term)))))
+  :hints (("Goal" :in-theory (disable len)
+           :expand ((pseudo-termp term))))))
+
+(defthm pseudo-termp-of-beta-reduce
+  (implies (and (pseudo-termp term)
+                (consp term)
+                (consp (car term)))
+           (pseudo-termp (beta-reduce term)))
+  :hints (("Goal" :expand ((pseudo-termp term)
+                           (nth 2 (car term))
+                           (nth 1 (cdr (car term))))
+           :in-theory (enable beta-reduce nth))))
 
 ;; where should this go?
 ;; Negate TERM by adding or removing a call of not (avoids double negation)
@@ -620,6 +401,12 @@
       (farg1 term) ;negation of (not x) is just x
     `(not ,term)))
 
+;; Kept disabled for speed
+;; Matches the one in std.
+(defthmd pseudo-term-listp-when-symbol-listp
+  (implies (symbol-listp syms)
+           (pseudo-term-listp syms)))
+
 (defthm pseudo-term-listp-when-symbol-listp-cheap
   (implies (symbol-listp x)
            (pseudo-term-listp x))
@@ -645,47 +432,11 @@
                          (length b))))))
   :hints (("Goal" :in-theory (enable pseudo-termp))))
 
+;move
 (defthm pseudo-termp-car-last
   (implies (pseudo-term-listp term)
-           (pseudo-termp (car (last term)))))
-
-(mutual-recursion
- (defun lambda-free-termp (term)
-   (declare (xargs :guard (pseudo-termp term)))
-   (if (variablep term)
-       t
-     (let ((fn (ffn-symb term)))
-       (if (eq 'quote fn)
-           t
-         (and (symbolp fn) ;excludes a lambda application
-              (lambda-free-termsp (fargs term)))))))
- (defun lambda-free-termsp (terms)
-   (declare (xargs :guard (pseudo-term-listp terms)))
-   (if (endp terms)
-       t
-     (and (lambda-free-termp (first terms))
-          (lambda-free-termsp (rest terms))))))
-
-(defthm lambda-free-termp-of-cdr-of-assoc-equal
-  (implies (lambda-free-termsp (strip-cdrs alist))
-           (lambda-free-termp (cdr (assoc-equal form alist)))))
-
-(defthm-flag-my-sublis-var
-  (defthm lambda-free-termp-of-my-sublis-var
-    (implies (and (lambda-free-termp form)
-                  (lambda-free-termsp (strip-cdrs alist)))
-             (lambda-free-termp (my-sublis-var alist form)))
-    :flag my-sublis-var)
-  (defthm lambda-free-termsp-of-my-sublis-var-lst
-    (implies (and (lambda-free-termsp l)
-                  (lambda-free-termsp (strip-cdrs alist)))
-             (lambda-free-termsp (my-sublis-var-lst alist l)))
-    :flag my-sublis-var-lst))
-
-(defthm lambda-free-termsp-of-true-list-fix
-  (equal (lambda-free-termsp (true-list-fix terms))
-         (lambda-free-termsp terms)))
-
+           (pseudo-termp (car (last term))))
+  :hints (("Goal" :in-theory (enable last))))
 
 ;; (thm
 ;;  (implies (natp n)
@@ -700,18 +451,6 @@
            (equal (strip-cdrs (pairlis$ x y))
                   (true-list-fix y))))
 
-(defthm-flag-expand-lambdas-in-term
-  (defthm lambda-free-termp-of-expand-lambdas-in-term
-    (implies (pseudo-termp term)
-             (lambda-free-termp (expand-lambdas-in-term term)))
-    :flag expand-lambdas-in-term)
-  (defthm lambda-free-term-listp-of-expand-lambdas-in-terms
-    (implies (pseudo-term-listp terms)
-             (lambda-free-termsp (expand-lambdas-in-terms terms)))
-    :flag expand-lambdas-in-terms)
-  :hints (("Goal" :in-theory (enable expand-lambdas-in-term
-                                     expand-lambdas-in-terms))))
-
 (defthm symbolp-of-car-of-expand-lambdas-in-term
   (implies (and (symbolp (car term))
                 ;; (consp term)
@@ -795,22 +534,24 @@
   (implies (not (member-equal fn (fns-in-terms (strip-cdrs alist))))
            (not (member-equal fn (fns-in-term (cdr (assoc-equal form alist)))))))
 
-(defthm-flag-my-sublis-var
-  (defthm not-member-equal-of-fns-in-term-of-my-sublis-var
+(defthm-flag-sublis-var-simple
+  (defthm not-member-equal-of-fns-in-term-of-sublis-var-simple
     (implies (and (not (member-equal fn (fns-in-term form)))
                   (not (member-equal fn (fns-in-terms (strip-cdrs alist))))
                   (pseudo-termp form))
-             (not (member-equal fn (fns-in-term (my-sublis-var alist form)))))
-    :flag my-sublis-var)
-  (defthm not-member-equal-of-fns-in-term-of-my-sublis-var-lst
+             (not (member-equal fn (fns-in-term (sublis-var-simple alist form)))))
+    :flag sublis-var-simple)
+  (defthm not-member-equal-of-fns-in-term-of-sublis-var-simple-lst
     (implies (and (not (member-equal fn (fns-in-terms l)))
                   (not (member-equal fn (fns-in-terms (strip-cdrs alist))))
                   (pseudo-term-listp l))
-             (not (member-equal fn (fns-in-terms (my-sublis-var-lst alist l)))))
-    :flag my-sublis-var-lst)
+             (not (member-equal fn (fns-in-terms (sublis-var-simple-lst alist l)))))
+    :flag sublis-var-simple-lst)
   :hints (("Goal" :expand ((FNS-IN-TERM (CONS (CAR FORM)
-                                              (MY-SUBLIS-VAR-LST ALIST (CDR FORM)))))
-           :in-theory (enable fns-in-term))))
+                                              (SUBLIS-VAR-SIMPLE-LST ALIST (CDR FORM)))))
+           :in-theory (enable fns-in-term
+                              sublis-var-simple
+                              sublis-var-simple-lst))))
 
 (defthm-flag-expand-lambdas-in-term
   (defthm not-member-equal-of-fns-in-term-of-expand-lambdas-in-term
@@ -826,3 +567,18 @@
   :hints (("Goal" :do-not '(generalize eliminate-destructors)
            :in-theory (enable expand-lambdas-in-term
                               expand-lambdas-in-terms))))
+
+;; (defthm-flag-expand-lambdas-in-term
+;;   (defthm member-equal-of-fns-in-term-of-expand-lambdas-in-term
+;;     (implies (and (pseudo-termp term)
+;;                   (member-equal fn (fns-in-term term)))
+;;              (member-equal fn (fns-in-term (expand-lambdas-in-term term))))
+;;     :flag expand-lambdas-in-term)
+;;   (defthm member-equal-of-fns-in-terms-of-expand-lambdas-in-terms
+;;     (implies (and (pseudo-term-listp terms)
+;;                   (member-equal fn (fns-in-terms terms)))
+;;              (member-equal fn (fns-in-terms (expand-lambdas-in-terms terms))))
+;;     :flag expand-lambdas-in-terms)
+;;   :hints (("Goal" :do-not '(generalize eliminate-destructors)
+;;            :in-theory (enable expand-lambdas-in-term
+;;                               expand-lambdas-in-terms))))
