@@ -35,6 +35,8 @@
 (include-book "std/strings/decimal" :dir :system)
 (include-book "std/strings/cat" :dir :system)
 
+(local (in-theory (disable w)))
+
 (defsection tempfile
   :parents (oslib)
   :short "Generate a suitable name for a temporary file."
@@ -80,7 +82,11 @@ fail for whatever reason, @('filename') may be @('nil').</p>
 
     (defthm state-p1-of-tempfile-fn
       (implies (force (state-p1 state))
-               (state-p1 (mv-nth 1 (tempfile-fn basename state)))))))
+               (state-p1 (mv-nth 1 (tempfile-fn basename state)))))
+
+    (defthm w-state-of-tempfile-fn
+      (equal (w (mv-nth 1 (tempfile-fn basename state)))
+             (w state)))))
 
 
 
@@ -92,7 +98,7 @@ fail for whatever reason, @('filename') may be @('nil').</p>
                          (or (stringp tempfile)
                              (not tempfile))
                          :rule-classes :type-prescription)
-               (state state-p1 :hyp (force (state-p1 state))))
+               (new-state state-p1 :hyp (force (state-p1 state))))
 
   :parents (tempfile)
   :short "Join together a temp directory, the user name, the PID, and the base
@@ -108,7 +114,11 @@ name to create a temporary filename."
         (mv nil state))
        (filename (str::cat user "-" (str::natstr pid) "-" basename))
        (path     (catpath tempdir filename)))
-    (mv path state)))
+    (mv path state))
+  ///
+  (defret w-state-of-<fn>
+    (equal (w new-state)
+           (w state))))
 
 
 (defun pathname-to-unix (str)
@@ -123,7 +133,7 @@ name to create a temporary filename."
 (define default-tempdir (state)
   :returns (mv (tempdir "Directory to use for temporary files."
                         stringp :rule-classes :type-prescription)
-               (state state-p1 :hyp (force (state-p1 state))))
+               (new-state state-p1 :hyp (force (state-p1 state))))
 
   :parents (tempfile)
   :short "Figure out what directory to use for temporary files."
@@ -137,7 +147,11 @@ Otherwise, we just default to @('/tmp').</p>"
        (tmpdir (or (and (stringp tempdir) tempdir)
                    (and (stringp temp) (pathname-to-unix temp)) ;; ACL2 traffics in unix-style pathnames
                    "/tmp")))
-    (mv tmpdir state)))
+    (mv tmpdir state))
+  ///
+  (defret w-state-of-<fn>
+    (equal (w new-state)
+           (w state))))
 
 
 (define default-tempfile ((basename stringp)
@@ -145,11 +159,15 @@ Otherwise, we just default to @('/tmp').</p>"
   :returns (mv (tempfile (or (stringp tempfile)
                              (not tempfile))
                          :rule-classes :type-prescription)
-               (state state-p1 :hyp (force (state-p1 state))))
+               (new-state state-p1 :hyp (force (state-p1 state))))
   :parents (tempfile)
   :short "Default way to generate temporary file names."
   (b* (((mv dir state) (default-tempdir state)))
-    (default-tempfile-aux dir basename state)))
+    (default-tempfile-aux dir basename state))
+  ///
+  (defret w-state-of-<fn>
+    (equal (w new-state)
+           (w state))))
 
 
 (defattach tempfile-fn default-tempfile)
