@@ -92,18 +92,27 @@
 
 (defconst *blacklisted-apply$-fns*
 
+; Warning: Keep this constant in sync with the value in ACL2 source file
+; apply-prim.lisp.
+
 ; The functions listed here are not safe to apply, primarily because their
 ; behavior differs from their logical definitions.
 
-   '(SYNP                                      ; bad
-     HIDE                                      ; stupid
-     WORMHOLE1                                 ; restricts arguments
-     WORMHOLE-EVAL                             ; restricts arguments
-     SYS-CALL                                  ; bad -- requires trust tag
-     HONS-CLEAR!                               ; bad -- requires trust tag
-     HONS-WASH!                                ; bad -- requires trust tag
-     UNTOUCHABLE-MARKER                        ; bad -- untouchable
-     ))
+; This list should contain every defined built-in function symbol that belongs
+; to *initial-untouchable-fns* or (strip-cars *ttag-fns*) and traffics only in
+; non-stobjs, and we check that property in check-built-in-constants.  These
+; three restrictions need not be enforced here because unless a trust tag is
+; used, then only defined functions that avoid stobjs can have warrants, and
+; the ttag and untouchable restrictions already prevent warrants.
+
+   '(SYNP                                   ; bad
+     WORMHOLE1                              ; restricts arguments
+     WORMHOLE-EVAL                          ; restricts arguments
+     SYS-CALL                               ; bad -- requires trust tag
+     HONS-CLEAR!                            ; bad -- requires trust tag
+     HONS-WASH!                             ; bad -- requires trust tag
+     UNTOUCHABLE-MARKER                     ; bad -- untouchable
+     ASET1-TRUSTED                          ; bad -- untouchable (added 7/2021)
 
 ; At one time we considered disallowing these functions but we now allow them.
 ; We list them here just to document that we considered them and concluded that
@@ -124,6 +133,7 @@
 ;    MEMOIZE-SUMMARY
 ;    CLEAR-MEMOIZE-TABLES
 ;    CLEAR-MEMOIZE-TABLE
+     ))
 
 (defun first-order-like-terms-and-out-arities (world)
 
@@ -507,6 +517,11 @@
 ;                                           (:executable-counterpart break$))))
 ;       :rule-classes ((:meta :trigger-fns (apply$-prim)))))
 
+    (local
+     (defthm hide-is-identity
+       (equal (hide x) x)
+       :hints (("Goal" :expand ((hide x))))))
+
     (defthm apply$-prim-meta-fn-correct
       (equal (apply$-prim-meta-fn-ev term alist)
              (apply$-prim-meta-fn-ev (meta-apply$-prim term)
@@ -538,3 +553,4 @@
       :rule-classes :forward-chaining)))
 
 (in-theory (disable apply$-prim apply$-primp))
+
