@@ -4,7 +4,7 @@
 ;; ACL2.
 
 ;; Cuong Chau <ckcuong@cs.utexas.edu>
-;; April 2019
+;; May 2019
 
 (in-package "ADE")
 
@@ -39,7 +39,6 @@
 (defconst *interl$go-num* (+ *interl$prim-go-num*
                              (* 2 *queue40-l$go-num*)
                              *arb-merge$go-num*))
-(defconst *interl$st-len* 3)
 
 (defun interl$data-ins-len (data-size)
   (declare (xargs :guard (natp data-size)))
@@ -166,9 +165,9 @@
 ;; Constraints on the state of INTERL
 
 (defund interl$st-format (st data-size)
-  (b* ((q40-l0 (get-field *interl$q40-l0* st))
-       (q40-l1 (get-field *interl$q40-l1* st))
-       (arb-merge (get-field *interl$arb-merge* st)))
+  (b* ((q40-l0 (nth *interl$q40-l0* st))
+       (q40-l1 (nth *interl$q40-l1* st))
+       (arb-merge (nth *interl$arb-merge* st)))
     (and (< 0 data-size)
          (queue40-l$st-format q40-l0 data-size)
          (queue40-l$st-format q40-l1 data-size)
@@ -181,9 +180,9 @@
   :rule-classes :forward-chaining)
 
 (defund interl$valid-st (st data-size)
-  (b* ((q40-l0 (get-field *interl$q40-l0* st))
-       (q40-l1 (get-field *interl$q40-l1* st))
-       (arb-merge (get-field *interl$arb-merge* st)))
+  (b* ((q40-l0 (nth *interl$q40-l0* st))
+       (q40-l1 (nth *interl$q40-l1* st))
+       (arb-merge (nth *interl$arb-merge* st)))
     (and (< 0 data-size)
          (queue40-l$valid-st q40-l0 data-size)
          (queue40-l$valid-st q40-l1 data-size)
@@ -249,7 +248,7 @@
 
          (go-in0 (nth 0 go-signals))
 
-         (q40-l0 (get-field *interl$q40-l0* st))
+         (q40-l0 (nth *interl$q40-l0* st))
          (q40-l0-ready-in- (queue40-l$ready-in- q40-l0)))
 
       (joint-act full-in0 q40-l0-ready-in- go-in0)))
@@ -269,7 +268,7 @@
 
          (go-in1 (nth 1 go-signals))
 
-         (q40-l1 (get-field *interl$q40-l1* st))
+         (q40-l1 (nth *interl$q40-l1* st))
          (q40-l1-ready-in- (queue40-l$ready-in- q40-l1)))
 
       (joint-act full-in1 q40-l1-ready-in- go-in1)))
@@ -294,8 +293,8 @@
                                                   *queue40-l$go-num*)
                                                go-signals)))
 
-         (q40-l0 (get-field *interl$q40-l0* st))
-         (q40-l1 (get-field *interl$q40-l1* st))
+         (q40-l0 (nth *interl$q40-l0* st))
+         (q40-l1 (nth *interl$q40-l1* st))
 
          (q40-l0-ready-out (queue40-l$ready-out q40-l0))
          (q40-l0-data-out (queue40-l$data-out q40-l0))
@@ -309,7 +308,7 @@
 
   (defund interl$out-act0 (inputs st data-size)
     (b* ((arb-merge-inputs (interl$arb-merge-inputs inputs st data-size))
-         (arb-merge (get-field *interl$arb-merge* st)))
+         (arb-merge (nth *interl$arb-merge* st)))
       (arb-merge$act0 arb-merge-inputs arb-merge data-size)))
 
   (defthm interl$out-act0-inactive
@@ -322,7 +321,7 @@
 
   (defund interl$out-act1 (inputs st data-size)
     (b* ((arb-merge-inputs (interl$arb-merge-inputs inputs st data-size))
-         (arb-merge (get-field *interl$arb-merge* st)))
+         (arb-merge (nth *interl$arb-merge* st)))
       (arb-merge$act1 arb-merge-inputs arb-merge data-size)))
 
   (defthm interl$out-act1-inactive
@@ -364,7 +363,7 @@
                                   (nthcdr *interl$prim-go-num*
                                           go-signals)))
 
-         (arb-merge (get-field *interl$arb-merge* st))
+         (arb-merge (nth *interl$arb-merge* st))
 
          (arb-merge-inputs (interl$arb-merge-inputs inputs st data-size))
          (out-act0 (arb-merge$act0 arb-merge-inputs arb-merge data-size)))
@@ -386,7 +385,7 @@
                                              *queue40-l$go-num*)
                                           go-signals)))
 
-         (arb-merge (get-field *interl$arb-merge* st))
+         (arb-merge (nth *interl$arb-merge* st))
 
          (arb-merge-inputs (interl$arb-merge-inputs inputs st data-size))
          (out-act1 (arb-merge$act1 arb-merge-inputs arb-merge data-size)))
@@ -398,7 +397,7 @@
 
   (defund interl$data-out (inputs st data-size)
     (b* ((arb-merge-inputs (interl$arb-merge-inputs inputs st data-size))
-         (arb-merge (get-field *interl$arb-merge* st)))
+         (arb-merge (nth *interl$arb-merge* st)))
       (arb-merge$data-out arb-merge-inputs arb-merge data-size)))
 
   (defthm len-interl$data-out-1
@@ -460,9 +459,9 @@
 ;; This function specifies the next state of INTERL.
 
 (defun interl$step (inputs st data-size)
-  (b* ((q40-l0     (get-field *interl$q40-l0* st))
-       (q40-l1     (get-field *interl$q40-l1* st))
-       (arb-merge (get-field *interl$arb-merge* st))
+  (b* ((q40-l0     (nth *interl$q40-l0* st))
+       (q40-l1     (nth *interl$q40-l1* st))
+       (arb-merge (nth *interl$arb-merge* st))
 
        (q40-l0-inputs (interl$q40-l0-inputs inputs st data-size))
        (q40-l1-inputs (interl$q40-l1-inputs inputs st data-size))
@@ -475,10 +474,6 @@
      ;; Joint arb-merge
      (arb-merge$step arb-merge-inputs arb-merge data-size))))
 
-(defthm len-of-interl$step
-  (equal (len (interl$step inputs st data-size))
-         *interl$st-len*))
-
 ;; The state lemma for INTERL
 
 (defthm interl$state
@@ -567,8 +562,7 @@
              (nth *interl$q40-l0* st)
              data-size))
    :hints (("Goal"
-            :in-theory (e/d (get-field
-                             f-and4
+            :in-theory (e/d (f-and4
                              f-and5
                              booleanp-car-of-bv
                              queue40-l$input-format
@@ -594,8 +588,7 @@
              (nth *interl$q40-l1* st)
              data-size))
    :hints (("Goal"
-            :in-theory (e/d (get-field
-                             f-and4
+            :in-theory (e/d (f-and4
                              f-and5
                              booleanp-car-of-bv
                              queue40-l$input-format
@@ -719,11 +712,11 @@
 ;; The extraction functions for INTERL
 
 (defund interl$extract0 (st)
-  (b* ((q40-l0 (get-field *interl$q40-l0* st)))
+  (b* ((q40-l0 (nth *interl$q40-l0* st)))
     (queue40-l$extract q40-l0)))
 
 (defund interl$extract1 (st)
-  (b* ((q40-l1 (get-field *interl$q40-l1* st)))
+  (b* ((q40-l1 (nth *interl$q40-l1* st)))
     (queue40-l$extract q40-l1)))
 
 (defthm interl$extract0-not-empty
@@ -861,8 +854,7 @@
                     (equal (interl$extract1 next-st)
                            (interl$extracted1-step inputs st data-size)))))
     :hints (("Goal"
-             :in-theory (e/d (get-field
-                              queue40-l$extracted-step
+             :in-theory (e/d (queue40-l$extracted-step
                               queue40-l$extracted-step
                               interl$extracted0-step
                               interl$extracted1-step
@@ -885,8 +877,7 @@
            (interl$valid-st (interl$step inputs st data-size)
                             data-size))
   :hints (("Goal"
-           :in-theory (e/d (get-field
-                            interl$valid-st
+           :in-theory (e/d (interl$valid-st
                             interl$step)
                            ()))))
 
@@ -900,8 +891,7 @@
               (equal (interl$data-out inputs st data-size)
                      (queue40-l$data-out (nth *interl$q40-l0* st))))
      :hints (("Goal"
-              :in-theory (enable get-field
-                                 f-and
+              :in-theory (enable f-and
                                  f-and4
                                  f-and5
                                  queue40-l$valid-st=>constraint
@@ -923,8 +913,7 @@
               (equal (interl$data-out inputs st data-size)
                      (queue40-l$data-out (nth *interl$q40-l1* st))))
      :hints (("Goal"
-              :in-theory (enable get-field
-                                 f-and
+              :in-theory (enable f-and
                                  f-and4
                                  f-and5
                                  queue40-l$valid-st=>constraint
@@ -964,8 +953,7 @@
                             (interl$extract0 st))))
     :hints (("Goal"
              :use interl$input-format=>q40-l0$input-format
-             :in-theory (e/d (get-field
-                              interl$valid-st
+             :in-theory (e/d (interl$valid-st
                               interl$extract0)
                              (interl$input-format=>q40-l0$input-format)))))
 
@@ -978,8 +966,7 @@
                             (interl$extract1 st))))
     :hints (("Goal"
              :use interl$input-format=>q40-l1$input-format
-             :in-theory (e/d (get-field
-                              interl$valid-st
+             :in-theory (e/d (interl$valid-st
                               interl$extract1)
                              (interl$input-format=>q40-l1$input-format)))))
   )
