@@ -4,7 +4,7 @@
 ;; ACL2.
 
 ;; Cuong Chau <ckcuong@cs.utexas.edu>
-;; November 2018
+;; May 2019
 
 (in-package "ADE")
 
@@ -35,7 +35,6 @@
 (defconst *queue20-l$prim-go-num* 1)
 (defconst *queue20-l$go-num* (+ *queue20-l$prim-go-num*
                                 (* 2 *queue10-l$go-num*)))
-(defconst *queue20-l$st-len* 2)
 
 (defun queue20-l$data-ins-len (data-size)
   (declare (xargs :guard (natp data-size)))
@@ -131,8 +130,8 @@
 ;; Constraints on the state of QUEUE20-L
 
 (defund queue20-l$st-format (st data-size)
-  (b* ((q10-l0 (get-field *queue20-l$q10-l0* st))
-       (q10-l1 (get-field *queue20-l$q10-l1* st)))
+  (b* ((q10-l0 (nth *queue20-l$q10-l0* st))
+       (q10-l1 (nth *queue20-l$q10-l1* st)))
     (and (queue10-l$st-format q10-l0 data-size)
          (queue10-l$st-format q10-l1 data-size))))
 
@@ -143,8 +142,8 @@
   :rule-classes :forward-chaining)
 
 (defund queue20-l$valid-st (st data-size)
-  (b* ((q10-l0 (get-field *queue20-l$q10-l0* st))
-       (q10-l1 (get-field *queue20-l$q10-l1* st)))
+  (b* ((q10-l0 (nth *queue20-l$q10-l0* st))
+       (q10-l1 (nth *queue20-l$q10-l1* st)))
     (and (queue10-l$valid-st q10-l0 data-size)
          (queue10-l$valid-st q10-l1 data-size))))
 
@@ -203,8 +202,8 @@
          (q10-l0-go-signals (take *queue10-l$go-num*
                                   (nthcdr *queue20-l$prim-go-num* go-signals)))
 
-         (q10-l0 (get-field *queue20-l$q10-l0* st))
-         (q10-l1 (get-field *queue20-l$q10-l1* st))
+         (q10-l0 (nth *queue20-l$q10-l0* st))
+         (q10-l1 (nth *queue20-l$q10-l1* st))
 
          (trans-act (joint-act (queue10-l$ready-out q10-l0)
                                (queue10-l$ready-in- q10-l1)
@@ -225,8 +224,8 @@
                                              *queue10-l$go-num*)
                                           go-signals)))
 
-         (q10-l0 (get-field *queue20-l$q10-l0* st))
-         (q10-l1 (get-field *queue20-l$q10-l1* st))
+         (q10-l0 (nth *queue20-l$q10-l0* st))
+         (q10-l1 (nth *queue20-l$q10-l1* st))
 
          (trans-act (joint-act (queue10-l$ready-out q10-l0)
                                (queue10-l$ready-in- q10-l1)
@@ -239,7 +238,7 @@
   ;; Extract the "ready-in-" signal
 
   (defund queue20-l$ready-in- (st)
-    (b* ((q10-l0 (get-field *queue20-l$q10-l0* st)))
+    (b* ((q10-l0 (nth *queue20-l$q10-l0* st)))
       (queue10-l$ready-in- q10-l0)))
 
   (defthm booleanp-queue20-l$ready-in-
@@ -252,7 +251,7 @@
   ;; Extract the "ready-out" signal
 
   (defund queue20-l$ready-out (st)
-    (b* ((q10-l1 (get-field *queue20-l$q10-l1* st)))
+    (b* ((q10-l1 (nth *queue20-l$q10-l1* st)))
       (queue10-l$ready-out q10-l1)))
 
   (defthm booleanp-queue20-l$ready-out
@@ -265,7 +264,7 @@
   ;; Extract the output data
 
   (defund queue20-l$data-out (st)
-    (b* ((q10-l1 (get-field *queue20-l$q10-l1* st)))
+    (b* ((q10-l1 (nth *queue20-l$q10-l1* st)))
       (queue10-l$data-out q10-l1)))
 
   (defthm len-queue20-l$data-out-1
@@ -321,8 +320,8 @@
 ;; This function specifies the next state of QUEUE20-L.
 
 (defun queue20-l$step (inputs st data-size)
-  (b* ((q10-l0 (get-field *queue20-l$q10-l0* st))
-       (q10-l1 (get-field *queue20-l$q10-l1* st))
+  (b* ((q10-l0 (nth *queue20-l$q10-l0* st))
+       (q10-l1 (nth *queue20-l$q10-l1* st))
 
        (q10-l0-inputs (queue20-l$q10-l0-inputs inputs st data-size))
        (q10-l1-inputs (queue20-l$q10-l1-inputs inputs st data-size)))
@@ -332,10 +331,6 @@
      ;; Q10-L1
      (queue10-l$step q10-l1-inputs q10-l1 data-size))))
 
-(defthm len-of-queue20-l$step
-  (equal (len (queue20-l$step inputs st data-size))
-         *queue20-l$st-len*))
-
 (defthm queue20-l$step-v-threefix-of-data-in-canceled
   (implies
    (and (true-listp data-in)
@@ -420,8 +415,7 @@
              (nth *queue20-l$q10-l0* st)
              data-size))
    :hints (("Goal"
-            :in-theory (e/d (get-field
-                             queue10-l$valid-st=>constraint
+            :in-theory (e/d (queue10-l$valid-st=>constraint
                              queue10-l$input-format
                              queue10-l$in-act
                              queue10-l$out-act
@@ -441,8 +435,7 @@
              (nth *queue20-l$q10-l1* st)
              data-size))
    :hints (("Goal"
-            :in-theory (e/d (get-field
-                             joint-act
+            :in-theory (e/d (joint-act
                              queue10-l$valid-st=>constraint
                              queue10-l$input-format
                              queue10-l$in-act
@@ -455,14 +448,14 @@
                             ())))))
 
 (defthm booleanp-queue20-l$in-act
-  (implies (queue20-l$input-format inputs st data-wisth)
+  (implies (queue20-l$input-format inputs st data-size)
            (booleanp (queue20-l$in-act inputs)))
   :hints (("Goal" :in-theory (enable queue20-l$input-format
                                      queue20-l$in-act)))
   :rule-classes (:rewrite :type-prescription))
 
 (defthm booleanp-queue20-l$out-act
-  (implies (queue20-l$input-format inputs st data-wisth)
+  (implies (queue20-l$input-format inputs st data-size)
            (booleanp (queue20-l$out-act inputs)))
   :hints (("Goal" :in-theory (enable queue20-l$input-format
                                      queue20-l$out-act)))
@@ -478,8 +471,8 @@
 ;; sequence from the current state.
 
 (defund queue20-l$extract (st)
-  (b* ((q10-l0 (get-field *queue20-l$q10-l0* st))
-       (q10-l1 (get-field *queue20-l$q10-l1* st)))
+  (b* ((q10-l0 (nth *queue20-l$q10-l0* st))
+       (q10-l1 (nth *queue20-l$q10-l1* st)))
     (append (queue10-l$extract q10-l0)
             (queue10-l$extract q10-l1))))
 
@@ -547,8 +540,7 @@
      :hints (("Goal"
               :use (:instance queue10-l$valid-st=>constraint
                               (st (nth *queue20-l$q10-l0* st)))
-              :in-theory (enable get-field
-                                 queue10-l$valid-st
+              :in-theory (enable queue10-l$valid-st
                                  queue10-l$data-in
                                  queue10-l$data-out
                                  queue20-l$q10-l1-inputs)))))
@@ -598,8 +590,7 @@
                       (queue20-l$extracted-step inputs st data-size))))
     :hints (("Goal"
              :use queue20-l$input-format=>q10-l0$input-format
-             :in-theory (e/d (get-field
-                              queue10-l$valid-st=>constraint
+             :in-theory (e/d (queue10-l$valid-st=>constraint
                               queue10-l$extracted-step
                               queue20-l$extracted-step
                               queue20-l$input-format
@@ -625,8 +616,7 @@
            (queue20-l$valid-st (queue20-l$step inputs st data-size)
                              data-size))
   :hints (("Goal"
-           :in-theory (e/d (get-field
-                            queue20-l$valid-st
+           :in-theory (e/d (queue20-l$valid-st
                             queue20-l$step)
                            ()))))
 
@@ -651,8 +641,7 @@
     :hints (("Goal"
              :do-not-induct t
              :use queue20-l$input-format=>q10-l1$input-format
-             :in-theory (e/d (get-field
-                              queue10-l$valid-st=>constraint
+             :in-theory (e/d (queue10-l$valid-st=>constraint
                               queue20-l$input-format
                               queue20-l$valid-st
                               queue20-l$extract
