@@ -4,7 +4,7 @@
 ;; ACL2.
 
 ;; Cuong Chau <ckcuong@cs.utexas.edu>
-;; November 2018
+;; May 2019
 
 (in-package "ADE")
 
@@ -35,7 +35,6 @@
 (defconst *queue8-l$prim-go-num* 1)
 (defconst *queue8-l$go-num* (+ *queue8-l$prim-go-num*
                                (* 2 *queue4-l$go-num*)))
-(defconst *queue8-l$st-len* 2)
 
 (defun queue8-l$data-ins-len (data-size)
   (declare (xargs :guard (natp data-size)))
@@ -131,8 +130,8 @@
 ;; Constraints on the state of QUEUE8-L
 
 (defund queue8-l$st-format (st data-size)
-  (b* ((q4-l0 (get-field *queue8-l$q4-l0* st))
-       (q4-l1 (get-field *queue8-l$q4-l1* st)))
+  (b* ((q4-l0 (nth *queue8-l$q4-l0* st))
+       (q4-l1 (nth *queue8-l$q4-l1* st)))
     (and (queue4-l$st-format q4-l0 data-size)
          (queue4-l$st-format q4-l1 data-size))))
 
@@ -143,8 +142,8 @@
   :rule-classes :forward-chaining)
 
 (defund queue8-l$valid-st (st data-size)
-  (b* ((q4-l0 (get-field *queue8-l$q4-l0* st))
-       (q4-l1 (get-field *queue8-l$q4-l1* st)))
+  (b* ((q4-l0 (nth *queue8-l$q4-l0* st))
+       (q4-l1 (nth *queue8-l$q4-l1* st)))
     (and (queue4-l$valid-st q4-l0 data-size)
          (queue4-l$valid-st q4-l1 data-size))))
 
@@ -203,8 +202,8 @@
                                  (nthcdr *queue8-l$prim-go-num*
                                          go-signals)))
 
-         (q4-l0 (get-field *queue8-l$q4-l0* st))
-         (q4-l1 (get-field *queue8-l$q4-l1* st))
+         (q4-l0 (nth *queue8-l$q4-l0* st))
+         (q4-l1 (nth *queue8-l$q4-l1* st))
 
          (trans-act (joint-act (queue4-l$ready-out q4-l0)
                                (queue4-l$ready-in- q4-l1)
@@ -225,8 +224,8 @@
                                             *queue4-l$go-num*)
                                          go-signals)))
 
-         (q4-l0 (get-field *queue8-l$q4-l0* st))
-         (q4-l1 (get-field *queue8-l$q4-l1* st))
+         (q4-l0 (nth *queue8-l$q4-l0* st))
+         (q4-l1 (nth *queue8-l$q4-l1* st))
 
          (trans-act (joint-act (queue4-l$ready-out q4-l0)
                                (queue4-l$ready-in- q4-l1)
@@ -239,7 +238,7 @@
   ;; Extract the "ready-in-" signal
 
   (defund queue8-l$ready-in- (st)
-    (b* ((q4-l0 (get-field *queue8-l$q4-l0* st)))
+    (b* ((q4-l0 (nth *queue8-l$q4-l0* st)))
       (queue4-l$ready-in- q4-l0)))
 
   (defthm booleanp-queue8-l$ready-in-
@@ -252,7 +251,7 @@
   ;; Extract the "ready-out" signal
 
   (defund queue8-l$ready-out (st)
-    (b* ((q4-l1 (get-field *queue8-l$q4-l1* st)))
+    (b* ((q4-l1 (nth *queue8-l$q4-l1* st)))
       (queue4-l$ready-out q4-l1)))
 
   (defthm booleanp-queue8-l$ready-out
@@ -265,7 +264,7 @@
   ;; Extract the output data
 
   (defund queue8-l$data-out (st)
-    (b* ((q4-l1 (get-field *queue8-l$q4-l1* st)))
+    (b* ((q4-l1 (nth *queue8-l$q4-l1* st)))
       (queue4-l$data-out q4-l1)))
 
   (defthm len-queue8-l$data-out-1
@@ -321,8 +320,8 @@
 ;; This function specifies the next state of QUEUE8-L.
 
 (defun queue8-l$step (inputs st data-size)
-  (b* ((q4-l0 (get-field *queue8-l$q4-l0* st))
-       (q4-l1 (get-field *queue8-l$q4-l1* st))
+  (b* ((q4-l0 (nth *queue8-l$q4-l0* st))
+       (q4-l1 (nth *queue8-l$q4-l1* st))
 
        (q4-l0-inputs (queue8-l$q4-l0-inputs inputs st data-size))
        (q4-l1-inputs (queue8-l$q4-l1-inputs inputs st data-size)))
@@ -332,10 +331,6 @@
      ;; Q4-L1
      (queue4-l$step q4-l1-inputs q4-l1 data-size))))
 
-(defthm len-of-queue8-l$step
-  (equal (len (queue8-l$step inputs st data-size))
-         *queue8-l$st-len*))
-
 (defthm queue8-l$step-v-threefix-of-data-in-canceled
   (implies
    (and (true-listp data-in)
@@ -420,8 +415,7 @@
              (nth *queue8-l$q4-l0* st)
              data-size))
    :hints (("Goal"
-            :in-theory (e/d (get-field
-                             queue4-l$valid-st=>constraint
+            :in-theory (e/d (queue4-l$valid-st=>constraint
                              queue4-l$input-format
                              queue4-l$in-act
                              queue4-l$out-act
@@ -441,8 +435,7 @@
              (nth *queue8-l$q4-l1* st)
              data-size))
    :hints (("Goal"
-            :in-theory (e/d (get-field
-                             joint-act
+            :in-theory (e/d (joint-act
                              queue4-l$valid-st=>constraint
                              queue4-l$input-format
                              queue4-l$in-act
@@ -455,14 +448,14 @@
                             ())))))
 
 (defthm booleanp-queue8-l$in-act
-  (implies (queue8-l$input-format inputs st data-wisth)
+  (implies (queue8-l$input-format inputs st data-size)
            (booleanp (queue8-l$in-act inputs)))
   :hints (("Goal" :in-theory (enable queue8-l$input-format
                                      queue8-l$in-act)))
   :rule-classes (:rewrite :type-prescription))
 
 (defthm booleanp-queue8-l$out-act
-  (implies (queue8-l$input-format inputs st data-wisth)
+  (implies (queue8-l$input-format inputs st data-size)
            (booleanp (queue8-l$out-act inputs)))
   :hints (("Goal" :in-theory (enable queue8-l$input-format
                                      queue8-l$out-act)))
@@ -478,8 +471,8 @@
 ;; sequence from the current state.
 
 (defund queue8-l$extract (st)
-  (b* ((q4-l0 (get-field *queue8-l$q4-l0* st))
-       (q4-l1 (get-field *queue8-l$q4-l1* st)))
+  (b* ((q4-l0 (nth *queue8-l$q4-l0* st))
+       (q4-l1 (nth *queue8-l$q4-l1* st)))
     (append (queue4-l$extract q4-l0)
             (queue4-l$extract q4-l1))))
 
@@ -545,8 +538,7 @@
                         data-size)
                        (queue4-l$data-out q4-l0))))
      :hints (("Goal"
-              :in-theory (enable get-field
-                                 queue4-l$valid-st
+              :in-theory (enable queue4-l$valid-st
                                  queue4-l$data-in
                                  queue4-l$data-out
                                  queue8-l$q4-l1-inputs)))))
@@ -596,8 +588,7 @@
                       (queue8-l$extracted-step inputs st data-size))))
     :hints (("Goal"
              :use queue8-l$input-format=>q4-l0$input-format
-             :in-theory (e/d (get-field
-                              queue4-l$valid-st=>constraint
+             :in-theory (e/d (queue4-l$valid-st=>constraint
                               queue4-l$extracted-step
                               queue8-l$extracted-step
                               queue8-l$input-format
@@ -623,8 +614,7 @@
            (queue8-l$valid-st (queue8-l$step inputs st data-size)
                             data-size))
   :hints (("Goal"
-           :in-theory (e/d (get-field
-                            queue8-l$valid-st
+           :in-theory (e/d (queue8-l$valid-st
                             queue8-l$step)
                            ()))))
 
@@ -649,8 +639,7 @@
     :hints (("Goal"
              :do-not-induct t
              :use queue8-l$input-format=>q4-l1$input-format
-             :in-theory (e/d (get-field
-                              queue4-l$valid-st=>constraint
+             :in-theory (e/d (queue4-l$valid-st=>constraint
                               queue8-l$input-format
                               queue8-l$valid-st
                               queue8-l$extract
