@@ -20,7 +20,7 @@
                   :measure (nfix (+ a b))))
   (cond ((zp a) (nfix b))
         ((zp b) (nfix a))
-        ((equal a b) (nfix a))
+        ((equal a b) a)
         ((< a b)
          (gcd-alg a (- b a)))
         (t (gcd-alg (- a b) b))))
@@ -30,6 +30,10 @@
            (< 0 (gcd-alg a b)))
   :rule-classes :linear)
 
+(defthmd gcd-alg-commutative
+  (equal (gcd-alg a b) (gcd-alg b a))
+  :rule-classes ((:rewrite :loop-stopper ((a b)))))
+
 (defthm gcd-alg-is-COMMON-divisor
   (implies (and (natp a)
                 (natp b))
