@@ -4,7 +4,7 @@
 ;; ACL2.
 
 ;; Cuong Chau <ckcuong@cs.utexas.edu>
-;; February 2019
+;; May 2019
 
 (in-package "ADE")
 
@@ -56,7 +56,6 @@
 (defconst *serial-sub$go-num* (+ *serial-sub$prim-go-num*
                                  *piso2-sreg$go-num*
                                  *sipo-sreg$go-num*))
-(defconst *serial-sub$st-len* 8)
 
 (defun serial-sub$data-ins-len (data-size)
   (declare (xargs :guard (natp data-size)))
@@ -204,8 +203,8 @@
 ;; Constraints on the state of SERIAL-SUB
 
 (defund serial-sub$st-format (st data-size cnt-size)
-  (b* ((piso2 (get-field *serial-sub$piso2* st))
-       (sipo (get-field *serial-sub$sipo* st)))
+  (b* ((piso2 (nth *serial-sub$piso2* st))
+       (sipo (nth *serial-sub$sipo* st)))
     (and (piso2-sreg$st-format piso2 data-size cnt-size)
          (sipo-sreg$st-format sipo data-size cnt-size))))
 
@@ -218,14 +217,14 @@
   :rule-classes :forward-chaining)
 
 (defund serial-sub$valid-st (st data-size cnt-size)
-  (b* ((a (get-field *serial-sub$a* st))
-       (b (get-field *serial-sub$b* st))
-       (ci (get-field *serial-sub$ci* st))
-       (s (get-field *serial-sub$s* st))
-       (co (get-field *serial-sub$co* st))
-       (done (get-field *serial-sub$done* st))
-       (piso2 (get-field *serial-sub$piso2* st))
-       (sipo (get-field *serial-sub$sipo* st)))
+  (b* ((a (nth *serial-sub$a* st))
+       (b (nth *serial-sub$b* st))
+       (ci (nth *serial-sub$ci* st))
+       (s (nth *serial-sub$s* st))
+       (co (nth *serial-sub$co* st))
+       (done (nth *serial-sub$done* st))
+       (piso2 (nth *serial-sub$piso2* st))
+       (sipo (nth *serial-sub$sipo* st)))
     (and (link1$valid-st a)
          (link1$valid-st b)
          (link1$valid-st ci)
@@ -300,10 +299,10 @@
                                   (nthcdr *serial-sub$prim-go-num*
                                           go-signals)))
 
-         (a (get-field *serial-sub$a* st))
-         (a.s (get-field *link1$s* a))
-         (b (get-field *serial-sub$b* st))
-         (b.s (get-field *link1$s* b)))
+         (a (nth *serial-sub$a* st))
+         (a.s (nth *link1$s* a))
+         (b (nth *serial-sub$b* st))
+         (b.s (nth *link1$s* b)))
 
       (list* full-in0 (f-buf (car a.s)) (f-buf (car b.s))
              (append data0-in data1-in piso2-go-signals))))
@@ -319,11 +318,11 @@
                                            *piso2-sreg$go-num*)
                                         go-signals)))
 
-         (s (get-field *serial-sub$s* st))
-         (s.s (get-field *link1$s* s))
-         (s.d (get-field *link1$d* s))
-         (done (get-field *serial-sub$done* st))
-         (done.s (get-field *link1$s* done))
+         (s (nth *serial-sub$s* st))
+         (s.s (nth *link1$s* s))
+         (s.d (nth *link1$d* s))
+         (done (nth *serial-sub$done* st))
+         (done.s (nth *link1$s* done))
 
          (sipo-full-in (f-and (car s.s) (f-not (car done.s)))))
 
@@ -334,7 +333,7 @@
 
   (defund serial-sub$in-act (inputs st data-size)
     (b* ((piso2-inputs (serial-sub$piso2-inputs inputs st data-size))
-         (piso2 (get-field *serial-sub$piso2* st)))
+         (piso2 (nth *serial-sub$piso2* st)))
       (piso2-sreg$in-act piso2-inputs piso2 data-size)))
 
   (defthm serial-sub$in-act-inactive
@@ -347,7 +346,7 @@
 
   (defund serial-sub$out-act (inputs st data-size)
     (b* ((sipo-inputs (serial-sub$sipo-inputs inputs st data-size))
-         (sipo (get-field *serial-sub$sipo* st)))
+         (sipo (nth *serial-sub$sipo* st)))
       (sipo-sreg$out-act sipo-inputs sipo)))
 
   (defthm serial-sub$out-act-inactive
@@ -359,7 +358,7 @@
   ;; Extract the output data
 
   (defund serial-sub$data-out (st)
-    (b* ((sipo (get-field *serial-sub$sipo* st)))
+    (b* ((sipo (nth *serial-sub$sipo* st)))
       (sipo-sreg$data-out sipo)))
 
   (defthm len-serial-sub$data-out-1
@@ -432,29 +431,29 @@
        (go-sub (nth 0 go-signals))
        (go-c-buf (nth 1 go-signals))
 
-       (a (get-field *serial-sub$a* st))
-       (b (get-field *serial-sub$b* st))
-       (ci (get-field *serial-sub$ci* st))
-       (s (get-field *serial-sub$s* st))
-       (co (get-field *serial-sub$co* st))
-       (done (get-field *serial-sub$done* st))
-       (piso2 (get-field *serial-sub$piso2* st))
-       (sipo (get-field *serial-sub$sipo* st))
+       (a (nth *serial-sub$a* st))
+       (b (nth *serial-sub$b* st))
+       (ci (nth *serial-sub$ci* st))
+       (s (nth *serial-sub$s* st))
+       (co (nth *serial-sub$co* st))
+       (done (nth *serial-sub$done* st))
+       (piso2 (nth *serial-sub$piso2* st))
+       (sipo (nth *serial-sub$sipo* st))
 
        (piso2-inputs (serial-sub$piso2-inputs inputs st data-size))
        (sipo-inputs (serial-sub$sipo-inputs inputs st data-size))
 
-       (a.s (get-field *link1$s* a))
-       (a.d (get-field *link1$d* a))
-       (b.s (get-field *link1$s* b))
-       (b.d (get-field *link1$d* b))
-       (ci.s (get-field *link1$s* ci))
-       (ci.d (get-field *link1$d* ci))
-       (s.s (get-field *link1$s* s))
-       (co.s (get-field *link1$s* co))
-       (co.d (get-field *link1$d* co))
-       (done.s (get-field *link1$s* done))
-       (done.d (get-field *link1$d* done))
+       (a.s (nth *link1$s* a))
+       (a.d (nth *link1$d* a))
+       (b.s (nth *link1$s* b))
+       (b.d (nth *link1$d* b))
+       (ci.s (nth *link1$s* ci))
+       (ci.d (nth *link1$d* ci))
+       (s.s (nth *link1$s* s))
+       (co.s (nth *link1$s* co))
+       (co.d (nth *link1$d* co))
+       (done.s (nth *link1$s* done))
+       (done.d (nth *link1$d* done))
 
        (piso2-out0-act
         (piso2-sreg$out0-act piso2-inputs piso2 data-size))
@@ -501,10 +500,6 @@
      ;; Joint SIPO
      (sipo-sreg$step sipo-inputs sipo data-size cnt-size))))
 
-(defthm len-of-serial-sub$step
-  (equal (len (serial-sub$step inputs st data-size cnt-size))
-         *serial-sub$st-len*))
-
 ;; The state lemma for SERIAL-SUB
 
 (defthm serial-sub$state
@@ -657,23 +652,23 @@
 ;; output sequence from the current state.
 
 (defund serial-sub$extract (st data-size)
-  (b* ((a (get-field *serial-sub$a* st))
-       (b (get-field *serial-sub$b* st))
-       (ci (get-field *serial-sub$ci* st))
-       (s (get-field *serial-sub$s* st))
-       (co (get-field *serial-sub$co* st))
-       (piso2 (get-field *serial-sub$piso2* st))
-       (sipo (get-field *serial-sub$sipo* st))
+  (b* ((a (nth *serial-sub$a* st))
+       (b (nth *serial-sub$b* st))
+       (ci (nth *serial-sub$ci* st))
+       (s (nth *serial-sub$s* st))
+       (co (nth *serial-sub$co* st))
+       (piso2 (nth *serial-sub$piso2* st))
+       (sipo (nth *serial-sub$sipo* st))
 
-       (a.s (get-field *link1$s* a))
-       (a.d (get-field *link1$d* a))
-       (b.s (get-field *link1$s* b))
-       (b.d (get-field *link1$d* b))
-       (ci.s (get-field *link1$s* ci))
-       (ci.d (get-field *link1$d* ci))
-       (s.s (get-field *link1$s* s))
-       (s.d (get-field *link1$d* s))
-       (co.d (get-field *link1$d* co))
+       (a.s (nth *link1$s* a))
+       (a.d (nth *link1$d* a))
+       (b.s (nth *link1$s* b))
+       (b.d (nth *link1$d* b))
+       (ci.s (nth *link1$s* ci))
+       (ci.d (nth *link1$d* ci))
+       (s.s (nth *link1$s* s))
+       (s.d (nth *link1$d* s))
+       (co.d (nth *link1$d* co))
 
        (a.valid-d (if (fullp a.s) a.d nil))
        (b.valid-d (if (fullp b.s) b.d nil))
@@ -715,26 +710,26 @@
 
 (progn
   (defund serial-sub$inv (st data-size)
-    (b* ((a (get-field *serial-sub$a* st))
-         (b (get-field *serial-sub$b* st))
-         (ci (get-field *serial-sub$ci* st))
-         (s (get-field *serial-sub$s* st))
-         (co (get-field *serial-sub$co* st))
-         (done (get-field *serial-sub$done* st))
-         (piso2 (get-field *serial-sub$piso2* st))
-         (sipo (get-field *serial-sub$sipo* st))
+    (b* ((a (nth *serial-sub$a* st))
+         (b (nth *serial-sub$b* st))
+         (ci (nth *serial-sub$ci* st))
+         (s (nth *serial-sub$s* st))
+         (co (nth *serial-sub$co* st))
+         (done (nth *serial-sub$done* st))
+         (piso2 (nth *serial-sub$piso2* st))
+         (sipo (nth *serial-sub$sipo* st))
 
-         (a.s (get-field *link1$s* a))
-         (a.d (get-field *link1$d* a))
-         (b.s (get-field *link1$s* b))
-         (b.d (get-field *link1$d* b))
-         (ci.s (get-field *link1$s* ci))
-         (ci.d (get-field *link1$d* ci))
-         (s.s (get-field *link1$s* s))
-         (s.d (get-field *link1$d* s))
-         (co.s (get-field *link1$s* co))
-         (done.s (get-field *link1$s* done))
-         (done.d (get-field *link1$d* done))
+         (a.s (nth *link1$s* a))
+         (a.d (nth *link1$d* a))
+         (b.s (nth *link1$s* b))
+         (b.d (nth *link1$d* b))
+         (ci.s (nth *link1$s* ci))
+         (ci.d (nth *link1$d* ci))
+         (s.s (nth *link1$s* s))
+         (s.d (nth *link1$d* s))
+         (co.s (nth *link1$s* co))
+         (done.s (nth *link1$s* done))
+         (done.d (nth *link1$d* done))
 
          (a.valid-d (if (fullp a.s) a.d nil))
          (b.valid-d (if (fullp b.s) b.d nil))
@@ -793,8 +788,7 @@
                 (not (piso2-sreg$out0-act piso2-inputs
                                                     piso2
                                                     data-size))))
-     :hints (("Goal" :in-theory (enable get-field
-                                        serial-sub$piso2-inputs)))))
+     :hints (("Goal" :in-theory (enable serial-sub$piso2-inputs)))))
 
   (local
    (defthm serial-sub$piso2-out1-act-inactive
@@ -806,8 +800,7 @@
                 (not (piso2-sreg$out1-act piso2-inputs
                                                     piso2
                                                     data-size))))
-     :hints (("Goal" :in-theory (enable get-field
-                                        serial-sub$piso2-inputs)))))
+     :hints (("Goal" :in-theory (enable serial-sub$piso2-inputs)))))
 
   (local
    (defthm serial-sub$sipo-in-act-inactive
@@ -820,8 +813,7 @@
        (implies (or (emptyp s.s)
                     (fullp done.s))
                 (not (sipo-sreg$in-act sipo-inputs sipo))))
-     :hints (("Goal" :in-theory (enable get-field
-                                        serial-sub$sipo-inputs)))))
+     :hints (("Goal" :in-theory (enable serial-sub$sipo-inputs)))))
 
   (local
    (defthm v-to-nat-of-v-zp
@@ -864,8 +856,7 @@
       :hints (("Goal"
                :use (serial-sub$input-format=>piso2$input-format
                      serial-sub$input-format=>sipo$input-format)
-               :in-theory (e/d (get-field
-                                f-sr
+               :in-theory (e/d (f-sr
                                 pos-len=>cons
                                 sipo-sreg$valid-st=>constraint
                                 piso2-sreg$extracted0-step
@@ -927,8 +918,8 @@
 
   (local
    (defthm serial-sub-aux-3
-     (b* ((s (get-field *serial-sub$s* st))
-          (s.d (get-field *link1$d* s))
+     (b* ((s (nth *serial-sub$s* st))
+          (s.d (nth *link1$d* s))
           (sipo-inputs (serial-sub$sipo-inputs inputs st data-size)))
        (equal (sipo-sreg$bit-in sipo-inputs)
               (f-buf (car s.d))))
@@ -967,8 +958,7 @@
     :hints (("Goal"
              :use (serial-sub$input-format=>piso2$input-format
                    serial-sub$input-format=>sipo$input-format)
-             :in-theory (e/d (get-field
-                              f-sr
+             :in-theory (e/d (f-sr
                               list-rewrite-1
                               consp-is-pos-len
                               v-adder
@@ -1009,8 +999,7 @@
   :hints (("Goal"
            :use (serial-sub$input-format=>piso2$input-format
                  serial-sub$input-format=>sipo$input-format)
-           :in-theory (e/d (get-field
-                            f-sr
+           :in-theory (e/d (f-sr
                             sipo-sreg$valid-st=>constraint
                             serial-sub$valid-st
                             serial-sub$step)
@@ -1028,8 +1017,7 @@
                           (serial-sub$extract st data-size))))
   :hints (("Goal"
            :do-not-induct t
-           :in-theory (e/d (get-field
-                            serial-sub$valid-st
+           :in-theory (e/d (serial-sub$valid-st
                             serial-sub$inv
                             serial-sub$extract
                             serial-sub$out-act
