@@ -17,20 +17,20 @@
      ((mv errmsg & extra-args) (parse-ls-opts argv))
      ;; Parsing error.
      ((when errmsg)
-      (mv (good-bye 1) fat32-in-memory state))
+      (mv (good-bye 1) fat32$c state))
      ((mv & disk-path state)
       (getenv$ "DISK" state))
      ((mv & prefix state)
       (getenv$ "LS_PREFIX" state))
      ((mv & output-path state)
       (getenv$ "LS_OUTPUT" state))
-     ((mv fat32-in-memory &)
+     ((mv fat32$c &)
       (disk-image-to-lofat
-       fat32-in-memory disk-path state))
-     ((mv fat32-in-memory ls-list exit-status)
-      (ls-2 fat32-in-memory state extra-args disk-path))
+       fat32$c disk-path state))
+     ((mv fat32$c ls-list exit-status)
+      (ls-2 fat32$c state extra-args disk-path))
      ((mv channel state)
       (open-output-channel output-path :character state))
      (state (ls-output prefix ls-list channel state))
      (state (close-output-channel channel state)))
-  (mv (good-bye exit-status) fat32-in-memory state))
+  (mv (good-bye exit-status) fat32$c state))
