@@ -5,7 +5,7 @@
 ;; See the README for historical information.
 
 ;; Cuong Chau <ckcuong@cs.utexas.edu>
-;; October 2016
+;; February 2021
 
 (in-package "FM9001")
 
@@ -18,12 +18,6 @@
 ; The atoms of the DE description language are symbols.  We define
 ; functions to aid their manipulation.
 
-;; (defthm alistp-symbol-listp-symbol-alistp-are-true-listp
-;;   (implies (or (alistp        lst)
-;; 	       (symbol-listp  lst)
-;; 	       (symbol-alistp lst))
-;; 	   (true-listp lst)))
-
 (defun true-list-alistp (x)
   (declare (xargs :guard t))
   (cond ((atom x) (eq x nil))
@@ -396,7 +390,7 @@
 
 (encapsulate
   ()
-  
+
   (local
    (defthm subsetp-cons
      (implies (subsetp x y)
@@ -498,11 +492,11 @@
   (and
    (implies (and (consp tree)
                  (equal size (tree-size tree)))
-            (equal (- size (tree-size (car tree))) 
+            (equal (- size (tree-size (car tree)))
                    (tree-size (cdr tree))))
    (implies (and (consp tree)
                  (equal size (tree-size tree)))
-            (equal (- size (tree-size (cdr tree))) 
+            (equal (- size (tree-size (cdr tree)))
                    (tree-size (car tree))))))
 
 (defthmd make-list-append-tree-crock
@@ -545,9 +539,9 @@
 (defun make-tree (n)
   (declare (xargs :guard (natp n)))
   (if (zp n)
-      0
+      nil
     (if (equal n 1)
-        0
+        nil
       (cons (make-tree (floor n 2))
             (make-tree (- n (floor n 2)))))))
 
@@ -663,17 +657,25 @@
             (not (equal x y)))
    :hints (("Goal" :in-theory (enable prefixp)))))
 
+;; Mihir M. mod: disabled a newly added lemma.
 (defthm prefixp-append-relation-1
   (implies (and (not (prefixp x y))
                 (not (prefixp y x)))
            (and (not (prefixp (append x a) (append y b)))
                 (not (prefixp (append y b) (append x a)))))
-  :hints (("Goal" :in-theory (enable prefixp))))
+  :hints
+  (("goal" :in-theory (e/d (prefixp)
+                           ((:rewrite acl2::prefixp-of-append-arg1)
+                            (:rewrite acl2::prefixp-of-append-arg2))))))
 
-(defthm prefixp-append-relation-2
- (implies (not (prefixp x y))
-          (not (prefixp (append x a) y)))
- :hints (("Goal" :in-theory (enable prefixp))))
+(defthm
+  prefixp-append-relation-2
+  (implies (not (prefixp x y))
+           (not (prefixp (append x a) y)))
+  :hints
+  (("goal" :in-theory (e/d (prefixp)
+                           ((:rewrite acl2::prefixp-of-append-arg1)
+                            (:rewrite acl2::prefixp-of-append-arg2))))))
 
 (local
  (defthm istrprefixp-prefixp-explode-relation
@@ -809,5 +811,3 @@
                     (sis s i n)))))
 
 (in-theory (disable sis))
-
-
