@@ -15,6 +15,7 @@
      (include-book "quadratic-reciprocity/euler")
      (include-book "sidekick/server")
      (include-book "x86isa/doc")
+     (include-book "execloader/top")
      (include-book "farray/farray")
      (include-book "sat/proof-checker-itp13/top")
      (include-book "sat/proof-checker-array/top")
@@ -23,9 +24,14 @@
      (include-book "rp-rewriter/top")
      (include-book "rp-rewriter/lib/mult/doc")
      (include-book "rp-rewriter/lib/mult2/doc")
-     
+     (include-book "rp-rewriter/lib/mult3/doc")
+
+     (include-book "smtlink/doc")
      (ifdef "OS_HAS_SMTLINK"
-            (include-book "smtlink/doc")
+            (include-book "smtlink/examples/examples")
+            :endif)
+     (ifdef "OS_HAS_SMTLINK"
+            (include-book "smtlink/examples/ringosc")
             :endif)
 
 
@@ -729,4 +735,3 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.</li>
 
 </ul>")))
-
