@@ -329,8 +329,9 @@
      merge-adder-b+
      adder-sum
      sum
-     resolve-adder-and-order
-     resolve-adder-sum-order)))
+     ;;resolve-adder-and-order
+     ;;resolve-adder-sum-order
+     )))
 
 
 (local
@@ -631,7 +632,7 @@
             :in-theory (e/d (resolve-adder-sum-order
                              resolve-adder-sum-order-rec) ())))))
 
-(defthm resolve-adder-and-order-is-valid-rp-meta-rulep
+#|(defthm resolve-adder-and-order-is-valid-rp-meta-rulep
   (implies (and (adder-rule-formula-checks state)
                 (rp-evl-meta-extract-global-facts :state state))
            (let ((rule (make rp-meta-rule-rec
@@ -647,9 +648,9 @@
                            (RP-TERMP
                             
                             
-                            VALID-SC)))))
+                            VALID-SC)))))||#
 
-(defthm resolve-adder-sum-order-is-valid-rp-meta-rulep
+#|(defthm resolve-adder-sum-order-is-valid-rp-meta-rulep
   (implies (and (adder-rule-formula-checks state)
                 (rp-evl-meta-extract-global-facts :state state))
            (let ((rule (make rp-meta-rule-rec
@@ -665,9 +666,9 @@
                            (RP-TERMP
                             
                             
-                            VALID-SC)))))
+                            VALID-SC)))))||#
 
-(rp::add-meta-rules
+#|(rp::add-meta-rules
  adder-rule-formula-checks
  (list
   (make rp-meta-rule-rec
@@ -679,4 +680,18 @@
         :fnc 'resolve-adder-sum-order
         :trig-fnc 'merge-adder-b+
         :dont-rw t
-        :valid-syntax t)))
+        :valid-syntax t)))||#
+
+
+(rp::add-meta-rule
+ :meta-fnc resolve-adder-and-order
+ :trig-fnc merge-adder-and
+ :valid-syntaxp t
+ :formula-checks adder-rule-formula-checks ;
+ :returns (mv term dont-rw))
+(rp::add-meta-rule
+ :meta-fnc resolve-adder-sum-order
+ :trig-fnc merge-adder-b+
+ :valid-syntaxp t
+ :formula-checks adder-rule-formula-checks
+ :returns (mv term dont-rw))
