@@ -47,6 +47,8 @@
 
 (include-book "projects/rp-rewriter/top" :dir :system)
 
+(include-book "fnc-defs")
+
 ;(include-book "mult-defs")
 
 (local
@@ -1113,3 +1115,265 @@
                             mult-byrow-spec-is-mult-simple-spec
                             mult-bycol-spec-is-mult-byrow-spec)
                            ()))))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Plus to 2vec-adder
+
+(define 2vec-adder ((x integerp)
+                    (y integerp)
+                    (carry-in integerp)
+                    (size natp))
+  (if (zp size)
+      0
+    (let ((sum (list (loghead 1 x)
+                     (loghead 1 y)
+                     carry-in)))
+      (logapp
+       1
+       (s-spec sum)
+       (2vec-adder (ash x -1)
+                   (ash y -1)
+                   (c-spec sum)
+                   (1- size))))))
+
+(add-rp-rule ACL2::LOGHEAD-TYPE)
+
+(def-rp-rule integerp-of-+
+  (implies (and (integerp x)
+                (integerp y))
+           (integerp (+ x y))))
+
+(encapsulate
+  nil
+
+  (local
+   (include-book "lemmas"))
+
+  (local
+   (defthmd equal-of-loghead-logapp-1-lemma
+     (implies (not (zp size))
+              (equal (logapp 1 (bit-of x 0) (loghead (1- size) (ash x -1)))
+                     (loghead size x)))
+     :hints (("Goal"
+              :in-theory (e/d* (bitops::ihsext-inductions
+                                bit-of
+                                bitops::ihsext-recursive-redefs)
+                               ())))))
+
+  (local
+   (defthm equal-of-loghead-logapp-1
+     (implies (not (zp size))
+              (equal (equal (loghead size x)
+                            (logapp 1 a b))
+                     (equal (logapp 1 (bit-of x 0) (loghead (1- size) (ash x -1)))
+                            (logapp 1 a b))))
+     :hints (("Goal"
+              :use ((:instance equal-of-loghead-logapp-1-lemma))
+              :in-theory (e/d* ()
+                               ())))))
+
+  (local
+   (defthm to-m2
+     (equal (ACL2::BOOL->BIT (LOGBITP 0 X))
+            (m2 x))
+     :hints (("Goal"
+              :in-theory (e/d (LOGBITP
+                               floor
+                               ifix
+                               oddp
+                               evenp
+                               mod
+                               m2
+                               ACL2::BOOL->BIT
+                               (:REWRITE ACL2::|(* (if a b c) x)|)
+                               (:REWRITE ACL2::|(* 1 x)|)
+                               (:REWRITE ACL2::|(* y x)|)
+                               (:REWRITE ACL2::|(equal (if a b c) x)|)
+                               (:REWRITE ACL2::|(mod (if a b c) x)|)
+                               (:REWRITE ACL2::|(mod x 2)| . 1)
+                               (:REWRITE ACL2::INTEGERP==>DENOMINATOR-=-1)
+                               (:REWRITE ACL2::INTEGERP==>NUMERATOR-=-X))
+                              (mod2-is-m2
+                               +-is-sum
+                               floor2-if-f2))))))
+
+  (local
+   (defthm logtail-to-floor
+     (implies (integerp x)
+              (equal
+               (logtail 1 x)
+               (floor x 2)))
+     :hints (("Goal"
+              :in-theory (e/d (floor
+                               logtail)
+                              ())))))
+
+  (local
+   (defthm logapp-of-the-same
+     (equal (equal (logapp 1 x y)
+                   (logapp 1 x z))
+            (equal (ifix y) (ifix z)))
+     :hints (("Goal"
+              :in-theory (e/d (logapp
+                               sum
+                               ifix)
+                              (+-is-sum))))))
+
+  (local
+   (encapsulate
+     nil
+     (local
+      (use-arithmetic-5 t))
+     (defthm f2-of-m2
+       (and (equal (sum other (f2 (sum (m2 x) y)))
+                   (sum other
+                        (f2 (sum x y))
+                        (-- (f2 x))))
+            (equal (sum (f2 (sum (m2 x) y)) other)
+                   (sum other
+                        (f2 (sum x y))
+                        (-- (f2 x)))))
+       :hints (("Goal"
+                :in-theory (e/d (f2
+                                 m2
+                                 sum --)
+                                (+-is-sum
+                                 floor2-if-f2
+                                 mod2-is-m2)))))))
+
+  (local
+   (defthm bitp-of-f2
+     (and (implies (and (bitp x)
+                        (bitp y)
+                        (bitp z))
+                   (bitp (f2 (sum x y z))))
+          (implies (and (bitp x)
+                        (bitp y))
+                   (bitp (f2 (sum x y)))))
+     :hints (("Goal"
+              :in-theory (e/d (bitp) ())))))
+
+  (local
+   (defthmd
+     loghead-binary-+-of-loghead
+     (implies (and (integerp x)
+                   (integerp y))
+              (equal (loghead size (+ (loghead size x) y))
+                     (loghead size (+ x y))
+                     ))
+     :hints (("Goal"
+              :in-theory (e/d* ((:REWRITE ACL2::|(* (expt c m) (expt d n))|)
+                                (:REWRITE ACL2::|(* 1 x)|)
+                                (:REWRITE ACL2::|(* x (- y))|)
+                                (:REWRITE ACL2::|(+ x (- x))|)
+                                (:REWRITE ACL2::|(+ y (+ x z))|)
+                                (:REWRITE ACL2::|(+ y x)|)
+                                (:REWRITE ACL2::|(/ (expt x n))|)
+                                (:REWRITE ACL2::|(mod (+ x y) z) where (<= 0 z)|
+                                          . 1)
+                                (:REWRITE ACL2::BUBBLE-DOWN-*-MATCH-1)
+                                (:REWRITE ACL2::BUBBLE-DOWN-+-MATCH-1)
+                                (:DEFINITION ACL2::EXPT2$INLINE)
+                                (:DEFINITION ACL2::IMOD$INLINE)
+                                (:DEFINITION ACL2::LOGHEAD$INLINE)
+                                (:REWRITE ACL2::SIMPLIFY-MOD-+-MOD)
+                                ;;(:TYPE-PRESCRIPTION ACL2::EXPT-TYPE-PRESCRIPTION-INTEGERP)
+                                ;;(:TYPE-PRESCRIPTION ACL2::EXPT-TYPE-PRESCRIPTION-NONZERO)
+                                ;;(:TYPE-PRESCRIPTION ACL2::EXPT-TYPE-PRESCRIPTION-POSITIVE)
+                                (:TYPE-PRESCRIPTION IFIX)
+                                (:TYPE-PRESCRIPTION ACL2::INTEGERP-MOD)
+                                (:TYPE-PRESCRIPTION ACL2::MOD-TYPE . 4)
+                                (:TYPE-PRESCRIPTION NFIX)
+                                (:TYPE-PRESCRIPTION ACL2::RATIONALP-MOD)
+                                (:REWRITE ACL2::NORMALIZE-ADDENDS)
+                                (:REWRITE ACL2::NORMALIZE-FACTORS-GATHER-EXPONENTS)
+                                (:REWRITE ACL2::SIMPLIFY-MOD-+-MOD)
+                                (:TYPE-PRESCRIPTION ACL2::EXPT-TYPE-PRESCRIPTION-INTEGERP-BASE)
+                                (:TYPE-PRESCRIPTION ACL2::EXPT-TYPE-PRESCRIPTION-NONNEGATIVE-BASE)
+                                (:TYPE-PRESCRIPTION ACL2::EXPT-TYPE-PRESCRIPTION-POSITIVE-BASE))
+                               (+-IS-SUM))))))
+
+  (local
+   (defthmd logcdr-is-f2
+     (EQUAL (LOGHEAD size
+                         (ACL2::LOGCDR (SUM CARRY X Y)))
+                (LOGHEAD size
+                         (F2 (SUM CARRY X Y))))
+     :hints (("Goal"
+              :in-theory (e/d (f2 acl2::logcdr) ())))))
+  
+  (local
+   (defthmd loghead-of-+-is-2vec-adder-lemma
+     (implies (and (integerp x)
+                   (integerp y)
+                   (bitp carry))
+              (and (equal (loghead size (+ x y carry))
+                          (2vec-adder x y carry size))
+                   (equal (loghead size (+ x carry y))
+                          (2vec-adder x y carry size))
+                   (equal (loghead size (+ carry x y))
+                          (2vec-adder x y carry size))))
+     :otf-flg t
+     :hints (("Goal"
+              :induct (2vec-adder x y carry size)
+              :do-not-induct t
+              :in-theory (e/d* (2vec-adder
+                                s-spec
+                                bitops::ihsext-recursive-redefs
+                                bitops::ihsext-inductions
+                                BITOPS::LOGCDR-OF-LOGHEAD
+                                sum-list
+                                BIT-OF
+                                c-spec
+                                ;; bit-concat
+                                ;;f2
+                                logapp
+                                logcdr-is-f2
+                                )
+                               (D2-OF-MINUS
+                                SUM-OF-F2S))))))
+
+  (defthm loghead-of-+-is-2vec-adder
+    (implies (and (integerp x)
+                  (integerp y)
+                  (bitp carry))
+             (and (equal (loghead size (+ x y carry))
+                         (2vec-adder (loghead size x) (loghead size y) carry size))
+                  (equal (loghead size (+ x carry y))
+                         (2vec-adder (loghead size x) (loghead size y) carry size))
+                  (equal (loghead size (+ carry x y))
+                         (2vec-adder (loghead size x) (loghead size y) carry size))))
+    :otf-flg t
+    :hints (("Goal"
+             :do-not-induct t
+             :in-theory (disable +-IS-SUM)
+             ;;:in-theory (theory 'ground-zero)
+             :use ((:instance loghead-of-+-is-2vec-adder-lemma
+                              (x (loghead size x))
+                              (y (loghead size y)))
+                   (:instance loghead-binary-+-of-loghead
+                              (x x)
+                              (y (+ (loghead size y) carry))
+                              )
+                   (:instance loghead-binary-+-of-loghead
+                              (x y)
+                              (y (+ x carry))
+                              ))
+             )))
+
+  (defthm loghead-of-+-is-2vec-adder-without-carry
+    (implies (and (integerp x)
+                  (integerp y))
+             (equal (loghead size (+ x y))
+                    (2vec-adder (loghead size x) (loghead size y) 0 size)))
+    :otf-flg t
+    :hints (("Goal"
+             :use ((:instance loghead-of-+-is-2vec-adder
+                              (carry 0)))
+             :do-not-induct t
+             :in-theory (e/d ()
+                             (loghead-of-+-is-2vec-adder))))))
+
+(add-rp-rule loghead-of-+-is-2vec-adder-without-carry)
+(add-rp-rule loghead-of-+-is-2vec-adder)
