@@ -156,7 +156,16 @@
                (mv y t 1))
               (('bit-of & &)
                (mv y t 1))
-              (& (mv y nil -1)))))
+              (& (mv y nil -1))))
+
+           #|((mv x y atom-x atom-y len-x len-y)
+           (if (and (equal len-x 2)
+           (equal len-y 2))
+           (mv x y atom-x atom-y len-x len-y)
+           (mv y x atom-x atom-y len-x len-y)))||#
+
+
+           )
         (cond
          ((not (equal len-x len-y))
           (cond
@@ -473,9 +482,25 @@
                  (hard-error 'c/d-fix-pp-args "" nil)
                  (mv ''nil pp)))))
 
-  (define c/d-fix-s-args-disabled ()
-    :inline t
+
+  (encapsulate
+    (((c/d-remove-repeated-s) => *))
+    (local
+     (defun c/d-remove-repeated-s ()
+       nil)))
+
+  (define return-t ()
     t)
+  (define return-nil ()
+    nil)
+
+  (defmacro c/d-remove-repeated-s-enable ()
+    `(defattach  c/d-remove-repeated-s return-t))
+
+  (defmacro c/d-remove-repeated-s-disable ()
+    `(defattach  c/d-remove-repeated-s return-nil))
+
+  (c/d-remove-repeated-s-disable)
 
   (define c/d-fix-s-args ((s))
     ;; same as c/d-pp-fix but don't touch the negated terms
@@ -484,19 +509,19 @@
                             :hyp (rp-termp s))
                  (cleaned-s rp-termp
                             :hyp (rp-termp s)))
-    (if (c/d-fix-s-args-disabled)
-        (if (equal s '(list))
-            (mv ''nil ''nil)
-          (mv ''nil s))
-      (case-match s
-        (('list . s-lst)
-         (b* (((mv coughed-lst res-lst) (c/d-fix-arg-aux s-lst nil (expt 2 30))))
-           (mv (if coughed-lst `(list . ,coughed-lst) ''nil)
-               (if res-lst (cons-with-hint 'list res-lst s) ''nil))))
-        (''nil (mv ''nil ''nil))
-        (& (progn$ (cw "Unexpected s form= ~p0 ~%" s)
-                   (hard-error 'c/d-fix-s-args "" nil)
-                   (mv ''nil s)))))))
+    (if (c/d-remove-repeated-s)
+        (case-match s
+          (('list . s-lst)
+           (b* (((mv coughed-lst res-lst) (c/d-fix-arg-aux s-lst nil (expt 2 30))))
+             (mv (if coughed-lst `(list . ,coughed-lst) ''nil)
+                 (if res-lst (cons-with-hint 'list res-lst s) ''nil))))
+          (''nil (mv ''nil ''nil))
+          (& (progn$ (cw "Unexpected s form= ~p0 ~%" s)
+                     (hard-error 'c/d-fix-s-args "" nil)
+                     (mv ''nil s))))
+      (if (equal s '(list))
+          (mv ''nil ''nil)
+        (mv ''nil s)))))
 
 (define get-c/d-args ((c/d))
   :inline t
@@ -598,7 +623,7 @@
 ;;                'nil))
 
 (define can-c-merge-fast-aux (s-lst pp c/d)
-  :inline t
+  ;;:inline t
   (if (atom s-lst)
       nil
     (or (b* ((cur-s (ex-from-rp-loose (car s-lst))))
@@ -606,10 +631,10 @@
             (('s pp-arg c/d-arg)
              (progn$
               (and
-               (equal pp-arg pp) (equal c/d-arg c/d) 
+               (equal pp-arg pp) (equal c/d-arg c/d)
                ;;(rp-equal-cnt c/d-arg c/d 10) (rp-equal-cnt pp-arg pp 10)
                )))))
-;(can-c-merge-fast-aux (cdr s-lst) pp c/d)
+        ;;(can-c-merge-fast-aux (cdr s-lst) pp c/d)
         )))
 
 (define can-c-merge-fast (c/d1 c/d2)
@@ -714,7 +739,6 @@
 
 ;; (memoize 'search-pattern)
 
-
 (local
  (defthm is-rp-of-evenpi
    (IS-RP `(RP 'EVENPI ,x))
@@ -728,7 +752,6 @@
             :in-theory (e/d (is-rp) ())))))
 
 
-
 (local
  (defthm rp-termp-of-d
    (iff (rp-termp `(d ,x))
@@ -745,7 +768,6 @@
         (rp-term-listp x))))
 
 
-
 (local
  (defthm rp-termp-of-d-sum
    (iff (rp-termp `(d-sum ,x ,y ,z))
@@ -764,7 +786,6 @@
   :inline t
   (case-match term
     (('list ('-- ('binary-and & &))) t)))
-  
 
 (define c/d-merge-slow-aux (pp-arg1 pp-arg2 pp-coughed-from-arg
                                     s-arg1 s-arg2 s-coughed-from-arg
@@ -832,11 +853,11 @@
         (cond (c/d1-is-c
                (if (is-a-negated-minterm extra-s-arg1)
                    (b* (((mv pp-arg pp-arg-cnt3)
-                        (pp-sum-merge pp-arg extra-s-arg1)))
+                         (pp-sum-merge pp-arg extra-s-arg1)))
                      (mv s-arg pp-arg pp-arg-cnt3))
                  (mv (s-sum-merge extra-s-arg1 s-arg)
-                        pp-arg
-                        0)))
+                     pp-arg
+                     0)))
               (t (mv s-arg pp-arg 0))))
        ((mv s-arg pp-arg pp-arg-cnt4)
         (cond (c/d2-is-c
@@ -1028,7 +1049,8 @@
        (case-match cur-s
          (('s pp-cur c/d-cur)
           (b* (((mv s-coughed pp-coughed c/d)
-                (c/d-merge c/d c/d-cur nil (1- limit)))
+                (c/d-merge c/d c/d-cur nil  (1- limit) ;;--hereeeee
+                           ))
                ((mv pp &) (pp-sum-merge pp pp-cur))
                ((mv pp &) (pp-sum-merge pp pp-coughed)))
             (case-match s-coughed
@@ -1160,7 +1182,8 @@
                  (mv ''nil ''nil `(binary-sum ,c/d1 ,c/d2))))
 
         ((mv s-coughed-from-arg pp-coughed-from-arg c/d-arg)
-         (c/d-merge c/d-arg1 c/d-arg2 t (1- limit)))
+         (c/d-merge c/d-arg1 c/d-arg2 t ;;(expt 2 40)))
+                    (1- limit)));; --hereeeee
 
         )
      (cond
@@ -1168,60 +1191,16 @@
        (c/d-merge-fast-aux pp-arg1 pp-arg2 pp-coughed-from-arg
                            s-arg2 s-coughed-from-arg
                            c/d-arg
-                           clean-flg)
-       #|(b* ((- (fast-merge-profile))
-
-       ((mv pp-arg pp-arg-cnt1) (pp-sum-merge pp-arg1 pp-arg2))
-       ((mv pp-arg pp-arg-cnt2) (pp-sum-merge pp-coughed-from-arg pp-arg))
-       (pp-arg-cnt (+ ;;(expt 2 20) ;;test and remove later when sure...
-       pp-arg-cnt1 pp-arg-cnt2))
-
-       (s-arg `(list . ,(cddr s-arg2)))
-       ;;(s-arg (remove-s-from-for-fast-merge s-arg2 pp-arg1 c/d-arg1))
-       ;;(s-arg (s-sum-merge s-arg2 `(list (-- (s ,pp-arg1 ,c/d-arg1)))))
-       (s-arg (s-sum-merge s-arg s-coughed-from-arg))
-
-       ((mv s-coughed s-arg pp-coughed pp-arg)
-       (clean-c/d-args s-arg pp-arg pp-arg-cnt clean-flg))
-       (c-merged
-       (create-c-instance s-arg pp-arg c/d-arg))
-       #|(if (and (equal s-arg ''nil)
-       (equal c/d-arg ''0)
-       (case-match pp (('list ('binary-and & &)) t)))
-       ''0
-       `(c ,s-arg ,pp-arg ,c/d-arg))||#)
-       (mv s-coughed pp-coughed c-merged))||#)
+                           clean-flg))
       (t
        (b* ((c/d1-is-c (eq type1 'c))
             (c/d2-is-c (eq type2 'c))
-
-            ;; ((mv pp-arg pp-arg-cnt1) (pp-sum-merge pp-arg1 pp-arg2))
-            ;; ((mv pp-arg pp-arg-cnt2) (pp-sum-merge pp-coughed-from-arg pp-arg))
-            ;; (pp-arg-cnt (+ ;;(expt 2 20) ;;test and remove later when sure...
-            ;;              pp-arg-cnt1 pp-arg-cnt2))
-
-            ;; (s-arg (s-sum-merge s-arg2 s-coughed-from-arg))
-            ;; (s-arg (s-sum-merge s-arg1 s-arg))
-
             (extra-s-arg1 (and c/d1-is-c
                                (get-extra-s-arg s-arg1 pp-arg1
                                                 c/d-arg1 (1- limit))))
             (extra-s-arg2 (and c/d2-is-c
                                (get-extra-s-arg s-arg2 pp-arg2
-                                                c/d-arg2 (1- limit))))
-
-            ;; (s-arg (cond ((and c/d1-is-c c/d2-is-c)
-            ;;               (s-sum-merge s-arg (s-sum-merge extra-s-arg1 extra-s-arg2)))
-            ;;              (c/d1-is-c (s-sum-merge s-arg extra-s-arg1))
-            ;;              (c/d2-is-c (s-sum-merge s-arg extra-s-arg2))
-            ;;              (t s-arg)))
-
-            ;; ((mv s-coughed s-arg pp-coughed pp-arg)
-            ;;  (clean-c/d-args s-arg pp-arg pp-arg-cnt clean-flg))
-
-            ;; (d-res `(d (rp 'evenpi (d-sum ,s-arg ,pp-arg ,c/d-arg))))
-            ;; (c/d-merged (if clean-flg (d-to-c d-res) d-res))
-            )
+                                                c/d-arg2 (1- limit)))))
          (c/d-merge-slow-aux pp-arg1 pp-arg2 pp-coughed-from-arg
                              s-arg1 s-arg2 s-coughed-from-arg
                              extra-s-arg1
@@ -1229,9 +1208,7 @@
                              c/d-arg
                              clean-flg
                              c/d1-is-c
-                             c/d2-is-c)
-         ;;(mv s-coughed pp-coughed c/d-merged)
-         ))))))
+                             c/d2-is-c)))))))
 
 ;; (define c/d-merge-aux (c/d1-is-c
 ;;                        c/d2-is-c
@@ -1337,7 +1314,7 @@
    rp-termp-of--<fn>
    (rp-termp pp-term)
    :hyp (rp-termp term)
-   
+
    :hints (("Goal"
             :induct (4vec->pp-term term)
             :do-not-induct t
@@ -1353,30 +1330,30 @@
                                 pp-term-p)))))
 
   #|(local
-   (defthm lemma1
-     (IMPLIES (AND (PP-HAS-BITP-RP TERM))
-              (equal (PP-TERM-P TERM)
-                     (B* ((ORIG TERM) (TERM (EX-FROM-RP TERM)))
-                       (CASE-MATCH TERM
-                         (('BINARY-AND X Y)
-                          (AND (PP-TERM-P X) (PP-TERM-P Y)))
-                         (('BINARY-OR X Y)
-                          (AND (PP-TERM-P X) (PP-TERM-P Y)))
-                         (('BINARY-XOR X Y)
-                          (AND (PP-TERM-P X) (PP-TERM-P Y)))
-                         (('BINARY-? X Y Z)
-                          (AND (PP-TERM-P X)
-                               (PP-TERM-P Y)
-                               (PP-TERM-P Z)))
-                         (('BINARY-NOT X) (AND (PP-TERM-P X)))
-                         (('BIT-OF & &) T)
-                         (''1 T)
-                         (& (PP-HAS-BITP-RP ORIG))))))
-     :hints (("goal"
-              :do-not-induct t
-              :expand (pp-term-p term)
-              :in-theory (e/d () (pp-has-bitp-rp))))))||#
-  
+  (defthm lemma1
+  (IMPLIES (AND (PP-HAS-BITP-RP TERM))
+  (equal (PP-TERM-P TERM)
+  (B* ((ORIG TERM) (TERM (EX-FROM-RP TERM)))
+  (CASE-MATCH TERM
+  (('BINARY-AND X Y)
+  (AND (PP-TERM-P X) (PP-TERM-P Y)))
+  (('BINARY-OR X Y)
+  (AND (PP-TERM-P X) (PP-TERM-P Y)))
+  (('BINARY-XOR X Y)
+  (AND (PP-TERM-P X) (PP-TERM-P Y)))
+  (('BINARY-? X Y Z)
+  (AND (PP-TERM-P X)
+  (PP-TERM-P Y)
+  (PP-TERM-P Z)))
+  (('BINARY-NOT X) (AND (PP-TERM-P X)))
+  (('BIT-OF & &) T)
+  (''1 T)
+  (& (PP-HAS-BITP-RP ORIG))))))
+  :hints (("goal"
+  :do-not-induct t
+  :expand (pp-term-p term)
+  :in-theory (e/d () (pp-has-bitp-rp))))))||#
+
   (acl2::defret
    pp-term-p-of--<fn>
    :hyp (good-4vec-term-p term)
@@ -1393,11 +1370,10 @@
     :returns (res booleanp)
     (case-match term
       (('cons x rest)
-       (b* ((x (ex-from-rp-loose x))
+       (b* ((?orig x)
+            (x (ex-from-rp-loose x))
             (rest-res (well-formed-new-sum rest)))
-         (cond ((good-4vec-term-p x)
-                rest-res)
-               ((case-match x (('s & &) t))
+         (cond ((case-match x (('s & &) t))
                 rest-res)
                ((case-match x (('c & & &) t))
                 rest-res)
@@ -1409,6 +1385,8 @@
                 rest-res)
                ((equal x ''0)
                 rest-res)
+               ((good-4vec-term-p orig)
+                rest-res)
                (t
                 nil))))
       (('quote x)
@@ -1442,12 +1420,7 @@
             (limit (expt 2 40))
             (x-orig x)
             (x (ex-from-rp-loose x)))
-         (cond ((good-4vec-term-p x) ;; (pp-term-p x)
-                (b* ((x (4vec->pp-term x)))
-                  (mv s-rest
-                      (mv-nth-0-of-2 (pp-sum-merge (pp-flatten x nil) pp-rest))
-                      c/d-rest)))
-               ((case-match x (('s & &) t))
+         (cond ((case-match x (('s & &) t))
                 (mv (s-sum-merge `(list ,x-orig) s-rest)
                     pp-rest
                     c/d-rest))
@@ -1479,6 +1452,11 @@
                     c/d-rest))
                ((equal x ''0)
                 (mv s-rest pp-rest c/d-rest))
+               ((good-4vec-term-p x-orig) ;; (pp-term-p x)
+                (b* ((x (4vec->pp-term x-orig)))
+                  (mv s-rest
+                      (mv-nth-0-of-2 (pp-sum-merge (pp-flatten x nil) pp-rest))
+                      c/d-rest)))
                (t
                 (progn$ (cw "not well-formed term for new sums= ~p0 ~%" x)
                         (mv ''nil ''nil ''0))))))
@@ -1788,8 +1766,8 @@
                     (c-res (c-spec-meta-aux s pp c/d quarternaryp))
                     (res `(cons ,s-res (cons ,c-res 'nil)))
                     #|(- (if (search-pattern res)
-                           (cw "pattern found s-c-spec-meta ~%")
-                         nil))||#)
+                    (cw "pattern found s-c-spec-meta ~%")
+                    nil))||#)
                  res)
              (progn$ (cw "term is not well-formed-new-sum ~p0 ~%" term)
                      term)))
@@ -1806,13 +1784,13 @@
              (progn$ (cw "term is not well-formed-new-sum ~p0 ~%" term)
                      term)))
           (('s-spec sum)
-                  (cond ((well-formed-new-sum sum)
-                         (b* (((mv s pp c/d)
-                               (new-sum-merge sum)))
-                           (s-spec-meta-aux s pp c/d)))
-                        (t
-                         (progn$ (cw "term is not well-formed-new-sum ~p0 ~%" term)
-                                 term))))
+           (cond ((well-formed-new-sum sum)
+                  (b* (((mv s pp c/d)
+                        (new-sum-merge sum)))
+                    (s-spec-meta-aux s pp c/d)))
+                 (t
+                  (progn$ (cw "term is not well-formed-new-sum ~p0 ~%" term)
+                          term))))
           (('c-spec sum)
            (if (well-formed-new-sum sum)
                (b* (((mv s pp c/d)
@@ -1988,25 +1966,25 @@
   (local
    (defthm EXTRA-S-CAN-BE-PP-def
      (equal (EXTRA-S-CAN-BE-PP pp c/d)
-             (AND (EQUAL C/D ''0)
-                            (CASE-MATCH PP (('LIST ('BINARY-AND & &)) T))))))
-  
+            (AND (EQUAL C/D ''0)
+                 (CASE-MATCH PP (('LIST ('BINARY-AND & &)) T))))))
+
   (with-output
     :off :all
     :gag-mode nil
 
     (def-formula-checks
       mult-formula-checks
-      (pp-sum-merge
-       s-sum-merge
+      (;pp-sum-merge
+;s-sum-merge
        binary-append
-       pp-lists-to-term-pp-lst
-       pp-term-to-pp-lists
+;pp-lists-to-term-pp-lst
+;pp-term-to-pp-lists
        --
        sum-list
-       s-c-spec-meta
-       s-spec-meta
-       c-spec-meta
+;s-c-spec-meta
+;s-spec-meta
+;c-spec-meta
        binary-and
        and-list
        sort-sum
@@ -2036,9 +2014,16 @@
        m2 d2 f2 times2
        s
        binary-sum
-       sort-sum-meta
+;sort-sum-meta
        evenpi
        d-sum
        sv::3vec-fix
        sv::4vec-fix
-       c-s-spec-meta))))
+;c-s-spec-meta
+       ))))
+
+(defmacro ss (&rest args)
+  `(s-spec (list . ,args)))
+
+(defmacro cc (&rest args)
+  `(c-spec (list . ,args)))
