@@ -225,7 +225,7 @@
                                        (append (bind-bindings-aux bindings a)
                                                a))))
               t))||#
-     
+
      (implies (and
                ;(not (consp term))
                ;(symbolp term)
@@ -343,6 +343,7 @@
                 (alistp a)
                 (rp-evl (rp-apply-bindings (rp-hyp rule) bindings) a)
                 (bindings-alistp bindings)
+                (rp-rule-rwp rule)
                 (rp-iff-flag rule))
            (iff (rp-evl (rp-apply-bindings (rp-lhs rule) bindings) a)
                 (rp-evl (rp-apply-bindings (rp-rhs rule) bindings) a)))
@@ -359,6 +360,7 @@
                 (alistp a)
                 (rp-evl (rp-apply-bindings (rp-hyp rule) bindings) a)
                 (bindings-alistp bindings)
+                (rp-rule-rwp rule)
                 (not (rp-iff-flag rule)))
            (equal (rp-evl (rp-apply-bindings (rp-lhs rule) bindings) a)
                   (rp-evl (rp-apply-bindings (rp-rhs rule) bindings) a)))
@@ -605,6 +607,9 @@
      :hints (("Goal"
               :in-theory (e/d (is-rp) ())))))
 
+; Matt K. mod 7/2021: The following lemma is no longer accepted due to a
+; strengthening of remove-guard-holders.
+#||
   (local
    (defthm lemma3
      (implies (is-if term)
@@ -612,6 +617,7 @@
                 (& NIL)))
      :hints (("Goal"
               :in-theory (e/d (is-if) ())))))
+||#
 
   (local
    (in-theory (disable is-synp)))
@@ -755,6 +761,7 @@
 
   (defthm valid-sc-apply-bindings-for-hyp
     (implies (and (valid-rulep rule)
+                  (rp-rule-rwp rule)
                   (bindings-alistp bindings)
                   (valid-sc-bindings bindings a))
              (valid-sc (rp-apply-bindings (rp-hyp rule) bindings) a))
@@ -860,7 +867,7 @@
                                   EX-FROM-RP-LEMMA1))))))
 
      (defthm lemma202
-       (implies (and 
+       (implies (and
                      (rp-termp term)
                      (alistp a)
                      (bindings-alistp bindings)
@@ -957,7 +964,7 @@
               :in-theory (e/d (is-rp
                                is-synp) ())))))
 
-  
+
   (local
    (defthm include-fnc-lemma
      (implies (and (NOT (INCLUDE-FNC TERM 'LIST))
@@ -1008,6 +1015,7 @@
   (defthm valid-sc-apply-bindings-for-rhs
     (implies
      (and (valid-rulep rule)
+          (rp-rule-rwp rule)
           (not (include-fnc (rp-hyp rule) 'list))
           (not (include-fnc (rp-rhs rule) 'list))
           (alistp a)
