@@ -172,11 +172,16 @@
   (local
    (in-theory (enable is-rp)))
 
+;; Removed in July 2021 by Matt Kaufmann since the right-hand sides of the
+;; generated rules are now all constants, due to enhancements to
+;; remove-guard-holders.
+#||
   (defthm is-synp-implies
     (implies (is-synp term)
              (CASE-MATCH TERM (('SYNP & & &) T)
                (& NIL)))
     :hints (("Goal" :in-theory (enable is-synp))))
+||#
 
   (defthm pseudo-termlistp-extract-from-rp
     (implies (and (rp-termp term)
@@ -485,27 +490,26 @@
   (MV-NTH 0 (GET-LAMBDA-FREE-VARS-LST SUBTERMS)))))||#
 
   #|(local
-   ;; this was used when lambda expressions were allowed
-   (defthm rp-termp-cons-car-term-subterms-lemma1
-     (implies (and (rp-termp term)
-                   (consp term)
-                   (consp (car term))
-                   (equal (len subterms)
-                          (len (cdr term)))
-;(mv-nth 0 (get-lambda-free-vars-lst subterms))
-                   #|(lambda-exp-free-listp subterms)||#
-                   (rp-term-listp subterms))
-              (rp-termp (cons (car term) subterms)))
-     :otf-flg t
-     :hints (("Goal"
-              :do-not-induct t
-              :expand ((rp-termp term)
-                       (GET-LAMBDA-FREE-VARS (CONS (CAR TERM) SUBTERMS))
-                       (is-lambda-strict (cons (car term) subterms))
-                       (IS-LAMBDA (CONS (CAR TERM) SUBTERMS)))
-              :in-theory (e/d () (get-lambda-free-vars))))))||#
+  ;; this was used when lambda expressions were allowed
+  (defthm rp-termp-cons-car-term-subterms-lemma1
+  (implies (and (rp-termp term)
+  (consp term)
+  (consp (car term))
+  (equal (len subterms)
+  (len (cdr term)))
+;(mv-nth 0 (get-lambda-free-vars-lst subterms)) ;
+  #|(lambda-exp-free-listp subterms)||#
+  (rp-term-listp subterms))
+  (rp-termp (cons (car term) subterms)))
+  :otf-flg t
+  :hints (("Goal"
+  :do-not-induct t
+  :expand ((rp-termp term)
+  (GET-LAMBDA-FREE-VARS (CONS (CAR TERM) SUBTERMS))
+  (is-lambda-strict (cons (car term) subterms))
+  (IS-LAMBDA (CONS (CAR TERM) SUBTERMS)))
+  :in-theory (e/d () (get-lambda-free-vars))))))||#
 
-  
 
   (defthm rp-termp-cons-car-term-subterms
     (implies (and (rp-termp term)
@@ -518,12 +522,12 @@
                   (force (or (not (equal (car term) 'rp))
                              (is-rp (cons (car term) subterms))))
                   (force (or (not (equal (car term) 'falist))
-                       (falist-consistent (cons (car term) subterms))))
+                             (falist-consistent (cons (car term) subterms))))
                   (rp-term-listp subterms))
              (rp-termp (cons (car term) subterms)))
     :hints (("Goal"
              :in-theory (e/d (is-rp) ())))
-     ))
+    ))
 
 (defthm rp-term-listp-append
   (implies (and (rp-term-listp lst1)
@@ -753,7 +757,8 @@
            :in-theory (e/d (is-rp) ()))))
 
 (defthmd rule-syntaxp-implies
-  (implies (rule-syntaxp rule)
+  (implies (and (rule-syntaxp rule :warning warning)
+                (not (rp-rule-metap rule)))
            (and
             (weak-custom-rewrite-rule-p rule)
             (rp-termp (rp-hyp rule))
@@ -774,6 +779,16 @@
   :rule-classes (:rewrite :forward-chaining)
   :hints (("Goal" :in-theory (enable rule-syntaxp))))
 
+(defthmd rule-syntaxp-implies-2
+  (implies (and (rule-syntaxp rule :warning warning)
+                (rp-rule-metap rule))
+           (AND (SYMBOLP (RP-RULE-TRIG-FNC RULE))
+                (RP-RULE-TRIG-FNC RULE)
+                (SYMBOLP (RP-RULE-META-FNC RULE))
+                (RP-RULE-META-FNC RULE)))
+  :rule-classes (:rewrite :forward-chaining)
+  :hints (("Goal" :in-theory (enable rule-syntaxp))))
+
 (defthm ex-from-rp-loose-of-ex-from-rp-loose
   (equal (ex-from-rp-loose (ex-from-rp-loose term))
          (ex-from-rp-loose term))
@@ -890,7 +905,6 @@
                             ex-from-rp-loose
                             is-rp-loose) ()))))||#
 
-
 (defthm rp-termp-caddr
   (implies (and (rp-termp term)
                 (consp term)
@@ -918,6 +932,21 @@
                             ex-from-rp-loose
                             is-rp-loose) ()))))
 
+(defthm rp-termp-caddddr
+  (implies (and (rp-termp term)
+                (consp term)
+                (not (quotep term))
+                (consp (cdr term))
+                (consp (cddr term))
+                (consp (cdddr term))
+                (consp (cddddr term)))
+           (rp-termp (car (cddddr term))))
+  :hints (("Goal"
+           :in-theory (e/d (rp-termp
+                            is-rp
+                            ex-from-rp-loose
+                            is-rp-loose) ()))))
+
 #|(defthm rp-syntaxp-caddr
   (implies (and (rp-syntaxp term)
                 (consp term)
@@ -975,7 +1004,6 @@
   (implies (rp-termp term)
 	   (dont-rw-syntaxp term)))
 
-
 (defthm rp-termp-ex-from-falist
   (implies (rp-termp x)
            (rp-termp (ex-from-falist x)))
@@ -984,13 +1012,11 @@
            :expand (ex-from-falist x)
            :in-theory (e/d () ()))))
 
-
 (defthm is-falist-strict-to-is-falist
   (implies (rp-termp term)
            (equal (is-falist-strict term)
                   (is-falist term))))
 
-
 (defthm rp-termp-trans*-list
   (implies (and (rp-term-listp lst)
                 (consp lst))
@@ -1010,7 +1036,6 @@
   :hints (("Goal"
            :in-theory (e/d (is-rp) ()))))
 
-
 (defthm-rp-trans
   (defthm rp-termp-of-rp-trans
     (implies (rp-termp term)
@@ -1025,18 +1050,59 @@
            :in-theory (e/d ()
                            ()))))
 
-
 #|(defthm-rp-trans
-  (defthm rp-trans-is-term-when-list-is-absent
-    (implies (not (include-fnc term 'list))
-             (equal (rp-trans term) term))
-    :flag rp-trans)
-  (defthm rp-trans-lst-is-lst-when-list-is-absent
-    (implies (not (include-fnc-subterms lst 'list))
-             (equal (rp-trans-lst lst) lst))
-    :flag rp-trans-lst))||#
+(defthm rp-trans-is-term-when-list-is-absent
+(implies (not (include-fnc term 'list))
+(equal (rp-trans term) term))
+:flag rp-trans)
+(defthm rp-trans-lst-is-lst-when-list-is-absent
+(implies (not (include-fnc-subterms lst 'list))
+(equal (rp-trans-lst lst) lst))
+:flag rp-trans-lst))||#
+
+(defthm rp-state-new-run-returns-rp-statep
+  (implies (rp-statep rp-state)
+           (rp-statep (rp-state-new-run rp-state)))
+  :hints (("Goal"
+           :in-theory (e/d (rp-state-new-run
+                            rp-statep)
+                           ()))))
 
 
+(defthm RP-TERM-LISTP-of-append
+  (implies (and (rp-term-listp lst1)
+                (rp-term-listp lst2))
+           (rp-term-listp (append lst1 lst2))))
 
 
+(defthm rp-state-preservedp-of-the-same
+  (implies (rp-statep rp-state)
+           (rp-state-preservedp rp-state rp-state))
+  :hints (("Goal"
+           :in-theory (e/d (rp-state-preservedp) ()))))
 
+(defthm rp-state-preservedp-implies-rp-statep
+  (implies (and (rp-statep rp-state)
+                (rp-state-preservedp rp-state new-rp-state))
+           (rp-statep new-rp-state))
+  :hints (("Goal"
+           :in-theory (e/d (rp-state-preservedp) ()))))
+
+(defthm rp-state-preservedp-implies-valid-rp-state-syntaxp
+  (implies (and (valid-rp-state-syntaxp rp-state)
+                (rp-state-preservedp rp-state new-rp-state))
+           (valid-rp-state-syntaxp new-rp-state))
+  :hints (("Goal"
+           :do-not-induct t
+           :use ((:instance RP-STATE-PRESERVEDP-SK-necc
+                            (key (VALID-RP-STATE-SYNTAXP-AUX-WITNESS
+                                  NEW-RP-STATE))
+                            (OLD-RP-STATE rp-state))
+                 (:instance VALID-RP-STATE-SYNTAXP-AUX-necc
+                            (key (VALID-RP-STATE-SYNTAXP-AUX-WITNESS NEW-RP-STATE))))
+           :expand (VALID-RP-STATE-SYNTAXP-AUX NEW-RP-STATE)
+           :in-theory (e/d (rp-state-preservedp
+                            valid-rp-state-syntaxp)
+                           (RP-STATE-PRESERVEDP-SK
+                            VALID-RP-STATE-SYNTAXP-AUX
+                            RP-STATEP)))))
