@@ -111,6 +111,9 @@
                                                sublhs context acc-bindings))))
     :flag rp-match-lhs-subterms))
 
+; Matt K. mod 7/2021: The following lemma is no longer accepted due to a
+; strengthening of remove-guard-holders.
+#||
 (local
  (defthm should-term-be-in-cons-lemma1
    (implies (should-term-be-in-cons rule-lhs term)
@@ -120,12 +123,17 @@
                  (case-match rule-lhs (('cons & &) t)
                    (& nil))))
    :hints (("Goal" :in-theory (enable should-term-be-in-cons)))))
+||#
 
+; Matt K. mod 7/2021: The following lemma is no longer accepted due to a
+; strengthening of remove-guard-holders.
+#||
 (local
  (defthm is-snyp-props
    (implies (is-synp term)
             (case-match term (('synp & & &) t)
               (& nil)))))
+||#
 
 (defmacro bindings-from (x)
   `(mv-nth 1 ,x))
@@ -376,7 +384,8 @@
                               (:DEFINITION MEMBER-EQUAL)
                               (:DEFINITION FALIST-CONSISTENT)
                               (:REWRITE RP-TERMP-IMPLIES-CDR-LISTP)
-                              (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
+; Obsolete (see Matt K. comment above):
+;                             (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
                               (:REWRITE NOT-INCLUDE-RP)
                               (:DEFINITION FALIST-CONSISTENT-AUX)
                               (:REWRITE ACL2::O-P-O-INFP-CAR)
@@ -438,7 +447,8 @@
                             (:DEFINITION VALID-RULEP-SK)
                             (:REWRITE RP-TERMP-IMPLIES-CDR-LISTP)
                             (:DEFINITION VALID-SC)
-                            (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
+; Obsolete (see Matt K. comment above):
+;                           (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
                             (:REWRITE
                              ACL2::TRUE-LIST-LISTP-IMPLIES-ALWAYS$-TRUE-LISTP)
                             (:DEFINITION FALIST-CONSISTENT))))))
@@ -458,7 +468,8 @@
                                (:DEFINITION VALID-RULEP-SK)
                                (:REWRITE RP-TERMP-IMPLIES-CDR-LISTP)
                                (:DEFINITION VALID-SC)
-                               (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
+; Obsolete (see Matt K. comment above):
+;                              (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
                                (:REWRITE
                                 ACL2::TRUE-LIST-LISTP-IMPLIES-ALWAYS$-TRUE-LISTP)
                                (:DEFINITION FALIST-CONSISTENT))))))
@@ -470,7 +481,8 @@
                                                    context iff-flg state))))
   :hints (("Goal"
            :in-theory (disable (:DEFINITION LEN)
-                               (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
+; Obsolete (see Matt K. comment above):
+;                              (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
                                (:REWRITE RP-EQUAL2-BINDINGS-1TO1-CONSP)
                                (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA4)
                                (:REWRITE DEFAULT-<-1)
@@ -511,7 +523,8 @@
 
   (local
    (defthm lemma1
-     (implies (rule-syntaxp rule)
+     (implies (and (rule-syntaxp rule)
+                   (rp-rule-rwp rule))
               (not (include-fnc (rp-lhs rule) 'if)))
      :hints (("Goal"
               :in-theory (e/d (rule-syntaxp) ())))))
@@ -519,6 +532,7 @@
   (local
    (defthm lemma2
      (implies (and (consp rules-for-term)
+                   (rp-rule-rw-listp rules-for-term)
                    (rule-list-syntaxp rules-for-term))
               (NOT (INCLUDE-FNC (RP-LHS (CAR RULES-FOR-TERM))
                                 'IF)))
@@ -531,6 +545,7 @@
       (declare (ignorable rules-rest rp-context))
       (implies (and rule
                     (rule-list-syntaxp rules-for-term)
+                    (rp-rule-rwp rule)
                     (valid-sc term a))
                (valid-sc-bindings bindings a)))
     :hints (("Goal"
@@ -644,8 +659,10 @@
 
 (defthm rp-rw-rule-aux-does-not-return-rule-with-iff-flg-when-iff-flg=nil
   (implies
-   (mv-nth 0
-           (rp-rw-rule-aux term rules-for-term context nil state))
+   (and (mv-nth 0
+                (rp-rw-rule-aux term rules-for-term context nil state))
+        (rp-rule-rwp (mv-nth 0
+                             (rp-rw-rule-aux term rules-for-term context nil state))))
    (not (rp-iff-flag
          (mv-nth 0
                  (rp-rw-rule-aux term rules-for-term context nil state)))))
@@ -679,7 +696,8 @@
                               (:REWRITE CAR-OF-EX-FROM-RP-IS-NOT-RP)
                               (:REWRITE VALID-SC-CONS)
                               (:DEFINITION INCLUDE-FNC)
-                              (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
+; Obsolete (see Matt K. comment above):
+;                             (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
                               (:REWRITE VALID-SC-CADR)
                               (:REWRITE EVL-OF-EXTRACT-FROM-RP-2)
                               (:TYPE-PRESCRIPTION INCLUDE-FNC)
@@ -781,7 +799,8 @@
                              (ex-from-rp-lemma1
 
                               (:DEFINITION ACL2::APPLY$-BADGEP)
-                              (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
+; Obsolete (see Matt K. comment above):
+;                             (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
                               (:REWRITE RP-TERM-LISTP-IS-TRUE-LISTP)
                               (:REWRITE NOT-INCLUDE-RP-MEANS-VALID-SC)
                               (:DEFINITION RP-TERMP))))))
@@ -841,7 +860,8 @@
                               (:DEFINITION TRUE-LISTP)
                               (:REWRITE VALID-SC-CONS)
                               (:REWRITE NOT-INCLUDE-RP)
-                              (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
+; Obsolete (see Matt K. comment above):
+;                             (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
                               (:REWRITE NOT-INCLUDE-RP-MEANS-VALID-SC)
                               (:REWRITE ACL2::O-P-O-INFP-CAR)
                               (:DEFINITION FALIST-CONSISTENT)
@@ -880,6 +900,7 @@
                                      (:DEFINITION FALIST-CONSISTENT-AUX)
                                      rp-does-lhs-match
                                      rp-match-lhs))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (defthm SHOULD-TERM-BE-IN-CONS-implies
@@ -944,7 +965,8 @@
   (defthm-rp-match-lhs
     (defthm rp-match-lhs-binds-all
       (implies (and (mv-nth 2 (rp-match-lhs term rule-lhs
-                                            context acc-bindings)))
+                                            context acc-bindings))
+                    (rp-termp rule-lhs))
                (all-vars-bound
                 rule-lhs
                 (mv-nth 1 (rp-match-lhs term rule-lhs
@@ -953,7 +975,8 @@
     (defthm rp-match-lhs-binds-all-subterms
       (implies (and (mv-nth 2 (rp-match-lhs-subterms subterms
                                                      sublhs
-                                                     context acc-bindings)))
+                                                     context acc-bindings))
+                    (rp-term-listp sublhs))
                (all-vars-bound-subterms
                 sublhs
                 (mv-nth 1 (rp-match-lhs-subterms subterms sublhs
@@ -965,7 +988,9 @@
   (local
    (in-theory (e/d (bind-bindings-aux
                     put-term-in-cons)
-                   ((:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
+                   (
+; Obsolete (see Matt K. comment above):
+;                   (:REWRITE SHOULD-TERM-BE-IN-CONS-LEMMA1)
                     (:REWRITE ACL2::O-P-O-INFP-CAR)
                     (:LINEAR ACL2::APPLY$-BADGEP-PROPERTIES . 1)
                     (:DEFINITION ACL2::APPLY$-BADGEP)
@@ -1481,6 +1506,7 @@
       (implies (and rule
                     (alistp a)
                     (rule-list-syntaxp rules-for-term)
+                    (rp-rule-rwp rule)
                     (rp-termp term))
                (and (equal (rp-evl (rp-apply-bindings (rp-lhs rule)
                                                       (rp-trans-bindings bindings))
@@ -1498,7 +1524,8 @@
                               rp-equal2-is-symmetric
                               no-free-variablep
                               VALID-RULESP-IMPLIES-RULE-LIST-SYNTAXP
-                              SHOULD-TERM-BE-IN-CONS-LEMMA1
+; Obsolete (see Matt K. comment above):
+;                             SHOULD-TERM-BE-IN-CONS-LEMMA1
                               INCLUDE-FNC
                               rule-syntaxp
                               weak-custom-rewrite-rule-p
