@@ -42,28 +42,7 @@
 
 (include-book "projects/apply/top" :dir :system)
 
-(defstobj rp-state
-  (show-used-rules-flg :type (satisfies booleanp) :initially nil)
-  (count-used-rules-flg :type (satisfies booleanp) :initially nil)
-  (rules-used :type (satisfies alistp) :initially nil)
 
-  (rp-brr :type (satisfies booleanp) :initially nil)
-  (rw-stack-size :type (satisfies integerp) :initially 0)
-  (rw-stack :type (satisfies alistp) :initially nil)
-  (rule-frame-cnts :type (satisfies alistp) :initially nil)
-
-  (rw-step-limit :type (unsigned-byte 58) :initially 100000)
-
-  (not-simplified-action :type (satisfies symbolp) :initially :error))
-
-(defund rp-state-new-run (rp-state)
-  (declare (xargs :stobjs (rp-state)))
-  (b* ((- (fast-alist-free (rules-used rp-state)))
-       (rp-state (update-rules-used nil rp-state))
-       (rp-state (update-rw-stack-size 0 rp-state))
-       (rp-state (update-rw-stack nil rp-state))
-       (rp-state (update-rule-frame-cnts nil rp-state)))
-    rp-state))
 
 (progn
   (defun rule-result-comperator (x y)
@@ -96,14 +75,13 @@ which submits an event.
  :short "Sets whether or not RP-Rewriter should print used rules when rewriting
  a conjecture."
  :long
- "<p>(show-rules @('<nil-OR-t-OR-:cnt>')) submits an event that changes
- RP-Rewriter's behaviour on saving and printing used rules. For best
- performance, it is set to nil by default. When set to t, it prints rule in a
- fashion similar to the built-in rewriter but only differently for
- meta-rules. When set to :cnt, it also attaches a number to each rune showing
- how many times they are used, and how many times they failed due to unrelieved
- hypotheses. These entries are saved in rules-used field of stobj rp::rp-state. </p>"
- )
+ "<p>(show-rules  @('<nil-OR-t-OR-:cnt>'))   submits  an  event   that  changes
+ RP-Rewriter's behaviour on  saving and printing used rules. When  set to t, it
+ prints rule in a fashion similar to the built-in rewriter but only differently
+ for meta-rules.  When set  to :cnt,  it also  attaches a  number to  each rune
+ showing how many  times they are used,  and how many times they  failed due to
+ unrelieved hypotheses.  These entries are  saved in rules-used field  of stobj
+ rp::rp-state. </p>" )
 
 
 (encapsulate
@@ -131,117 +109,101 @@ which submits an event.
         (mv nil `(value-triple `(not-simplifed-action ,',',flg)) state
             rp-state))))
 
-  (defund rp-stat-add-to-rules-used (rule failed rp-state)
-    (declare (xargs :guard (and (weak-custom-rewrite-rule-p rule))
+  (defund rp-stat-add-to-rules-used (rule failed ex-counterpart-flg rp-state)
+    (declare (xargs :guard (or ex-counterpart-flg
+                               (weak-custom-rewrite-rule-p rule))
+                    :guard-hints (("Goal"
+                                   :in-theory (e/d () (rp-statep
+                                                       COUNT-USED-RULES-FLG
+                                                       SHOW-USED-RULES-FLG
+                                                       weak-custom-rewrite-rule-p))))
                     :stobjs (rp-state)))
     (if (show-used-rules-flg rp-state)
         (cond ((count-used-rules-flg rp-state)
-               (b* ((old-lst (rules-used rp-state))
-                    (rule-name (rp-rune rule))
-                    (rule-name
+               (b* ((rune (if ex-counterpart-flg
+                              `(:executable-counterpart ,rule)
+                            (rp-rune rule)))
+                    (rune
                      (cond (failed
-                            (cons failed rule-name))
-                           (t rule-name)))
-                    (entry (hons-get rule-name old-lst))
-                    (val (if (consp entry) (1+ (nfix (cdr entry))) 1)))
-                 (update-rules-used
-                  (hons-acons rule-name
-                              val
-                              old-lst)
-                  rp-state)))
+                            (cons rune failed))
+                           (t rune)))
+                    (rp-state
+                     (if (not (rules-used-boundp rune rp-state))
+                         (rules-used-put 'all-rules
+                                         (cons rune
+                                               (rules-used-get 'all-rules
+                                                               rp-state))
+                                         rp-state)
+                       rp-state))
+                         
+                    (val (1+ (nfix (rules-used-get rune rp-state)))))
+                 (rules-used-put rune val rp-state)))
               (t
-               (b* ((old-lst (rules-used rp-state))
-                    (rule-name (rp-rune rule))
-                    (entry (hons-get rule-name old-lst))
-                    ((when entry) rp-state))
-                 (update-rules-used (hons-acons rule-name nil old-lst)
-                                    rp-state))))
+               (b* (((when failed)
+                     rp-state)
+                    (rune1 (if ex-counterpart-flg
+                               rule
+                             (rp-rune rule)))
+                    ((when (rules-used-boundp rune1  rp-state))
+                     rp-state)
+                    (rune2 (if ex-counterpart-flg
+                               `(:executable-counterpart ,rule)
+                             (rp-rune rule)))
+                    (rp-state
+                     (rules-used-put 'all-rules
+                                     (cons rune2
+                                           (rules-used-get 'all-rules
+                                                           rp-state))
+                                     rp-state))
+                    (rp-state (rules-used-put rune1 nil rp-state)))
+                 rp-state)))
       rp-state))
 
-  (defund rp-stat-add-to-rules-used-ex-cnt (rule-name rp-state)
+
+  (defund rp-state-print-rules-used-aux (all-rules rp-state)
     (declare (xargs :stobjs (rp-state)))
-    (if (show-used-rules-flg rp-state)
-        (cond
-         ((count-used-rules-flg rp-state)
-          (b* ((old-lst (rules-used rp-state))
-               (rule-name `(:executable-counterpart ,rule-name))
-               (entry (hons-get rule-name old-lst))
-               (val (if (consp entry) (1+ (nfix (cdr entry))) 1)))
-            (update-rules-used
-             (hons-acons rule-name
-                         val
-                         old-lst)
-             rp-state)))
-         (t
-          (b* ((old-lst (rules-used rp-state))
-               (rule-name `(:executable-counterpart ,rule-name))
-               (entry (hons-get rule-name old-lst))
-               ((when entry) rp-state))
-            (update-rules-used (hons-acons rule-name nil old-lst)
-                               rp-state))))
-      rp-state))
-
-  (defund rp-stat-add-to-rules-used-meta-cnt (meta-rule rp-state)
-    (declare (xargs :stobjs (rp-state)
-                    :guard (weak-rp-meta-rule-rec-p meta-rule)))
-    (if (show-used-rules-flg rp-state)
-        (cond
-         ((count-used-rules-flg rp-state)
-          (b* ((old-lst (rules-used rp-state))
-               (rule-name `(:meta ,(rp-meta-fnc meta-rule)
-                                  :trig ,(rp-meta-trig-fnc meta-rule)))
-               (entry (hons-get rule-name old-lst))
-               (val (if (consp entry) (1+ (nfix (cdr entry))) 1)))
-            (update-rules-used
-             (hons-acons rule-name
-                         val
-                         old-lst)
-             rp-state)))
-         (t
-          (b* ((old-lst (rules-used rp-state))
-               (rule-name `(:meta ,(rp-meta-fnc meta-rule)
-                                  :trig ,(rp-meta-trig-fnc meta-rule)))
-               (entry (hons-get rule-name old-lst))
-               ((when entry) rp-state))
-            (update-rules-used (hons-acons rule-name nil old-lst)
-                               rp-state))))
-      rp-state))
-
+    (if (atom all-rules)
+        nil
+      (acons (car all-rules)
+             (rules-used-get (car all-rules) rp-state)
+             (rp-state-print-rules-used-aux (cdr all-rules) rp-state))))
+  
   (defund rp-state-print-rules-used (rp-state)
-    (declare (xargs :stobjs (rp-state)))
+    (declare (xargs :stobjs (rp-state)
+                    :guard-hints (("Goal"
+                                   :in-theory (e/d () (rp-statep
+                                                       COUNT-USED-RULES-FLG
+                                                       SHOW-USED-RULES-FLG
+                                                       weak-custom-rewrite-rule-p))))))
     (if (show-used-rules-flg rp-state)
-        (cw "Rules used so far: ~p0 ~%"
-            (let* ((alist (rules-used rp-state)))
+        (cw "~%List of rules used: ~p0 ~%"
+            (let* ((all-rules (true-list-fix (rules-used-get 'all-rules rp-state))))
               (if (count-used-rules-flg rp-state)
                   (merge-comperator-sort
-                   (b* ((alist (fast-alist-clean alist))
-                        (alist (if (true-listp alist) alist nil)))
-                     alist)
+                   (true-list-fix (rp-state-print-rules-used-aux all-rules rp-state))
                    'rule-result-comperator)
-                #|(acl2::merge-sort-lexorder
-                (b* ((alist (fast-alist-clean alist))
-                (alist (if (true-listp alist) alist nil)))
-                alist))||#
                 (acl2::merge-sort-lexorder
-                 (strip-cars alist)))))
+                 all-rules))))
       nil)))
 
 (defund rp-state-push-to-try-to-rw-stack (rule var-bindings rp-context rp-state)
   (declare (xargs :stobjs (rp-state)
                   :guard (WEAK-CUSTOM-REWRITE-RULE-P RULE)))
-  (if (rp-brr rp-state)
+  (if (and (rp-brr rp-state)
+           (not (rp-rule-metap rule)))
       (b* ((old-rw-stack (rw-stack rp-state))
            (index (rw-stack-size rp-state))
-           (new-rw-stack (acons index
-                                (list
-                                 (list ':type 'trying)
-                                 (list ':rune (rp-rune rule))
-                                 (list ':lhs (rp-lhs rule))
-                                 (list ':rhs (rp-rhs rule))
-                                 (list ':hyp (rp-hyp rule))
-                                 (list ':context rp-context)
-                                 (list ':var-bindings var-bindings))
-                                old-rw-stack))
+           (new-rw-stack
+            (acons index
+                   (list
+                    (list ':type ':trying)
+                    (list ':rune (rp-rune rule))
+                    (list ':lhs (rp-lhs rule))
+                    (list ':rhs (rp-rhs rule))
+                    (list ':hyp (rp-hyp rule))
+                    (list ':context rp-context)
+                    (list ':var-bindings var-bindings))
+                   old-rw-stack))
            (rp-state (update-rw-stack new-rw-stack rp-state))
            (rp-state (update-rw-stack-size (1+ index) rp-state)))
         (mv index rp-state))
@@ -249,15 +211,16 @@ which submits an event.
 
 (defund rp-state-push-meta-to-rw-stack (meta-rule old-term new-term rp-state)
   (declare (xargs :stobjs (rp-state)
-                  :guard (weak-rp-meta-rule-rec-p meta-rule)))
+                  :guard (and (WEAK-CUSTOM-REWRITE-RULE-P meta-rule)
+                              (rp-rule-metap meta-rule))))
   (if (rp-brr rp-state)
       (b* ((old-rw-stack (rw-stack rp-state))
            (index (rw-stack-size rp-state))
            (new-rw-stack (acons index
                                 (list
-                                 (list ':type 'meta-applied)
-                                 (list ':meta-fnc (rp-meta-fnc meta-rule))
-                                 (list ':trig-fnc (rp-meta-trig-fnc meta-rule))
+                                 (list ':type ':meta-applied)
+                                 (list ':meta-fnc (rp-rule-meta-fnc meta-rule))
+                                 (list ':trig-fnc (rp-rule-trig-fnc meta-rule))
                                  (list ':new-term new-term)
                                  (list ':old-term old-term))
                                 old-rw-stack))
@@ -286,10 +249,10 @@ which submits an event.
            (old-rw-stack (rw-stack rp-state))
            (new-rw-stack (acons index
                                 (list*
-                                 (cons 'type failed)
-                                 (list 'rune rune)
-                                 (if new-term (list (list 'new-term new-term)
-                                                    (list 'old-term old-term))
+                                 (cons ':type failed)
+                                 (list ':rune rune)
+                                 (if new-term (list (list ':new-term new-term)
+                                                    (list ':old-term old-term))
                                    nil))
                                 old-rw-stack)))
         (update-rw-stack new-rw-stack rp-state))
@@ -311,14 +274,14 @@ which submits an event.
   (if (atom keys)
       nil
     (acons (car keys)
-           (cond ((or (equal (car keys) 'new-term)
-                      (equal (car keys) 'old-term)
-                      (equal (car keys) 'rhs)
-                      (equal (car keys) 'lhs)
-                      (equal (car keys) 'hyp))
+           (cond ((or (equal (car keys) ':new-term)
+                      (equal (car keys) ':old-term)
+                      (equal (car keys) ':rhs)
+                      (equal (car keys) ':lhs)
+                      (equal (car keys) ':hyp))
                   (list (untranslate (cadr (assoc-eq (car keys) alist)) t (w
                                                                            state))))
-                 ((equal (car keys) 'var-bindings)
+                 ((equal (car keys) ':var-bindings)
                   (list (untranslate-var-bindinds (cadr (assoc-eq (car keys) alist)) t
                                                   (w state))))
                  (t (cdr (assoc-eq (car keys) alist))))
@@ -337,11 +300,11 @@ which submits an event.
   (if (atom rw-stack)
       nil
     (b* ((current (car rw-stack))
-         (type (cdr (assoc-equal 'type (cdr current))))
-         ((unless (or (eq type 'success)
-                      (eq type 'meta-applied)))
+         (type (cdr (assoc-equal ':type (cdr current))))
+         ((unless (or (eq type ':success)
+                      (eq type ':meta-applied)))
           (search-source-in-stack (cdr rw-stack) term)))
-      (if (subtermp (cadr (assoc-equal 'new-term (cdr current))) term)
+      (if (subtermp (cadr (assoc-equal ':new-term (cdr current))) term)
           (car current)
         (search-source-in-stack (cdr rw-stack) term)))))
 
@@ -363,11 +326,11 @@ which submits an event.
         state
       (b* ((entry (car rw-stack))
            ((when (and only
-                       (not (or (member-equal (cdr (assoc-equal 'type (cdr entry)))
+                       (not (or (member-equal (cdr (assoc-equal ':type (cdr entry)))
                                               only)
-                                (member-equal (cadr (assoc-equal 'rune (cdr entry)))
+                                (member-equal (cadr (assoc-equal ':rune (cdr entry)))
                                               only)
-                                (member-equal (cadr (assoc-equal 'meta-fnc (cdr entry)))
+                                (member-equal (cadr (assoc-equal ':meta-fnc (cdr entry)))
                                               only)))))
             (pp-rw-stack-aux (cdr rw-stack)
                              omit
@@ -376,11 +339,11 @@ which submits an event.
                              untranslate
                              search-source
                              state))
-           ((when (or (member-equal (cdr (assoc-equal 'type (cdr entry)))
+           ((when (or (member-equal (cdr (assoc-equal ':type (cdr entry)))
                                     omit)
-                      (member-equal (cadr (assoc-equal 'rune (cdr entry)))
+                      (member-equal (cadr (assoc-equal ':rune (cdr entry)))
                                     omit)
-                      (member-equal (cadr (assoc-equal 'meta-fnc (cdr entry)))
+                      (member-equal (cadr (assoc-equal ':meta-fnc (cdr entry)))
                                     omit)))
             (pp-rw-stack-aux (cdr rw-stack)
                              omit
@@ -399,23 +362,23 @@ which submits an event.
                                           omit)
                          (cdr entry))))
            (sub-entries (if (and search-source
-                                 (equal (cdr (assoc-equal 'type (cdr entry))) 'trying))
+                                 (equal (cdr (assoc-equal ':type (cdr entry))) ':trying))
                             (append
                              sub-entries
-                             (list (cons 'var-bindings-sources
+                             (list (cons ':var-bindings
                                          (search-source-in-stack-var-bindings
                                           rw-stack
-                                          (cadr (assoc-equal 'var-bindings
+                                          (cadr (assoc-equal ':var-bindings
                                                              (cdr entry)))))))
                           sub-entries))
            (sub-entries (if (and search-source
-                                 (equal (cdr (assoc-equal 'type (cdr entry))) 'meta-applied))
+                                 (equal (cdr (assoc-equal ':type (cdr entry))) ':meta-applied))
                             (append
                              sub-entries
-                             (list (cons 'old-term-source
+                             (list (cons ':old-term
                                          (search-source-in-stack
                                           rw-stack
-                                          (cadr (assoc-equal 'old-term
+                                          (cadr (assoc-equal ':old-term
                                                              (cdr entry)))))))
                           sub-entries))
            (state (fms "~p0~%"
@@ -451,8 +414,8 @@ which submits an event.
                             (frames-offset '0)
                             (omit 'nil)
                             (only 'nil)
-                            (evisc-tuple ''(NIL 3 4 NIL))
-                            (untranslate 't)
+                            (evisc-tuple ''(NIL 6 8 NIL))
+                            (untranslate 'nil)
                             (search-source 'nil))
     (declare (xargs :stobjs (rp-state state)
                     :mode :program))
@@ -515,7 +478,9 @@ untranslate). Default value = t.
 
 (define increment-rw-stack-size (rp-state)
   (declare (xargs :stobjs (rp-state)))
-  (update-rw-stack-size (1+ (rw-stack-size rp-state)) rp-state))
+  (if (rp-brr rp-state)
+      (update-rw-stack-size (1+ (rw-stack-size rp-state)) rp-state)
+    rp-state))
 
 (in-theory (disable rp-statep))
 
