@@ -735,7 +735,7 @@
 
 (defun formula-p$ (formula a$)
 
-; We recognize null-terminated fast-alists (applicative hash tables), such that
+; We recognize nil-terminated fast-alists (applicative hash tables), such that
 ; that every index is bound to a clause or *deleted-clause*.
 
   (declare (xargs :stobjs a$ :guard (a$p a$)))
@@ -2125,7 +2125,8 @@
                 (clause-max-var-1 (cdr clause))))))
 
 (defthm clause-max-var-is-clause-max-var-1
-  (implies (and (<= 0 acc)
+  (implies (and (integerp acc)
+                (<= 0 acc)
                 (clausep clause))
            (equal (clause-max-var clause acc)
                   (max acc (clause-max-var-1 clause)))))
