@@ -6,7 +6,8 @@
                                                ("alu" . ALU_VAR)
                                                ("function" . FUNCTION0)))))
 
-(INCLUDE-BOOK "/Users/kaufmann/projects/l3-to-acl2/translator/l3")
+(INCLUDE-BOOK "projects/translators/l3-to-acl2/translator/l3"
+              :DIR :SYSTEM)
 
 (CONSTRUCT FUNCT
            (FADD FSUB FINC FDEC FAND FOR FXOR FRESERVED))
@@ -87,7 +88,7 @@
                            ('RCY1 (ASH A -1))
                            ('RCY8 (ASH A -8))
                            ('RCY16 (ASH A -16))
-                           (& (ASSERT! NIL (ARB (UNSIGNED-BYTE 32))))))
+                           (& (IMPOSSIBLE (ARB (UNSIGNED-BYTE 32))))))
 
 (DEFUN-STRUCT ALU
               (((FUNC (TYPE-FUNCT FUNC))
@@ -124,7 +125,7 @@
                  (LET ((ST$ (UPDATE-PCTR (N+ 10 (PCTR ST$) (IF (INRDY ST$) 2 1))
                                          ST$)))
                       (MV (UNIT-VALUE) ST$)))
-                (& (ASSERT! NIL (MV (ARB UTY) ST$)))))
+                (& (IMPOSSIBLE (MV (ARB UTY) ST$)))))
 
 (DEFUN-STRUCT NORM
               (((FUNC (TYPE-FUNCT FUNC))
@@ -301,7 +302,7 @@
                            (('OUT V6) (DFN-OUT V6 ST$))
                            (('STOREDM V7) (DFN-STOREDM V7 ST$))
                            (('STOREIM V8) (DFN-STOREIM V8 ST$))
-                           (& (ASSERT! NIL (MV (ARB UTY) ST$)))))
+                           (& (IMPOSSIBLE (MV (ARB UTY) ST$)))))
 
 (DEFUN-STRUCT
  DECODE ((OPC (UNSIGNED-BYTE-P 32 OPC)))
@@ -342,7 +343,7 @@
                      (6 (CALL-CONSTRUCTOR INSTRUCTION
                                           JUMP (TUPLE FUNC SHIFT RW RA RB)))
                      (7 'RESERVEDINSTR)
-                     (& (ASSERT! NIL (ARB INSTRUCTION))))))))
+                     (& (IMPOSSIBLE (ARB INSTRUCTION))))))))
 
 (DEFUN-STRUCT NEXT (ST$)
               (DECLARE (XARGS :STOBJS ST$))
@@ -387,7 +388,7 @@
                             (ENC (TUPLE (TUPLE FUNC SHIFT 'SKIPNEVER RW RA RB)
                                         6)))
                            ('RESERVEDINSTR 7)
-                           (& (ASSERT! NIL (ARB (UNSIGNED-BYTE 32))))))
+                           (& (IMPOSSIBLE (ARB (UNSIGNED-BYTE 32))))))
 
 (DEFUN-STRUCT LOADIM
               (((A (UNSIGNED-BYTE-P 10 A))
@@ -399,7 +400,7 @@
                            ((H . T_VAR)
                             (LET ((ST$ (UPDATE-IMI A (ENCODE H) ST$)))
                                  (LOADIM (TUPLE (N+ 10 A 1) T_VAR) ST$)))
-                           (& (ASSERT! NIL (MV (ARB UTY) ST$)))))
+                           (& (IMPOSSIBLE (MV (ARB UTY) ST$)))))
 
 (DEFUN-STRUCT INITIALIZE
               ((P (TYPE-INSTRUCTION-LIST P)) ST$)
