@@ -138,17 +138,16 @@
  mbe) and its friends like @(see acl2::mbt), and @(see acl2::assert*),
  @(see acl2::stobj), and @(see acl2::defabsstobj).</p>
 
- <p>ACL2 features like abstract stobjs and @('assert*') were a response
- by the ACL2 authors to provide mechanisms to mitigate the complexity
- of reasoning vs. execution efficiency that came up over the course of
- this project.  This x86 ISA model uses abstract stobjs to layer the
- state of the x86 machine such that the lower layer \(i.e., the
- concrete stobj\, see @(see concrete-state)\) can be optimized for
- execution efficiency and the upper layer \(i.e., the abstract stobj,
- see @(see abstract-state)\) can be optimized for reasoning
- efficiency, while a correspondence theorem is proved to hold between
- these two layers at all times.  Therefore, we get the benefit of both
- fast execution and effective reasoning.</p>
+ <p>ACL2 features like abstract stobjs and @('assert*') were a
+ response by the ACL2 authors to provide mechanisms to mitigate the
+ complexity of reasoning vs. execution efficiency that came up over
+ the course of this project.  This x86 ISA model uses abstract stobjs
+ to layer the state of the x86 machine such that the lower layer
+ \(i.e., the concrete stobj\) can be optimized for execution
+ efficiency and the upper layer \(i.e., the abstract stobj) can be
+ optimized for reasoning efficiency, while a correspondence theorem is
+ proved to hold between these two layers at all times.  Therefore, we
+ get the benefit of both fast execution and effective reasoning.</p>
 
  <h3>Modes of Operation</h3>
 
@@ -301,7 +300,7 @@
 
 (defxdoc Publications
   :parents (x86isa)
-  :short "Technical publications related to @('x86isa')"
+  :short "Technical publications related to these @('x86isa') books"
   :long "<p>From the oldest to the newest:</p>
  <ol>
 
@@ -346,6 +345,19 @@
   the x86 ISA. In the Fifteenth International Workshop on the ACL2 Theorem
   Prover and Its Applications (ACL2 Workshop), 2018</li>
 
+ <li>Alessandro Coglio and Shilpi Goel. Adding 32-bit Mode to the ACL2 Model of
+  the x86 ISA. In the Fifteenth International Workshop on the ACL2 Theorem
+  Prover and Its Applications (ACL2 Workshop), 2018</li>
+
+ <li>Shilpi Goel and Rob Sumners. Using @('x86isa') for Microcode
+ Verification. In the Workshop on Instruction Set Architecture
+ Specification (SpISA), 2019.</li>
+
+ <li>Shilpi Goel, Anna Slobodova, Rob Sumners, and Sol
+ Swords. Verifying x86 Instruction Implementations. In the proceedings
+ of the 9th ACM SIGPLAN International Conference on Certified Programs
+ and Proofs (CPP), 2020.</li>
+
  </ol>")
 
 (defxdoc Contributors
@@ -482,10 +494,7 @@
 
 (xdoc::order-subtopics
  machine
- (concrete-state
-  concrete-memory
-  abstract-state
-  state-field-theorems
+ (x86isa-state
   app-view
   rflag-specifications
   register-readers-and-writers
