@@ -64,7 +64,9 @@
    :rule-classes (:rewrite :type-prescription)))
 
 (def-inst x86-sal/sar/shl/shr/rcl/rcr/rol/ror
+
   :guard (not (equal (modr/m->reg modr/m) 6))
+
   :guard-hints (("Goal"
                  :in-theory (e/d ()
                                  (unsigned-byte-p
@@ -87,19 +89,19 @@
   "<p>
   Op/En: MI<br/>
   C0/0: ROL r/m8, imm8<br/>
-  C0/1: ROR r/m8,imm8<br/>
+  C0/1: ROR r/m8, imm8<br/>
   C0/2: RCL r/m8, imm8<br/>
   C0/3: RCR r/m8, imm8<br/>
-  C0/4: SAL/SHL r/m8 imm8<br/>
-  C0/5: SHR r/m8 imm8<br/>
-  C0/7: SAR r/m8 imm8<br/>
-  C1/0: ROL r/m16, r/m32, or r/m64, imm8<br/>
-  C1/1: ROR r/m16, r/m32, or r/m64, imm8<br/>
-  C1/2: RCL r/m16, r/m32, or r/m64, imm8<br/>
-  C1/3: RCR r/m16, r/m32, or r/m64, imm8<br/>
-  C1/4: SAL/SHL r/m8 r/m16 or r/m64 imm8<br/>
-  C1/5: SHR r/m16 r/m32 or r/m64 imm8<br/>
-  C1/7: SAR r/m16 r/m32 or r/m64 imm8<br/>
+  C0/4: SAL/SHL r/m8, imm8<br/>
+  C0/5: SHR r/m8, imm8<br/>
+  C0/7: SAR r/m8, imm8<br/>
+  C1/0: ROL r/m16/32/64, imm8<br/>
+  C1/1: ROR r/m16/32/64, imm8<br/>
+  C1/2: RCL r/m16/32/64, imm8<br/>
+  C1/3: RCR r/m16/32/64, imm8<br/>
+  C1/4: SAL/SHL r/m16/32/64, imm8<br/>
+  C1/5: SHR r/m16/32/64, imm8<br/>
+  C1/7: SAR r/m16/32/64. imm8<br/>
   </p>
 
   <p>
@@ -108,16 +110,16 @@
   D0/1: ROR r/m8, 1<br/>
   D0/2: RCL r/m8, 1<br/>
   D0/3: RCR r/m8, 1<br/>
-  D0/4: SAL/SHL r/m8 1<br/>
-  D0/5: SHR r/m8 1<br/>
-  D0/7: SAR r/m8 1<br/>
-  D1/0: ROL r/m16, r/m32, or r/m64, 1<br/>
-  D1/1: ROR r/m16, r/m32, or r/m64, 1<br/>
-  D1/2: RCL r/m16, r/m32, or r/m64, 1<br/>
-  D1/3: RCR r/m16, r/m32, or r/m64, 1<br/>
-  D1/4: SAL/SHL r/m16 r/m32 or r/m64 1<br/>
-  D1/5: SHR r/m16 r/m32 or r/m64 1<br/>
-  D1/7: SAR r/m16 r/m32 or r/m64 1<br/>
+  D0/4: SAL/SHL r/m8, 1<br/>
+  D0/5: SHR r/m8, 1<br/>
+  D0/7: SAR r/m8, 1<br/>
+  D1/0: ROL r/m16/32/64, 1<br/>
+  D1/1: ROR r/m16/32/64, 1<br/>
+  D1/2: RCL r/m16/32/64, 1<br/>
+  D1/3: RCR r/m16/32/64, 1<br/>
+  D1/4: SAL/SHL r/m16/32/64, 1<br/>
+  D1/5: SHR r/m16/32/64, 1<br/>
+  D1/7: SAR r/m16/32/64, 1<br/>
   </p>
 
   <p>
@@ -126,27 +128,23 @@
   D2/1: ROR r/m8, CL<br/>
   D2/2: RCL r/m8, CL<br/>
   D2/3: RCR r/m8, CL<br/>
-  D2/4: SAL/SHL r/m8 CL<br/>
-  D2/5: SHR r/m8 CL<br/>
+  D2/4: SAL/SHL r/m8, CL<br/>
+  D2/5: SHR r/m8, CL<br/>
   D2/7: SAR r/m8, CL<br/>
-  D3/0: ROL r/m16, r/m32, or r/m64, CL<br/>
-  D3/1: ROR r/m16, r/m32, or r/m64, CL<br/>
-  D3/2: RCL r/m16, r/m32, or r/m64, CL<br/>
-  D3/3: RCR r/m16, r/m32, or r/m64, CL<br/>
-  D3/4: SAL/SHL r/m16 r/m32 or r/m64 CL<br/>
-  D3/5: SHR r/m16 r/m32 or r/m64 CL<br/>
-  D3/7: SAR r/m16 r/m32 or r/m64 CL<br/>
+  D3/0: ROL r/m16/32/64, CL<br/>
+  D3/1: ROR r/m16/32/64, CL<br/>
+  D3/2: RCL r/m16/32/64, CL<br/>
+  D3/3: RCR r/m16/32/64, CL<br/>
+  D3/4: SAL/SHL r/m16/32/64, CL<br/>
+  D3/5: SHR r/m16/32/64, CL<br/>
+  D3/7: SAR r/m16/32/64, CL<br/>
   </p>"
 
+  :modr/m t
+
   :body
 
-  (b* ((ctx 'x86-sal/sar/shl/shr/rcl/rcr/rol/ror)
-
-       (r/m (modr/m->r/m modr/m))
-       (mod (modr/m->mod modr/m))
-       (reg (modr/m->reg modr/m))
-
-       (p2 (prefixes->seg prefixes))
+  (b* ((p2 (prefixes->seg prefixes))
        (p4? (equal #.*addr-size-override* (prefixes->adr prefixes)))
 
        (byte-operand? (or (equal opcode #xC0)
@@ -264,3 +262,166 @@
     x86))
 
 ;; ======================================================================
+;; INSTRUCTION: SHLD/SHRD
+;; ======================================================================
+
+(def-inst x86-shld/shrd
+
+  :returns (x86 x86p :hyp (x86p x86))
+
+  :parents (one-byte-opcodes)
+
+  :short "Double-precision shift left or right."
+
+  :long
+  "<p>
+   Op/En: MRI<br/>
+   0F A4: SHLD r/m16, r16, imm8<br/>
+   0F A4: SHLD r/m32, r32, imm8<br/>
+   0F A4: SHLD r/m64, r64, imm8<br/>
+   </p>
+
+   <p>
+   Op/En: MRC<br/>
+   0F A5: SHLD r/m16, r16, CL<br/>
+   0F A5: SHLD r/m32, r32, CL<br/>
+   0F A5: SHLD r/m64, r64, CL<br/>
+   </p>
+
+   <p>
+   Op/En: MRI<br/>
+   0F AC: SHRD r/m16, r16, imm8<br/>
+   0F AC: SHRD r/m32, r32, imm8<br/>
+   0F AC: SHRD r/m64, r64, imm8<br/>
+   </p>
+
+   <p>
+   Op/En: MRC<br/>
+   0F AD: SHRD r/m16, r16, CL<br/>
+   0F AD: SHRD r/m32, r32, CL<br/>
+   0F AD: SHRD r/m64, r64, CL<br/>
+   </p>"
+
+  :modr/m t
+
+  :body
+
+  (b* ((p2 (prefixes->seg prefixes))
+       (p4? (equal #.*addr-size-override* (prefixes->adr prefixes)))
+
+       ((the (integer 2 8) operand-size)
+        (select-operand-size proc-mode
+                             nil ; not a byte operand
+                             rex-byte
+                             nil ; not an immediate operand
+                             prefixes
+                             nil ; no 64-bit default in 64-bit mode
+                             nil ; don't ignore REX in 64-bit mode
+                             nil ; don't ignore P3 in 64-bit mode
+                             x86))
+
+       (seg-reg (select-segment-register proc-mode p2 p4? mod r/m sib x86))
+
+       ;; read destination operand:
+
+       (inst-ac? t)
+       ((mv flg dst-value increment-rip-by dst-addr x86)
+        (x86-operand-from-modr/m-and-sib-bytes proc-mode
+                                               #.*gpr-access*
+                                               operand-size
+                                               inst-ac?
+                                               nil ; not memory pointer operand
+                                               seg-reg
+                                               p4?
+                                               temp-rip
+                                               rex-byte
+                                               r/m
+                                               mod sib
+                                               1 ; imm8
+                                               x86))
+       ((when flg) (!!ms-fresh :x86-operand-from-modr/m-and-sib-bytes flg))
+
+       ((mv flg (the (signed-byte #.*max-linear-address-size*) temp-rip))
+        (add-to-*ip proc-mode temp-rip increment-rip-by x86))
+       ((when flg) (!!ms-fresh :rip-increment-error flg))
+
+       ;; read source operand:
+
+       (src-value (rgfi-size operand-size
+                             (reg-index reg rex-byte #.*r*)
+                             rex-byte
+                             x86))
+
+       ;; read count operand:
+
+       ((mv flg count x86)
+        (case opcode
+          ((#xA4 #xAC) (rme-size-opt proc-mode 1 temp-rip #.*cs* :x nil x86))
+          ((#xA5 #xAD) (mv nil (rr08 *rcx* rex-byte x86) x86))
+          (otherwise (mv nil 0 x86)))) ; unreachable
+       ((when flg) (!!ms-fresh :rme-size-error flg))
+
+       ((mv flg (the (signed-byte #.*max-linear-address-size*) temp-rip))
+        (case opcode
+          ((#xA4 #xAC) (add-to-*ip proc-mode temp-rip 1 x86))
+          ((#xA5 #xAD) (mv nil temp-rip))
+          (otherwise (mv nil 0)))) ; unreachable
+       ((when flg) (!!ms-fresh :rip-increment-error flg))
+
+       ;; check instruction length now that we have read all of it:
+
+       (badlength? (check-instruction-length start-rip temp-rip 0))
+       ((when badlength?)
+        (!!fault-fresh :gp 0 :instruction-length badlength?)) ;; #GP(0)
+
+       ;; mask count according to pseudocode (the text only mentions masking
+       ;; when CL is used, but the pseudocode masks also if imm8 is used):
+
+       (count-mask (if (logbitp #.*w* rex-byte)
+                       #x3f
+                     #x1f))
+       (count (logand count-mask count))
+
+       ;; compute result and flags:
+
+       (input-rflags (the (unsigned-byte 32) (rflags x86)))
+
+       ((mv result
+            result-undefined?
+            (the (unsigned-byte 32) output-rflags)
+            (the (unsigned-byte 32) undefined-flags))
+        (case opcode
+          ((#xA4 #xA5) (shld-spec
+                        operand-size dst-value src-value count input-rflags))
+          ((#xAC #xAD) (shrd-spec
+                        operand-size dst-value src-value count input-rflags))
+          (otherwise (mv 0 nil 0 0)))) ; unreachable
+
+       ((mv result x86)
+        (if result-undefined?
+            (undef-read x86)
+          (mv result x86)))
+
+       ;; update the state:
+
+       (x86 (write-user-rflags output-rflags undefined-flags x86))
+
+       ((mv flg x86)
+        (x86-operand-to-reg/mem proc-mode
+                                operand-size
+                                inst-ac?
+                                nil ;; not memory pointer operand
+                                (trunc operand-size result) ; TODO: remove trunc
+                                seg-reg
+                                dst-addr
+                                rex-byte
+                                r/m
+                                mod
+                                x86))
+       ((when flg) (!!ms-fresh :x86-operand-to-reg/mem flg))
+
+       (x86 (write-*ip proc-mode temp-rip x86)))
+
+    x86))
+
+;; ======================================================================
