@@ -42,8 +42,10 @@
 (local (include-book "centaur/bitops/ihs-extensions" :dir :system))
 (local (include-book "centaur/bitops/signed-byte-p" :dir :system))
 
-(local (in-theory (e/d (multiple-of-8-disjoint-with-addr-range-and-open-qword-paddr-list-to-member-p)
-                       (unsigned-byte-p signed-byte-p))))
+(local
+ (in-theory
+  (e/d (multiple-of-8-disjoint-with-addr-range-and-open-qword-paddr-list-to-member-p)
+       (unsigned-byte-p signed-byte-p))))
 
 ;; ======================================================================
 
@@ -117,6 +119,28 @@
                   (mv t 0 x86)))
   :hints (("Goal" :in-theory (e/d* (ia32e-la-to-pa-pml4-table) ()))))
 
+
+(defthm xlate-equiv-entries-and-ia32e-pml4ebits->pdpt
+  (implies (xlate-equiv-entries e-1 e-2)
+           (equal (ia32e-pml4ebits->pdpt e-1)
+                  (ia32e-pml4ebits->pdpt e-2)))
+  :hints (("goal" :in-theory (e/d* (ia32e-pml4ebits->pdpt
+                                    ia32e-pml4ebits-fix
+                                    xlate-equiv-entries
+                                    ia32e-page-tablesbits->xd
+                                    ia32e-page-tablesbits->res2
+                                    ia32e-page-tablesbits->reference-addr
+                                    ia32e-page-tablesbits->res1
+                                    ia32e-page-tablesbits->ps
+                                    ia32e-page-tablesbits->pcd
+                                    ia32e-page-tablesbits->pwt
+                                    ia32e-page-tablesbits->u/s
+                                    ia32e-page-tablesbits->r/w
+                                    ia32e-page-tablesbits->p
+                                    ia32e-page-tablesbits-fix)
+                                   ())))
+  :rule-classes :congruence)
+
 (defthmd xlate-equiv-memory-and-ia32e-la-to-pa-pml4-table
   (implies (xlate-equiv-memory (double-rewrite x86-1) x86-2)
            (and
@@ -372,7 +396,8 @@
                lin-addr base-addr
                wp smep smap ac nxe r-w-x cpl x86))
     (double-rewrite x86)))
-  :hints (("Goal" :in-theory (e/d* (ia32e-la-to-pa-pml4-table)
+  :hints (("Goal" :in-theory (e/d* (ia32e-la-to-pa-pml4-table
+                                    ia32e-pml4ebits->pdpt)
                                    (bitops::logand-with-negated-bitmask
                                     accessed-bit
                                     dirty-bit
