@@ -49,7 +49,7 @@
 (include-book "std/strings/case-conversion" :dir :system)
 (include-book "centaur/bitops/part-install" :dir :system)
 (include-book "centaur/bitops/fast-logext" :dir :system)
-(include-book "centaur/gl/def-gl-rule" :dir :system)
+(include-book "centaur/gl/defthm-using-gl" :dir :system)
 (local (include-book "centaur/bitops/ihs-extensions" :dir :system))
 (local (include-book "centaur/bitops/logbitp-bounds" :dir :system))
 
@@ -79,36 +79,6 @@
 
 ;; ======================================================================
 
-;; The following macro is useful to prove a theorem using GL in a book while
-;; including GL in that book only locally: the local GL::DEF-GL-RULEDL needs
-;; GL, but the non-local DEFTHM does not.  Note that GL::DEF-GL-RULEDL does not
-;; automatically include all of GL already.  This macro is more general than
-;; the x86 ISA model and could be moved to the GL library, perhaps renamed.
-
-(defmacro defthm-using-gl
-  (name &key hyp concl g-bindings rule-classes)
-
-  (if (and hyp concl g-bindings)
-
-      (let ((gl-name (mk-name name "-GL")))
-
-        `(progn
-
-           (gl::def-gl-ruledl
-            ,gl-name
-            :hyp ,hyp
-            :concl ,concl
-            :g-bindings ,g-bindings)
-
-           (defthm ,name
-             (implies ,hyp ,concl)
-             :hints (("Goal" :in-theory (theory 'minimal-theory)
-                      :use ((:instance ,gl-name))))
-             :rule-classes ,(or rule-classes :rewrite))))
-    nil))
-
-;; ======================================================================
-
 ;; Convenient forcing idiom:
 
 (defun formal-force-list (x)
@@ -128,99 +98,79 @@
   convert between @('natp') and @('integerp'), etc."
 
   :long "<p>Definitions of constants (of the form @('2^W')) and
-functions/macros grouped in @('ruleset')s of the following form are defined (where W is at
-least a two-digit natural number; @('8') is represented as
+functions/macros grouped in @('ruleset')s of the following form are defined
+(where @('W') is at least a two-digit natural number; @('8') is represented as
 @('08')):</p>
 
 <ul>
-<li> @('nWp') belongs to @('nwp-defs') ruleset.
+
+<li>@('nWp') belongs to @('nwp-defs') ruleset.
 @({
     (define nWp (x)
       (unsigned-byte-p W x))
 })</li>
 
-<li> @('nW') belongs to @('nw-defs') ruleset.
+<li>@('nW') belongs to @('nw-defs') ruleset.
 @({
     (define nW ((x integerp))
       (mbe :logic (loghead W x)
            :exec (logand 2^W-1 x)))
 })</li>
 
-<li> @('iWp') belongs to @('iwp-defs') ruleset.
+<li>@('iWp') belongs to @('iwp-defs') ruleset.
 @({
     (define iWp (x)
       (signed-byte-p W x))
 })</li>
 
-<li> @('iW') belongs to @('iw-defs') ruleset.
+<li>@('iW') belongs to @('iw-defs') ruleset.
 @({
     (define iW ((x integerp))
       (logext W x))
 })</li>
 
-<li> @('nW-to-iW') belongs to @('nw-to-iw-defs') ruleset.
+<li>@('nW-to-iW') belongs to @('nw-to-iw-defs') ruleset.
 @({
     (define nW-to-iW ((x nWp :type (unsigned-byte W)))
         (mbe :logic (logext W x)
              :exec (if (< x 2^(W-1)) x (- x 2^W))))
 })</li>
 
-<li> @('iW-to-nW') belongs to @('iw-to-nw-defs') ruleset.
+<li>@('iW-to-nW') belongs to @('iw-to-nw-defs') ruleset.
 @({
     (define iW-to-Nw ((x iWp :type (signed-byte W)))
-        (mbe :logic (logext W x)
+        (mbe :logic (loghead W x)
              :exec (if (>= x 0) x (+ x 2^W))))
 })</li>
 
 </ul>
 
-<p> The function @('np-def-n') is used to automatically create these
-constants and functions; it also proves some associated lemmas.</p>"
+<p>The function @('np-def-n') is used to automatically create these
+constants and functions; it also proves some associated lemmas.</p>")
 
+;; Lemmas to help in the MBE proof obligations
+;; of the generated NTOI and ITON functions below:
 
-  )
+(defruledl logext-when-unsigned-byte-p-and-sign-changes
+  (implies (and (unsigned-byte-p size x)
+                (<= (expt 2 (1- size)) x))
+           (equal (logext size x) (- x (expt 2 size))))
+  :prep-books ((include-book "arithmetic-5/top" :dir :system))
+  :enable (logext logapp loghead))
 
-;; Lemmas to help in the MBE proof obligation of ntoi rules:
-
-(local
- (encapsulate
-  ()
-
-  (local (include-book "arithmetic/top-with-meta" :dir :system))
-
-  (defthm logext-is-the-same-as-ntoi-helper-1
-    (implies (and (unsigned-byte-p (1+ n) x)
-                  (<= 0 n)
-                  (< x (expt 2 n)))
-             (equal (loghead n x) x)))))
-
-(local
- (encapsulate
-  ()
-  (local (include-book "arithmetic-5/top" :dir :system))
-
-  (defthm logext-is-the-same-as-ntoi-helper-2
-    (implies (and (unsigned-byte-p (1+ n) x)
-                  (not (zp (1+ n)))
-                  (<= (expt 2 n) x))
-             (equal (logapp n x -1)
-                    (+ x (- (expt 2 (1+ n))))))
-    :hints (("Goal" :in-theory (e/d (logapp loghead) ()))))
-
-  (defthm logext-is-the-same-as-ntoi-helper-3
-    (implies (and (unsigned-byte-p (1+ n) x)
-                  (logbitp n x)
-                  (natp n)
-                  (< x (expt 2 n)))
-             (equal (logapp n x -1)
-                    x))
-    :hints (("Goal" :in-theory (e/d (logapp logbitp) ()))))
-
-  (defthm loghead-is-the-same-as-iton-helper
-    (implies (signed-byte-p n x)
-             (equal (loghead n x)
-                    (if (>= x 0) x (+ x (expt 2 n)))))
-    :hints (("Goal" :in-theory (enable loghead** logcons))))))
+(defruledl loghead-when-signed-byte-p-and-sign-changes
+  (implies (and (signed-byte-p size x)
+                (< x 0))
+           (equal (loghead size x) (+ (expt 2 size) x)))
+  :prep-books ((include-book "arithmetic-5/top" :dir :system))
+  :enable loghead
+  :prep-lemmas
+  ((defrule lemma
+     (implies (posp size)
+              (< (expt 2 (1- size))
+                 (expt 2 size)))
+     :rule-classes :linear
+     :prep-books ((include-book "arithmetic/top" :dir :system)))))
 
 (def-ruleset nwp-defs       nil)
 (def-ruleset nw-defs        nil)
@@ -230,25 +180,27 @@ constants and functions; it also proves some associated lemmas.</p>"
 (def-ruleset iw-to-nw-defs  nil)
 
 (define np-def-n (n)
-  :mode :program ;; NP-DEF-N is a :program mode function
+  :mode :program
   :guard (posp n)
   :parents (constants-conversions-and-bounds)
-  (let* ((str-n          (symbol-name (if (< n 10)
-                                          (acl2::packn (list 0 n))
-                                        (acl2::packn (list n)))))
-         (digits         (symbol-name (acl2::packn (list n))))
-         (2^XY           (mk-name "*2^" digits "*"))
-         (nXYp           (mk-name "N" str-n "P"))
-         (nXY            (mk-name "N" str-n))
-         (iXYp           (mk-name "I" str-n "P"))
-         (iXY            (mk-name "I" str-n))
-         (ntoi           (mk-name "N" str-n "-TO-I" str-n))
-         (iton           (mk-name "I" str-n "-TO-N" str-n)))
+  (let* ((str-n  (symbol-name (if (< n 10)
+                                  (acl2::packn (list 0 n))
+                                (acl2::packn (list n)))))
+         (digits (symbol-name (acl2::packn (list n))))
+         (2^XY   (mk-name "*2^" digits "*"))
+         (nXYp   (mk-name "N" str-n "P"))
+         (nXY    (mk-name "N" str-n))
+         (iXYp   (mk-name "I" str-n "P"))
+         (iXY    (mk-name "I" str-n))
+         (ntoi   (mk-name "N" str-n "-TO-I" str-n))
+         (iton   (mk-name "I" str-n "-TO-N" str-n)))
     (list
 
      `(defconst ,2^XY
         (expt 2 ,n))
 
+     ;; All the following functions are kept enabled.
+
      `(define ,nXYp (x)
         ;; XY-bit natural number recognizer
         :inline t
@@ -260,8 +212,7 @@ constants and functions; it also proves some associated lemmas.</p>"
      `(define ,nXY ((x integerp))
         ;; Truncate input to an XY-bit natural number
         ;; This function can be used to convert, say a 32-bit integer
-        ;; to a 32-bit natural number.  We choose to keep this function
-        ;; enabled.
+        ;; to a 32-bit natural number.
         :inline t
         :no-function t
         :enabled t
@@ -270,18 +221,17 @@ constants and functions; it also proves some associated lemmas.</p>"
              :exec (logand ,(1- (expt 2 n)) x)))
 
      `(define ,iXYp (x)
+        ;; XY-bit integer recognizer
         :inline t
         :no-function t
         :enabled t
         :parents (constants-conversions-and-bounds)
-        ;; XY-bit integer recognizer
         (signed-byte-p ,n x))
 
      `(define ,iXY ((x integerp))
         ;; Truncate input to an XY-bit signed integer number
         ;; This function can be used to convert, say a 32-bit natural number
-        ;; to a 32-bit integer.  We choose to keep this function
-        ;; enabled.
+        ;; to a 32-bit integer.
         :inline t
         :no-function t
         :enabled t
@@ -290,12 +240,14 @@ constants and functions; it also proves some associated lemmas.</p>"
              :exec (bitops::fast-logext ,n x)))
 
      `(define ,ntoi
+        ;; Convert natural number to integer
         :inline t
         :no-function t
         :enabled t
         :parents (constants-conversions-and-bounds)
-        ;; Convert natural number to integer
-        :guard-hints (("Goal" :in-theory (enable logext)))
+        :guard-hints (("Goal"
+                       :in-theory
+                       (enable logext-when-unsigned-byte-p-and-sign-changes)))
         ((x ,nXYp :type (unsigned-byte ,n)))
 
         (mbe :logic (logext ,n x)
@@ -304,11 +256,14 @@ constants and functions; it also proves some associated lemmas.</p>"
                      (- x ,(expt 2 n)))))
 
      `(define ,iton
+        ;; Convert integer to natural number
         :inline t
         :no-function t
         :enabled t
         :parents (constants-conversions-and-bounds)
-        ;; Convert integer to natural number
+        :guard-hints (("Goal"
+                       :in-theory
+                       (enable loghead-when-signed-byte-p-and-sign-changes)))
         ((x ,iXYp :type (signed-byte ,n)))
 
         (mbe :logic (loghead ,n x)
@@ -338,22 +293,18 @@ constants and functions; it also proves some associated lemmas.</p>"
 (defuns-np 1 2 3 4 5 6 8 9 11 12 16 17 18 20 21 22 24 25 26 27 28
   30 32 33 35 43 44 45 47 48 49 51 52 55 59 60 64 65 80 112 120 128 256 512)
 
-
 (defmacro n-size (n x)
-  ;; I prefer using n-size in functions that generate functions. E.g.,
-  ;; see gpr-add-spec-gen-fn in
-  ;; machine/instructions-spec/add-adc-sub-sbb-or-and-xor-cmp-test.lisp.
+  ;; We prefer using n-size in functions that generate functions. E.g.,
+  ;; see gpr-add-spec-gen-fn in machine/instructions/add-spec.lisp.
   (let* ((fn-name (mk-name "N"
                            (symbol-name (if (< n 10)
                                             (acl2::packn (list 0 n))
                                           (acl2::packn (list n)))))))
     `(,fn-name ,x)))
 
-
 (defmacro ntoi (n x)
-  ;; I prefer using ntoi in functions that generate functions. E.g.,
-  ;; see idiv-spec-gen in
-  ;; machine/instructions-spec/div-idiv.lisp
+  ;; We prefer using ntoi in functions that generate functions. E.g.,
+  ;; see idiv-spec-gen in machine/instructions/divide-spec.lisp.
   (let* ((val (symbol-name (if (< n 10)
                                (acl2::packn (list 0 n))
                              (acl2::packn (list n)))))
@@ -361,7 +312,7 @@ constants and functions; it also proves some associated lemmas.</p>"
     `(,fn-name ,x)))
 
 (define trunc
-  ;; I prefer using trunc in function definitions.
+  ;; We prefer using trunc in function definitions.
   ((n :type (integer 0 *))
    (x :type integer))
   :inline t
@@ -395,7 +346,7 @@ constants and functions; it also proves some associated lemmas.</p>"
                        (alistp y))))
   :hints (("Goal" :induct (revappend x y))))
 
-(defthm alist-list-to-alist
+(defthm alistp-of-list-to-alist
   (implies (alistp acc)
            (alistp (list-to-alist x i acc))))
 
@@ -435,6 +386,12 @@ constants and functions; it also proves some associated lemmas.</p>"
                           :name ,name)
                  ,@alist))))
 
+;; =============================================================================
+
+;; Maps a list of integers to a corresponding list of Booleans,
+;; treating 0 as false.  Example: (ints-to-booleans '(0 1 0 0 1)) ==>
+;; (nil t nil nil t).
+
 (defun ints-to-booleans-acc (x acc)
   (declare (xargs :guard (and (integer-listp x)
                               (true-listp acc))))
@@ -444,11 +401,6 @@ constants and functions; it also proves some associated lemmas.</p>"
                                        acc)))))
 
 (defun ints-to-booleans (x)
-
-  ;; Maps a list of integers to a corresponding list of Booleans,
-  ;; treating 0 as false.  Example: (ints-to-booleans '(0 1 0 0 1)) ==>
-  ;; (nil t nil nil t).
-
   (declare (xargs :guard (integer-listp x)))
   (ints-to-booleans-acc x nil))
 
