@@ -60,9 +60,11 @@
 
 (defnd dp () '(nil 53 11))
 
+(defnd bf () '(nil 8 8))
+
 (defnd ep () '(t 64 15))
 
-(in-theory (disable (sp) (dp) (hp) (ep)))
+(in-theory (disable (sp) (dp) (hp) (bf) (ep)))
 
 (defthm formatp-sp
   (formatp (sp)))
@@ -303,6 +305,10 @@
        (<= (+ (expo x) (bias f)) 0)
        (exactp x (+ (1- (prec f)) (bias f) (expo x)))))
 
+(defthmd drepp-exactp
+  (implies (drepp x f)
+           (exactp x (prec f))))
+
 (defund dencode (x f)
   (declare (xargs :guard (drepp x f)))
   (cat (if (= (sgn x) 1) 0 1)
@@ -330,6 +336,10 @@
            (equal (ddecode (dencode x f) f)
                   x)))
 
+(defthmd drepp<spn
+  (implies (drepp x f)
+           (< (abs x) (spn f))))
+
 ;; Smallest positive denormal:
 
 (defund spd (f)
