@@ -274,15 +274,6 @@
 	     (equal (logand x (- (expt 2 n) (expt 2 k)))
 		    (* (expt 2 k) (bits x (1- n) k)))))
 
-(defthmd logand-expt-4
-    (implies (and (natp n)
-		  (natp k)
-		  (natp l)
-		  (< l k)
-		  (<= k n))
-	     (equal (logand (- (1- (expt 2 n)) (expt 2 l)) (- (expt 2 n) (expt 2 k)))
-		    (- (expt 2 n) (expt 2 k)))))
-
 (defthmd logand-shift
     (implies (and (integerp x)
 		  (integerp y)
