@@ -30,9 +30,6 @@
 
 (include-book "round") ;floating-point rounding
 
-(include-book "add") ;support for reasoning about floating-point addition
-;                      (leading one prediction and sticky bit computation)
-
 ; Users may prefer to replace the (include-book "arith") below with:
 ; (include-book "../arithmetic/top")
 
