@@ -1,5 +1,6 @@
 ; Copyright (C) 2019, Regents of the University of Texas
 ; Written by Mihir Mehta
+; Contributing Author: Alessandro Coglio (coglio@kestrel.edu)
 ; License: A 3-clause BSD license.  See the LICENSE file distributed with ACL2.
 ; Lemmas about remove1-equal
 
@@ -27,6 +28,10 @@
   (defthm member-equal-of-remove1-equal
     (implies (not (equal x1 x2))
              (iff (member-equal x1 (remove1-equal x2 l))
-                  (member-equal x1 l)))))
+                  (member-equal x1 l))))
+
+  (defthm subsetp-equal-of-remove1-equal-left
+    (implies (subsetp-equal x y)
+             (subsetp-equal (remove1-equal a x) y))))
 
 (in-theory (disable remove1-equal))
