@@ -66,10 +66,6 @@
     fty::flexprod->ctor-name
     fty::nice-cond
     fty::find-prod-by-kind
-    fty::flexsum-case-macro-kinds
-    fty::flexsum-case-macro-conds
-    fty::flexsum-case-macro-member-special-form-expand
-    fty::flexsum-case-macro-fn
     print-base-fix
     print-base-equiv
     acl2::pos-fix
