@@ -473,7 +473,7 @@ encapsulate), and is mainly meant as a tool for macro developers.</dd>
 (defun collect-flag-mapping (gutslist)
   (if (atom gutslist)
       nil
-    (cons (cons (defguts->name-fn (car gutslist))
+    (cons (list (defguts->name-fn (car gutslist))
                 (guts->flag (car gutslist)))
           (collect-flag-mapping (cdr gutslist)))))
 
@@ -680,7 +680,8 @@ encapsulate), and is mainly meant as a tool for macro developers.</dd>
                name 'define 'defines))
        ((when (and long sub-long))
         (raise "Can't give :long in (~x1 ~x0 ...) because there is a global ~
-                :long in the (~x2 ~x0 ...) form."))
+                :long in the (~x2 ~x0 ...) form."
+               name 'define 'defines))
 
        (kwd-alist guts1.kwd-alist)
        (kwd-alist (remove1-assoc :short kwd-alist))
@@ -783,7 +784,7 @@ encapsulate), and is mainly meant as a tool for macro developers.</dd>
                               (gutslist-find-hints gutslist)))
        (flag-mapping      (collect-flag-mapping gutslist))
 
-       ((unless (no-duplicatesp-eq (strip-cdrs flag-mapping)))
+       ((unless (no-duplicatesp-eq (strip-cadrs flag-mapping)))
         (raise "Error: duplicated flag names!"))
 
        (returns-induct    (and flag-name
@@ -902,7 +903,7 @@ the @(':hyp') keyword.</p>
 <p>Using the following mutual recursion as an example:</p>
 @({
  (defines pseudo-term-vars
-   (define pseudo-term-vars ((x psuedo-termp))
+   (define pseudo-term-vars ((x pseudo-termp))
      :returns (vars)
      ...)
    (define pseudo-term-list-vars ((x pseudo-term-listp))
@@ -985,7 +986,7 @@ lemma for the overall mutually recursive proof, and doesn't export it.</p>
         (raise "~x0: The function name must be specified (with :fn)." ctx))
        (fnguts (defgutslist-find fn guts.gutslist))
        ((defguts fnguts))
-       (flag (cdr (assoc fnguts.name-fn guts.flag-mapping)))
+       (flag (cadr (assoc fnguts.name-fn guts.flag-mapping)))
        ((unless flag)
         (raise "~x0: Invalid function name ~x1 (no flag mapping)." ctx fn))
        (fnguts (defgutslist-find fn guts.gutslist))
@@ -1027,7 +1028,7 @@ lemma for the overall mutually recursive proof, and doesn't export it.</p>
         (raise "Defret-mutual needs a mutual recursion created with defines to work on."))
        (guts (cdr (assoc mutual-recursion defines-alist)))
        ((unless guts)
-        (raise "~x0 is not the name of a mutual recursion created with defines."))
+        (raise "~x0 is not the name of a mutual recursion created with defines." mutual-recursion))
        ((defines-guts guts))
        ((unless guts.flag-defthm-macro)
         (raise "No flag defthm macro for ~x0." mutual-recursion))
