@@ -13,8 +13,7 @@
 
 (in-package "ACL2")
 
-(include-book "std/testing/assert" :dir :system)
-(include-book "std/testing/eval" :dir :system)
+(include-book "std/testing/assert-bang" :dir :system)
 
 (defttag raw-lisp-for-parallelism-tests)
 
