@@ -4,7 +4,7 @@
 
 (in-package "ACL2")
 
-(include-book "std/testing/eval" :dir :system)
+(include-book "std/testing/must-fail" :dir :system)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ; First, an example from Eric Smith
