@@ -10,6 +10,8 @@
 ; nesting of inductions, suggesting considering formulation of additional
 ; lemmas.
 
+; See also nested-stobj-errors-input.lsp for additional error testing.
+
 ; See also ACL2 community book demos/modeling/nested-stobj-toy-isa.lisp for a
 ; nicer, self-contained example.
 
@@ -21,8 +23,8 @@
 
 (in-package "ACL2")
 
-(include-book "std/testing/eval" :dir :system) ; defines must-fail
-(include-book "std/testing/assert" :dir :system)
+(include-book "std/testing/must-fail" :dir :system)
+(include-book "std/testing/assert-bang" :dir :system)
 
 (defmacro local-test (&key defs run check)
 
@@ -43,8 +45,8 @@
 
 ; As promised in :doc stobj-let, we begin with an example from that :doc.
 
-; WARNING: Many of these events appear in :DOC STOBJ-LET.  If they are changed
-; below, then we should make corresponding changes to that :doc topic.
+; WARNING: Many of these events appear in :DOC NESTED-STOBJS.  If they are
+; changed below, then we should make corresponding changes to that :doc topic.
 
 (defstobj kid1 fld1)
 
@@ -691,7 +693,7 @@ TOP1-FLD (:EXEC):
         :RULE-CLASSES NIL)
 
 (defabsstobj top1{abs}
-  :concrete top1
+  :foundation top1
   :recognizer (top1{abs}p :exec top1p :logic top1$ap)
   :creator (create-top1{abs} :exec create-top1 :logic create-top1$a)
   :corr-fn corr
@@ -1197,7 +1199,7 @@ ACL2 !>
                                      (c-hidden-a-fld-update)))))
 
 (must-fail
- (defthm true-and-false-is-contradictory
+ (defthm true-and-false-is-contradictory-2
    nil
    :hints (("goal" :use true-and-false-is-true
             :in-theory (disable true-and-false-is-true)))
@@ -1256,7 +1258,7 @@ ACL2 !>
 
 (must-fail
 
-; The following should cause an error, or else the next assertion coudl fail:
+; The following should cause an error, or else the next assertion could fail:
 
  (cause-invariant-violation-program n-arr)
  :expected :hard)
@@ -1305,7 +1307,7 @@ ACL2 !>
      (mv a b))
    dd))
 
-;; want these both to remain illegal 
+;; want these both to remain illegal
 (must-fail
  (defun access-dd-as-both (dd)
    (declare (xargs :stobjs dd))
@@ -1368,21 +1370,22 @@ ACL2 !>
    (list (a-fld a) (a-fld aa))
    x))
 
-;; want this to fail its guard check
-(must-fail
- (defun access-aa-arr-as-a-arr-insufficient-guard (n m aa-arr)
-   (declare (xargs :stobjs aa-arr
-                   :guard (and (natp n)
-                               (natp m)
-                               (< n 2)
-                               (< m 2))))
-   (stobj-let
-    ((a (asi n aa-arr))
-     (aa (asi m aa-arr)))
-    (x)
-    (list (a-fld a) (a-fld aa))
-    x))
- :with-output-off nil)
+;; This formerly failed its guard verification because aliasing in array
+;; accesses is possible when n = m.  But starting in May 2021 (at the same time
+;; that ACL2 was changed to support stobj fields of abstract stobjs), such
+;; aliasing is permitted provided there is no updating involved.
+(defun access-aa-arr-as-a-arr-with-possible-aliasing (n m aa-arr)
+  (declare (xargs :stobjs aa-arr
+                  :guard (and (natp n)
+                              (natp m)
+                              (< n 2)
+                              (< m 2))))
+  (stobj-let
+   ((a (asi n aa-arr))
+    (aa (asi m aa-arr)))
+   (x)
+   (list (a-fld a) (a-fld aa))
+   x))
 
 (defstobj scal-and-arr
   (saa-sc1 :type a)
@@ -1408,5 +1411,21 @@ ACL2 !>
    (let ((aaaa (update-a-fld (not (a-fld aaaa)) aaaa)))
      (mv (list (a-fld a) (a-fld aa) (a-fld aaa)) aaaa))
    (mv x scal-and-arr)))
-  
+
 )))
+
+; After 6/28/2021, duplicate expressions in the stobj-let bindings do not cause
+; failure provided the bound stobjs aren't among the producer variables.
+(local-test
+ :defs
+ ((defstobj s1 s1-fld)
+  (defstobj s2 s2-fld :congruent-to s1)
+  (defstobj top (st :type s1))
+  (defun foo (top)
+    (declare (xargs :stobjs top))
+    (stobj-let ((s1 (st top))
+                (s2 (st top)))
+               (val1 val2)
+               (mv (s1-fld s1)
+                   (s2-fld s2))
+               (mv val1 val2 top)))))
