@@ -4,7 +4,7 @@
 
 (in-package "ACL2")
 
-(include-book "std/testing/assert" :dir :system)
+(include-book "std/testing/assert-bang" :dir :system)
 
 ;;; cw
 
