@@ -113,7 +113,7 @@
        (acc   (str::printtree-rconcat "<index_head><see topic=\"" acc))
        (acc   (file-name-mangle name acc))
        (acc   (str::printtree-rconcat "\">" acc))
-       (acc   (sym-mangle-cap name index-pkg acc))
+       (acc   (sym-mangle-cap name index-pkg nil acc))
        (acc   (str::printtree-rconcat "</see>" acc))
        (acc   (str::printtree-rconcat "</index_head>" acc))
        (acc   (cons #\Newline acc))
@@ -164,7 +164,7 @@
        (acc (str::printtree-rconcat "<parent topic=\"" acc))
        (acc (file-name-mangle (car parents) acc))
        (acc (str::printtree-rconcat "\">" acc))
-       (acc (sym-mangle-cap (car parents) base-pkg acc))
+       (acc (sym-mangle-cap (car parents) base-pkg nil acc))
        (acc (str::printtree-rconcat "</parent>" acc))
        (acc (cons #\Newline acc)))
     (add-parents (cdr parents) base-pkg acc)))
@@ -241,7 +241,7 @@
        (acc    (str::printtree-rconcat *xml-entity-stuff* acc))
        (acc    (str::printtree-rconcat "<page>" acc))
        (acc    (str::printtree-rconcat "<topic name=\"" acc))
-       (acc    (sym-mangle-cap name base-pkg acc))
+       (acc    (sym-mangle-cap name base-pkg nil acc))
        (acc    (str::printtree-rconcat "\">" acc))
        (acc    (cons #\Newline acc))
        (acc    (add-parents parents base-pkg acc))
