@@ -1,5 +1,5 @@
-; ACL2 Version 8.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2020, Regents of the University of Texas
+; ACL2 Version 8.4 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2021, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
@@ -64,6 +64,15 @@
 ; We keep this list sorted, since that makes defpkg more efficient when users
 ; choose to import these symbols; it avoids having to sort them then.
 
+; Here is the documentation string that was supplied before the optional
+; argument to defconst was eliminated after Version_8.3.
+
+;     "This is the list of ACL2 symbols that the ordinary user is extremely
+;   likely to want to include in the import list of any package created
+;   because these symbols are the basic hooks for using ACL2.  However,
+;   it is never necessary to do such importing: one can always use the
+;   acl2:: prefix."
+
   (sort-symbol-listp
    (append
     *hons-primitives* ; even for non-hons version, for compatibility of the two
@@ -89,7 +98,7 @@
         ACL2-COUNT
         ACL2-INPUT-CHANNEL-PACKAGE ACL2-NUMBER-LISTP ACL2-NUMBERP
         ACL2-ORACLE ACL2-OUTPUT-CHANNEL-PACKAGE
-        ACL2-PACKAGE ACONS ACTIVE-OR-NON-RUNEP ACTIVE-RUNEP
+        ACL2-PACKAGE ACL2-UNWIND-PROTECT ACONS ACTIVE-OR-NON-RUNEP ACTIVE-RUNEP
         ADD-BINOP ADD-CUSTOM-KEYWORD-HINT
         ADD-DEFAULT-HINTS ADD-DEFAULT-HINTS!
         ADD-INCLUDE-BOOK-DIR ADD-INCLUDE-BOOK-DIR!
@@ -100,7 +109,7 @@
         ADD-PAIR ADD-PAIR-PRESERVES-ALL-BOUNDP
         ADD-RAW-ARITY ADD-SUFFIX ADD-SUFFIX-TO-FN ADD-TIMERS
         ADD-TO-SET ADD-TO-SET-EQ ADD-TO-SET-EQL ADD-TO-SET-EQUAL
-        ALISTP ALISTP-FORWARD-TO-TRUE-LISTP
+        ALISTP ALISTP-FORWARD-TO-TRUE-LISTP ALL-ATTACHMENTS
         ALL-BOUNDP ALL-BOUNDP-PRESERVES-ASSOC
         ALL-VARS ALL-VARS1 ALL-VARS1-LST
         ALLOCATE-FIXNUM-RANGE ALPHA-CHAR-P
@@ -112,7 +121,7 @@
         ARRAY1P-FORWARD ARRAY1P-LINEAR
         ARRAY2P ARRAY2P-CONS ARRAY2P-FORWARD
         ARRAY2P-LINEAR ASET-32-BIT-INTEGER-STACK
-        ASET-T-STACK ASET1 ASET2
+        ASET-T-STACK ASET1 ASET1-TRUSTED ASET2
         ASH ASSERT$ ASSERT* ASSERT-EVENT ASSIGN ASSOC
         ASSOC-ADD-PAIR ASSOC-EQ ASSOC-EQ-EQUAL
         ASSOC-EQ-EQUAL-ALISTP ASSOC-EQUAL
@@ -166,6 +175,7 @@
         CODE-CHAR-TYPE COERCE COERCE-INVERSE-1
         COERCE-INVERSE-2 COERCE-OBJECT-TO-STATE
         COERCE-STATE-TO-OBJECT
+        COLLECT$ COLLECT$+
         COMMENT
         COMMUNITY-BOOKS
         COMMUTATIVITY-OF-*
@@ -190,12 +200,12 @@
         COMPLEX-IMPLIES1 COMPLEX-RATIONALP
         COMPLEX/COMPLEX-RATIONALP
         COMPRESS1 COMPRESS11 COMPRESS2
-        COMPRESS21 COMPRESS211 CONCATENATE COUNT-KEYS
-        DOPPELGANGER-APPLY$-USERFN DOPPELGANGER-BADGE-USERFN
+        COMPRESS21 COMPRESS211 CONCATENATE
         COND COND-CLAUSESP COND-MACRO
         CONJUGATE CONS CONS-EQUAL CONS-WITH-HINT CONSP
-        CONSP-ASSOC-EQUAL CONSTRAINT-INFO COROLLARY CPU-CORE-COUNT
-        CTX ; perhaps useful for computed hints
+        CONSP-ASSOC-EQUAL CONSTRAINT-INFO COROLLARY
+        COUNT-KEYS CPU-CORE-COUNT
+        CTX CTXP
         CURRENT-PACKAGE CURRENT-THEORY
         CW CW! CW-GSTACK CW-PRINT-BASE-RADIX CW-PRINT-BASE-RADIX!
         DECLARE
@@ -220,7 +230,7 @@
         DEFAULT-UNARY-/ DEFAULT-UNARY-MINUS
         DEFAULT-VERIFY-GUARDS-EAGERNESS
         DEFAULT-WELL-FOUNDED-RELATION
-        DEFAXIOM DEFCHOOSE DEFCONG DEFCONST
+        DEFAXIOM DEFBADGE DEFCHOOSE DEFCONG DEFCONST
         DEFEQUIV DEFEVALUATOR DEFEXEC
         DEFINE-PC-ATOMIC-MACRO DEFINE-PC-HELP
         DEFINE-PC-MACRO DEFINE-PC-META
@@ -239,7 +249,9 @@
         DIMENSIONS DISABLE DISABLE-FORCING
         DISABLE-IMMEDIATE-FORCE-MODEP
         DISABLEDP DISASSEMBLE$ DISTRIBUTIVITY
-        DMR-START DMR-STOP DOC DOC! DOCS DOUBLE-REWRITE
+        DMR-START DMR-STOP DOC DOC! DOCS
+        DOPPELGANGER-APPLY$-USERFN DOPPELGANGER-BADGE-USERFN
+        DOUBLE-REWRITE DOUBLET-LISTP
         DUMB-OCCUR DUMB-OCCUR-VAR
         DUPLICATES E/D E0-ORD-< E0-ORDINALP
         EC-CALL EIGHTH ELIMINATE-DESTRUCTORS
@@ -260,17 +272,18 @@
         EXPT-TYPE-PRESCRIPTION-NON-ZERO-BASE
         EXTEND-32-BIT-INTEGER-STACK
         EXTEND-PE-TABLE EXTEND-T-STACK
-        EXTEND-WORLD EXTRA-INFO F-GET-GLOBAL F-PUT-GLOBAL
+        EXTEND-WORLD EXTRA-INFO F-BOUNDP-GLOBAL F-GET-GLOBAL F-PUT-GLOBAL
         FAST-ALIST-CLEAN FAST-ALIST-CLEAN! FAST-ALIST-FORK FAST-ALIST-FORK!
         FAST-ALIST-FREE-ON-EXIT FC-REPORT FERTILIZE
         FGETPROP FIFTH FILE-CLOCK FILE-CLOCK-P
         FILE-CLOCK-P-FORWARD-TO-INTEGERP FILE-LENGTH$ FILE-WRITE-DATE$
         FINALIZE-EVENT-USER
         FIRST FIRST-N-AC
-        FIX FIX-TRUE-LIST FLET FLOOR FLUSH-COMPRESS
+        FIX FIX-PKG FIX-TRUE-LIST FLET FLOOR FLUSH-COMPRESS
         FMS FMS!
         FMS!-TO-STRING FMS-TO-STRING
         FMT FMT! FMT!-TO-STRING
+        FMT-HARD-RIGHT-MARGIN FMT-SOFT-RIGHT-MARGIN
         FMT-TO-COMMENT-WINDOW FMT-TO-STRING FMT1 FMT1!
         FMT1!-TO-STRING FMT1-TO-STRING
         FMX FMX!-CW FMX-CW
@@ -278,8 +291,9 @@
         FNCALL-TERM FORALL FORCE FORMULA FOURTH FUNCTION-SYMBOLP
         FUNCTION-THEORY GAG-MODE GC$
         GC-STRATEGY GC-VERBOSE GCS GENERALIZE
-        GET-CHECK-INVARIANT-RISK GET-COMMAND-SEQUENCE GET-ENFORCE-REDUNDANCY
-        GET-EVENT-DATA GET-GLOBAL
+        GET-CHECK-INVARIANT-RISK GET-COMMAND-SEQUENCE
+        GET-DEFUN-EVENT GET-ENFORCE-REDUNDANCY
+        GET-EVENT-DATA GET-GLOBAL GET-GUARD-CHECKING
         GET-IN-THEORY-REDUNDANT-OKP GET-OUTPUT-STREAM-STRING$
         GET-REGISTER-INVARIANT-RISK GET-TIMER GET-WORMHOLE-STATUS
         GETENV$ GETPROP GETPROP-DEFAULT GETPROPC GETPROPS
@@ -339,6 +353,7 @@
         LOGEQV LOGIC
         LOGIC-FNS-LIST-LISTP LOGIC-FNS-LISTP LOGIC-FNSP
         LOGIC-TERM-LIST-LISTP LOGIC-TERM-LISTP LOGIC-TERMP
+        LOGICAL-DEFUN ; deprecated in favor of get-defun-event
         LOGIOR LOGNAND LOGNOR
         LOGNOT LOGORC1 LOGORC2 LOGTEST LOGXOR LOOP$
         LOWER-CASE-P LOWER-CASE-P-CHAR-DOWNCASE
@@ -359,13 +374,14 @@
         MAYBE-FLUSH-AND-COMPRESS1
         MBE MBT MBT*
         MEMBER MEMBER-EQ MEMBER-EQUAL MEMBER-SYMBOL-NAME
+        MEMOIZE-PARTIAL
         META-EXTRACT-CONTEXTUAL-FACT META-EXTRACT-FORMULA
         META-EXTRACT-GLOBAL-FACT META-EXTRACT-GLOBAL-FACT+ META-EXTRACT-RW+-TERM
         MFC
         MFC-ANCESTORS MFC-AP MFC-CLAUSE MFC-RDEPTH MFC-RELIEVE-HYP MFC-RW
         MFC-RW+ MFC-TS MFC-TYPE-ALIST MFC-UNIFY-SUBST MFC-WORLD
         MIN MINIMAL-THEORY MINUSP
-        MOD MOD-EXPT MONITOR MONITORED-RUNES
+        MOD MOD-EXPT MONITOR MONITOR! MONITORED-RUNES
         MORE MORE! MORE-DOC MSG MSGP MUST-BE-EQUAL
         MUTUAL-RECURSION MUTUAL-RECURSION-GUARDP
         MV MV-LET MV-LIST MV-NTH MV? MV?-LET
@@ -420,7 +436,8 @@
         POSITIVE POSP POWER-EVAL PPROGN PR
         PR! PREPROCESS PRIN1$ PRIN1-WITH-SLASHES
         PRIN1-WITH-SLASHES1 PRINC$ PRINT-BASE-P PRINT-CL-CACHE PRINT-GV
-        PRINT-OBJECT$ PRINT-OBJECT$-PRESERVING-CASE PRINT-OBJECT$-SER
+        PRINT-OBJECT$ PRINT-OBJECT$+ PRINT-OBJECT$-FN
+        PRINT-OBJECT$-PRESERVING-CASE
         PRINT-RATIONAL-AS-DECIMAL
         PRINT-TIMER PROFILE
         PROG2$ PROGN PROGN! PROGN$ PROGRAM
@@ -433,7 +450,7 @@
         PUT-ASSOC PUT-ASSOC-EQ PUT-ASSOC-EQL
         PUT-ASSOC-EQUAL PUT-GLOBAL PUTPROP
         QUICK-AND-DIRTY-SUBSUMPTION-REPLACEMENT-STEP
-        QUIT QUOTE QUOTEP R-EQLABLE-ALISTP R-SYMBOL-ALISTP
+        QUIT QUOTE QUOTEP QUOTE~ R-EQLABLE-ALISTP R-SYMBOL-ALISTP
         RANDOM$ RASSOC RASSOC-EQ RASSOC-EQUAL
         RATIO RATIONAL RATIONAL-IMPLIES1
         RATIONAL-IMPLIES2 RATIONAL-LISTP
@@ -482,7 +499,8 @@
         RESET-PREHISTORY RESET-PRINT-CONTROL
         RESIZE-LIST REST RESTORE-MEMOIZATION-SETTINGS RETRACT-WORLD
         RETRIEVE RETURN-LAST RETURN-LAST-TABLE
-        REVAPPEND REVERSE REVERT-WORLD REWRITE-EQUIV REWRITE-STACK-LIMIT
+        REVAPPEND REVERSE REVERT-WORLD REWRITE-EQUIV
+        REWRITE-QUOTED-CONSTANT REWRITE-STACK-LIMIT
         RFIX ROUND RW-CACHE SATISFIES
         SAVE-AND-CLEAR-MEMOIZATION-SETTINGS SAVE-EXEC
         SEARCH SECOND
@@ -596,11 +614,15 @@
         SUBLIS SUBLIS-FN SUBLIS-FN-LST-SIMPLE SUBLIS-FN-SIMPLE
         SUBSEQ SUBSEQ-LIST SUBSEQUENCEP
         SUBSETP SUBSETP-EQ SUBSETP-EQUAL
-        SUBST SUBSTITUTE SUBSTITUTE-AC SUITABLY-TAMEP-LISTP SUMMARY
-        SWAP-STOBJS
-        SYMBOL SYMBOL-< SYMBOL-<-ASYMMETRIC
-        SYMBOL-<-IRREFLEXIVE SYMBOL-<-TRANSITIVE
-        SYMBOL-<-TRICHOTOMY SYMBOL-ALISTP
+        SUBST SUBSTITUTE SUBSTITUTE-AC SUITABLY-TAMEP-LISTP
+        SUM$ SUM$+ SUMMARY SWAP-STOBJS
+        SYMBOL
+; The following are deprecated in ACL2 Version  8.4.  See SYMBOL< etc. below
+; for current versions.
+        SYMBOL-< SYMBOL-<-ASYMMETRIC SYMBOL-<-IRREFLEXIVE
+        SYMBOL-<-TRANSITIVE SYMBOL-<-TRICHOTOMY
+; End of deprecated symbols.
+        SYMBOL-ALISTP
         SYMBOL-ALISTP-FORWARD-TO-EQLABLE-ALISTP
         SYMBOL-DOUBLET-LISTP
         SYMBOL-EQUALITY SYMBOL-LISTP
@@ -608,7 +630,10 @@
         SYMBOL-NAME
         SYMBOL-NAME-INTERN-IN-PACKAGE-OF-SYMBOL
         SYMBOL-NAME-LST
-        SYMBOL-PACKAGE-NAME SYMBOLP
+        SYMBOL-PACKAGE-NAME
+        SYMBOL< SYMBOL<-ASYMMETRIC SYMBOL<-IRREFLEXIVE
+        SYMBOL<-TRANSITIVE SYMBOL<-TRICHOTOMY
+        SYMBOLP
         SYMBOLP-INTERN-IN-PACKAGE-OF-SYMBOL
         SYNP SYNTAXP SYS-CALL SYS-CALL* SYS-CALL+ SYS-CALL-STATUS
         T T-STACK T-STACK-LENGTH T-STACK-LENGTH1
@@ -627,7 +652,7 @@
         THIRD
         THM TIME$ TIME-TRACKER TIME-TRACKER-TAU TIMER-ALISTP
         TIMER-ALISTP-FORWARD-TO-TRUE-LIST-LISTP-AND-SYMBOL-ALISTP
-        TOGGLE-PC-MACRO
+        TOGGLE-INHIBIT-WARNING TOGGLE-INHIBIT-WARNING! TOGGLE-PC-MACRO
         TOP-LEVEL TRACE! TRACE$ TRANS
         TRANS!
         TRANS-EVAL TRANS-EVAL-DEFAULT-WARNING TRANS-EVAL-NO-WARNING
@@ -664,6 +689,7 @@
         UPPER-CASE-P-FORWARD-TO-ALPHA-CHAR-P
         USER-STOBJ-ALIST USER-STOBJ-ALIST1
         VALUE VALUE-TRIPLE VERBOSE-PSTACK VERIFY
+        VERIFY-GUARD-IMPLICATION
         VERIFY-GUARDS VERIFY-GUARDS+ VERIFY-GUARDS-FORMULA
         VERIFY-TERMINATION W WALKABOUT
         WARNING! WARRANT WATERFALL-PARALLELISM WATERFALL-PRINTING
@@ -697,15 +723,7 @@
 
         DEFTHM-STD DEFUN-STD DEFUNS-STD
         I-CLOSE I-LARGE I-LIMITED I-SMALL
-        REAL-LISTP STANDARD-PART STANDARDP)))
-
-  "This is the list of ACL2 symbols that the ordinary user is extremely
-likely to want to include in the import list of any package created
-because these symbols are the basic hooks for using ACL2.  However,
-it is never necessary to do such importing: one can always use the
-acl2:: prefix."
-
-  )
+        REAL-LISTP STANDARD-PART STANDARDP))))
 
 (defpkg "ACL2-USER"
   (union-eq *acl2-exports*
