@@ -1,5 +1,5 @@
-; ACL2 Version 8.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2020, Regents of the University of Texas
+; ACL2 Version 8.4 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2021, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
@@ -26,7 +26,7 @@
 
 (defun topic-to-url-list (url chars names)
 ; Url is a url ending in "/", e.g.,
-; http://www.cs.utexas.edu/users/moore/acl2/manuals/current/manual/ .
+; http://www.cs.utexas.edu/users/moore/acl2/manuals/latest/ .
   (cond ((endp names) nil)
         (t (cons (cons (car chars)
                        (concatenate 'string
@@ -41,10 +41,10 @@
   "manual/")
 
 (defconst *combined-manual*
-  "http://www.cs.utexas.edu/users/moore/acl2/v8-3/combined-manual/")
+  "http://www.cs.utexas.edu/users/moore/acl2/v8-4/combined-manual/")
 
 (defconst *bleeding-edge-manual*
-  "http://www.cs.utexas.edu/users/moore/acl2/manuals/current/manual/")
+  "http://www.cs.utexas.edu/users/moore/acl2/manuals/latest/")
 
 (defconst *home-page-references*
   '(|The_02Tours|                       ;;; a
@@ -53,7 +53,7 @@
     programming                         ;;; d
     rule-classes                        ;;; e
     books                               ;;; f
-    note-8-3                            ;;; g   ; current release notes
+    note-8-4                            ;;; g   ; current release notes
     the-method                          ;;; h
     introduction-to-the-theorem-prover  ;;; i   ; This is not used right now.
     interesting-applications            ;;; j
@@ -277,7 +277,7 @@ alt=\"FTP icon\" border=0></a>
 <a href=\"~sg\"><img src=\"HTML/note02.gif\" alt=\"paper note icon\" border=0></a>
 </td>
 <td>
-<a href=\"~sg\">Differences from Version 8.2</a><a href=\"~sq\"> <img
+<a href=\"~sg\">Differences from Version 8.3</a><a href=\"~sq\"> <img
 src=\"HTML/twarning.gif\" alt=\"tiny warning icon\"></a>
 </td>
 <td align=center valign=MIDDLE>
@@ -404,12 +404,12 @@ href=\"~ss\">DOCUMENTATION</a></CODE>.
 
 <ul>
 
-<li><a href=\"~s2\">ACL2+Books Manual</a> (Version 8.3)</li>
+<li><a href=\"~s2\">ACL2+Books Manual</a> (Version 8.4)</li>
 
 <li><a href=\"~s3\">ACL2+Books Manual</a> (for
-\"<a href=\"HTML/installation/obtaining-and-installing.html#Bleeding-edge\">bleeding edge</a>\" distributions)</li>
+<a href=\"HTML/installation/obtaining-and-installing.html#GitHub\">GitHub</a> distributions)</li>
 
-<li><a href=\"~s1index.html?topic=ACL2____ACL2\">ACL2 User's Manual</a> (Version 8.3)</li>
+<li><a href=\"~s1index.html?topic=ACL2____ACL2\">ACL2 User's Manual</a> (Version 8.4)</li>
 
 </ul>
 
@@ -445,12 +445,19 @@ file <code>books/doc/manual/index.html</code> under your ACL2 sources directory.
 <br><hr><br>
 <H2><a NAME=\"Tools\">Community Books: Lemma Libraries and Utilities, and How to Contribute</a></H2>
 
-A companion to ACL2 is the library of <em>community books</em>, which have
-been developed by many users over the years.  These books contain definitions
-and theorems that you might find useful in your models and proofs.  In
-addition, some books contain ACL2 reasoning or analysis tools built by users.
-The <a href=\"HTML/installation/installation.html\">installation instructions</a>
-explain how to download and install the community books.
+A companion to ACL2 is the library of <em>community books</em>, which have been
+developed by many users over the years.  These books contain definitions and
+theorems that you might find useful in your models and proofs.  In addition,
+some books contain ACL2 tools built by users to help with reasoning,
+programming, interfaces, debugging, and testing; see <a
+href='http://www.cs.utexas.edu/users/moore/acl2/v8-4/combined-manual/index.html'>
+the documentation</a>.  Some relevant papers may be found by following links in
+the pages on <a
+href='http://www.cs.utexas.edu/users/moore/publications/acl2-papers.html'>
+Books and Papers about ACL2 and Its Applications</a> and the <a
+href='http://www.cs.utexas.edu/users/moore/acl2/workshops.html'>ACL2 Workshops
+Series</a>.  The <a href=\"HTML/installation/installation.html\">installation
+instructions</a> explain how to download and install the community books.
 
 <p>
 
@@ -466,14 +473,9 @@ documentation, and project members are welcome to improve it.
 
 <p>
 
-We also distribute a few interface
-tools.  For these, see the <a
-href=\"http://www.cs.utexas.edu/users/moore/publications/acl2-papers.html#Utilities\">Utilities</a>
-section of <a href=
-\"http://www.cs.utexas.edu/users/moore/publications/acl2-papers.html\">
-Books and Papers about ACL2 and Its Applications</a>.  Some of the
-papers mentioned in that collection contain utilities, scripts, or
-ACL2 books for the problem domains in question.
+(Prior to ACL2 Version 7.0 (January, 2015) books were <a
+href='http://acl2.org/books-pre-7.0/'>distributed through a different
+mechanism</a>.)
 
 <br><hr><br>
 <H2><a NAME=\"search\">Searching documentation</a></H2>
@@ -496,7 +498,7 @@ href=\"manual/index.html?topic=ACL2____ACL2-DOC\">ACL2-Doc browser</a>.
 (defun write-home-page (channel state url)
 
 ; Url is a url ending in "/", e.g.,
-; http://www.cs.utexas.edu/users/moore/acl2/manuals/current/manual/ .
+; http://www.cs.utexas.edu/users/moore/acl2/manuals/latest/ .
 
   (mv-let
    (n state)
