@@ -1,5 +1,5 @@
-; ACL2 Version 8.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2020, Regents of the University of Texas
+; ACL2 Version 8.4 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2021, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
@@ -165,13 +165,6 @@
                              (not action))))
   `(table hons 'slow-alist-warning ,action))
 
-(defn get-slow-alist-action (state)
-  (declare (xargs :stobjs state))
-  (let* ((alist   (table-alist 'hons (w state)))
-         (warning (hons-assoc-equal 'slow-alist-warning alist)))
-    (and (consp warning)
-         (cdr warning))))
-
 #+(or acl2-loop-only (not hons))
 (defn hons-acons! (key val alist)
   ;; Has an under-the-hood implementation
