@@ -1,5 +1,5 @@
-; ACL2 Version 8.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2020, Regents of the University of Texas
+; ACL2 Version 8.4 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2021, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
@@ -202,7 +202,7 @@
 
 (defun acl2-traced-fns ()
   (sort (delete-duplicates (strip-cars (ccl:advisedp t)))
-        #'symbol-<))
+        #'symbol<))
 
 (let ((temp ccl::*warn-if-redefine-kernel*))
   (setf ccl::*warn-if-redefine-kernel* nil)
